'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfAlphaFunc" "3pf"
.SH NAME
.upperok
.B pfAlphaFunc,
.B pfGetAlphaFunc
\- 
Specify alpha function and reference value
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfAlphaFunc\fP(float\ ref,
'in +2
'll -2
int\ mode);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfGetAlphaFunc\fP(float\ *ref,
'in +2
'll -2
int\ *mode);
.in
.ll
.fi
.SH PARAMETERS
.TP \w'mode\ \ 'u
\f2ref\fP
is a reference value with which to compare source alpha at
each pixel.  This value should be a float in the range 0 through 1.
.TP
\f2mode\fP
is a symbolic constant that specifies the conditional
comparison that source alpha and \f2ref\fP must pass for a pixel to be
drawn.
.RE
.SH DESCRIPTION

\f3\%pfAlphaFunc\fP sets the alpha function mode and reference value which
affects all subsequent geometry.
\f2mode\fP is a symbolic constant that specifies the conditional
comparison that source alpha and \f2ref\fP must pass for a pixel to be
drawn.
For example:
.Ex
.nf

if (source alpha \f2mode\fP \f2ref\fP)
    draw the pixel
.Ee
.fi
.na

where the alpha value boolean function \f2mode\fP is be one of:
.RS
\f3\%PFAF_ALWAYS\fP
.br
\f3\%PFAF_EQUAL\fP
.br
\f3\%PFAF_GEQUAL\fP
.br
\f3\%PFAF_GREATER\fP
.br
\f3\%PFAF_LEQUAL\fP
.br
\f3\%PFAF_LESS\fP
.br
\f3\%PFAF_NEVER\fP
.br
\f3\%PFAF_NOTEQUAL\fP
.br
\f3\%PFAF_OFF\fP
.RE

If it was desired to only draw pixels whose alpha value was greater
than or equal to 50% of the representable range, then a \f2mode\fP of
\f3\%PFAF_GEQUAL\fP and a \f2ref\fP of 0.5 would produce the hardware pixel
rendering conditional:
.Ex
.nf

if (source alpha \f3\%PFAF_GEQUAL\fP \f3\%0.5\fP)
    draw the pixel
.Ee
.fi
.na


The the default \f2mode\fP is \f3\%PFAF_OFF\fP and default \f2ref\fP value is
0.  The alpha function and reference value state elements are
identified by the \f3\%PFSTATE_ALPHAFUNC\fP mode and \f3\%PFSTATE_ALPHAREF\fP value tokens
respectively.  Use these tokens with \f3\%pfGeoState::setMode\fP and
\f3\%pfGeoState::setVode\fP, respectively,
to set the alpha function and reference value of a
pfGeoState and with \f3\%pfOverride\fP to override subsequent alpha
function and reference value changes.

Here is an example:
.Ex
.nf

/*
 * Setup pfGeoState so that only pixels whose alpha is > 40
 * are drawn once the pfGeoState is applied with pfGeoState::apply.
 */
gstate->setMode(PFSTATE_ALPHAFUNC, PFAF_GREATER);
gstate->setVal(PFSTATE_ALPHAREF, (40.0f/255.0f));

/*
 * Override alpha function.  The alpha reference value can still
 * be changed.
 */
pfOverride(PFSTATE_ALPHAFUNC, PF_ON);

/*
 * All subsequent attempts to set alpha function will be ignored
 * until pfOverride is called to unlock it.
 */
.Ee
.fi
.na


\f3\%pfAlphaFunc\fP is a display-listable command.  If a pfDispList has been
opened by \f3\%pfDispList::open\fP, \f3\%pfAlphaFunc\fP will not have immediate
effect but will be captured by the pfDispList and will only have effect
when that pfDispList is later drawn with \f3\%pfDispList::draw\fP.

\f3\%pfGetAlphaFunc\fP copies the current alpha function reference value
and mode into \f2ref\fP and \f2mode\fP respectively.
.SH NOTES

\f3\%pfAlphaFunc\fP is typically used for textures with alpha that simulate
trees and other complicated geometry having many holes.
See the OpenGL \f3\%glAlphaFunc\fP manual page for further details.
.SH SEE ALSO
afunction,
glAlphaFunc,
pfDispList,
pfGeoState,
pfState
