'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfAntialias" "3pf"
.SH NAME
.upperok
.B pfAntialias,
.B pfGetAntialias
\- 
Specify antialiasing mode
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfAntialias\fP(int\ mode);
.nf
.fi
.TP
int
\f3\%pfGetAntialias\fP(void);
.fi
.SH PARAMETERS
.TP \w'mode\ \ 'u
\f2mode\fP
is a symbolic constant and is one of:
.RS
.TP 20
\f3\%PFAA_OFF\fP
Antialiasing will be disabled.
.TP
\f3\%PFAA_ON\fP
Antialiasing will be enabled.
The antialiasing mechanism used depends on the machine type.
.RE
.SH DESCRIPTION

\f3\%pfAntialias\fP sets the hardware antialiasing mode.  Geometry drawn
subsequent to calling \f3\%pfAntialias\fP will be antialiased according to
\f2mode\fP.  The antialiasing mechanism used is machine-dependent:
multisampling on InfiniteReality and RealityEngine systems and
non-multisampling on all others.  In addition, if available,
\f3\%pfAntialias\fP will enable a special hardware mode that efficiently
renders points using multisampled circles rather than squares.  See the
\f3\%SGIS_multisample\fP
section of the OpenGL \f3\%glIntro(3g)\fP man page for more detailed
information on multisampled antialiasing.

If \f2mode\fP is \f3\%PFAA_ON\fP, then antialiasing will be enabled.  On
machines which do not support multisampling, \f3\%PFAA_ON\fP will enable
line and/or point antialiasing if either are supported in hardware.
Polygons will not be antialiased.  In this case it is recommended that
\f3\%pfAntialias\fP be enabled only for points and lines since it may reduce
the speed of polygon rendering.  Line or point antialiasing can be
explicitly enabled by specifying either \f3\%PFAA_LINE_SMOOTH\fP or
\f3\%PFAA_POINT_SMOOTH\fP as the \f2mode\fP for \f3\%pfAntialias\fP.

X windows cannot have their framebuffer resources reconfigured.  X
windows for OpenGL are, by default, created with
multisample buffers if they are available in the current hardware
configuration.  The default configuration, if available will be 8
subsamples, 24 bits of depth buffer, and 4 bits of stencil.  If this is
not available, 1 bit of stencil will be used, and then 4 subsamples
will be allocated if 8 are not still available.  The exact framebuffer
configuration of windows can be specified via
\f3\%pfWindow::setFBConfigAttrs\fP.

For X windows, if \f2mode\fP is \f3\%PFAA_OFF\fP, the antialiasing mode will
be disabled but the buffers cannot be deallocated and there might be
associated framebuffer operations that are not truly disabled. Because
of this, the full performance benefit expected by turning off
antialiasing may not be achieved.

The antialiasing mode state element is identified by the
\f3\%PFSTATE_ANTIALIAS\fP token.  Use this token with
\f3\%pfGeoState::setMode\fP to set the antialiasing
mode of a pfGeoState and with \f3\%pfOverride\fP to override subsequent
antialiasing mode changes.

\f3\%pfAntialias\fP is a display-listable command.  If a pfDispList has
been opened by \f3\%pfDispList::open\fP,
\f3\%pfAntialias\fP will not have immediate effect but will be captured by
the pfDispList and will only have effect when that pfDispList is later
drawn with \f3\%pfDispList::draw\fP.

\f3\%pfGetAntialias\fP returns the current antialiasing mode.

Example 1:
.Ex
.nf

/* Set up 'antialiased' pfGeoState */
gstate->setMode(PFSTATE_ANTIALIAS, PFAA_ON);

/* Attach gstate to gset */
gset->setGState(gstate);

/* Draw antialiased gset */
gset->draw();
.Ee
.fi
.na


Example 2:
.Ex
.nf

/* Override antialiasing mode to PFAA_OFF */
pfAntialias(PFAA_OFF);
pfOverride(PFSTATE_ANTIALIAS, PF_ON);
.Ee
.fi
.na

.SH NOTES



\f3\%pfQueryFeature\fP can be used to determined what features
are available on the current hardware configuration.
\f3\%pfQuerySys\fP can be used to query the exact extent of hardware resources,
such as number of subsamples available for multisampling.

When using antialiasing without multisampling,
blending is used which may conflict with other transparency modes.  Specifically,
all geometry will be blended which may cause artifacts and may
substantially reduce performance.  For this reason \f3\%pfAntialias\fP
should be used with discretion on all but InfiniteReality and RealityEngine systems.

In the default framebuffer configurations, the 4 bit of stencil buffer
is allocated to support depth complexity fill statistics; see the
\f3\%pfStats\fP man page for more information.  1 bit of stencil is
required for the support of high quality decals; see the \f3\%pfDecal\fP man page
for more information.

Not all machines support stencil planes
and in these cases, stencil bits will not be allocated.
The Extreme graphics platforms only support stencil
with reduced depth buffer resolution and so stencil will not be
allocated by default.

If a window has been configured with
multisample buffers, the state of pfAntialias() is used internally
to track whether or not multisampling is being done.  This knowledge
is used for doing the fast TAG clear \f3\%pfClear()\fP, and for
drawing multisampled points. OpenGL Performer will not detect
a GL call made to enable or disable multisampling so if you
do this you must return state to match OpenGL Performer's internal
state or the results will be undefined.

.SH SEE ALSO
blendfunction,
glBlendFunc,
linesmooth,
glHint(GL_LINE_SMOOTH_HINT),
pntsmooth,
glHint(GL_POINT_SMOOTH_HINT),
mssize,
multisample,
glIntro,
pfQueryFeature,
pfQuerySys,
pfWindow,
pfChooseFBConfig,
pfDispList,
pfGeoState,
pfOverride,
pfState
