'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfBox" "3pf"
.SH NAME
.upperok
.B pfBox
\- 
Operate on axis-aligned bounding boxes
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void*\ \ \ 'u
#include <Performer/pr/pfGeoMath.h>
.nf
.fi
.TP
void*
\f3\%pfBox::operator new\fP(size_t);
.nf
.fi
.TP
void*
\f3\%pfBox::operator new\fP(size_t,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
\ 
\f3\%pfBox::pfBox\fP();
.nf
.fi
.TP
\ 
\f3\%pfBox\fP(const\ pfVec3&\ _min,
'in +2
'll -2
const\ pfVec3&\ _max);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBox::makeEmpty\fP(void);
.nf
.fi
.TP
void
\f3\%pfBox::extendBy\fP(const\ pfVec3\ &pt);
.nf
.fi
.TP
void
\f3\%pfBox::extendBy\fP(const\ pfBox\ *box);
.nf
.fi
.TP
void
\f3\%pfBox::around\fP(const\ pfVec3\ *pts,
'in +2
'll -2
int\ npt);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBox::around\fP(const\ pfBox\ **boxes,
'in +2
'll -2
int\ nbox);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBox::around\fP(const\ pfSphere\ **sphs,
'in +2
'll -2
int\ nsph);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBox::around\fP(const\ pfCylinder\ **cyls,
'in +2
'll -2
int\ ncyl);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBox::contains\fP(const\ pfVec3\ &pt);
.nf
.fi
.TP
int
\f3\%pfBox::contains\fP(const\ pfBox\ *box2);
.nf
.fi
.TP
int
\f3\%pfBox::contains\fP(const\ pfSphere\ *sphere);
.nf
.fi
.TP
int
\f3\%pfBox::isect\fP(const\ pfSeg*\ seg,
'in +2
'll -2
float*\ d1,
float*\ d2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBox::xform\fP(const\ pfBox\ *box,
'in +2
'll -2
const\ pfMatrix\ &xform);
.in
.ll
.fi

.Ex
.nf

struct pfBox
{
    pfVec3      min;
    pfVec3      max;
};
.Ee
.fi
.na

.SH DESCRIPTION

A pfBox is an axis-aligned box which can be used for intersection tests
and for maintaining bounding information about geometry.  A box
represents the axis-aligned hexahedral volume: (x, y, z) where min[0]
<= x <= max[0], min[1] <= y <= max[1] and min[2] <= z <= max[2].  pfBox
is a public struct whose data members \f2min\fP and \f2max\fP may be
operated on directly.

The default constructor \f3\%pfBox\fP() is empty and does no
initialization.  There is also a constructor
\f3\%pfBox\fP(const pfVec3& _min, const pfVec3& _max) that
will directly initialize the pfBox's max and min values.
\f3\%new\fP(arena) allocates a pfBox from the specified
memory arena, or from the heap if \f2arena\fP is NULL.  \f3\%new\fP
allocates a pfBox from the default memory arena (see
\f3\%pfGetSharedArena\fP).  pfBoxes can also be created automatically on
the stack or statically.  pfBoxes allocated with \f3\%new\fP can be
deleted with \f3\%delete\fP or \f3\%pfDelete\fP.

\f3\%pfBox::makeEmpty\fP sets the
pfBox to appear empty to extend operations.

\f3\%pfBox::extendBy\fP extends the size of the pfBox to include the point \f2pt\fP.

\f3\%pfBox::extendBy\fP extends the size of the pfBox to include the box \f2box\fP.

The variations of the member function \f3\%pfBox::around\fP
set the pfBox to be an axis-aligned box encompassing
the given primitives.  \f2npt\fP, \f2nbox\fP, \f2ncyls\fP and \f2nsph\fP are
the number of points, boxes, and spheres in the respective primitive
lists.

\f3\%pfBox::contains(const pfVec3, ...)\fP returns
\f3\%TRUE\fP or \f3\%FALSE\fP depending on whether the point \f2pt\fP is in the
interior of the specified box.

The return value from \f3\%pfBox::contains(const
pfBox*, ...)\fP is the OR of one or more bit fields.  The returned
value may be:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
The intersection of the  box argument and the
box is empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP:
The intersection of the  box argument and the
box is definitely non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP:
The  box argument is non-empty and lies entirely
inside the  box.
.RE

The return value from \f3\%pfBox::contains(const
pfSphere*)\fP is the OR of one or more bit fields.  The returned
value may be:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
The intersection of the box and the sphere is empty, or the box is empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP:
The intersection of the box and the sphere is non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP:
The sphere lies entirely inside the box, or the sphere is empty.
.RE

\f3\%pfBox::isect\fP intersect the line segment
\f2seg\fP with the volume of an axis-aligned pfBox.
The possible return values include all of the above as well as:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
\f2seg\fP lies entirely in the exterior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_START_IN\fP:
The starting point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_END_IN\fP:
The ending point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP | \f3\%PFIS_START_IN\fP | \f3\%PFIS_END_IN\fP:
Both end points of \f2seg\fP lie in the interior.
.RE

If \f2d1\fP and \f2d2\fP are non-NULL, on return from
\f3\%pfBox::isect\fP
they contain the starting and ending positions of the line segment
(0 <= d1 <= d2 <= seg->length) intersected with the specified volume.

\f3\%pfBox::xform\fP sets the pfBox to
a box which contains \f2box\fP as transformed by the matrix \f2xform\fP,
i.e. a box around (\f2box\fP * \f2xform\fP).  Because transformed boxes
must be axis-aligned, most rotations cause the box to grow, and the
transformation is not reversed by the inverse rotation.

.SH NOTES

The bit fields returned by the contains functions are structured so
that bitwise AND-ing the results of sequential tests can be used to
compute composite results, e.g. testing exclusion against a number
of half spaces.

Because pfBoxes are axially aligned, they tend to grow when
transformed.  Hence, they are best for static geometry or other cases
in which the bounding geometry does not need to be transformed.
.SH SEE ALSO
pfSeg,
pfSphere
