'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfClear" "3pf"
.SH NAME
.upperok
.B pfClear
\- 
Clear specified graphics buffers
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
void
\f3\%pfClear\fP(int\ which,
'in +2
'll -2
const\ pfVec4\ color);
.in
.ll
.fi
.SH PARAMETERS
.TP \w'which\ \ 'u
\f2which\fP
is a mask that specifies which buffers are to be cleared.
\f2which\fP is a bitwise OR of:
.RS
.TP 20
\f3\%PFCL_COLOR\fP
Clear color buffer to \f2color\fP.
.TP
\f3\%PFCL_DEPTH\fP
Clear depth buffer to maximum value of our defined depth range.
.TP
\f3\%PFCL_MSDEPTH\fP
Fast clear of the multisample depth buffer.
.TP
\f3\%PFCL_STENCIL\fP
Clear stencil buffer to 0.
.TP
\f3\%PFCL_DITHER\fP
Enable dithering during the color clear.  By default, \f3\%pfClear\fP
turns off dithering for color clears.
.RE
.TP
\f2color\fP
specifies the red, green, blue, and alpha components of
the color buffer clear color.  Each component is defined in the range
0.0 to 1.0.  If \f2color\fP is NULL then a black fully opaque color
will be used.
.SH DESCRIPTION

\f3\%pfClear\fP clears the buffers specified by \f2which\fP in the current
graphics window.  The actual screen area cleared depends on many
GL state settings including viewport and screen or scissor mask
(OpenGL \f3\%glScissor\fP),
current draw buffer (front, back, left, right, overlay, etc.),
and the existence of a depth buffer for \f3\%PFCL_DEPTH\fP and stencil
buffer for \f3\%PFCL_STENCIL\fP.
See the OpenGL \f3\%glClear(3g)\fP reference
page for more details.

If \f2which\fP includes \f3\%PFCL_COLOR\fP and \f2color\fP is NULL, then any
selected color buffer will be cleared to black fully opaque pixels
using \f3\%glColor4f(0,0,0,1)\fP in OpenGL.

\f3\%PFCL_MSDEPTH\fP has effect only when multisampling (See
\f3\%pfAntialias\fP).  In this case, instead of writing the maximum depth
value into each individual pixel subsample, each pixel is "tagged" as
having the maximum depth value.  This clear is much faster than a full
depth buffer clear; however, the color buffer is not cleared so
results from previous frames will be left in the color buffer if not
redrawn.  This requires that each pixel in the viewport be covered by
geometry.  Often this is accomplished by drawing one or more large
background polygons (often textured) at the far clip plane to "clear"
the framebuffer to an interesting background rather than depth buffer
and then incurring the additional cost of clearing drawing background
polygons.
This requires that the background rendering disable depth
buffer testing (e.g. \f3\%glDepthFunc(GL_ALWAYS)\fP in OpenGL).
Otherwise, a normal depth
buffer clear will be required if multisampling is not in use or not
supported in the current framebuffer configuration.
Note that the background drawing should leave depth buffering enabled
so that it's depth values will be written.

The follow example shows how to clear all buffers with one \f3\%pfClear\fP call:
.Ex
.nf

/*
 * Clear color buffer to black, depth buffer to the maximum depth value,
 * and stencil buffer to 0.
 */
pfClear(PFCL_DEPTH | PFCL_COLOR | PFCL_STENCIL, NULL);
.Ee
.fi
.na


\f3\%pfClear\fP is a display-listable command.  If a pfDispList has been
opened by \f3\%pfDispList::open\fP, \f3\%pfClear\fP will not have immediate effect
but will be captured by the pfDispList and will only have effect when
that pfDispList is later drawn with \f3\%pfDispList::draw\fP.
.SH NOTES

\f3\%PFCL_MSDEPTH\fP is only available on RealityEngine systems, and then
only in the multisample antialiasing mode.  For performance reasons,
the depth buffer for the entire window rather than just the current
viewport is cleared with OpenGL on Indy, i.e. scissoring is disabled.
.SH SEE ALSO
pfAntialias,
pfDispList,
glClear,
glDepthFunc,
clear,
multisample,
gconfig,
zclear,
zfunction,
czclear
