'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfCylinder" "3pf"
.SH NAME
.upperok
.B pfCylinder
\- 
Operations on cylinder definitions.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void*\ \ \ 'u
#include <Performer/pr/pfGeoMath.h>
.nf
.fi
.TP
void*
\f3\%pfCylinder::operator new\fP(size_t);
.nf
.fi
.TP
void*
\f3\%pfCylinder::operator new\fP(size_t,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
\ 
\f3\%pfCylinder::pfCylinder\fP();
.nf
.fi
.TP
\ 
\f3\%pfCylinder\fP(const\ pfVec3&\ _center,
'in +2
'll -2
float\ _radius,
const\ pfVec3&\ _axis,
float\ _halfLength);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylinder::makeEmpty\fP(void);
.nf
.fi
.TP
void
\f3\%pfCylinder::around\fP(const\ pfSeg\ **segs,
'in +2
'll -2
int\ nseg);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylinder::around\fP(const\ pfBox\ **boxes,
'in +2
'll -2
int\ nbox);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylinder::around\fP(const\ pfVec3\ *pts,
'in +2
'll -2
int\ npt);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylinder::around\fP(const\ pfCylinder\ **cyls,
'in +2
'll -2
int\ ncyl);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylinder::around\fP(const\ pfSphere\ **sphs,
'in +2
'll -2
int\ nsph);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylinder::extendBy\fP(const\ pfBox\ *sph);
.nf
.fi
.TP
void
\f3\%pfCylinder::extendBy\fP(const\ pfSphere\ *sph);
.nf
.fi
.TP
void
\f3\%pfCylinder::extendBy\fP(const\ pfCylinder\ *cyl);
.nf
.fi
.TP
int
\f3\%pfCylinder::contains\fP(const\ pfVec3\ &pt);
.nf
.fi
.TP
int
\f3\%pfCylinder::isect\fP(const\ pfSeg*\ seg,
'in +2
'll -2
float*\ d1,
float*\ d2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylinder::orthoXform\fP(const\ pfCylinder\ *cyl,
'in +2
'll -2
const\ pfMatrix\ &xform);
.in
.ll
.fi

.Ex
.nf

struct pfCylinder
{
    pfVec3   center;
    float    radius;
    pfVec3   axis;
    float    halfLength;
};
.Ee
.fi
.na

.SH DESCRIPTION

A pfCylinder represents a cylinder of finite length.  The routines
listed here provide means of creating and extending cylinders for use
as bounding geometry around groups of line segments.  The cylinder is
defined by its \f2center\fP, \f2radius\fP, \f2axis\fP and \f2halfLength\fP.
The routines assume \f2axis\fP is a vector of unit length, otherwise
results are undefined.  pfCylinder is a public struct whose data
members \f2center\fP, \f2radius\fP, \f2axis\fP and \f2halfLength\fP may be
operated on directly.

The default constructor \f3\%pfCylinder\fP() is empty and does no
initialization.  There is also a constructor that will do direct
initialization of the pfCylinder's center, radius, axis, and
half length.  \f3\%new\fP(arena) allocates a pfCylinder from the
specified memory arena, or from the heap if \f2arena\fP is NULL.
\f3\%new\fP allocates a pfCylinder from the default memory arena (see
\f3\%pfGetSharedArena\fP).  pfCylinders can also be created automatically
automatically on the stack or statically.  pfCylinders allocated with
\f3\%new\fP can be deleted with \f3\%delete\fP or \f3\%pfDelete\fP.

\f3\%pfCylinder::makeEmpty\fP sets the
pfCylinder so that it appears empty to other operations.

\f3\%pfCylinder::around(const pfSeg**, int)\fP,
\f3\%pfCylinder::around(const pfVec3*, int)\fP,
\f3\%pfSphere::around(const pfSphere**, int)\fP
and \f3\%pfCylinder::around(const pfBox**,
int)\fP set the pfCylinder to a cylinder which
contains a set of line segments, points, spheres or boxes,
respectively.  These routines are passed the address of an array of
pointers to the objects to be encompassed along with the number of
objects.

\f3\%pfCylinder::extendBy(const pfBox *)\fP,
\f3\%pfCylinder::extendBy(const pfSphere
*)\fP, and \f3\%pfCylinder::extendBy(const
pfCylinder *)\fP set the pfCylinder to a cylinder
which contains both the pfSphere  and the box \f2box\fP,
the sphere \f2sph\fP or the cylinder \f2cyl\fP, respectively.

\f3\%pfCylinder::contains\fP returns \f3\%TRUE\fP or
\f3\%FALSE\fP depending on whether the point \f2pt\fP is in the interior of
the specified pfCylinder.

\f3\%pfCylinder::isect\fP intersects the line segment \f2seg\fP with the volume
of the cylinder \f2cyl\fP.  The possible return values
are:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
\f2seg\fP lies entirely in the exterior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_START_IN\fP:
The starting point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_END_IN\fP:
The ending point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP | \f3\%PFIS_START_IN\fP | \f3\%PFIS_END_IN\fP:
Both end points of \f2seg\fP lie in the interior.
.RE

If \f2d1\fP and \f2d2\fP are non-NULL, on return from
\f3\%pfCylinder::isect\fP they contain the starting
and ending positions of the line segment (0 <= d1 <= d2 <=
seg->length) intersected with the cyl.

\f3\%pfCylinder::orthoXform\fP sets the
pfCylinder to the cylinder \f2cyl\fP transformed by the orthogonal
transformation \f2xform\fP.
the pfCylinder = \f2cyl\fP * \f2xform\fP.
If \f2xform\fP is not an orthogonal transformation the results are undefined.
.SH NOTES

C++ does not support array deletion (i.e. \f3\%delete[]\fP) for
arrays of objects allocated new operators that take additional
arguments.  Hence, the array deletion operator \f3\%delete[]\fP should not
be used on arrays of objects created with \f3\%new\fP(arena) pfVec3[n].
.SH SEE ALSO
pfBox,
pfCylinder,
pfSeg,
pfSphere,
pfVec3
