'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfFilePath" "3pf"
.SH NAME
.upperok
.B pfFilePath,
.B pfGetFilePath,
.B pfFindFile
\- 
Locate files using a search path.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'const\ char\ *\ \ \ 'u
#include <unistd.h>
.nf
.fi
.TP
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfFilePath\fP(const\ char\ *path);
.nf
.fi
.TP
const char *
\f3\%pfGetFilePath\fP(void);
.nf
.fi
.TP
int
\f3\%pfFindFile\fP(const\ char\ *file,
'in +2
'll -2
char\ path[PF_MAXSTRING],
int\ amode);
.in
.ll
.fi
.SH DESCRIPTION

\f3\%pfFilePath\fP specifies one or more search path locations.  These
locations are directories to be searched for data files by OpenGL
Performer applications.  This information is used by the
\f3\%pfFindFile\fP function.  The \f2path\fP argument to \f3\%pfFilePath\fP
is a colon-separated list of directory pathnames similar to the
\f3\%PATH\fP environment variable.  Here is a simple example:
.Ex
.nf

pfFilePath("/usr/bin:/usr/sbin:/usr/local/bin");
.Ee
.fi
.na

\f3\%pfGetFilePath\fP returns the path list set using \f3\%pfFilePath\fP
or \f3\%NULL\fP if no path has yet been set.  The string returned is
identical in format to the one set via \f3\%pfFilePath\fP; each
of the directory names is colon separated.

\f3\%pfFindFile\fP attempts to find \f2file\fP amongst the paths set
in the environment variable \f3\%PFPATH\fP and by \f3\%pfFilePath\fP.
It also tests the file's access mode against the \f2amode\fP
argument.  OpenGL Performer routines which access files use
\f3\%pfFindFile\fP.

The \f3\%PFPATH\fP environment variable is interpreted in the same
format used for the \f3\%pfFilePath\fP.  A C-shell example that specifies
the directories "/usr/data" and "/usr/share/Performer/data" is:
.Ex
.nf

setenv PFPATH "/usr/data:/usr/share/Performer/data"
.Ee
.fi
.na

The search logic used by \f3\%pfFindFile\fP is this:
.RS
.TP 5
1.
First the file is sought exactly as named by \f2file\fP.  If it exists and
passes the subsequent access test described below, then \f2file\fP
will be returned in the \f2path\fP argument.
.TP
2.
If the file was not found or was not accessible, then each of the
locations defined by the \f3\%PFPATH\fP environment variable are
prepended to the \f2file\fP argument and tested.  If it exists and
passes the subsequent access test described below, then the
complete path name will be returned in the \f2path\fP argument.
.TP
3.
If the file has still not been successfully located, then each of
the locations defined by the most recent call to \f3\%pfFilePath\fP will
be prepended to the \f2file\fP argument and tested.  If it exists and
passes the subsequent access test described below, then the
complete path name will be returned in the \f2path\fP argument.
.TP
4.
If all of these efforts are fruitless, then \f3\%pfFindFile\fP will give
up and return a NULL string in the \f2path\fP argument.
.RE

The mere existence of \f2file\fP in one of the indicated directories is
not sufficient, the file must also be accessible in the access mode
defined by \f2amode\fP.  This mode is a bitfield composed by OR-ing
together the permission attributes defined in \f2<unistd.h>\fP and
listed in the following table:

.TS
center tab($) ;
c | c | c
cB | c | l .
Mode Token$Mode Value$Action
_
R_OK$0x4$Read permission
W_OK$0x2$Write permission
X_OK$0x1$Execute and search
F_OK$0x0$Existence of file
.TE

If the bits set in  \f2amode\fP are also set in the file's access
permission mode, then the complete path is copied into \f2path\fP
and \f3\%TRUE\fP is returned indicating success.  If the access modes
are not similar, then the search continues until there are no
more paths to search and \f3\%FALSE\fP is returned indicating failure.
.SH NOTES

\f3\%pfFile::create\fP and
\f3\%pfFile::open\fP do not use \f3\%pfFindFile\fP.
This is because the search implied can be unpredictably slow when
remote directories are present in the search path.
.SH SEE ALSO
access
