'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfFrame" "3pf"
.SH NAME
.upperok
.B pfFrameRate,
.B pfGetFrameRate,
.B pfFieldRate,
.B pfGetFieldRate,
.B pfVideoRate,
.B pfGetVideoRate,
.B pfSync,
.B pfFrame,
.B pfAppFrame,
.B pfGetFrameCount,
.B pfFrameTimeStamp,
.B pfGetFrameTimeStamp,
.B pfGetFrameTimeFlux,
.B pfPhase,
.B pfGetPhase
\- 
Set and get system frame and video rate, phase, and frame count.
Synchronize and initiate frame.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfFlux*\ \ \ 'u
#include <Performer/pf.h>
.nf
.fi
.TP
float
\f3\%pfFrameRate\fP(float\ rate);
.nf
.fi
.TP
float
\f3\%pfGetFrameRate\fP(void);
.nf
.fi
.TP
int
\f3\%pfFieldRate\fP(int\ fields);
.nf
.fi
.TP
int
\f3\%pfGetFieldRate\fP(void);
.nf
.fi
.TP
void
\f3\%pfVideoRate\fP(float\ vrate);
.nf
.fi
.TP
float
\f3\%pfGetVideoRate\fP(void);
.nf
.fi
.TP
int
\f3\%pfSync\fP(void);
.nf
.fi
.TP
int
\f3\%pfFrame\fP(void);
.nf
.fi
.TP
int
\f3\%pfAppFrame\fP(void);
.nf
.fi
.TP
int
\f3\%pfGetFrameCount\fP(void);
.nf
.fi
.TP
void
\f3\%pfFrameTimeStamp\fP(double\ time);
.nf
.fi
.TP
double
\f3\%pfGetFrameTimeStamp\fP(void);
.nf
.fi
.TP
pfFlux*
\f3\%pfGetFrameTimeFlux\fP(void);
.nf
.fi
.TP
void
\f3\%pfPhase\fP(int\ phase);
.nf
.fi
.TP
int
\f3\%pfGetPhase\fP(void);
.fi
.SH DESCRIPTION

OpenGL Performer is designed to run at a fixed frame rate.  The \f2rate\fP
argument to \f3\%pfFrameRate\fP specifies the desired rate in units of
frames per second.  The actual rate used is based on the video timing
of the display hardware.  \f2rate\fP is rounded to the nearest frame rate
which corresponds to an integral multiple of video fields.

For a 60Hz video rate, possible frame rates are (in Hz) 60.0, 30.0,
20.0, 15.0, 12.0, 10.0, 8.57, 7.5, 6.67, and 6.0.  These rates would
mean that the number of fields per frame would range from 1 (for 60Hz)
to 10 (for 6Hz).  \f3\%pfFrameRate\fP returns the actual frame rate used or
-1.0 if it is called before \f3\%pfConfig\fP.

\f3\%pfVideoRate\fP specifies the system video rate as \f2vrate\fP fields per
second. If \f3\%pfVideoRate\fP is not called, then OpenGL Performer
determines the video field rate at \f3\%pfConfig\fP time and will not be
aware of changes in video timing made during application run-time until
\f3\%pfVideoRate\fP is called.

\f3\%pfGetVideoRate\fP returns the video timing in number of video fields
per second or -1.0 if it is called before the video rate has been
determined.  The OpenGL Performer video clock (see \f3\%pfInitVClock\fP) runs
at this video field rate and is initialized to 0 by \f3\%pfConfig\fP.

An alternate way of specifying a desired frame rate is
\f3\%pfFieldRate\fP.  \f2fields\fP is the number of video fields per
simulation frame.  The corresponding frame rate will then be the video
field rate (see \f3\%pfGetVideoRate\fP) divided by \f2fields\fP.
\f3\%pfGetFieldRate\fP returns the number of video fields per simulation
frame.

Frame rate is a per-machine metric and is used by all pfPipes.  It
controls the rate at which multiprocessing pipelines run and affects
computed system load and related stress metrics (see
\f3\%pfChannel::setStress\fP).  Since frame rate is
global it follows that all hardware pipelines used by a single OpenGL
Performer application should be genlocked, i.e., the video signals are
synchronized by hardware.  Otherwise the video signals of the pipes
will be out of phase, reducing graphics throughput and increasing
latency.  Genlock is crucial for proper multipipe operation and
requires some simple, platform-specific cabling and software
configuration through the \f3\%setmon\fP call.

Depending on the phase as is discussed below, \f3\%pfSync\fP synchronizes
the application process with the frame rate specified by
\f3\%pfFrameRate\fP (when phase is \f3\%PFPHASE_LOCK\fP or \f3\%PFPHASE_FLOAT\fP),
or to the system rendering rate (when phase is \f3\%PFPHASE_FREE_RUN\fP or
\f3\%PFPHASE_LIMIT\fP).  In the first case, \f3\%pfSync\fP sleeps until the
next frame boundary, then awakens and returns control to the
application.  In the second case, \f3\%pfSync\fP sleeps until the draw
process begins rendering a new frame or returns immediately if in
single-process operation.  \f3\%pfSync\fP returns the current frame count
and should only be called by the application process when
multiprocessing.

\f3\%pfFrame\fP initiates a new frame of OpenGL Performer processing
by doing the following:

.RS
.TP 5
1.
Calls \f3\%pfFlux::syncComplete()\fP to complete any
pfFlux sync groups that are ready for completion.
.TP
2.
Triggers all processing stages that are configured as a separate process.
.TP
3.
Inlines all processing stages that are not configured as a separate process.
.TP
4.
Sets the current flux frame (\f3\%pfFlux::setFrame\fP).
.TP
5.
Sets the current, global pfCycleBuffer index
(see \f3\%pfCycleBuffer::setCurIndex\fP)
which is guaranteed not to be in use by any other OpenGL Performer process.
.TP
6.
Sets the frame's time stamp (\f3\%pfFrameTimeStamp\fP).
.TP
7.
Sets the global frame time flux if one has been requested
(\f3\%pfGetFrameTimeFlux\fP).
.RE

\f3\%pfFrame\fP triggers all OpenGL Performer processing stages (\f3\%APP\fP,
\f3\%ISECT\fP, \f3\%DBASE\fP, \f3\%CULL\fP, \f3\%DRAW\fP, and the optional \f3\%LPOINT\fP).
If a stage is partitioned into a separate process, \f3\%pfFrame\fP will
allow that process to run.  Otherwise, \f3\%pfFrame\fP itself will carry out
the processing associated with the stage. \f3\%pfFrame\fP will directly invoke
all user callbacks that are in the same process as that which called
\f3\%pfFrame\fP.  Otherwise, a callback will be invoked by the process of
which it is a part, e.g., the \f3\%ISECT\fP callback will be invoked by
the \f3\%ISECT\fP process if \f3\%PFMP_FORK_ISECT\fP is set in the argument to
\f3\%pfMultiprocess\fP.

All OpenGL Performer stage callbacks have a block of associated data
known as "user data." User data is passed as an argument to the stage
callback. To simplify data flow in a multiprocessing environment, OpenGL
Performer copies user data into internal buffers and propagates the
data down multiprocessing pipelines. To restrict data copying to only
those frames in which user data changes, use the \f3\%pfPass<*>Data\fP
and pfChannel::passChanData functions. \f3\%pfPass<*>Data\fP and
pfChannel::passChanData signify that the user data has
changed and needs to be copied. \f3\%pfFrame\fP will then copy the data
into its internal buffer and the stage callback will receive the
updated user data.  Stage callbacks and user data functions are listed
below.

.TS
center tab($) ;
c | c | c | c .
Stage$Callback$Allocation$Pass
_
APP$pfChannel::setTravFunc$pfChannel::allocChanData$pfChannel::passChanData$
CULL$pfChannel::setTravFunc$pfChannel::allocChanData$pfChannel::passChanData$
DRAW$pfChannel::setTravFunc$pfChannel::allocChanData$pfChannel::passChanData$
ISECT$pfIsectFunc$pfAllocIsectData$pfPassIsectData
DBASE$pfDBaseFunc$pfAllocDBaseData$pfPassDBaseData
.TE

\f3\%pfFrame\fP triggers the APP, CULL and DRAW stages of all pfPipes so
it must be called every frame a new display is desired.  OpenGL
Performer will attempt to cull and draw all active pfChannels on all
pfPipes within a single frame period.  Multiple pfChannels on a single
pfPipe will be processed in the order they were added to the pfPipe.
\f3\%pfFrame\fP returns the current frame count and should only be called
by the application process when multiprocessing.

If specified, pfChannel cull and draw callbacks
(\f3\%pfChannel::setTravFunc\fP) will be invoked by
the appropriate process which may or may not be the same process that
called \f3\%pfFrame\fP.  If these callbacks are not specified, \f3\%pfCull\fP
and \f3\%pfDraw\fP will be called instead.  pfChannel passthrough data
which is passed to pfChannel function callbacks (see
\f3\%pfChannel::passChanData\fP) is copied into
internal memory at \f3\%pfFrame\fP time.

If the light points process is used, pfFrame will invoke the corresponding
callback if one is set using \f3\%pfChannel::setTravFunc\fP.
If no callback is specified, \f3\%pfLPoint\fP will be called instead. The light points
callback share the same passthrough as the draw process.

In typical operation, \f3\%pfFrame\fP should closely follow \f3\%pfSync\fP in
the main application loop.  Since the \f3\%CULL\fP does not start until
\f3\%pfFrame\fP is called, considerable processing between \f3\%pfSync\fP and
\f3\%pfFrame\fP can reduce system throughput.  However, any updates to the
database or view made at this time will be applied to the current
frame so latency is reduced for these updates.  Updates made
after \f3\%pfFrame\fP will be applied to the next frame.
\f3\%pfFrame\fP returns the current frame count.

\f3\%pfFrame\fP will automatically call \f3\%pfSync\fP if the application did
not call \f3\%pfSync\fP before calling \f3\%pfFrame\fP.  This means the
application need not call \f3\%pfSync\fP.

It is crucial to keep the time spent in the application process less
than a frame's time so the system can meet the desired frame rate.  If
the application process exceeds a single frame's time, \f3\%pfFrame\fP will
not be called often enough to meet the frame rate.

The following code fragment is an example of an application's main
processing loop:

Example 1: Main simulation loop.
.Ex
.nf

pfFrameRate(30.0f);     /* Set desired frame rate to 30Hz */

while (!done)
{
    app_funcs();        /* Perform application-specific functions */
    update_positions(); /* Update moving models for frame N */

    pfSync();           /* Sleep until next frame boundary */

    update_view();      /* Set view for frame N */
    pfFrame();          /* Trigger cull and draw for frame N */
}
.Ee
.fi
.na


\f3\%pfAppFrame\fP triggers a traversal that updates the state of the
scene graph for the next frame.  This includes updating the state of
pfSequence nodes and invoking APP callbacks on nodes in the scene
graph.  If \f3\%pfAppFrame\fP is not invoked directly, \f3\%pfSync\fP or
\f3\%pfFrame\fP invokes it automatically.  Note that when the view is not
set until after \f3\%pfSync\fP, as in the example above, the view point in
the channel during the application traversal contains the eye point
from the previous frame.

\f3\%pfGetFrameCount\fP returns the current frame count.  The frame count
is initialized to 0 by \f3\%pfConfig\fP and is incremented by each call
to \f3\%pfFrame\fP.

\f3\%pfGetFrameRate\fP returns the current system frame rate (possibly
rounded) previously set by \f3\%pfFrameRate\fP.  Note that this is not
necessarily the same as the achieved frame rate.

\f3\%pfSync\fP synchronizes the application process to a particular rate.
This rate may be fixed, for example a steady 20Hz or may vary with the
rendering rate.  In addition, the drawing process may be synchronized
to either a steady or a varying rate.  \f3\%pfPhase\fP specifies the
synchronization methods used by \f3\%pfSync\fP and the drawing process (if
it is a separate process).  \f2phase\fP is a symbolic constant that
specifies the phase of all process pipeline(s). It can take on the
following values:
.RS
.TP 5
\f3\%PFPHASE_LOCK\fP
\f3\%pfSync\fP synchronizes to the next frame boundary and the drawing process
begins drawing and swaps its rendering buffers only at fixed frame boundaries.
.TP
\f3\%PFPHASE_FLOAT\fP
\f3\%pfSync\fP synchronizes to the next frame boundary but the drawing process
can begin drawing and swap its rendering buffers at non-frame boundaries.
.TP
\f3\%PFPHASE_FREE_RUN\fP
\f3\%pfSync\fP synchronizes to the rendering rate so the application runs at
its peak (and usually non-constant) capability.
.TP
\f3\%PFPHASE_LIMIT\fP
\f3\%pfSync\fP synchronizes to the rendering rate but the rendering rate is
limited to that frame rate specified by \f3\%pfFrameRate\fP.
.RE

Additionally, the modifier \f3\%PFPHASE_SPIN_DRAW\fP may be or-ed in with the
main phase mode from the above list.

\f3\%pfPhase\fP is a global mode so all pfPipes in the system are forced to
run with the same phase.

If running in locked phase, the drawing process will swap buffers only on
frame boundaries.  A benefit of locking is that such pipelines are
self-regulating so synchronizing two pfPipes together is simple, even
across different machines.  Another benefit is that latency is minimized
and predictable.  The major drawback is that if a view takes slightly
longer than a frame to render (it has 'frame-extended'), then an entire
frame is skipped rather than a single vertical retrace period.
However, if minimal distraction is crucial, the phase can float so that
buffer swapping may happen on non-frame boundaries.  In this case it is
not guaranteed that the windows on pfPipes will swap together; they may
get out of phase resulting in inconsistent images if the displays are
adjacent and are displaying the same scene.

The difference between phase lock and phase float becomes less apparent
with increasing frame rate.  At a rate equal to the vertical retrace rate,
there is no difference.  Also, if pfPipes do not 'frame extend', then
there is no difference.

Applications which do not require a fixed frame rate may use
\f3\%PFPHASE_FREE_RUN\fP or \f3\%PFPHASE_LIMIT\fP.  \f3\%PFPHASE_FREE_RUN\fP essentially
disables OpenGL Performer's fixed frame rate mechanisms and will cause the
application to run at its rendering rate so it slows down when rendering
complex scenes and speeds up when rendering simple scenes.
In this case,
the frame rate specified by \f3\%pfFrameRate\fP no longer affects the system
frame rate but is still used to compute system load and stress.
Additionally, if the APP, CULL, and/or DRAW stages are in separate processes,
they will run in lock step and each stage will wait for the downstream stage to
get its results and likewise, downstream stages wait for upstream stages to finish.
This is the desired mode if you need to be sure that every
APP frame is actually drawn and that the APP can not spin ahead of the
draw is slow and extends past its goal frame time.

\f3\%PFPHASE_LIMIT\fP is equivalent to \f3\%PFPHASE_FREE_RUN\fP except that
the application can go no faster than the frame rate specified by
\f3\%pfFrameRate\fP although it may go slower.  Thus fixed frame rate behavior
is achieved if the time required to process a frame never takes longer than
that specified by \f3\%pfFrameRate\fP.

\f3\%PFPHASE_SPIN_DRAW\fP allows forked draw processes to redraw the previous frame
if an upstream stage slowed down an missed a frame.  This only affects
locked and float phase modes.  The default behavior
is without \f3\%PFPHASE_SPIN_DRAW\fP, in which case a forked draw process will
wait for new results from an upstream stage before continuing.  In
locked or float modes, the draw process will only check for new upstream
results on frame boundaries.

\f3\%pfPhase\fP may be called any time after \f3\%pfConfig\fP.

\f3\%pfGetPhase\fP returns the current phase. The default phase is
\f3\%PFPHASE_FREE_RUN\fP.

\f3\%pfFrameTimeStamp\fP sets the time stamp of the current frame to \f2time\fP.
The frame time stamp is used when evaluating all pfSequences. Normally,
\f3\%pfFrame\fP sets the frame time stamp immediately before returning
control to the application although the application may set it
to account for varying latency in a non-constant frame rate situation.
Time is relative to \f3\%pfInit\fP when the system clock is initialized to 0.

\f3\%pfGetFrameTimeStamp\fP returns the frame time stamp of the current frame.

\f3\%pfGetFrameTimeFlux\fP returns the global frame time flux.  The frame time
flux is a pfFlux that is updated with the frame time stamp of the current
frame whenever it changes.  By default the frame time flux is configured
with the \f3\%PFFLUX_PUSH\fP mode set to \f3\%PF_ON\fP.  The frame time flux is
useful for automatically driving pfEngine animations.

.SH NOTES


To achieve a steady frame rate synchronized with the graphics display
in an otherwise non-graphical application,
a screen must be set on the first pfPipe with \f3\%pfPipeScreen\fP and
\f3\%pfFrame\fP should be used in the main loop which will indirectly
call \f3\%pfSync\fP to obey the current phase.

.SH SEE ALSO
pfChannel,
pfPipeWindow,
pfConfig,
pfIsectFunc,
pfInitVClock,
pfCycleBuffer,
pfGetTime
