'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfIsectFunc" "3pf"
.SH NAME
.upperok
.B pfIsectFunc,
.B pfGetIsectFunc,
.B pfAllocIsectData,
.B pfGetIsectData,
.B pfPassIsectData
\- 
Set intersection callback, allocate and pass intersection data.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfIsectFuncType\ \ \ 'u
#include <Performer/pf.h>
.nf
.fi
.TP
void
\f3\%pfIsectFunc\fP(pfIsectFuncType\ func);
.nf
.fi
.TP
pfIsectFuncType
\f3\%pfGetIsectFunc\fP(void);
.nf
.fi
.TP
void *
\f3\%pfAllocIsectData\fP(int\ bytes);
.nf
.fi
.TP
void *
\f3\%pfGetIsectData\fP(void);
.nf
.fi
.TP
void
\f3\%pfPassIsectData\fP(void);
.fi

.Ex
.nf

typedef void (*pfIsectFuncType)(void *userData);
.Ee
.fi
.na

.SH DESCRIPTION

The \f2func\fP argument to \f3\%pfIsectFunc\fP specifies the intersection
callback function.  This function will be invoked by \f3\%pfFrame\fP and
will be passed a pointer to a data buffer allocated by
\f3\%pfAllocIsectData\fP.  If a separate process is allocated for
intersections by the \f3\%PFMP_FORK_ISECT\fP mode to \f3\%pfMultiprocess\fP,
then \f3\%pfFrame\fP will cause \f2func\fP to be called in the separate
process.  \f3\%pfGetIsectFunc\fP returns the intersection callback or
\f3\%NULL\fP if none is set.

Within the intersection callback, the user may further multiprocess
intersection queries through any IRIX multiprocessing mechanism
such as \f3\%fork\fP, \f3\%sproc\fP, or \f3\%m_fork\fP.  All of these processes
may call \f3\%pfNode::isect\fP in parallel.

When the intersection function is in a separate process, it will run
asynchronously with the rest of the rendering pipeline.  Specifically,
if the intersection function takes more than a frame time, the
rendering pipeline will not be affected and the next invocation of the
intersection function will be delayed until triggered by the next
\f3\%pfFrame\fP.  Changes to the scene graph made by the application
process are only propagated to the intersection process after the
intersection function returns.

Any modifications made to the scene graph by a forked intersection
function will not be reflected in the scene graph that is seen by any
other OpenGL Performer functions.  To be safe, only \f3\%pfNode::isect\fP
(which does not modify the scene graph) should be called from within
the intersection function.

\f3\%pfAllocIsectData\fP returns a pointer to a chunk of shared memory of
\f2bytes\fP bytes.  This memory buffer may be used to communicate
information between the intersection function and application.
Intersection data should only be allocated once.  \f3\%pfGetIsectData\fP
returns the previously allocated intersection data.

When the intersection function is forked, \f3\%pfPassIsectData\fP should be
used to copy the intersection data into internal OpenGL Performer memory
when the next \f3\%pfFrame\fP is called.  Once \f3\%pfFrame\fP is called, the
application may modify data in the intersection data buffer without
fear of colliding with the forked intersection function.

Example 1: Multiprocessed intersections.
.Ex
.nf

typedef struct
{
    int         frameCount;	/* For frame stamping collisions */
    pfNode      *collidee;	/* pfNode to collide with */
    int         numCollisions;	/* Number of collision vectors */
    pfSeg       *collisionVecs[MAXCOLLISIONS];
} IsectStuff;

void
isectFunc(void *data)
{
    IsectStuff 	istuff = (IsectStuff*) data;

    istuff->collidee->isect(etc...);
}
     :
    pfMultiprocess(PFMP_FORK_ISECT | PFMP_APP_CULL_DRAW);
    pfConfig();
     :
    pfIsectFunc(isectFunc);
    isectData = (IsectStuff*) pfAllocIsectData(sizeof(IsectStuff));

    isectData->collidee = (pfNode*) scene;

    while (!done)
    {
        pfSync();           /* Sleep until next frame boundary */

        update_view();      /* Set view for frame N */

        isectData->frameCount = pfGetFrameCount();

        pfPassIsectData();  /* Pass intersection data to */
                            /* intersection process */

        pfFrame();          /* Trigger cull, intersection for frame N */

        app_funcs();        /* Perform application-specific functions */
        update_positions(); /* Update moving models for frame N + 1 */

        /*
         * Act on result of previous collisions and set up isectData
         * for more collisions.
         */
        update_collisions(isectData);
    }
.Ee
.fi
.na


If \f3\%pfIsectFunc\fP is called before \f3\%pfConfig\fP and the multiprocessing
mode is \f3\%PFMP_DEFAULT\fP, then \f3\%pfConfig\fP will fork the intersection
process if there are enough processors.  Otherwise, you must explicitly
fork the intersection process by setting the \f3\%PFMP_FORK_ISECT\fP bit
in the argument passed to \f3\%pfMultiprocess\fP.
.SH SEE ALSO
pfConfig,
pfMultiprocess,
pfNode
