'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfOverride" "3pf"
.SH NAME
.upperok
.B pfOverride,
.B pfGetOverride,
.B pfGLOverride,
.B pfGetGLOverride
\- 
Override state element(s).
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'uint64_t\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfOverride\fP(uint64_t\ mask,
'in +2
'll -2
int\ val);
.in
.ll
.nf
.fi
.TP
uint64_t
\f3\%pfGetOverride\fP(void);
.nf
.fi
.TP
void
\f3\%pfGLOverride\fP(int\ which,
'in +2
'll -2
float\ val);
.in
.ll
.nf
.fi
.TP
float
\f3\%pfGetGLOverride\fP(int\ which);
.fi
.SH PARAMETERS
.TP \w'mask\ \ 'u
\f2mask\fP
is a bit mask that specifies the state elements that are
to be set to the override value \f2val\fP.  \f2mask\fP is a bitwise OR of:
.RS
.RS
\f3\%PFSTATE_ENLIGHTING\fP
.br
\f3\%PFSTATE_ENTEXTURE\fP
.br
\f3\%PFSTATE_ENFOG\fP
.br
\f3\%PFSTATE_ENWIREFRAME\fP
.br
\f3\%PFSTATE_ENHIGHLIGHTING\fP
.br
\f3\%PFSTATE_ENCOLORTABLE\fP
.br
\f3\%PFSTATE_FRONTMTL\fP
.br
\f3\%PFSTATE_BACKMTL\fP
.br
\f3\%PFSTATE_TEXTURE\fP
.br
\f3\%PFSTATE_TEXENV\fP
.br
\f3\%PFSTATE_ALPHAFUNC\fP
.br
\f3\%PFSTATE_TRANSPARENCY\fP
.br
\f3\%PFSTATE_ANTIALIAS\fP
.br
\f3\%PFSTATE_CULLFACE\fP
.br
\f3\%PFSTATE_DECAL\fP
.br
\f3\%PFSTATE_ALPHAREF\fP
.br
\f3\%PFSTATE_COLORTABLE\fP
.br
\f3\%PFSTATE_HIGHLIGHT\fP
.br
\f3\%PFSTATE_FOG\fP
.br
\f3\%PFSTATE_LIGHTS\fP
.br
\f3\%PFSTATE_LIGHTMODEL\fP
.RE
.RE
.TP
\f2val\fP
is a symbolic token and either \f3\%PF_ON\fP or \f3\%PF_OFF\fP
indicating whether the state elements in \f2mask\fP should be overridden
or not.
.SH DESCRIPTION

\f3\%pfOverride\fP is used to override individual state elements.  If a state
element is overridden, all subsequent attempts to modify it will be ignored.
An overridden state element is locked to the current value at the time
\f3\%pfOverride\fP is called.


Example 1:
.Ex
.nf

pfTransparency(PFTR_OFF);
tex->apply();
pfDisable(PFEN_LIGHTING);
pfOverride(PFSTATE_TRANSPARENCY | PFSTATE_TEXTURE | PFSTATE_ENLIGHTING, PF_ON);
.Ee
.fi
.na

This example turns off transparency and lighting and applies \f2tex\fP to all
subsequent geometry for which texturing is enabled.

\f3\%pfOverride\fP with a \f2val\fP of \f3\%PF_OFF\fP will free the state elements
specified by \f2mask\fP to be modified.  Although state elements will not be
restored to their pre-override condition, \f3\%pfPushState\fP and
\f3\%pfPopState\fP may be used to do so.  The override mask is pushed and
popped along with the rest of the state.

\f3\%pfGetOverride\fP returns the current override mask.

\f3\%pfOverride\fP
is a display-listable command.  If a pfDispList has been opened by
\f3\%pfDispList::open\fP,
\f3\%pfOverride\fP
will not have immediate effect but will be captured by the pfDispList
and will only have effect when that pfDispList is later drawn with
\f3\%pfDispList::draw\fP.

\f3\%pfGLOverride\fP overrides the graphics library mechanism used to
achieve a desired effect such as transparency and
visually correct coplanar geometry. \f2which\fP identifies the
mechanism to override and is one of the following:

.RS
.TP 5
\f3\%PFGL_TRANSPARENCY\fP
\f2val\fP may be either \f3\%PFGL_TRANSP_MSALPHA\fP or \f3\%PFGL_TRANSP_BLEND\fP.
in which case multisample ("screen-door") or blending will be used
when transparency is enabled (see \f3\%pfTransparency\fP).
.TP
\f3\%PFGL_DECAL\fP
\f2val\fP may be either \f3\%PFGL_DECAL_STENCIL\fP or \f3\%PFGL_DECAL_DISPLACE\fP
in which case stenciling or displacement will be used for all
decals (see \f3\%pfDecal\fP).
.RE

\f3\%pfGetGLOverride\fP returns the override mode corresponding to \f2which\fP.

.SH SEE ALSO
pfDispList,
pfDrawDList,
pfGeoState,
pfOpenDList,
pfState,
pfTransparency,
pfDecal
