'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfPipe" "3pf"
.SH NAME
.upperok
.B pfPipe,
.B pfGetPipeNum,
.B pfGetPipeDrawCount,
.B pfGetLastPipeFrameCount
\- 
Initialize and get window information for a pfPipe.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfPipeSwapFuncType\ \ \ 'u
#include <Performer/pf/pfPipe.h>
.nf
.fi
.TP
void
\f3\%pfPipe::getSize\fP(int\ *xsize,
'in +2
'll -2
int\ *ysize);
.in
.ll
.nf
.fi
.TP
static pfType *
\f3\%pfPipe::getClassType\fP(void);
.nf
.fi
.TP
void
\f3\%pfPipe::setScreen\fP(int\ screen);
.nf
.fi
.TP
int
\f3\%pfPipe::getScreen\fP(void);
.nf
.fi
.TP
int
\f3\%pfPipe::movePWin\fP(pfPipeWindow\ *pwin);
.nf
.fi
.TP
pfPipeWindow *
\f3\%pfPipe::getPWin\fP(int\ which);
.nf
.fi
.TP
void
\f3\%pfPipe::setSwapFunc\fP(pfPipeSwapFuncType\ func);
.nf
.fi
.TP
pfPipeSwapFuncType
\f3\%pfPipe::getSwapFunc\fP(void);
.nf
.fi
.TP
float
\f3\%pfPipe::getLoad\fP(void);
.nf
.fi
.TP
int
\f3\%pfPipe::getNumPWins\fP(void);
.nf
.fi
.TP
int
\f3\%pfPipe::getNumChans\fP(void);
.nf
.fi
.TP
pfChannel *
\f3\%pfPipe::getChan\fP(int\ which);
.nf
.fi
.TP
void
\f3\%pfPipe::bindVChans\fP(void);
.nf
.fi
.TP
void
\f3\%pfPipe::unbindPVChans\fP(void);
.nf
.fi
.TP
void
\f3\%pfPipe::setIncrementalStateChanNum\fP(int\ num);
.nf
.fi
.TP
int
\f3\%pfPipe::getIncrementalStateChanNum\fP(void);
.nf
.fi
.TP
void
\f3\%pfPipe::addMPClipTexture\fP(pfMPClipTexture\ *clip);
.nf
.fi
.TP
int
\f3\%pfPipe::removeMPClipTexture\fP(pfMPClipTexture\ *clip);
.nf
.fi
.TP
int
\f3\%pfPipe::getNumMPClipTextures\fP(void);
.nf
.fi
.TP
pfMPClipTexture*
\f3\%pfPipe::getMPClipTexture\fP(int\ which);
.nf
.fi
.TP
void
\f3\%pfPipe::setTotalTexLoadTime\fP(float\ msec);
.nf
.fi
.TP
float
\f3\%pfPipe::getTotalTexLoadTime\fP(void);
.nf
.fi
.TP
void
\f3\%pfGetPipeNum\fP(void);
.nf
.fi
.TP
void
\f3\%pfGetPipeDrawCount\fP(int\ pnum);
.nf
.fi
.TP
void
\f3\%pfGetLastPipeFrameCount\fP(int\ pnum);
.fi

.Ex
.nf

/* pfPipe-specific types */
typedef void (*pfPipeFuncType)(pfPipe *p);
typedef void (*pfPipeSwapFuncType)(pfPipe *p, pfPipeWindow *pw);
.Ee
.fi
.na

.SH DESCRIPTION

A pfPipe is a software rendering pipeline which renders
one or more pfChannels into one or more pfPipeWindows. A pfPipe
can be configured as multiple processes for increased throughput
on multiprocessor systems. Multiple pfPipes can operate in parallel
in support of platforms with multiple graphics pipelines.
The number of pfPipes and the multiprocessing mode used
are set by \f3\%pfMultipipe\fP and \f3\%pfMultiprocess\fP
respectively (see \f3\%pfConfig\fP).

A pfPipe references one or more pfPipeWindows which in turn reference
one or more pfChannels. A pfChannel is simply a view of a
scene which is rendered into a viewport of a pfPipeWindow.
A pfPipeWindow is a graphics window managed by its parent pfPipe.

pfPipes, pfPipeWindows, and pfChannels form a hierarchy with the
following rules:

.RS
.TP 5
1.
A screen (i.e. hardware graphics display) can have multiple
pfPipes but should only have one drawing to it
.TP
2.
A pfPipe may only draw to one screen
.TP
3.
A pfPipe may render to multiple pfPipeWindows
.TP
4.
A pfPipeWindow belongs to a single fixed pfPipe
and thus also to a single fixed screen
.TP
5.
A pfPipeWindow may have multiple pfChannels
.TP
6.
A pfChannel always belongs to a pfPipe but may
change pfPipeWindows or might not belong to any pfPipeWindow.
a channel not assigned to a pfPipeWindow is culled but not drawn.
.RE

The following is an example pfPipe->pfPipeWindow->pfChannel configuration.

Example 1:

.Ex
.nf

The screen:

    hardware screen/graphics pipeline
    +-----------------------------------------------+
    | pfPipe                                        |
    |                                               |
    |                    pfPipeWindow1              |
    |                   +----------------+          |
    |                   |                |          |
    |                   |                |          |
    |                   |                |          |
    |                   |                |          |
    |                   |  pfChannel2    |          |
    |                   |                |          |
    |    pfPipeWindow0  |                |          |
    |   +--------------------+           |          |
    |   |            |       |           |          |
    |   |            |       |-----------+          |
    |   | pfChannel0 | blank |                      |
    |   |            |       |                      |
    |   |--------------------|                      |
    |   |                    |                      |
    |   |    pfChannel1      |                      |
    |   |                    |                      |
    |   +--------------------+                      |
    |                                               |
    |                                               |
    +-----------------------------------------------+



The hierarchy:

                screen0 ....
                  |
                  |
                pfPipe0 ---------------------+
                 /   \\                       |
                /     \\                      |
               /       \\                     |
        pfPipeWindow0   pfPipeWindow1        |
          /     \\          \\                 |
         /       \\          \\                |
    pfChannel0  pfChannel1   pfChannel2  pfChannel3
                                         (not drawn)

The code: (in application process)

    /* Calls that create the hierarchy: */
    pfPipe		*pipe = pfGetPipe(0);
    pfPipeWindow 	*pwin0 = new pfPipeWindow(pipe);
    pfPipeWindow 	*pwin1 = new pfPipeWindow(pipe);

    pfChannel	*chan0 = new pfChannel(pipe);
    pfChannel	*chan1 = new pfChannel(pipe);
    pfChannel	*chan2 = new pfChannel(pipe);

    pwin0->addChan(chan0);
    pwin0->addChan(chan1);
    pwin1->addChan(chan2);

    /* Calls that cause the window to be opened at next pfFrame() */
    pwin0->open();
    pwin1->open();
    pfFrame();
.Ee
.fi
.na


If a pfPipe has no windows at the time \f3\%pfFrame\fP is called, a full
screen pfPipeWindow will be opened for \f2pipe\fP and all pfChannels of
\f2pipe\fP will be assigned to that pfPipeWindow.

\f3\%pfPipe::getClassType\fP returns the
\f3\%pfType\fP* for the class \f3\%pfPipe\fP.  The \f3\%pfType\fP* returned
by \f3\%pfPipe::getClassType\fP is the
same as the \f3\%pfType\fP* returned by invoking the
virtual function \f3\%getType\fP on any instance of class
\f3\%pfPipe\fP.  Because OpenGL Performer allows subclassing of
built-in types, when decisions are made based on the type of an
object, it is usually better to use  the member
function \f3\%isOfType\fP to test if an object is of a type derived from
a Performer type rather than to test for strict equality of the
\f3\%pfType\fP*'s.

\f3\%pfPipe::setScreen\fP specifies the hardware screen,
\f2screen\fP, (graphics pipeline) used for rendering by the pfPipe,
the pfPipe.  The screen of the pfPipe may be specified in the application
process before the call to open or configure any pfPipeWindows
(\f3\%pfPipeWindow::open\fP, \f3\%pfPipeWindow::config\fP) on
the pfPipe, or may be specified implicitly by
the screen of the first opened pfPipeWindow.
A pfPipe is tied
to a specific hardware pipeline and the screen of a pfPipe cannot be
changed once determined.
For single pipe operation, if the screen of a
pfPipe or pfPipeWindow is never explicitly set in single pipe
configuration, the screen will be taken from the default screen of the
current pfWSConnection, or current X Display.
For multipipe operation, if the screen of a pfPipe
or pfPipeWindow is never explicitly set and pfMultipipe() has been used
to configure multiple pfPipes, then pfPipes will automatically be
assigned to hardware screens in order, i.e., pfGetPipe(0) -> screen 0,
pfGetPipe(1) -> screen 1, etc.  If a custom mapping of pfPipes to screens
is desired, the screens of all pfPipes must be specified before the
configuration of the first pfPipe which will happen at the first call to
\f3\%pfFrame\fP.
See the \f3\%pfGetCurWSConnection\fP reference  page for
more details on how to manage X display connections.

\f3\%pfPipe::setWSConnectionName\fP
allows you to specify both a window server target and screen
for the pfPipe.  This is useful for doing remote rendering, or for running
on a system with multiple window servers. This call should be made in
the application process, before the first call to pfFrame.
\f3\%pfPipe::getWSConnectionName\fP will
return the current window server target name.
A window server target specified on a pfPipe will take precedence over
any such targets specified on pfPipeWindows of that pfPipe.
If the window server target of a pfPipe has not been set, it may be
implicitly set from the first such setting on a child pfPipeWindow.
The window server target of a pfPipe may not be changed after the first
call to \f3\%pfFrame\fP.
See the \f3\%pfGetCurWSConnection\fP reference  page for
more details on how to manage X display connections.

\f3\%pfPipe::getScreen\fP can be used to get the screen of a pfPipe.  A
return value of (-1) indicates that the screen of the pfPipe is
undefined.  \f3\%pfPipe::getSize\fP returns the size of the screen used by the pfPipe.

For best performance only one pfPipe should render to a given hardware
pipeline. If multiple views on a single screen are desired, use
multiple pfChannels, and if necessary, multiple pfPipeWindows.

Normally a pfPipe swaps the color buffers at the end of each frame.
However, if special control is needed over buffer swapping,
\f3\%pfPipe::setSwapFunc\fP will register \f2func\fP as the buffer swapping
function for the pfPipe.  Instead of swapping buffers, \f2func\fP will be
called and will be expected to swap the color buffers of the provided
pfPipeWindow.
\f3\%pfPipe::getSwapFunc\fP returns the buffer swapping function of the pfPipe
or \f3\%NULL\fP if none is set.

If you wish to frame lock multiple pfPipes so that each pfPipe swaps
its color buffers at the same time, then you should create a channel
group consisting of one or more pfChannels on each pfPipe and make sure
\f3\%PFCHAN_SWAPBUFFERS\fP is shared. In addition, separate hardware
graphics pipelines *must* be genlocked for proper frame-locking.

\f3\%pfPipe::getPWin\fP
returns the pointer to the pfPipeWindow at the location specified by
\f2which\fP in the pfPipeWindow list on the pfPipe.

\f3\%pfPipe::getNumPWins\fP
returns the number of pfPipeWindows that have been created on
the pfPipe.
\f3\%pfPipe::getNumChans\fP
returns the number of pfChannels that have been created on
the pfPipe.

\f3\%pfPipe::movePWin\fP moves the specified
pfPipeWindow \f2pwin\fP to the location specified by \f2where\fP in the
pfPipeWindow list on the pfPipe.  The move includes
removing \f2pwin\fP from its current location by moving up the elements
in the list that follow it and then inserting \f2pwin\fP into its new
location.  If \f2pwin\fP is attached to the pfPipe,
(-1) is returned and \f2pwin\fP is not inserted into the list.
Otherwise, \f2where\fP is returned to indicate success.  \f2where\fP must
be within the range [0 .. n] where n is the number returned by
\f3\%pfPipe::getNumPWins\fP(), or else
(-1) is returned and no move is executed.

\f3\%pfPipe::getChan\fP
returns the pointer to the pfChannel at location
\f2which\fP in the list of pfChannels on the pfPipe.

\f3\%pfPipe::getLoad\fP returns the current
instantaneous load (ratio of time required to process a frame
and the time allotted for that frame's processing) for the pipe.

\f3\%pfPipe::bindVChans\fP binds all
pfPipeVideoChannels associated with the pfPipe 
to the video hardware.
\f3\%pfPipe::unbindPVChans\fP performs
the opposite action, unbinding each from the video hardware.


Example 2: How to frame lock pfPipes
.Ex
.nf

leftChan  = new pfChannel(pfGetPipe(0));
rightChan = new pfChannel(pfGetPipe(1));

/* BPFCHAN_SWAPBUFFERS is shared by default */
leftChan->attach(rightChan);

/* Pipe 0 and pipe 1 are now frame-locked */
.Ee
.fi
.na


.SS USING PFPIPES TO APPLY INCREMENTAL GRAPHICS STATE

pfPipe's are also the level at which hardware pipeline specific graphics state can be applied.
The only two currently supported incremental graphics state elements that can be applied on a per-pipe
basis are pfMPClipTexture's.  Both of the libpf primitives provide support for incrementally
paging texture from memory into texture memory which contains a texture cache that differs on a per frame basis.
Thus by attaching these structures to a pfPipe, the pfPipe will provide frame accurate applying and downloading
of these texture primitives.  Note that this incremental downloading should be attached to a channel such
that loading statistics can be accumulated.

\f3\%pfPipe::setIncrementalStateChanNum\fP sets the channel with which the
incremental state should be processed.  If unspecified the first channel enabled for drawing will be used
to process the incremental state.
\f3\%pfPipe::getIncrementalStateChanNum\fP returns the channel with
which all incremental state attached to the pfPipe will be processed.

\f3\%pfPipe::addMPClipTexture\fP is used to associate a pfMPClipTexture with a particular
pfPipe.  This association forces the pfPipe to apply the pfMPClipTexture and download the resulting incremental
texture changes in a frame accurate manner. If a pfMPClipTexture is added to a pipe that is already connected to
another pipe, the function silently creates a new pfMPClipTexture, makes it a slave of the pfMPClipTexture that is
already connected to another pipe, and adds the slave to the pipe in place of the one passed as an argument to
the function.

\f3\%pfPipe::removeMPClipTexture\fP is used to remove the association of a
pfMPClipTexture from a pfPipe such that the pfPipe will no longer manage the incremental
application of the pfMPClipTexture. If a pfMPClipTexture is removed that is the master of other pfMPClipTextures, the
slaves will be removed from their pipes as well.

\f3\%pfPipe::getNumMPClipTextures\fP returns
the number of pfMPClipTextures that have been attached to the pfPipe.

\f3\%pfPipe::getMPClipTexture\fP returns the
\f2which\fPth pfMPClipTexture that has been attached to the pfPipe.

\f3\%pfPipe::setTotalTexLoadTime\fP
sets the total amount of texture download time in (milliseconds)
allocated for all the pfMPClipTextures attached to this pfPipe.
This download time is used by the pfMPClipTexture's DTR load control
system to update texture memory as the pfMPClipTexture's center is moved.

The total download time is distributed among the pfMPClipTextures attached
to the pfPipe by examining the value of each pfMPClipTexture's load time
fraction parameter. This floating point number is queried from each
pfMPClipTexture attached to the pfPipe and the results are summed. The
resulting value used to normalize each pfMPClipTexture's load time fraction
value.

Each pfMPClipTexture's normalized load time fraction value is used to
scale the total download time. The result is the download time available
for that pfMPClipTexture. If the pfMPClipTexture has had a texture
download time set via
\f3\%pfMPClipTexture::setTexLoadTime\fP,
then that value takes precedence, and the the scaled total download time
is ignored by that pfMPClipTexture.

\f3\%pfPipe::getTotalTexLoadTime\fP
return the current value for total texture download time. The default
value is 6 milliseconds.

\f3\%pfGetPipeNum\fP will return the pipeline number for the current
OpenGL Performer pipeline process. A process that is not
associated with a pfPipe will have a pipe number of -1.
Only CULL, DRAW, and LPOINT processes will have valid pipe numbers.

\f3\%pfGetPipeDrawCount\fP will return the frame count of the last
draw to complete for the pfPipe. This can be called in the application
process to find out where the draw is.

\f3\%pfGetLastPipeFrameCount\fP will return the frame count of the last
process associated between ISECT and the DRAW of \f2pipe\fP to finish.
This can tell you what processes might be using old
pfGeoSets in the scene graph.

.SH NOTES

pfPipes cannot be deleted.
.SH SEE ALSO
pfChannel,
pfConfig,
pfMultipipe,
pfMultiprocess,
pfPipeWindow,
pfGetCurWSConnection,
pfMPClipTexture
