'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfPlane" "3pf"
.SH NAME
.upperok
.B pfPlane
\- 
Set and operate on planes
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void*\ \ \ 'u
#include <Performer/pr/pfGeoMath.h>
.nf
.fi
.TP
void*
\f3\%pfPlane::operator new\fP(size_t);
.nf
.fi
.TP
void*
\f3\%pfPlane::operator new\fP(size_t,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
\ 
\f3\%pfPlane::pfPlane\fP();
.nf
.fi
.TP
\ 
\f3\%pfPlane\fP(const\ pfVec3&\ p1,
'in +2
'll -2
const\ pfVec3&\ p2,
const\ pfVec3&\ p3);
.in
.ll
.nf
.fi
.TP
\ 
\f3\%pfPlane\fP(const\ pfVec3&\ _norm,
'in +2
'll -2
const\ pfVec3&\ _pos);
.in
.ll
.nf
.fi
.TP
\ 
\f3\%pfPlane\fP(const\ pfVec3&\ _norm,
'in +2
'll -2
float\ _offset);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfPlane::makePts\fP(const\ pfVec3\ &pt1,
'in +2
'll -2
const\ pfVec3\ &pt2,
const\ pfVec3\ &pt3);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfPlane::makeNormPt\fP(const\ pfVec3\ &norm,
'in +2
'll -2
const\ pfVec3\ &pos);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfPlane::displace\fP(float\ d);
.nf
.fi
.TP
void
\f3\%pfPlane::closestPtOn\fP(const\ pfVec3\ &pt,
'in +2
'll -2
pfVec3\ &dst);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfPlane::isect\fP(const\ pfSeg\ *seg,
'in +2
'll -2
float\ *d);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfPlane::isect\fP(const\ pfSeg*\ seg,
'in +2
'll -2
float*\ d1,
float*\ d2);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfPlane::contains\fP(const\ pfVec3\ &pt);
.nf
.fi
.TP
int
\f3\%pfPlane::contains\fP(const\ pfBox\ *box);
.nf
.fi
.TP
int
\f3\%pfPlane::contains\fP(const\ pfSphere\ *sph);
.nf
.fi
.TP
int
\f3\%pfPlane::contains\fP(const\ pfCylinder\ *cyl);
.nf
.fi
.TP
void
\f3\%pfPlane::orthoXform\fP(const\ pfPlane\ *pln,
'in +2
'll -2
const\ pfMatrix\ &xform);
.in
.ll
.fi

.Ex
.nf

struct pfPlane
{
    pfVec3      normal;
    float       offset;
};
.Ee
.fi
.na

.SH DESCRIPTION

A pfPlane represents an infinite 2D plane as a normal and a distance
offset from the origin in the normal direction.  A point on the plane
satisfies the equation \f2normal\fP dot (x, y, z) = offset.  pfPlane is
a public struct whose data members \f2normal\fP and \f2offset\fP may be
operated on directly.

The default constructor \f3\%pfPlane\fP() is empty and does no
initialization.  There are also three other constructors
to construct a plane from three points, a normal and a point,
and a normal and an offset from the origin.  These constructors
correspond directly with makePts, makeNormPt, and direct initialization.
\f3\%new\fP(arena) allocates a pfPlane from the specified
memory arena, or from the heap if \f2arena\fP is NULL.  \f3\%new\fP
allocates a pfPlane from the default memory arena (see
\f3\%pfGetSharedArena\fP).  pfPlanes can also be created automatically on
the stack or statically.  pfPlanes allocated with \f3\%new\fP can be
deleted with \f3\%delete\fP or \f3\%pfDelete\fP.

\f3\%pfPlane::makePts\fP sets the pfPlane to the plane which passes through the
three points \f2pt1\fP, \f2pt2\fP and \f2pt3\fP.

\f3\%pfPlane::makeNormPt\fP sets the pfPlane to the plane which passes through
the point \f2pt\fP with normal \f2norm\fP.

\f3\%pfPlane::displace\fP moves the pfPlane by a distance \f2d\fP in the
direction of the plane normal.

\f3\%pfPlane::closestPtOn\fP
sets the pfPlane to
the closest point to \f2pt\fP
which lies in the plane \f2pln\fP.  The line segment connecting \f2pt\fP and
the pfPlane is perpendicular to \f2pln\fP.

\f3\%pfPlane::contains(const pfVec3)\fP
returns \f3\%TRUE\fP or \f3\%FALSE\fP depending on whether the point
given is in the interior of the specified half-space.
The half-space is defined with plane normal pointing to the exterior.

\f3\%pfPlane::contains(const pfSphere
*)\fP, \f3\%pfPlane::contains(const pfBox
*)\fP and \f3\%pfPlane::contains(const
pfCylinder *)\fP test whether the half space specified by pfPlane
contains a non-empty portion of the volume specified by the
argument, a sphere, box or cylinder, respectively.

The return value from the these functions is the OR of one or
more bit fields.  The returned value may be:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
The intersection of the primitive
and the half space is empty.
.TP
\f3\%PFIS_MAYBE\fP:
The intersection of the primitive and the
half space might be non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP:
The intersection of the primitive and the
half space is definitely non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP:
The primitive
is non-empty and lies entirely inside the half space.
.RE

indicating indicate that the  argument is entirely
outside, potentially partly inside, partially inside or entirely
inside the half space specified by the pfPlane.

\f3\%pfPlane::isect\fP tests the line segment \f2seg\fP for intersection
with the half space specified by the pfPlane.
The possible test results are:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
\f2seg\fP lies entirely in the exterior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_START_IN\fP:
The starting point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_END_IN\fP:
The ending point of \f2seg\fP lies in the interior.
.RE

If \f2d\fP is non-NULL, on return it contains the position along the
line segment (0 <= d <= seg->length) at which the intersection occurred.

\f3\%pfPlane::isect\fP intersects the line
segment \f2seg\fP with the half space specified by the
pfPlane and has return values the same as
\f3\%pfPlane::isect\fP
except that it also returns a non-zero value when both points are
inside the half-space.  In this case it returns:
.RS
.TP 5
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP | \f3\%PFIS_START_IN\fP | \f3\%PFIS_END_IN\fP:
Both end points of \f2seg\fP lie in the interior.
.RE

If \f2d1\fP and \f2d2\fP are non-NULL, on return from
\f3\%pfPlane::isect\fP they contain the
starting and ending positions of the line segment (0 <= d1 <= d2 <=
seg->length) intersected with the half space.

\f3\%pfPlane::orthoXform\fP sets the pfPlane
to the plane as transformed by the orthogonal transformation \f2xform\fP;
\f2dst\fP = \f2pln\fP * \f2xform\fP.  If \f2xform\fP is not an orthogonal transformation the
results are undefined.
.SH NOTES

The bit fields returned by the contains functions are structured so
that bitwise AND-ing the results of sequential tests can be used to
compute composite results, e.g. testing exclusion against a number
of half spaces.

C++ does not support array deletion (i.e. \f3\%delete[]\fP) for
arrays of objects allocated new operators that take additional
arguments.  Hence, the array deletion operator \f3\%delete[]\fP should not
be used on arrays of objects created with \f3\%new\fP(arena) pfVec3[n].
.SH SEE ALSO
pfBox,
pfMatrix,
pfSeg,
pfSphere,
pfVec3
