'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfQuerySys" "3pf"
.SH NAME
.upperok
.B pfGetMachString,
.B pfGetRelString,
.B pfQuerySys,
.B pfMQuerySys,
.B pfQueryPerf,
.B pfPerf,
.B pfGetIRIXRelease
\- 
Routines for querying parameters of system configuration
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'const\ char\ *\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
const char *
\f3\%pfGetMachString\fP(void);
.nf
.fi
.TP
const char *
\f3\%pfGetRelString\fP(void);
.nf
.fi
.TP
int
\f3\%pfQuerySys\fP(int\ which,
'in +2
'll -2
int\ *dst);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfMQuerySys\fP(int\ *which,
'in +2
'll -2
int\ *dst);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfQueryPerf\fP(int\ which,
'in +2
'll -2
void\ *dst);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfPerf\fP(int\ which,
'in +2
'll -2
void\ *dst);
.in
.ll
.nf
.fi
.TP
float
\f3\%pfGetIRIXRelease\fP(void);
.fi
.SH DESCRIPTION

Graphics platforms provide a range of hardware configurations whose
parameters may affect the usability of graphics features.
These functions provide the ability to query these parameters.
OpenGL Performer makes use of the following useful routines in determining
its information: \f3\%getgdesc(3g)\fP, \f3\%XGetVisualInfo(3X11)\fP, and
\f3\%glXGetConfig(3g)\fP.

\f3\%pfGetMachString\fP returns the renderer string for the
current graphics platform:
.RS
.TP 15
\f3\%VGXT\fP
VGX
.TP
\f3\%LIGHT\fP
Indigo Starter
.TP
\f3\%EXTREME\fP
Extreme
.TP
\f3\%RE\fP
RealityEngine
.TP
\f3\%VTX\fP
RealityEngine - VTX
.TP
\f3\%NEWPORT\fP
Indy
.TP
\f3\%CRM\fP
O2
.TP
\f3\%MGRAS\fP
Impact
.TP
\f3\%IR\fP
InfiniteReality
.TP
\f3\%IRL\fP
InfiniteReality on Onyx2
.TP
\f3\%IRE\fP
iR Reality
.RE

\f3\%pfGetRelString\fP returns the string describing the current
OpenGL Performer release: PERFORMER_X.Y.Z[aAAA] where X is the
major number, Y is the minor number, and Z is the maintenance
number of the release.  Maintenance releases are binary
compatible with their base X.Y minor release.  The potential
AAA at the end is an internal attribution string on unreleased
versions, such as an alpha or build number.  \f3\%pfQuerySys\fP
can also be used to directly query this same information.

\f3\%pfQuerySys\fP takes a \f3\%PFQSYS_\fP token and returns in \f2dst\fP
the value for the requested parameter.
The return value is the number of bytes successfully written.

The \f3\%pfQuerySys\fP query token must be one of:
.RS
.TP
\f3\%PFQSYS_GL\fP
returns the GL type currently being used:
\f3\%PFGL_OPENGL\fP, or \f3\%PFGL_NOGL\fP if no graphics
driver is currently running.
.TP
\f3\%PFQSYS_MAJOR_VERSION\fP
returns the major number of the OpenGL Performer release in use.
.TP
\f3\%PFQSYS_MINOR_VERSION\fP
returns the minor number of the OpenGL Performer release in use.
.TP
\f3\%PFQSYS_MAINT_VERSION\fP
returns the maintenance release of the OpenGL Performer release in use.
.TP
\f3\%PFQSYS_NUM_CPUS\fP
returns the number of CPUs on the system.
.TP
\f3\%PFQSYS_NUM_CPUS_AVAILABLE\fP
returns the number of available (non-isolated) CPUs on the system.
.TP
\f3\%PFQSYS_NUM_SCREENS\fP
returns the number of screens, or hardware graphics pipelines.
.TP
\f3\%PFQSYS_SIZE_PIX_X\fP
returns the width of the default screen of the current display in pixels.
The current display is that returned by \f3\%pfGetCurWSConnection\fP.
.TP
\f3\%PFQSYS_SIZE_PIX_Y\fP
returns the height of the default screen of the current display in pixels.
The current display is that returned by \f3\%pfGetCurWSConnection\fP.
.TP
\f3\%PFQSYS_MAX_SNG_RGB_BITS\fP
returns the maximum number of configurable single-buffered bits per
RGB color component.
.TP
\f3\%PFQSYS_MAX_DBL_RGB_BITS\fP
returns the maximum number of configurable double-buffered bits per
RGB color component.
.TP
\f3\%PFQSYS_MAX_SNG_ALPHA_BITS\fP
returns the maximum number of configurable single-buffered bits per
alpha color component.
.TP
\f3\%PFQSYS_MAX_DBL_ALPHA_BITS\fP
returns the maximum number of configurable double-buffered bits per
alpha color component.
.TP
\f3\%PFQSYS_MAX_SNG_ACCUM_BITS\fP
returns the maximum number of configurable single-buffered bits per
accumulation buffer color component.
.TP
\f3\%PFQSYS_MAX_DBL_ACCUM_BITS\fP
returns the maximum number of configurable double-buffered bits per
accumulation buffer color component.
.TP
\f3\%PFQSYS_MAX_SNG_CI_BITS\fP
returns the maximum number of configurable single-buffered bits for
colorindex.
.TP
\f3\%PFQSYS_MAX_DBL_CI_BITS\fP
returns the maximum number of configurable double-buffered bits for
colorindex.
.TP
\f3\%PFQSYS_MAX_SNG_OVERLAY_CI_BITS\fP
returns the maximum number of configurable bits for colorindex
single-buffered overlay buffers.
.TP
\f3\%PFQSYS_MAX_DBL_OVERLAY_CI_BITS\fP
returns the maximum number of configurable bits for colorindex
double-buffered overlay buffers.
.TP
\f3\%PFQSYS_MAX_DEPTH_BITS\fP
returns the maximum number of configurable bits for depth buffers.
.TP
\f3\%PFQSYS_MAX_STENCIL_BITS\fP
returns the maximum number of configurable bits for stencil buffers.
.TP
\f3\%PFQSYS_MAX_MS_SAMPLES\fP
returns the maximum number of configurable subsamples for
multisample buffers.  Multisample buffers are used for antialiasing.
See the \f3\%pfWindow\fP and \f3\%pfAntialias\fP man pages for more
information.
.TP
\f3\%PFQSYS_MAX_MS_DEPTH_BITS\fP
returns the maximum number of configurable bits for
multisampled depth buffers.
.TP
\f3\%PFQSYS_MAX_MS_STENCIL_BITS\fP
returns the maximum number of configurable bits for
multisampled stencil buffers.
.TP
\f3\%PFQSYS_MAX_LIGHTS\fP
returns the maximum allowable number of GL lights that may be on
at one time.
.TP
\f3\%PFQSYS_TEXTURE_MEMORY_BYTES\fP
returns the number of bytes of hardware texture memory.
.TP
\f3\%PFQSYS_MAX_TEXTURE_SIZE\fP
returns the number of bytes in the largest single texture that can
can be allocated in hardware texture memory.
.TP
\f3\%PFQSYS_MAX_CLIPTEXTURE_SIZE\fP
returns the maximum virtual size of both the s and t dimensions
of a pfClipTexture.
.TP
\f3\%PFQSYS_CLIPTEXTURE_CENTER_ALIGNMENT\fP
returns the pfClipTexture alignment constraint based on this system.
The pfClipTexture center must be set to a multiple
of this value in each dimension.
.TP
\f3\%PFQSYS_MIN_CLIPTEXTURE_INVALID_BORDER\fP
returns the minimum invalid border that a pfClipTexture can use on this
system (in order to generate 'correct' texture based of the pfClipTexture).
.TP
\f3\%PFQSYS_TEX_SUBLOAD_ALIGNMENT\fP
returns the minimum texel alignment allowed for pfTexture subloads.
.TP
\f3\%PFQSYS_CALLIGRAPHIC_PIPE0\fP
returns a bit field that indicates which video channels are enabled on the Light Point Board.
A value of 0 either means that there is no board on the Pipe0, or there are no enabled channel
on this board, so no calligraphics lights can be used on the pipe.
\f3\%PFQSYS_CALLIGRAPHIC_PIPE1\fP and \f3\%PFQSYS_CALLIGRAPHIC_PIPE2\fP does the same test for the pipe 1 and the pipe 2.
.TP
\f3\%PFQSYS_MAX_ANISOTROPY\fP
returns the maximum anisotropy supported on the current platform.
.RE

\f3\%pfMQuerySys\fP takes an NULL-terminated array of query tokens and a destination buffer
and will do multiple queries.
The return value will be the number of bytes successfully written.
This routine is more efficient than \f3\%pfQuerySys\fP
if multiple queries are desired.

\f3\%pfQueryPerf\fP
returns performance information for the requested query token.
The currently supported queries deal with texture download timings.
The possible queries are currently values indicating
which texture download cost table the application is interested in
querying.  The token values may be one of:
\f3\%PFQPERF_DEFAULT_TEXLOAD_TABLE\fP - the default
cost table appropriate the current hardware configuration,
\f3\%PFQPERF_USER_TEXLOAD_TABLE\fP - the user defined cost table, if one has been
defined, and \f3\%PFQPERF_CUR_TEXLOAD_TABLE\fP - which cost table, either the
default or the user-defined one, which is currently in use. If a
user-defined cost table is supplied via \f3\%pfPerf\fP,
that table takes precedence.

The cost table is returned as a pointer to a pfTexSubloadCostTable
structure in the \f2dst\fP ptr. This structure contains the width and height
of the cost table arrays of floating point times (in milliseconds), the
minimum legal cost table alignment, and a set of cost table arrays.
There are four arrays, one for 1 byte, 2 byte, 3 byte and 4 byte per
texel formats. Alignment values are required since most machines
require that texture subloads be aligned to some value.

Texture download cost table is internally used by cliptexture load
control (DTR) to estimate the time it will take in the DRAW process to
update the cliptextures texture memory. DTR adjusts the download size
appropriately to try and
keep the total download cost (measured in milliseconds) below a application
defined maximum limit.   Users can use these tables in similar ways for
doing their own texture management for individual pfTextures.
Performer stores a number of cost tables for a
set of important hardware configurations. In addition, a user defined
cost table can be supplied by the application with \f3\%pfPerf\fP.

\f3\%pfPerf\fP
allows the application to provide a custom cost information.
table. The function expects a single argument, a pointer to a token and
associated values.  Currently supported is specifying
pfTexSubloadCost table structure that has been properly filled in.
These times will be then used by the pfClipTexture DTR load management
of texture downloads.  Application-defined cost tables have
precedence, and are used instead of the default table when they are supplied.
The cost table can be checked by reading it back by calling
\f3\%pfQueryPerf\fP with \f3\%PFQPERF_USER_TEXLOAD_TABLE\fP for the \f2which\fP argument.

\f3\%pfGetIRIXRelease\fP will return the X.Y IRIX release number
corresponding to the host system.

.SH NOTES

\f3\%pfWindow::query\fP can be used to query the configuration
parameters of a given pfWindow.  \f3\%pfFeature\fP can be used to query the
availability of specific features on the current hardware configuration.

.SH SEE ALSO
pfFeature,
pfGetCurWSConnection,
pfWindow,
pfAntialias,
pfTexture,
pfClipTexture,
XGetVisualInfo,
glXGetConfig
