'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfSeg" "3pf"
.SH NAME
.upperok
.B pfSeg,
.B pfTriIsectSeg
\- 
Set and operate on line segments
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr/pfGeoMath.h>
.nf
.fi
.TP
\ 
\f3\%pfSeg::pfSeg\fP();
.nf
.fi
.TP
\ 
\f3\%pfSeg\fP(const\ pfVec3&\ p1,
'in +2
'll -2
const\ pfVec3&\ p2);
.in
.ll
.nf
.fi
.TP
\ 
\f3\%pfSeg\fP(const\ pfVec3&\ _pos,
'in +2
'll -2
float\ azi,
float\ elev,
float\ len);
.in
.ll
.nf
.fi
.TP
\ 
\f3\%pfSeg\fP(const\ pfVec3&\ _pos,
'in +2
'll -2
const\ pfVec3&\ _dir,
float\ _length);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSeg::clip\fP(const\ pfSeg\ *seg,
'in +2
'll -2
float\ d1,
float\ d2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSeg::makePts\fP(const\ pfVec3\ &p1,
'in +2
'll -2
const\ pfVec3\ &p2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSeg::makePolar\fP(const\ pfVec3\ &pos,
'in +2
'll -2
float\ azi,
float\ elev,
float\ len);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfSeg::closestPtsOn\fP(const\ pfSeg\ *seg2,
'in +2
'll -2
pfVec3\ &ptOn1,
pfVec3\ &ptOn2);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfTriIsectSeg\fP(const\ pfVec3\ v1,
'in +2
'll -2
const\ pfVec3\ v2,
const\ pfVec3\ v3,
const\ pfSeg\ *seg,
float\ *d);
.in
.ll
.fi

.Ex
.nf

struct pfSeg
{
    pfVec3      pos;
    pfVec3      dir;
    float       length;
};
.Ee
.fi
.na

.SH DESCRIPTION


A \f3\%pfSeg\fP represents a line segment starting at \f2pos\fP, extending
for a length \f2length\fP in the direction \f2dir\fP.  The routines assume
that \f2dir\fP is of unit length, otherwise the results are undefined.
pfSeg is a public struct whose data members \f2pos\fP, \f2dir\fP and
\f2length\fP may be operated on directly.

The default constructor \f3\%pfSeg\fP() is empty and does no
initialization.  There are also three other constructors
to initialize the pfSeg from two points , polar coordinates,
or position/direction/length.  These constructors correspond
directly to makePts, makePolar, and direct initialization.
\f3\%new\fP(arena) allocates a pfSeg from the specified
memory arena, or from the heap if \f2arena\fP is NULL.  \f3\%new\fP
allocates a pfSeg from the default memory arena (see
\f3\%pfGetSharedArena\fP).  pfSegs can also be created automatically on
the stack or statically.  pfSegs allocated with \f3\%new\fP can be deleted
with \f3\%delete\fP or \f3\%pfDelete\fP.

\f3\%pfSeg::clip\fP is used to select a subset of the pfSeg.  It sets \f2dst\fP to the portion
of the pfSegment clipped to start at distance
\f2d1\fP and end at \f2d2\fP.  When \f2d1\fP = 0 and \f2d2\fP =
\f2seg\fP->length, \f3\%pfSeg::clip\fP returns the
original segment.  Values of \f2d1\fP < 0 and \f2d2\fP > \f2seg\fP->length
can be used to extend the segment.

\f3\%pfSeg::makePts\fP sets the pfSeg to the segment which starts at the point
\f2p1\fP and ends at the point \f2p2\fP.

\f3\%pfSeg::makePolar\fP sets the pfSeg to the segment which starts at \f2pos\fP
and has length \f2length\fP and points in the direction specified by
\f2azi\fP and \f2elev\fP.  \f2azi\fP specifies the azimuth (or heading),
which is the angle which the projection of the segment in the X-Y plane
makes with the +Y axis.  \f2elev\fP specifies the elevation (or pitch),
the angle with respect to the X-Y plane.  The positive Y axis is
\f2azi\fP=0 and \f2elev\fP=0.  Azimuth follows the right hand rule about
the Z axis, i.e. +90 degrees is the -X axis.  Similarly, elevation
follows the right hand rule about the X axis, i.e. +90 degrees is the
+Z axis.

\f3\%pfSeg::closestPtsOn\fP returns the two closest
points on the
implicit pfSeg
and \f2seg2\fP.  If the two segments are parallel FALSE
is returned and the contents of ptOn1 and ptOn2 are undefined.

\f3\%pfTriIsectSeg\fP tests the line segment \f2seg\fP for intersection with
the triangle defined by the three vertices \f2v1\fP, \f2v2\fP, and \f2v3\fP.
\f3\%pfTriIsectSeg\fP returns TRUE or FALSE.  If \f2d\fP is non-null, on
return it contains the length position of the intersection between 0
and seg->length.

.SH SEE ALSO
pfNodeIsectSegs,
pfGSetIsectSegs,
pfVec3
