'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfShadeModel" "3pf"
.SH NAME
.upperok
.B pfShadeModel,
.B pfGetShadeModel
\- 
Set and get the shading model
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfShadeModel\fP(int\ model);
.nf
.fi
.TP
int
\f3\%pfGetShadeModel\fP(void);
.fi
.SH PARAMETERS
.TP \w'model\ \ 'u
\f2model\fP
is a symbolic constant and is one of:
.RS
.TP 20
\f3\%PFSM_FLAT\fP
Use flat shading,
.TP
\f3\%PFSM_GOURAUD\fP
Use Gouraud shading.
.RE
.SH DESCRIPTION

\f3\%pfShadeModel\fP sets the shading model to \f2model\fP.  When flat shading
is enabled, the last vertex in a geometric primitive defines the color
of the entire geometric primitive.  When Gouraud shading, vertex colors
are interpolated across the primitive.

The following example shows how data equivalent to OpenGL immediate
mode graphics commands would be interpreted in both \f3\%PFSM_FLAT\fP and
\f3\%PFSM_GOURAUD\fP shade models.

Example 1:
.Ex
.nf

/*
 * Draw a three-primitive triangle strip in OpenGL
 */
glColor3f(0, 0, 0);	    /****************************************/
glBegin(GL__TRIANGLE_STRIP);/*   Actual Rendered Triangle Colors    */
glVertex3v(v0);		    /* ------------------------------------ */
glVertex3v(v1);		    /* Tri  PFSM_FLAT       PFSM_GOURAUD    */
glColor3f(1, 0, 0);	    /* ------------------------------------ */
glVertex3v(v2);		    /*  0   red             black/black/red */
glColor3f(0, 1, 0);	    /*                                      */
glVertex3v(v3);		    /*  1   green           black/red/green */
glColor3f(0, 0, 1);	    /*                                      */
glVertex3v(v4);		    /*  2   blue            red/green/blue  */
glEnd();		    /****************************************/
.Ee
.fi
.na


Consequently, strips (triangle \f3\%PFGS_TRISTRIPS\fP or line
\f3\%PFGS_LINESTRIPS\fP) which are composed of different colored primitives
\f2must\fP have flat shading enabled in order to be rendered properly.
The pfGeoSet primitive types of \f3\%PFGS_FLAT_TRISTRIPS\fP and
\f3\%PFGS_FLAT_LINESTRIPS\fP ensure that flat shading will be enabled when
the pfGeoSet is drawn.

Another subtlety of the shading model is related to the current
lighting model.  If the lighting model is local due to either the
pfLightModel (\f3\%pfLightModel::setLocal\fP) or if any pfLights are local
(\f3\%pfLight::setPos\fP), then Gouraud shading must be enabled since lighting
effects should be different at each vertex.  This means that even if a
triangle has a constant color and normal, it should still be drawn with
Gouraud shading so the effects of the local lighting can be seen.  The
exception to this rule are the flat strips discussed above.

The shading model state element is identified by the
\f3\%PFSTATE_SHADEMODEL\fP token.  Use this token with \f3\%pfGeoState::setMode\fP to
set the shading model of a pfGeoState and with \f3\%pfOverride\fP to
override subsequent shading model changes.

Example 2:
.Ex
.nf

/* Set up flat shaded pfGeoState. */
gstate->setMode(PFSTATE_SHADEMODEL, PFSM_FLAT);

/* Attach gstate to gset */
gset->setGState(gstate);

/* Draw flat shaded gset */
gset->draw();
.Ee
.fi
.na


Example 3:
.Ex
.nf

/* Alternative way to draw flat-shaded pfGeoSet */
gset->setDrawMode(PFGS_FLATSHADE, PF_ON);
gset->draw();
.Ee
.fi
.na


Example 4:
.Ex
.nf

/*
 * Draw flat-shaded triangle strip pfGeoSet.  PFGS_FLATSHADE
 * and pfShadeModel are not required.
 */
gset->setPrimType(PFGS_FLAT_TRISTRIPS);
gset->draw();
.Ee
.fi
.na


Example 5:
.Ex
.nf

pfShadeModel(PFSM_FLAT);

/* Override shading model to PFSM_FLAT */
pfOverride(PFSTATE_SHADEMODEL, PF_ON);
.Ee
.fi
.na


\f3\%pfShadeModel\fP
is a display-listable command.  If a pfDispList has been opened by
\f3\%pfDispList::open\fP,
\f3\%pfShadeModel\fP
will not have immediate effect but will be captured by the pfDispList
and will only have effect when that pfDispList is later drawn with
\f3\%pfDispList::draw\fP.

The selection of which shading model a pfGeoSet uses is based upon the
following decision hierarchy:
.RS
.TP 5
1.
Use flat shading if pfGeoSet is \f3\%PFGS_FLAT_TRISTRIPS\fP or
\f3\%PFGS_FLAT_LINESTRIPS\fP or if \f3\%PFGS_FLATSHADE\fP is enabled through
\f3\%pfGeoSet::setDrawMode\fP.
.TP
2.
Use the shading model set by the attached pfGeoState, if any
(see \f3\%pfGeoSet::setGState\fP).
.TP
3.
Use the shading model set by \f3\%pfShadeModel\fP.
.RE

The default shading model is Gouraud.

\f3\%pfGetShadeModel\fP returns the current shading model.
.SH NOTES

Overriding the shading model to \f3\%PFSM_FLAT\fP can be a useful debugging
aid since it reveals the facets of a normally smooth surface.
.SH SEE ALSO
pfGSetGState,
pfGeoSet,
pfGeoState,
pfLModelLocal,
pfLightPos,
pfState
