'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfSphere" "3pf"
.SH NAME
.upperok
.B pfSphere
\- 
Set, transform and extend a sphere
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void*\ \ \ 'u
#include <Performer/pr/pfGeoMath.h>
.nf
.fi
.TP
void*
\f3\%pfSphere::operator new\fP(size_t);
.nf
.fi
.TP
void*
\f3\%pfSphere::operator new\fP(size_t,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
\ 
\f3\%pfSphere::pfSphere\fP();
.nf
.fi
.TP
\ 
\f3\%pfSphere\fP(const\ pfVec3&\ _center,
'in +2
'll -2
float\ _radius);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphere::makeEmpty\fP(void);
.nf
.fi
.TP
void
\f3\%pfSphere::extendBy\fP(const\ pfVec3\ &pt);
.nf
.fi
.TP
void
\f3\%pfSphere::extendBy\fP(const\ pfSphere*\ sph);
.nf
.fi
.TP
void
\f3\%pfSphere::extendBy\fP(const\ pfCylinder*\ cyl);
.nf
.fi
.TP
void
\f3\%pfSphere::around\fP(const\ pfVec3\ *pts,
'in +2
'll -2
int\ npt);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphere::around\fP(const\ pfSphere\ **sphs,
'in +2
'll -2
int\ nsph);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphere::around\fP(const\ pfBox\ **boxes,
'in +2
'll -2
int\ nbox);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphere::around\fP(const\ pfCylinder\ **cyls,
'in +2
'll -2
int\ ncyl);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfSphere::contains\fP(const\ pfVec3\ &pt);
.nf
.fi
.TP
int
\f3\%pfSphere::contains\fP(const\ pfSphere\ *sph2);
.nf
.fi
.TP
int
\f3\%pfSphere::contains\fP(const\ pfCylinder\ *cyl);
.nf
.fi
.TP
int
\f3\%pfSphere::contains\fP(const\ pfBox\ *box);
.nf
.fi
.TP
int
\f3\%pfSphere::isect\fP(const\ pfSeg*\ seg,
'in +2
'll -2
float*\ d1,
float*\ d2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphere::orthoXform\fP(const\ pfSphere\ *sph,
'in +2
'll -2
const\ pfMatrix\ &xform);
.in
.ll
.fi

.Ex
.nf

struct pfSphere
{
    pfVec3 center;
    float radius;
};
.Ee
.fi
.na

.SH DESCRIPTION


A pfSphere represents a sphere as a \f2center\fP and a \f2radius\fP.  The
routines listed here provide means of creating and extending spheres
for use as bounding geometry.  pfSphere is a public struct whose data
members \f2center\fP and \f2radius\fP may be operated on directly.

The default constructor \f3\%pfSphere\fP() is empty and does no
initialization.  There is also a constructor to directly
initialize the center and radius of the pfSphere. \f3\%new\fP(arena)
allocates a pfSphere from the specified memory arena, or from the
heap if \f2arena\fP is NULL.  \f3\%new\fP allocates a pfSphere from the
default memory arena (see \f3\%pfGetSharedArena\fP).  pfSpheres can
also be created automatically on the stack or statically.
pfSpheres allocated with \f3\%new\fP can be deleted with \f3\%delete\fP
or \f3\%pfDelete\fP.

\f3\%pfSphere::makeEmpty\fP sets
the pfSphere so that it appears empty to extend and
around operations.

\f3\%pfSphere::extendBy(const pfVec3 pt)\fP,
\f3\%pfSphere::extendBy(const pfSphere
*sph)\fP, and \f3\%pfSphere::extendBy(const
pfCylinder *cyl)\fP set the pfSphere to a sphere
which contains both the pfSphere and the point \f2pt\fP,
the sphere \f2sph\fP or the cylinder \f2cyl\fP, respectively.

\f3\%pfSphere::around(const pfVec3 *, int)\fP,
\f3\%pfSphere::around(const pfBox **, int)\fP,
\f3\%pfSphere::around(const pfSphere **,
int)\fP and \f3\%pfCylinder::around(const
pfCylinder **, int)\fP set the pfSphere to a sphere
which contains a set of points, boxes, spheres or cylinders,
respectively.  These routines are passed the address of an array of
pointers to the objects being bounded along with the number of
objects.

\f3\%pfSphere::contains(const pfVec3 pt)\fP,
returns \f3\%TRUE\fP or \f3\%FALSE\fP depending on whether the point \f2pt\fP is
in the interior of the specified sphere.

\f3\%pfSphere::contains(const pfSphere
*)\fP, \f3\%pfSphere::contains(const
pfCylinder *)\fP and \f3\%pfSphere::contains(const
pfBox *)\fP test whether the sphere  contains a non-empty portion of the volume specified by
the  argument, a sphere, a cylinder or a box, respectively.

The return value from the these functions is the OR of one or
more bit fields.  The returned value may be:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
The intersection of the  argument and the sphere is empty.
.TP
\f3\%PFIS_MAYBE\fP:
The intersection of the  argument
and the sphere might be non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP:
The intersection of the 
argument and the sphere is  definitely non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP:
The 
argument is non-empty and lies entirely inside the sphere.
.RE

\f3\%pfSphere::isect\fP intersects the line
segment \f2seg\fP with the volume of the pfSphere \f2sphere\fP.  The
possible return values are:

.RS
.TP 5
\f3\%PFIS_FALSE\fP:
\f2seg\fP lies entirely in the exterior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_START_IN\fP:
The starting point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_END_IN\fP:
The ending point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP | \f3\%PFIS_START_IN\fP | \f3\%PFIS_END_IN\fP:
Both end points of \f2seg\fP lie in the interior.
.RE

If \f2d1\fP and \f2d2\fP are non-NULL, on return from
\f3\%pfSphere::isect\fP they contain the starting
and ending positions of the line segment (0 <= d1 <= d2 <=
seg->length) intersected with the sphere.

\f3\%pfSphere::orthoXform\fP
sets the pfSphere to be
the sphere \f2sph\fP
transformed by the orthogonal transform \f2xform\fP.

.SH NOTES

The bit fields returned by the contains functions are structured so
that bitwise AND-ing the results of sequential tests can be used to
compute composite results, e.g. testing exclusion against a number
of half spaces.

Some of the extend and around operations are time consuming and should
be used sparingly.  In general, the quality of a bound generated by a
series of extend operations will be no better, and sometimes much
worse, than a bound generated by a single around operation.

C++ does not support array deletion (i.e. \f3\%delete[]\fP) for
arrays of objects allocated new operators that take additional
arguments.  Hence, the array deletion operator \f3\%delete[]\fP should not
be used on arrays of objects created with \f3\%new\fP(arena) pfVec3[n].

.SH SEE ALSO
pfBox,
pfCylinder,
pfSeg,
pfVec3
