'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfTraverser" "3pf"
.SH NAME
.upperok
.B pfTraverser,
.B pfCullResult,
.B pfGetCullResult,
.B pfGetParentCullResult
\- 
Set and get traversal masks, callback functions and callback data, and get pfTraverser attributes.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'const\ pfPath\ *\ \ \ 'u
#include <Performer/pf/pfTraverser.h>
.nf
.fi
.TP
pfChannel *
\f3\%pfTraverser::getChan\fP(void);
.nf
.fi
.TP
void
\f3\%pfTraverser::getMat\fP(pfMatrix\ &mat);
.nf
.fi
.TP
pfNode *
\f3\%pfTraverser::getNode\fP(void);
.nf
.fi
.TP
int
\f3\%pfTraverser::getIndex\fP(void);
.nf
.fi
.TP
const pfPath *
\f3\%pfTraverser::getPath\fP(void);
.nf
.fi
.TP
void
\f3\%pfCullResult\fP(int\ result);
.nf
.fi
.TP
int
\f3\%pfGetCullResult\fP(void);
.nf
.fi
.TP
int
\f3\%pfGetParentCullResult\fP(void);
.fi

.Ex
.nf

typedef int (*pfNodeTravFuncType)(pfTraverser *trav, void *userData);
.Ee
.fi
.na

.SH PARAMETERS
.TP \w'which\ \ 'u
\f2which\fP
identifies the traversal: \f3\%PFTRAV_ISECT\fP, \f3\%PFTRAV_APP\fP,
\f3\%PFTRAV_CULL\fP or \f3\%PFTRAV_DRAW\fP, denoting the intersection, application,
cull or draw traversals respectively.
.SH DESCRIPTION

OpenGL Performer provides four major traversals: intersection,
application, cull, and draw that are often abbreviated as \f3\%ISECT\fP,
\f3\%APP\fP, \f3\%CULL\fP, and \f3\%DRAW\fP.  A traversal is typically an in-order
traversal of a directed acyclic graph of pfNodes otherwise known as a
subgraph.  The actual traversal method, traverser structure, and
traversal initiation used depends on the traversal type as well as the
multiprocessing mode as shown in the following table.

.ne 15
.TS
center tab($);
C | C | C | C
L | L | L | L .
Traversal$Traverser$Traversee$Trigger
_
PFTRAV_ISECT$pfSegSet$subgraph$pfNode::isect(),
$$$pfChanNodeIsectSegs
PFTRAV_APP$pfTraverser$pfScene$pfApp()

CULL_DL_DRAW is set$$$
\ PFTRAV_CULL$pfChannel$pfScene$pfCull()
\ PFTRAV_DRAW$pfChannel$pfDispList$pfDraw()

CULL_DL_DRAW is not set$$$
\ PFTRAV_CULL$pfChannel$pfScene$pfDraw()
\ PFTRAV_DRAW$$$
.TE

Typical traversal callback usage:

.RS
.TP 5
\f3\%ISECT\fP
Collision detection, terrain following, line of sight
.TP
\f3\%APP\fP
Application-specific behavior, motors
.TP
\f3\%CULL\fP
Custom level-of-detail selection, culling
.TP
\f3\%DRAW\fP
Custom rendering
.RE

When \f3\%PFMPCULL_DL_DRAW\fP is not set in the multiprocessing mode
argument to \f3\%pfMultiprocess\fP (and the cull and draw stages are in
the same process), then \f3\%pfDraw\fP simultaneously culls and draws the
pfScene attached to the pfChannel by
\f3\%pfChannel::setScene\fP.  Otherwise, \f3\%pfCull\fP
culls and builds up a pfDispList which is later rendered by
\f3\%pfDraw\fP.

If the traversal CULL mask and node CULL mask AND to zero at a node,
the CULL traversal disables view culling and trivially accepts the
node and all its descendents.  Note that unlike other traversals, a
mask result of 0 does not prune the node.

If the traversal DRAW mask and node DRAW mask AND to zero at a node,
the CULL traversal prunes the node, so descendents are neither
CULL-traversed nor drawn.

If the traversal APP mask and the node APP mask AND to zero, the APP
traversal prunes the node and its descendents.

If the ISECT masks AND to zero, the ISECT traversal prunes the node.
The intersection mask is typically used to control traversals of
different types of objects, e.g.  different bits may indicate ground,
water, foliage, and buildings, so they may be intersected selectively.
See (\f3\%pfNode::setTravMask\fP).

In many respects a traversal appears to the user as an atomic action.
The user configures a traverser, triggers it with the appropriate
routine and awaits the results.  Node callbacks are supported to
provide user extensibility and configuration into this scenario.  They
are user-supplied routines that are invoked in the course of a
traversal.  Callbacks return a value which can control traversal on a
coarse-grained basis.  In addition, draw callbacks can render custom
geometry and cull callbacks can substitute custom culling for the
default OpenGL Performer culling.

The pre- or post-callbacks for the cull and intersection traversals may
return \f3\%PFTRAV_CONT\fP, \f3\%PFTRAV_PRUNE\fP, \f3\%PFTRAV_TERM\fP to indicate
that traversal should continue normally, skip this node or terminate
the traversal, respectively.  \f3\%PFTRAV_PRUNE\fP is equivalent to
\f3\%PFTRAV_CONT\fP for the post-callback. Currently, the return value from
the draw callbacks is ignored.

\f3\%pfCullResult\fP, \f3\%pfGetCullResult\fP, and \f3\%pfGetParentCullResult\fP
can all be called in the pre-cull callback and all but \f3\%pfCullResult\fP
may be called in the post-cull callback.  \f3\%pfGetCullResult\fP returns
the result of the cull for the node that the cull callback is
associated with.  \f3\%pfGetParentCullResult\fP returns the cull result for
the parent of the node that the cull callback is associated with.  When
called within the pre-cull callback, \f3\%pfCullResult\fP specifies the
result of cull for the node that the pre-cull callback is associated
with.  This essentially replaces default OpenGL Performer cull processing
with user-defined culling.  \f2result\fP is a token which specifies the
result of the cull test and should be one of:
.RS
.TP 5
\f3\%PFIS_FALSE\fP
Node is entirely outside the viewing frustum and should be pruned.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP
Node is partially inside the viewing frustum and the children of the
node should be cull-tested.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP
Node is totally inside the viewing frustum so all the children of the
node should be trivially accepted without further cull testing.
.RE

If \f3\%pfCullResult\fP is not called within the pre-cull callback, OpenGL
Performer will use its default geometric culling mechanism that
compares node bounding volumes to the current culling frustum to
determine if the node may be within view.

In the post-cull callback \f3\%pfGetCullResult\fP will return the
result of the cull set by \f3\%pfCullResult\fP or the result of
the default cull if \f3\%pfCullResult\fP was not called.

The evaluation order of the cull and draw traversal masks and callbacks
is illustrated in the following pseudo-code:

Example 1: Cull and draw traversal mask and callback evaluation order.
.Ex
.nf

/* Return if draw mask test fails */
if ((drawMask & nodeDrawMask) == 0)
    return PFTRAV_CONT;

/* Call pre-cull callback */
if (preCull)
{
    rtn = (*preCull)(traverser, cullData);

    if (rtn == PFTRAV_PRUNE)
        return PFTRAV_CONT;
    else if (rtn == PFTRAV_TERM)
        return PFTRAV_TERM;
}

/* Disable view culling if cull mask test fails */
if ((cullMask & nodeCullMask) == 0)
    disableViewCulling();

/* Perform default culling if pfCullResult was not called */
if (!userCalledpfCullResultInThePreCullCallback)
    cullResult = cullTest(node);

if (cullResult == PFIS_FALSE)
{
    /* Call post-cull callback */
    if (postCull)
    {
        rtn = (*postCull)(traverser, cullData);

        if (rtn == PFTRAV_PRUNE)
            return PFTRAV_CONT;
        else if (rtn == PFTRAV_TERM)
            return PFTRAV_TERM;
    }
    return PFTRAV_CONT;
}
else
/* Trivially accept node and all its children */
if (cullResult == PFIS_ALL_IN)
    disableViewCulling();

/* Call pre-draw callback */
if (preDraw)
    (*preDraw)(traverser, drawData);

evaluateNodeAndItsChildren();

/* Call post-draw callback */
if (postDraw)
    (*postDraw)(traverser, drawData);

/* Call post-cull callback */
if (postCull)
{
    rtn = (*postCull)(traverser, cullData);

    if (rtn == PFTRAV_PRUNE)
        return PFTRAV_CONT;
    else
    if (rtn == PFTRAV_TERM)
        return PFTRAV_TERM;
}

return PFTRAV_CONT;
.Ee
.fi
.na


Example 2: Use of DRAW callbacks to save and restore state.
.Ex
.nf

extern int
preDraw(pfTraverser *trav, void *data)
{
    pfPushState();
    pfEnable(PFEN_TEXGEN);
    ((pfTexGen*)data)->apply;

    return PFTRAV_CONT;
}

extern int
postDraw(pfTraverser *trav, void *data)
{
    pfPopState();

    return PFTRAV_CONT;
}

/*
 * Set up draw callbacks and user data to draw 'geode' in
 * EYE_LINEAR texgen mode.
 */


pfTexGen	*tgen;

tgen = new pfTexGen(pfGetSharedArena());

tgen->setMode(PF_S, PFTG_EYE_LINEAR);
tgen->setMode(PF_T, PFTG_EYE_LINEAR);

geode->setTravFuncs(PFTRAV_DRAW, preDraw, postDraw);
geode->setTravData(PFTRAV_DRAW, tgen);

.Ee
.fi
.na

\f3\%libpr\fP graphics calls like \f3\%pfTexGen::apply\fP should be made in a DRAW callback
only. Specifically, \f3\%libpr\fP graphics calls made in a CULL callback are not
legal and have undefined behavior.

The intersection, application, cull and draw callbacks are passed a
pfTraverser which can be used to query the channel, current
transformation matrix and current node.
\f3\%pfTraverser::getChan\fP returns the current
channel for the cull, and draw traversal.  It returns the current
channel for intersection traversals initiated with
\f3\%pfChanNodeIsectSegs\fP and \f3\%NULL\fP for intersection traversals
initiated with \f3\%pfNode::isect\fP.

\f3\%pfTraverser::getMat\fP sets
\f2mat\fP to the current transformation matrix, which is the
concatenation of the matrices from the root of the scene down to and
including the current node.  Since no transformation hierarchy is
retained in the draw process, in a draw callback, the current matrix
should be queried using the \f3\%getmatrix\fP or
\f3\%pfGetModelMat\fP/\f3\%pfGetViewMat\fP routines.

\f3\%pfTraverser::getNode\fP returns the current
node being traversed and
\f3\%pfTraverser::getIndex\fP returns the child
index of the current node, i.e.- the index of the current node in its
parent's list of children.
\f3\%pfTraverser::getPath\fP returns a pointer to
the list of nodes which defines the path from the scene graph root to
the current node.

.SH NOTES

The post-cull callback is a good place to implement custom level-of-detail
mechanisms.
.SH BUGS

The path returned by \f3\%pfGetTravPath\fP is valid only when invoked
from a cull callback.
.SH SEE ALSO
pfGroup,
pfClone,
pfFrame,
pfNode
