'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfViewMat" "3pf"
.SH NAME
.upperok
.B pfModelMat,
.B pfGetModelMat,
.B pfViewMat,
.B pfGetViewMat,
.B pfInvViewMat,
.B pfGetInvViewMat,
.B pfProjMat,
.B pfGetProjMat,
.B pfInvModelMat,
.B pfGetInvModelMat,
.B pfNearPixDist,
.B pfGetNearPixDist,
.B pfTexMat,
.B pfGetTexMat
\- 
Set/get shadows of viewing, modeling, texturing, and inverse modeling matrices; set/get near plane pixel distance
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'float\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfModelMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfGetModelMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfViewMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfGetViewMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfInvViewMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfGetInvViewMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfProjMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfGetProjMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfInvModelMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfGetInvModelMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfNearPixDist\fP(float\ npd);
.nf
.fi
.TP
float
\f3\%pfGetNearPixDist\fP(void);
.nf
.fi
.TP
void
\f3\%pfTexMat\fP(pfMatrix\ mat);
.nf
.fi
.TP
void
\f3\%pfGetTexMat\fP(pfMatrix\ mat);
.fi
.SH DESCRIPTION

Certain eyepoint-dependent features require knowledge about the
viewing and modeling matrices as well as knowledge about the
mapping of eye to window coordinates. Current eyepoint-dependent features
include pfLPointState, which renders \f3\%PFGS_POINTS\fP pfGeoSets
as "light points" and pfSprite, which rotates geometry to face the
viewer.

\f3\%pfViewMat\fP, \f3\%pfProjMat\fP, \f3\%pfModelMat\fP, \f3\%pfTexMat\fP
and \f3\%pfNearPixDist\fP set the current viewing matrix, projection matrix,
modeling matrix, texture matrix,
and distance to the near (view) plane in pixels. These routines
in no way affect the graphics library or its transformation stack.
They only specify values used internally by \f3\%libpr\fP. \f3\%libpf\fP applications
do not need to call these routines unless they modify associated
values through the graphics library instead of through
\f3\%libpr\fP matrix routines, e.g., if the viewing transform is set with
\f3\%pfLoadMatrix\fP instead of \f3\%pfChanViewMat\fP. \f3\%pfTexMat\fP is provided
so \f3\%libpr\fP can change, then restore the current texture matrix without
incurring the cost of querying the graphics library for the matrix value.

pfLPointState and pfSprite both require the viewing and modeling matrices.
The inverse modeling matrix is automatically computed by OpenGL Performer
when necessary unless it is specified by the application with
\f3\%pfInvModelMat\fP. pfLPointState also requires the distance, measured
in pixels, from the eyepoint to the near plane of the viewing frustum.
This value is required in order to render light points at their proper
screen size. Use \f3\%pfNearPixDist\fP to set the "near pixel distance"
and \f3\%pfGetNearPixDist\fP to get it.
pfCalligraphic requieres the projection matrix also to be set before
caling pfDispList::preprocess. It also
make use of the inverse viewing matrix that is automatically computed
unless specified with \f3\%pfInvViewMat\fP.

\f3\%pfViewMat\fP makes \f2mat\fP the current viewing matrix.
\f2mat\fP is a 4x4 homogeneous matrix which defines the view
coordinate system such that the upper 3x3 submatrix defines the
coordinate system axes and the bottom vector defines the coordinate
system origin.  OpenGL Performer defines the view direction to be along
the positive Y axis and the up direction to be the positive Z
direction, e.g., the second row of \f2mat\fP defines the viewing direction
and the third row defines the up direction in world coordinates.
\f2mat\fP must be orthonormal or results are undefined.
\f3\%pfGetViewMat\fP copies the current viewing matrix into \f2mat\fP.

\f3\%pfModelMat\fP makes \f2mat\fP the current modeling matrix. \f3\%pfGetModelMat\fP
copies the current modeling matrix into \f2mat\fP.

\f3\%pfProjMat\fP makes \f2mat\fP the current projection matrix. \f3\%pfGetProjMat\fP
copies the current projection matrix into \f2mat\fP.

\f3\%pfViewMat\fP, \f3\%pfModelMat\fP, \f3\%pfProjMat\fP, \f3\%pfTexMat\fP, \f3\%pfInvModelMat\fP,
\f3\%pfInvViewMat\fP, and \f3\%pfNearPixDist\fP
are all display-listable commands.  If a pfDispList has been
opened by \f3\%pfOpenDList\fP, these commands will not have immediate effect
but will be captured by the pfDispList and will only have effect when
that pfDispList is later drawn with \f3\%pfDrawDList\fP.

.SH SEE ALSO
pfLPointState,
pfSprite,
pfDispList
