'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfWSConnection" "3pf"
.SH NAME
.upperok
.B pfGetCurWSConnection,
.B pfOpenWSConnection,
.B pfSelectWSConnection,
.B pfOpenScreen,
.B pfCloseWSConnection,
.B pfGetWSConnectionName,
.B pfGetScreenSize,
.B pfGetNumScreenVChans,
.B pfChooseFBConfig,
.B pfChooseFBConfigData
\- 
Window system utility routines
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfWSConnection\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
pfWSConnection
\f3\%pfGetCurWSConnection\fP(void);
.nf
.fi
.TP
pfWSConnection
\f3\%pfOpenWSConnection\fP(const\ char\ *str,
'in +2
'll -2
int\ shared);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSelectWSConnection\fP(pfWSConnection\ ws);
.nf
.fi
.TP
pfWSConnection
\f3\%pfOpenScreen\fP(int\ screen,
'in +2
'll -2
int\ shared);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCloseWSConnection\fP(pfWSConnection\ ws);
.nf
.fi
.TP
const char*
\f3\%pfGetWSConnectionName\fP(pfWSConnection\ ws);
.nf
.fi
.TP
void
\f3\%pfGetScreenSize\fP(int\ screen,
'in +2
'll -2
int\ *x,
int\ *y);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfGetNumScreenVChans\fP(int\ screen);
.nf
.fi
.TP
pfFBConfig
\f3\%pfChooseFBConfig\fP(pfWSConnection\ ws,
'in +2
'll -2
int\ screen,
int\ *attr);
.in
.ll
.nf
.fi
.TP
pfFBConfig
\f3\%pfChooseFBConfigData\fP(void\ **dst,
'in +2
'll -2
pfWSConnection\ ws,
int\ screen,
int\ *attr,
void\ *arena);
.in
.ll
.fi

.Ex
.nf

/* typedef of X-based Performer Types */
typedef Display         *pfWSConnection;
typedef XVisualInfo      pfFBConfig;
typedef Window           pfWSWindow;
typedef Drawable         pfWSDrawable;

/* typedef of GL-based Performer Types */
typedef GLXContext       pfGLContext;
.Ee
.fi
.na

.SH PARAMETERS
.TP \w'ws\ \ 'u
\f2ws\fP
identifies a pfWSConnection.
.SH DESCRIPTION

These functions provide a single API for communicating with the window
system.

These routines communicate with the window system via a \f3\%pfWSConnection\fP to a
window server.  This connection is \f2per-process\fP.  If a process tries to use
the pfWSConnection of another process, bad things are likely to result.  A
process may use its pfWSConnection to open and communication with all windows
on all screens managed by that window server.  Typically, a given machine will
have a single window server, even if they have multiple screens -- all screens
are managed by the single server.  Exceptions are the multiple-keyboard
machines.  A pfWSConnection has a default screen on which windows will be
opened if no other screen is explicitly named (such as through
\f3\%pfWindow::setScreen\fP).  This default screen is
determined by the \f3\%DISPLAY\fP environment variable, and is screen 0 if the
\f3\%DISPLAY\fP variable is unset.

A \f3\%pfWSWindow\fP is a window-server X window.
A \f3\%pfWSDrawable\fP is a
window system primitive that may be connected to a GL drawing context.
A \f3\%pfWSDrawable\fP includes Pixmaps.  \f3\%pfGLContext\fP is the GL context
that is attached to the \f3\%pfWSDrawable\fP.

Windows have framebuffer resources associated with them, such as a zbuffer,
stencil planes, and possibly multisample buffers.  A framebuffer configuration
can be created by \f3\%pfChooseFBConfig\fP, or
\f3\%pfWindow::chooseFBConfig\fP, and is returned in a
\f3\%pfFBConfig\fP.

\f3\%pfGetCurWSConnection\fP returns the current connection to the window
system.  If there is no current open connection, one is opened on the
default display (using the \f3\%DISPLAY\fP environment variable).

\f3\%pfOpenWSConnection\fP opens the window server connection named by
\f2str\fP and if \f2shared\fP is true, this connection will be shared with
OpenGL Performer and made the current libpr window system connection via
a call to \f3\%pfSelectWSConnection\fP.

\f3\%pfOpenScreen\fP opens a local window server connection with the
default screen of \f2screen\fP.  If \f2screen\fP is (-1), the default
screen as set by the \f3\%DISPLAY\fP environment variable will be used, or
screen 0 if this variable is unset. Note that all window system
connections can communication with all screens managed by that window
server, regardless of the value of the default screen.

\f3\%pfSelectWSConnection\fP sets the current OpenGL Performer libpr window
system connection to be \f2ws\fP. The \f3\%DISPLAY\fP environment variable
for that process will be set to the connection name for \f2ws\fP using the
\f3\%putenv(3C)\fP command.
This window system connection will be
that returned by \f3\%pfGetCurWSConnection\fP.

\f3\%pfCloseWSConnection\fP will close the specified window server
connection via XCloseDisplay. If \f2ws\fP was the currently selected libpr,
window server connection, the current connection will be reset to NULL.

\f3\%pfGetWSConnectionName\fP will return the string name for the window
system connection.  In the X window system, this corresponds to the string
returned by the \f3\%XDisplayString\fP call.

\f3\%pfGetScreenSize\fP returns the X and Y size of \f2screen\fP for the
server of the current window system connection in \f2x\fP and \f2y\fP.

\f3\%pfGetNumScreenVChans\fP returns the number of active hardware video channels
on \f2screen\fP for the current pfWSConnection.

\f3\%pfChooseFBConfig\fP takes an array of \f3\%PFFB_\fP attribute tokens in \f2attr\fP
and returns a matching pfFBConfig describing a framebuffer configuration
for the given window system connection, \f2ws\fP, and screen \f2screen\fP.
If \f2ws\fP is null, the current libpr window system connection will
be used. If \f2screen\fP is (-1), the default screen for the window
system connection will be used. The attribute tokens and their
values match the OpenGL GLX tokens (see \f3\%glXChooseVisual\fP for additional
information). These tokens and their values are (booleans are true if
present and should NOT be followed by any values):
.RS 5
.TP
\f3\%PFFB_USE_GL\fP
Boolean, true if present. Use GLX rendering (the default).
.TP
\f3\%PFFB_BUFFER_SIZE\fP
Depth of the color buffer
.TP
\f3\%PFFB_LEVEL\fP
Level in plane stacking.
0 is the main window.
negative numbers are underlay planes, positive numbers are
overlay planes.
.TP
\f3\%PFFB_RGBA\fP
Boolean, does RGBA mode if present
.TP
\f3\%PFFB_DOUBLEBUFFER\fP
Boolean, does double buffering if present
.TP
\f3\%PFFB_STEREO\fP
Boolean, does stereo buffering if present
.TP
\f3\%PFFB_AUX_BUFFERS\fP
Number of auxiliary buffers
.TP
\f3\%PFFB_RED_SIZE\fP
Number of red component bits
.TP
\f3\%PFFB_GREEN_SIZE\fP
Number of green component bits
.TP
\f3\%PFFB_BLUE_SIZE\fP
Number of blue component bits
.TP
\f3\%PFFB_ALPHA_SIZE\fP
Number of alpha component bits
.TP
\f3\%PFFB_DEPTH_SIZE\fP
Number of depth bits
.TP
\f3\%PFFB_STENCIL_SIZE\fP
Number of stencil bits
.TP
\f3\%PFFB_ACCUM_RED_SIZE\fP
Number of red accumulation bits
.TP
\f3\%PFFB_ACCUM_GREEN_SIZE\fP
Number of green accumulation bits
.TP
\f3\%PFFB_ACCUM_BLUE_SIZE\fP
Number of blue accumulation bits
.TP
\f3\%PFFB_ACCUM_ALPHA_SIZE\fP
Number of alpha accumulation bits
.TP
\f3\%PFFB_SAMPLES_SGIS\fP
Number of samples per pixel
.TP
\f3\%PFFB_SAMPLE_BUFFERS_SGIS\fP
The number of multisample buffers
.RE

The list must be terminated with a \f3\%NULL\fP or \f3\%None\fP.  OpenGL
Performer will try to use a multisample buffer with 8 samples per pixel
unless the number of samples or number of multisample buffers has been
explicitly set to 0 in the attribute array.  The default Performer
framebuffer configuration looks like:
.Ex
.nf

static int FBAttrs[] =
{
    PFFB_RGBA,
    PFFB_DOUBLEBUFFER,
    PFFB_DEPTH_SIZE, 24,
    PFFB_RED_SIZE, 8,
    PFFB_SAMPLES, 8,
    PFFB_STENCIL_SIZE, 4,
    None
};
.Ee
.fi
.na


.SH SEE ALSO
pfWindow,
glXChooseVisual,
GLXgetconfig,
XOpenDisplay,
XCloseDisplay
