'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfdAlignVerticesToASD, pfdProjectVerticesOnASD" "3pf"
.SH NAME
.upperok
.B pfdAlignVerticesToASD,
.B pfdProjectVerticesOnASD
\- 
Align an array of vertices to pfASD geometry.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pfdu.h>
.nf
.fi
.TP
void
\f3\%pfdAlignVerticesToASD\fP(\ pfASD\ *asd,
'in +2
'll -2
pfVec3\ *_v,
int\ _nofVertices,
float\ *_base,
float\ *_down,
float\ *_azimuth,
unsigned\ long\ _opcode,
pfVec3\ *_v_aligned);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdProjectVerticesOnASD\fP(pfASD\ *asd,
'in +2
'll -2
pfVec3\ *_v,
int\ _nofVertices,
float\ *_projection,
float\ *_down,
pfVec3\ *_v_projected);
.in
.ll
.fi
.SH DESCRIPTION

\f3\%pfdAlignVerticesToASD\fP takes an array of vertices \f2_v\fP and returns an
array of vertices \f2_v_aligned\fP. The two array pointers may points to the
same array.

\f3\%pfdAlignVerticesToASD\fP starts by projecting \f2_base\fP on the pfASD finest
level of detail, in direction \f2_down\fP. The projection point (\f3\%p\fP) and the
surface normal (\f3\%n\fP) at the point are used to align the input points.

\f3\%pfdAlignVerticesToASD\fP supports a number of operations on the
input vertics. \f2_opcode\fP specifies a bitwise OR of the desired operation.
It may contain the bits:

.RS
.TP 5
\f3\%PR_ALIGN_TRANSLATE\fP
- Translate all the points by \f3\%p\fP.
.TP
\f3\%PR_ALIGN_NORMAL\fP
- Rotate all input points to bring the Z axis to \f3\%n\fP, and the X axis to the
projection of \f2_azimuth\fP on the plane of \f3\%n\fP.
This is useful for aligning objects that follow the surface (e.g. cars on
terrain).
.TP
\f3\%PR_ALIGN_AZIMUTH\fP
- Rotate all input points to bring the Z axis to \f3\%(- _down)\fP, and the X axis
to the projection of \f2_azimuth\fP on the plane of \f3\%(- _down)\fP.
This is useful for aligning objects with a fixed UP orientation (e.g.
buildings on terrain).
.TP
\f3\%PR_ALIGN_AXIS_BILLBOARD\fP
- Rotate all input points to bring the Z axis to \f3\%(- _down)\fP, and the Y axis
to the projection of \f2_azimuth\fP on the plane of \f3\%(- _down)\fP.
This is useful for aligning pfBillboard objects with a fixed UP orientation
(e.g. trees on terrain).
.RE

\f3\%pfdAlignVerticesToASD\fP should be used to align objects to a pfASD finest
level of detail before projecting their shadow in the pfASD surface.

\f3\%pfdProjectVerticesOnASD\fP projects a list of vertices on the finest level of
detail of \f2asd\fP. Its input and output lists of vertices, \f2_v\fP and
\f2_v_projected\fP, may point to the same array. \f3\%pfdProjectVerticesOnASD\fP
uses the direction vector \f2_down\fP to determine whether a vertex falls under
the surface. Such vertices are clamped to the surface, and their projection is
set to the clampped point. \f2_projection\fP is the direction for projecting all
the input vertices.

The combination of \f3\%pfdAlignVerticesToASD\fP and \f3\%pfdProjectVerticesOnASD\fP
is useful for casting the shadow of geometry on pfASD surfaces. The results
of inviking these routines in a row on some geomtry is an outline of a shadow.
This outline may then be used as pfASD query triangles (see pfASD man page)..

The libpfdu source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfASD,
pfTerrain,
pfdExtractGraphTriangles
