'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfdBuildASD" "3pf"
.SH NAME
.upperok
.B pfdBuildASD,
.B pfdStoreASD,
.B pfdBreakTiles,
.B pfdPagingLookahead,
.B pfdWriteFile,
.B pfdLoadConfig
\- 
Construct pfASD node structure from surface data
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfASD\ *\ \ \ 'u
#include <Performer/pfdu.h>
.nf
.fi
.TP
pfASD *
\f3\%pfdBuildASD\fP(unsigned\ int\ numx,
'in +2
'll -2
unsigned\ int\ numy,
pfVec3\ *data,
int\ mode,
int\ inputlods,
int\ buildpaging,
char\ *prename,
char\ *confname,
char\ *pagename,
int\ *lookahead);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdStoreASD\fP(pfASD\ *asd,
'in +2
'll -2
FILE\ *f);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdBreakTiles\fP(int\ numlods,
'in +2
'll -2
pfASDLODRange\ *lods,
int\ numverts,
pfASDVert\ *verts,
int\ numfaces,
pfASDFace\ *faces,
int\ numfaces0,
char\ *prename,
char\ *conf);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdPagingLookahead\fP(char\ *confname,
'in +2
'll -2
char\ *pagename,
int\ *lookahead);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdWriteFile\fP(int\ ***FACE,
'in +2
'll -2
int\ **numf,
int\ ***V,
int\ **numv,
pfASDFace\ *faces,
pfASDVert\ *verts,
pfBox\ *facebounds,
int\ lod,
int\ x,
int\ y,
char\ *prename);
.in
.ll
.nf
.fi
.TP
pfASD *
\f3\%pfdLoadConfig\fP(char\ *fname,
'in +2
'll -2
char\ *pagename);
.in
.ll
.fi
.SH DESCRIPTION

\f3\%pfdBuildASD\fP accepts an input surface description (such as a terrain
height field) and builds a new pfASD node, which represents
the surface as hierarchical level of detail meshes that are combined
and morphed in real-time to model and render the surface efficiently.

A rectangular surface input to \f3\%pfdBuildASD\fP is specified as
an array \f2data\fP of \f2numx\fP columns and \f2numy\fP rows of vertices,
arranged in row-major format.  Adjacent vertices in the input array
are assumed to maintain the same adjacency in the pfASD node that is
created.  Coarser levels of detail of the surface are constructed
using an algorithm selected via \f2mode\fP. It will build \f2inputlods\fP
number of LODs as specified by the user.

\f3\%pfdBuildASD\fP will generate appropriate \f3\%PFTERRAIN_LODS\fP,
\f3\%PFTERRAIN_COORDS\fP, \f3\%PFTERRAIN_NORMALS\fP, \f3\%PFTERRAIN_FACES\fP,
and a default pfGeoState for the pfASD node that is created.
A uniform meshing will be performed
which yields efficient triangle strips.
\f2buildpaging\fP indicates if the pfASD node constructed should be a pageable node.
If a pageable node is desired, then a set of paging ASD patches will be
created under the base name \f2pagename\fP, with \f2lookahead\fP number of
extra tiles on the boundry during real-time paging.

\f3\%pfdBreakTiles\fP
constructs a hierarchical paging subtiles inside each LOD of
the pfASD node. It takes the already constructed pfASD structure
and divides the triangles and vertices into separate pageable
units and output them into a set of pages.

\f3\%pfdWriteFile\fP
writes pageable pfASD subtiles out in the fixed format.

\f3\%pfdLoadConfig\fP
constructs a paging pfASD node from config file and pagename file.

.SH NOTES

We provided a few examples in pguide/libpf/C
buildarcinfo.c, buildbw.c, builddem.c, builddted.c
that constructs from raw elevation data
a set of files that a pfASD node pages.
Unfortunately there is a size limit on
how many levels of ASD we can constructure before
we run out of memory.
Use environment varaible PFASDLODSIZE to specify
the number of LODs to build. I suggestion
setting the value to 9 for illustration purposes.

Once the files are built, the next step is to
convert them into intermediate run-time paging format
using a set of paging size that is calculated
based on LODRange and lookahead.
pguide/libpf/C/convasd.c provides the program
that converts the data.
Notice we assume the LODRange and lookahead are the ones calculated from
pfdBuildASD.c. User can modify the numbers based on
applications. This way, the intermediate paging data
can be set for a particular LODRange without
regenerating the paging tiles.

To summarize, an example of generating some paging ASD is:
create a dir bw/
buildbw pflogo.bw bw/tile bw/config bw/page
convasd bw/config bw/page
modify asdfly/paging.config to pick up the config and page file, then
run_paging.

The libpfdu source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfASD,
pfTerrain
