'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfdGSet" "3pf"
.SH NAME
.upperok
.B pfdNewCube,
.B pfdNewSphere,
.B pfdNewCylinder,
.B pfdNewPipe,
.B pfdNewCone,
.B pfdNewPyramid,
.B pfdNewArrow,
.B pfdNewDoubleArrow,
.B pfdNewCircle,
.B pfdNewRing,
.B pfdXformGSet,
.B pfdGSetColor
\- 
Construct simple pfGeoSets and perform simple pfGeoSet manipulations.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfGeoSet\ *\ \ \ 'u
#include <Performer/pfdu.h>
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewCube\fP(void\ *arena);
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewSphere\fP(int\ ntris,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewCylinder\fP(int\ ntris,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewPipe\fP(float\ botRadius,
'in +2
'll -2
float\ topRadius,
int\ ntris,
void\ *arena);
.in
.ll
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewCone\fP(int\ ntris,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewPyramid\fP(void\ *arena);
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewArrow\fP(int\ ntris,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewDoubleArrow\fP(int\ ntris,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewCircle\fP(int\ ntris,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdNewRing\fP(int\ ntris,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdXformGSet\fP(pfGeoSet\ *gset,
'in +2
'll -2
pfMatrix\ mat);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdGSetColor\fP(pfGeoSet\ *gset,
'in +2
'll -2
float\ r,
float\ g,
float\ b,
float\ a);
.in
.ll
.fi
.SH DESCRIPTION

These routines are provided to conveniently construct pfGeoSets for
various geometric objects.  The resulting objects are always
positioned and sized in canonical ways.  The user can then apply a
transformation to these pfGeoSets to achieve the desired shape and
position.

Some of these routines (such as \f3\%pfdNewSphere\fP) polygonalize smooth
surfaces.  These functions take an argument \f2ntris\fP which specifies
how many triangular faces to use when polygonizing the surface.  All
the constructor routines allocate storage in the shared memory
arena \f2arena\fP.

\f3\%pfdNewCube\fP creates a new pfGeoSet describing a unit cube.

\f3\%pfdNewSphere\fP creates a unit sphere centered at the origin.

\f3\%pfdNewCylinder\fP creates a new cylinder along the Z axis from -1 to 1
with a radius of 1.

\f3\%pfdNewPipe\fP creates a pipe (a cylinder without caps) extending
along the Z axis from -1 to 1.  The radius of the pipe at Z=-1 is
given by \f2botRadius\fP while \f2topRadius\fP determines the pipe's radius
at Z=1.

\f3\%pfdNewCone\fP creates a cone extending along the Z axis.  The base of
the cone is at Z=0 and has radius 1.  The cone extends to Z=1.  The
base of the cone is capped with a circle centered at the origin.

\f3\%pfdNewPyramid\fP creates a pyramid with a unit square base cap
centered at the origin and extending along the Z axis from Z=0 to Z=1.

\f3\%pfdNewArrow\fP constructs an arrow extending along the Z axis from
Z=0 to Z=1.

\f3\%pfdNewDoubleArrow\fP constructs a double-headed arrow.  The arrows
extend along the Z axis from Z=0 to Z=1 and Z=-1.

\f3\%pfdNewCircle\fP creates a filled unit circle centered at the origin
in the XY plane.  The circle is oriented so as to face in the
direction of the positive Z axis.

\f3\%pfdNewRing\fP also creates a circle, but it is unfilled.  The
perimeter of the circle is made up of connected lines.

\f3\%pfdXformGSet\fP transforms the coordinates in the given pfGeoSet by
the matrix \f2mat\fP.

\f3\%pfdGSetColor\fP is a simple convenience routine for setting the
global color of a pfGeoSet.  This function only works if the color
binding of \f2gset\fP is \f3\%PFGS_OVERALL\fP.
.SH NOTES

The libpfdu source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfGeoSet
