'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfdConverter" "3pf"
.SH NAME
.upperok
.B pfdOpenFile,
.B pfdLoadFile,
.B pfdStoreFile,
.B pfdConvertFrom,
.B pfdConvertTo,
.B pfdAddExtAlias,
.B pfdInitConverter,
.B pfdExitConverter,
.B pfdConverterMode,
.B pfdGetConverterMode,
.B pfdConverterVal,
.B pfdGetConverterVal,
.B pfdConverterAttr,
.B pfdGetConverterAttr,
.B pfdPrintSceneGraphStats
\- 
Utilities for loading object databases into Performer applications.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfNode*\ \ \ 'u
#include <Performer/pfdu.h>
.nf
.fi
.TP
FILE*
\f3\%pfdOpenFile\fP(const\ char\ *file);
.nf
.fi
.TP
pfNode*
\f3\%pfdLoadFile\fP(const\ char\ *fileName);
.nf
.fi
.TP
int
\f3\%pfdStoreFile\fP(pfNode\ *root,
'in +2
'll -2
const\ char\ *fileName);
.in
.ll
.nf
.fi
.TP
pfNode*
\f3\%pfdConvertFrom\fP(const\ char\ *ext,
'in +2
'll -2
void\ *root);
.in
.ll
.nf
.fi
.TP
void*
\f3\%pfdConvertTo\fP(const\ char\ *ext,
'in +2
'll -2
pfNode\ *root);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdAddExtAlias\fP(const\ char\ *ext,
'in +2
'll -2
const\ char\ *alias);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfdInitConverter\fP(const\ char\ *ext);
.nf
.fi
.TP
int
\f3\%pfdExitConverter\fP(const\ char\ *ext);
.nf
.fi
.TP
void
\f3\%pfdConverterMode\fP(const\ char\ *ext,
'in +2
'll -2
int\ mode,
int\ value);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfdGetConverterMode\fP(const\ char\ *ext,
'in +2
'll -2
int\ mode);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdConverterVal\fP(const\ char\ *ext,
'in +2
'll -2
int\ which,
float\ val);
.in
.ll
.nf
.fi
.TP
float
\f3\%pfdGetConverterVal\fP(const\ char\ *ext,
'in +2
'll -2
int\ which);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdConverterAttr\fP(const\ char\ *ext,
'in +2
'll -2
int\ which,
void\ *attr);
.in
.ll
.nf
.fi
.TP
void*
\f3\%pfdGetConverterAttr\fP(const\ char\ *ext,
'in +2
'll -2
int\ which);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdPrintSceneGraphStats\fP(pfNode\ *node,
'in +2
'll -2
double\ elapsedTime);
.in
.ll
.fi
.SH DESCRIPTION

\f3\%pfdOpenFile\fP searches through the OpenGL Performer search path
for the named file and opens it using fopen(). It is a convenience
function used by several of the functions described here.

\f3\%pfdLoadFile\fP builds in-memory data structures from an external
database file.  The filename's extension is used to determine the file
format.  If no path to the file is given, the directories in the
active Performer file search path (see \f3\%pfFilePath\fP) are scanned for
the given filename.   \f3\%pfdLoadFile\fP may only be called after
\f3\%pfConfig\fP.

\f3\%pfdStoreFile\fP writes a subgraph of a Performer scene rooted at
\f2root\fP to a file named \f2fileName\fP in the format specified by
the file name's extension string.

\f3\%pfdConvertFrom\fP converts the in-memory data structure \f2root\fP
of the format specified by \f2ext\fP into an in-memory Performer
scene.   \f3\%pfdConvertTo\fP reverses the process taking an in-memory
Performer scene and converting into the specified in-memory format.

\f3\%pfdAddExtAlias\fP registers an alias for the given file name
extension.  Whenever a file with extension \f2alias\fP is encountered,
the loader for type \f2ext\fP will be used.

\f3\%pfdInitConverter\fP dynamically links the converter corresponding to
the extension \f2ext\fP into the current executable.  This routine should
be called before \f3\%pfConfig\fP for all extensions that an executable
will use to ensure that any routines and static data required at
run-time are available in all Performer processes.  If the
corresponding loader is already in the executable, e.g. as a statically
linked object, \f3\%pfdInitConverter\fP takes no action.
\f3\%pfdInitConverter\fP returns TRUE if the loader is available or FALSE
if it could not be found or loaded.

The search for the converter DSO proceeds through the following
locations in order:
.RS
.TP 5
1.
In the directory indicated by the environment variable
PFLD_LIBRARY{,N32,64}_PATH, if it is defined.  The empty, "N32" and
"64" substrings correspond to O32, N32 and N64 modes
of compilation and execution.
.TP
2.
In the current directory.
.TP
3.
In the directory indicated by the rld environment variable
LD_LIBRARY{,N32,64}_PATH, if it is defined.
.TP
4.
In the directory "$PFHOME/usr/lib{,32,64}/libpfdb", if the
environment variable PFHOME is defined. The empty, "32" and
"64" lib suffix strings correspond to O32, N32 and N64 modes
of compilation and execution.
.TP
5.
In the directory "$PFHOME/usr/share/Performer/lib/libpfdb", if
the environment variable PFHOME is defined.
.RE

The loader DSO name is created as "libpf{-g}.so" where the
"-g" suffix is for a full symbol table debug version.
\f3\%pfdInitConverter\fP will only load the debug version of the converter
DSO if it is unable to find the optimized version of the DSO in any of
paths mentioned above.  \f3\%pfdLoadFile\fP also requires that the DSO
version number match that of libpfdu.  When PFLD_LIBRARY{,N32,64}_PATH is set,
\f3\%pfdLoadFile\fP prints diagnostic information about the DSO search
using \f3\%pfNotify\fP at the PFNFY_DEBUG notification level.

\f3\%pfdExitConverter\fP discards the database loader for the extension
\f2ext\fP and unlinks any dynamically linked objects from the current
executable.  The only reason to call this function is to save swap
space.  If \f2ext\fP is NULL, all converters are unlinked.

\f3\%pfdConverterMode\fP, \f3\%pfdGetConverterMode\fP, \f3\%pfdConverterAttr\fP,
\f3\%pfdGetConverterAttr\fP, \f3\%pfdConverterVal\fP and \f3\%pfdGetConverterVal\fP
allow the user to access and alter the modes, attributes and values of
specific loaders.  These modes, attributes and values are defined
inside each individual loader. These functions are provided as a means
for the user to communicate with the loaders which are usually loaded
as Dynamic Shared Objects at run-time.

\f3\%pfdPrintSceneGraphStats\fP uses \f3\%pfNotify\fP to print some simple
statistics about the primitives in the scene graph. The \f2elapsedTime\fP
argument is provided by the caller and indicates the time it took
for the scene graph to be loaded. It is used by \f3\%pfdPrintSceneGraphStats\fP
to print primitive loading rate statistics. When the value is zero, none
of the loading rate statistics are printed

The routines that take an extension as an argument may be passed a
full file name, in which case the extension is extracted and used.
.SH NOTES

The libpfdu source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.

Very few of the database DSOs provided in the current release support
\f3\%pfdStoreFile\fP, none of them support \f3\%pfdConvertTo\fP, and only the
Open Inventor loader supports \f3\%pfdConvertFrom\fP.  More pervasive
support of these operations is planned for future releases, however,
and developers of new database conversion tools are encouraged to
provide the full set of conversion functions in the database tools
they develop.

When statically linking a loader library into an executable that calls
\f3\%pfdLoadFile\fP, you can use the '-u' option to ld to force the inclusion
of the loader object even though it is never referenced in the executable,
e.g. "cc -o myapp myapp.o -u pfdLoadFile_iv libpfiv_igl.a ..."
.SH SEE ALSO
pfFilePath,
pfNotify,
ld
