'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfdShare" "3pf"
.SH NAME
.upperok
.B pfdNewShare,
.B pfdGetGlobalShare,
.B pfdSetGlobalShare,
.B pfdDelShare,
.B pfdPrintShare,
.B pfdCountShare,
.B pfdGetSharedList,
.B pfdFindSharedObject,
.B pfdAddSharedObject,
.B pfdRemoveSharedObject,
.B pfdNewSharedObject,
.B pfdMakeShared,
.B pfdMakeSharedScene,
.B pfdCleanShare,
.B pfdGetNodeGStateList
\- 
Facilitate the sharing of graphics state objects in a
Performer scene graph.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfdShare\ *\ \ \ 'u
#include <Performer/pfdu.h>
.nf
.fi
.TP
pfdShare *
\f3\%pfdNewShare\fP(void);
.nf
.fi
.TP
pfdShare *
\f3\%pfdGetGlobalShare\fP(void);
.nf
.fi
.TP
void *
\f3\%pfdSetGlobalShare\fP(pfdShare\ *share);
.nf
.fi
.TP
void
\f3\%pfdDelShare\fP(pfdShare\ *share,
'in +2
'll -2
int\ deepDelete);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdPrintShare\fP(pfdShare\ *share);
.nf
.fi
.TP
int
\f3\%pfdCountShare\fP(pfdShare\ *share);
.nf
.fi
.TP
pfList *
\f3\%pfdGetSharedList\fP(pfdShare\ *share,
'in +2
'll -2
pfType\ *type);
.in
.ll
.nf
.fi
.TP
pfObject *
\f3\%pfdFindSharedObject\fP(pfdShare\ *share,
'in +2
'll -2
pfObject\ *object);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfdAddSharedObject\fP(pfdShare\ *share,
'in +2
'll -2
pfObject\ *object);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfdRemoveSharedObject\fP(pfdShare\ *share,
'in +2
'll -2
pfObject\ *object);
.in
.ll
.nf
.fi
.TP
pfObject *
\f3\%pfdNewSharedObject\fP(pfdShare\ *share,
'in +2
'll -2
pfObject\ *object);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfdMakeShared\fP(pfNode\ *node);
.nf
.fi
.TP
void
\f3\%pfdMakeSharedScene\fP(pfScene\ *scene);
.nf
.fi
.TP
int
\f3\%pfdCleanShare\fP(pfdShare\ *share);
.nf
.fi
.TP
pfList *
\f3\%pfdGetNodeGStateList\fP(pfNode\ *node);
.fi
.SH DESCRIPTION

It is obviously desirable to share state between database objects in
OpenGL Performer whenever possible.  The notion of pervasive state
sharing underpins the entire \f3\%pfGeoState\fP mechanism.  Common data such
as texture, materials, and lighting models are often duplicated in
many different objects throughout a database.  This collection of
functions provides the means necessary to easily achieve sharing among
these objects by automatically producing a non-redundant set of
states.

\f3\%pfdNewShare\fP constructs a new \f3\%pfdShare\fP structure in shared memory.
This structure is the object used to track shared state objects.

\f3\%pfdGetGlobalShare\fP returns the global \f3\%pfdShare\fP structure.
This structure is used by both the \f3\%pfdBuilder\fP and by loaders
that do not use the \f3\%pfdBuilder\fP.

\f3\%pfdSetGlobalShare\fP sets the global \f3\%pfdShare\fP structure.

\f3\%pfdDelShare\fP deletes a \f3\%pfdShare\fP structure.  If \f2deepDelete\fP
is non-NULL, all the data referenced by the \f3\%pfdShare\fP will also be
deleted.

\f3\%pfdPrintShare\fP will print statistics about how many object
references are being held in the given sharing structure.

\f3\%pfdCountShare\fP returns the total number of shared objects
referenced by the given sharing structure.

Each \f3\%pfdShare\fP structure maintains lists of shared objects with
distinct types of objects stored in distinct lists.
\f3\%pfdGetSharedList\fP returns the list of shared objects of the given
type.

\f3\%pfdNewSharedObject\fP returns a shared object matching \f2object\fP.
If a matching object was already present in the sharing structure,
that object is returned.  If no such object exists, a new object is
created using \f2object\fP as a template.  This new object is entered
into the sharing structure and is returned to the caller.

\f3\%pfdFindSharedObject\fP looks through the given pfdShare structure for
an object matching \f2object\fP.  A NULL is returned if no matching
object is found.

\f3\%pfdAddSharedObject\fP adds \f2object\fP to the given sharing structure,
if it is not already present.  The object's index in the sharing list
is returned.

\f3\%pfdRemoveSharedObject\fP removes \f2object\fP from the given sharing structure,
if it is present.  The object's index in the sharing list is returned.
The object is deleted if its reference count reaches zero via
\f3\%pfUnrefDelete\fP.

\f3\%pfdMakeShared\fP can be used to force sharing of state within an
already existing scene graph.  It will traverse the graph rooted at
\f2node\fP looking for duplicate state objects.  Any references to such
objects will be made to point to a single shared copy, and the
duplicates will be freed.

\f3\%pfdMakeSharedScene\fP is similar to pfdMakeShared except that it works
on a scene and computes an optimal pfScene pfGeoState based on all of
the pfGeoStates referenced in \f2scene\fP (see \f3\%pfdMakeSceneGState\fP for
further information about the scene pfGeoState computation itself).
The pfGeoStates in \f2scene\fP are then optimized based on the new scene
GeoState (see \f3\%pfdOptimizeGStateList\fP).  Lastly, the optimized
pfGeoState is assigned as the scene's pfGeoState so that the inheritance
for the newly optimized states will be effective (see \f3\%pfSceneGState\fP).

\f3\%pfdCleanShare\fP removes all shared objects that are referenced only by
the shared structure itself.  It is useful to call \f3\%pfdCleanShare\fP
after deleting parts of database that were created using this share
structure to release the memory allocated for currently unused state
elements. \f3\%pfdCleanShare\fP returns the number of elements actually
removed.

\f3\%pfdGetNodeGStateList\fP creates and returns a pfList of unique pfGeoStates that
are referenced by geometry under \f2node\fP.
.SH NOTES

The libpfdu source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfGeoSet,
pfGeoState,
pfList
