'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfdTMesher" "3pf"
.SH NAME
.upperok
.B pfdMeshGSet,
.B pfdMesherMode,
.B pfdGetMesherMode,
.B pfdShowStrips
\- 
Create triangle meshes from pfGeoSets.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfGeoSet\ *\ \ \ 'u
#include <Performer/pfdu.h>
.nf
.fi
.TP
pfGeoSet *
\f3\%pfdMeshGSet\fP(pfGeoSet\ *gset);
.nf
.fi
.TP
void
\f3\%pfdMesherMode\fP(int\ mode,
'in +2
'll -2
int\ val);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfdGetMesherMode\fP(int\ mode);
.nf
.fi
.TP
void
\f3\%pfdShowStrips\fP(pfGeoSet\ *gset);
.fi
.SH DESCRIPTION

Forming independent triangles into triangle strips (or meshes) can
significantly improve rendering performance on most systems.  Strips
reduce the amount of work required by the CPU, bus, and graphics
subsystem.  The OpenGL Performer utility mesher is adapted from the
triangle mesh code supplied in "/usr/people/4Dgifts".  It is modified
to work with Performer \f3\%pfGeoSet\fP data structures and is optimized for
optimal performance.

\f3\%pfdMeshGSet\fP takes as input a \f3\%pfGeoSet\fP consisting of
independent triangles, (\f3\%PFGS_TRIS\fP).  The input may be either
indexed or non-indexed.  This routine outputs the input \f3\%pfGeoSet\fP
if it cannot strip the input or a single \f3\%pfGeoSet\fP that is a
collection of triangle strips (PFGS_\f3\%TRISTRIPS\fP) if it is
successful.  The output \f3\%pfGeoSet\fP is non-indexed and the input
\f3\%pfGeoSet\fP is not deleted although the application generally should
do so to avoid wasted memory.

The mesher attempts to maximize the average length of triangle strips
inside \f2gset\fP.  The code is complex but works well and can
significantly improve drawing performance if the average number of
triangles in the triangle strips in the output GeoSets is at least
four.  The length of triangle strips necessary to achieve peak drawing
performance is dependent on the exact hardware configuration.

\f3\%pfdMesherMode\fP sets the mode that the mesher will use when forming
triangle strips.  Currently two modes are supported:
.RS
.TP 5
\f3\%PFDMESH_SHOW_TSTRIPS\fP
Generate a random color for each triangle strip generated.  This is a
diagnostic mode which is extremely useful in understanding the
structure and efficiency of databases.
The OpenGL Performer sample
application \f3\%perfly\fP supports the specification of this mode on the
command line via the "-O1" flag.  Type "perfly -help" for full
instructions.  The default value is \f3\%FALSE\fP.
.TP
\f3\%PFDMESH_RETESSELLATE\fP
.RS
With this mode enabled the mesher will retessellate planar quads
to achieve longer strips.
The OpenGL Performer sample application
\f3\%perfly\fP allows retessellation to be enabled and disabled from the
command line.  Type "perfly -O4 ...." to enable retessellation.  The default value
is \f3\%TRUE\fP.
.RE
.TP
\f3\%PFDMESH_INDEXED\fP
.RS
Will construct indexed triangle strips.  If the pfdBuilder mode
\f3\%PFDBLDR_SHARE_INDEX_LISTS\fP is set, the index lists
will be shared between the different vertex attribute lists.
.RE

\f3\%pfdGetMesherMode\fP returns the current setting of \f2mode\fP.

.SH NOTES

If \f3\%pfdMeshGSet\fP generates a set of triangle strips whose lengths
are all three or four i.e. they are all independent triangles or quads, the
output \f3\%pfGeoSet\fP will be of type \f3\%PFGS_TRIS\fP or \f3\%PFGS_QUADS\fP,
respectively.

\f3\%pfdMeshGSet\fP also calls \f3\%pfuHashGSetVerts\fP which may delete the
attribute and index arrays of the input \f3\%pfGeoSet\fP.  Thus you may wish
to \f3\%pfRef\fP your arrays to avoid their deletion.

\f3\%pfdShowStrips\fP will assign each triangle strip a random color.  The
first triangle in each strip is distinguished by being slightly darker
than the rest of the strip

The libpfdu source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfGeoSet,
pfRef,
pfuHashGSetVerts,
pfdBldrMode,
pfdGeoBldrMode
