'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuPalettizer" "3pf"
.SH NAME
.upperok
.B pfuPalettizer
\- 
Class for creating one or more texture palettes of multiple textures
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfList\ *\ \ \ 'u
#include <Performer/pfutil/pfuPalettizer.h>
.nf
.fi
.TP
\ 
\f3\%pfuPalettizer::pfuPalettizer\fP(void);
.nf
.fi
.TP
pfList *
\f3\%pfuPalettizer::getPaletteList\fP(void);
.nf
.fi
.TP
pfList *
\f3\%pfuPalettizer::getCompleteList\fP(void);
.nf
.fi
.TP
int
\f3\%pfuPalettizer::palettize\fP(pfNode\ *node);
.nf
.fi
.TP
void
\f3\%pfuPalettizer::setST\fP(uint\ s,
'in +2
'll -2
uint\ t);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuPalettizer::setMargin\fP(uint\ margin);
.nf
.fi
.TP
void
\f3\%pfuPalettizer::setTexturePrefix\fP(const\ char\ *prefix);
.nf
.fi
.TP
void
\f3\%pfuPalettizer::setBackground\fP(float\ r,
'in +2
'll -2
float\ g,
float\ b);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuPalettizer::setMode\fP(int\ mode,
'in +2
'll -2
int\ val);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuPalettizer::getMode\fP(int\ mode);
.fi
.SH DESCRIPTION

The purpose of the palettizer is to consolidate textures into a single or
a smaller group of textures, thus reducing expensive texture binds during
rendering.  This is beneficial to databases that use many textures.
There are two cases in which the palettizer is of no use:
1) Textures that are referred to by GeoSets that contain texture coordinates
outside the range of 0.0 - 1.0.  That is, geosets that intend to "repeat"
textures.
2) Textures that use detail.  There is currently no practical way to produce
detail textures for a palette of multiple textures.

\f3\%pfuPalettizer::getPaletteList\fP(void) returns a list of newly created palettes after palettize has been called.

\f3\%pfuPalettizer::getCompleteList\fP(void) returns the list of textures including the newly created
palette that are referred to under 'node' after palettize has been called.

\f3\%pfuPalettizer::palettize\fP(pfNode *node) does the actual work of palettization.  \f3\%node\fP is traversed and
all textures are found.  Textures too large to fit in \f3\%s\fP or \f3\%t\fP, textures with detail,
or with pfGeoSets with texture coordinates outside of the
range 0.0-1.0 are passed over.  The remaining textures are bin-packed
(allowing for margins), using a multipass bin-packing algorithm into the
larger texture.  If all binable textures do not fit within \f3\%s\fP and \f3\%t\fP,
multiple textures are created until all textures are either palettized or
left alone.

\f3\%pfuPalettizer::setST\fP(uint s, uint t) allows the setting of maximum width \f3\%(s)\fP and maximum height \f3\%(t)\fP of newly
created palettes.  Actual \f3\%s\fP and \f3\%t\fP dimensions may change if space is wasted.

\f3\%pfuPalettizer::setMargin\fP(uint margin) allows the setting of the margin around texture tiles.  \f3\%margin\fP
is the number of pixels of value 0 to be placed around each original texture.
The space between adjacent textures then becomes 2 * \f3\%margin\fP.

\f3\%pfuPalettizer::setTexturePrefix\fP(const char *prefix) allows the specification of a file prefix \f3\%prefix\fP
to be used when naming the newly created texture palettes.  Palette names are prepended with
\f3\%prefix\fP and extended with \f3\%###\fP.rgba, where \f3\%###\fP is a sequence of numbers.

\f3\%pfuPalettizer::setMode\fP(int mode, int val) specifies a mode for creating the palette texture.
\f3\%mode\fP is a token to specify which mode to set and \f3\%val\fP is a boolean value.  Currently, the only
mode supported is \f3\%PFUPAL_MODE_MIPMAP\fP and \f3\%val\fP is a boolean.  By default, val is 1.  Force
non-mipmapped palettes by seting \f3\%BPFUPAL_MODE_MIPMAP\fP to 0.

Example 1 :
.Ex
.nf

    pfInit();
    pfConfig();
		...

    // Load a database
	pfNode *node;
    if( (node = pfdLoadFile( mydatabase )) == (pfNode *)0L )
        exit(1);

	// Create a palettizer
    p = new pfuPalettizer;
    // Set the prefix of the palette to be created
    p->setTexturePrefix( "myPalette" );
    // Use a margin of 32 pixels between textures
    p->setMargin( 32 );
    // Allow mipmapping
    p->setMode( PFUPAL_MODE_MIPMAP, 1 );
    // Set maximum width and height of palette
    p->setST( 512, 512 );

    // palettize!
    p->palettize( node );

    // store the altered scene graph
	pfdStoreFile( node, "mydatabase.pfb" );

	// Store the new textures
    pfList *texlist = p->getPaletteList();
    for( int i = 0; i < texlist->getNum(); i++ )
    {
        pfTexture *tex = (pfTexture *)texlist->get( i );
        tex->saveFile( tex->getName() );
    }

		...
.Ee
.fi
.na



.SH SEE ALSO
pfTexture(3pf)
