'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuProcessManager" "3pf"
.SH NAME
.upperok
.B pfuProcessManager,
.B pfuDefaultProcessManager,
.B pfuFreeAllCPUs
\- 
Priority, processes and processor assignment functions
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'static\ pfuBaseProcessManager\ *\ \ \ 'u
#include <Performer/pfutil/pfuProcessManager.h>
.nf
.fi
.TP
static void
\f3\%pfuProcessManager::select\fP(pfuBaseProcessManager\ *);
.nf
.fi
.TP
static void
\f3\%pfuProcessManager::setMode\fP(int\ mode,
'in +2
'll -2
int\ val);
.in
.ll
.nf
.fi
.TP
static int
\f3\%pfuProcessManager::setMode\fP(int\ mode);
.nf
.fi
.TP
void
\f3\%pfuProcessManager::setCreateFunc\fP(pfuProcessHandlerFuncType\ func,
'in +2
'll -2
uint\ mask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuProcessManager::getCreateFunc\fP(pfuProcessHandlerFuncType\ *func,
'in +2
'll -2
uint\ *mask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuProcessManager::setPlaceFunc\fP(pfuProcessHandlerFuncType\ func,
'in +2
'll -2
uint\ mask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuProcessManager::getPlaceFunc\fP(pfuProcessHandlerFuncType\ *func,
'in +2
'll -2
uint\ *mask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuProcessManager::setPrioritizeFunc\fP(pfuProcessHandlerFuncType\ func,
'in +2
'll -2
uint\ mask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuProcessManager::getPrioritizeFunc\fP(pfuProcessHandlerFuncType\ *func,
'in +2
'll -2
uint\ *mask);
.in
.ll
.nf
.fi
.TP
static void
\f3\%pfuProcessManager::reconfigure\fP(void);
.nf
.fi
.TP
static void
\f3\%pfuProcessManager::release\fP(void);
.nf
.fi
.TP
static pfuBaseProcessManager *
\f3\%pfuProcessManager::getCur\fP(void);
.nf
.fi
.TP
static void
\f3\%pfuDefaultProcessManager::init\fP(void);
.nf
.fi
.TP
int
\f3\%pfuFreeAllCPUs\fP(void);
.nf
.fi
.TP
typedef int
\f3\%(*pfuProcessHandlerFuncType)\fP(int\ _pipe,
'in +2
'll -2
uint\ _stage,
pid_t\ _pid);
.in
.ll
.fi
.SH DESCRIPTION

The pfuProcessManager class hierarchy implements an extensible process
manager framework bsed on the pfuBaseProcessManager class
with a default process manager class, pfuDefaultProcessManger.
The currently active process manager can be communicated with via
the pfuProcessManager global routines.

\f3\%pfuProcessManager::select\fP will
select the current process manager.
\f3\%pfuProcessManager::getCur\fP will
return a pointer to the current process manager.

\f3\%pfuProcessManager::setMode\fP
will set the specified \f2mode\fP for the currently active process
manager to the given value, \f2val\fP.  The modes and associated values
are to be supported by all process managers are:
.RS
.TP 25
\f3\%PFUPM_CONTINUOUS\fP
takes a boolean value to indicate continuous operation and placement
of new processes, or if explicit calls to reconfigure
the process manager need to be made to trigger process control.
.TP
\f3\%PFUPM_LOCK_DOWN\fP
takes a boolean value to indicate if locking of processes to
processors and potential isolation of those processors should occur.
The default value is \f3\%FALSE\fP.
.TP
\f3\%PFUPM_PRIORITIES\fP
takes a boolean value to indicate if processes should be given
non-degrading priorties.  The default value is \f3\%FALSE\fP.
.RE
\f3\%pfuProcessManager::getMode\fP
will return the current value of \f2mode\fP for the currently active
process manager.

\f3\%pfuProcessManager::reconfigure\fP
will force the current process manager to do a replacement of all
registered processes according to the current mode settings.

\f3\%pfuProcessManager::release\fP
will force the release of isolated CPUs and processes registered with the
current process manager.  Processes will have non-degrading priorties
removed and will be allowed to run on any CPU.

\f3\%pfuDefaultProcessManager::init\fP
will create and select a process manager with a default policy.  The
pfuDefaultProcessManger uses the \f3\%pfCreateProcessFunc\fP API to
register a callback function with libpf that will be called upone the
creation of any OpenGL Performer process.  \f3\%pfuGetProcessMiscCPU\fP is used to
determine which CPU is to be considered the non-real-time
miscellaneous CPU (typically CPU 0).  The pfuDefaultProcessManger
will place Performer processes onto processors and set non-degrading
priorities on critical processes as described below.  The routines
implementing these features utilize the IRIX REACT facilities.
Refer to the IRIX REACT technical report, and the \f3\%sysmp(2)\fP and
\f3\%realtime(5)\fP man pages for detailed information on these
concepts.

The policy implemented by the \f3\%pfuDefaultProcessManager\fP is:
1. First, if there are sufficient CPUs, each of \f3\%APP, CULL\fP and \f3\%DRAW\fP
gets its own processor and no critical path processes are placed on
the Misc CPU.
If not, the APP is placed on CPU 0 which which will not be isolated.
If there are still not enough processors, cull and draw processes
(if separate) of a given pfPipe are paired to run on the same CPU.
If there are still not sufficient CPUs,
extra cull and draw processes will be placed on the last CPU.
2. Next, the \f3\%LPOINT\fP process(es) are given their own CPUs if
there are enough available CPUs. Otherwise, they will be given the last CPU.
3. Next, \f3\%ISECT\fP and \f3\%COMPUTE\fP processes are given their CPUs or
paired if there is one additional CPU or else given the MISC CPU.
4. Finally, the \f3\%DBASE\fP and \f3\%QUEUE\fP process(es) are given their own CPUs
or paired if there is one additional CPU or else given the MISC CPU.

Multipipe mappings are as follows.
.Ex
.nf

If NumCPUs >= 2 + 2*NumPipes
then each cull and draw process can have its own CPU and are placed together
on a CPU node. The application is placed on CPU 1 with UNIX getting CPU 0.

If NumCPUs == 1 + 2*NumPipes
then the application shares CPU 0 with UNIX.

Otherwise, if NumCPUs >= 2 + NumPipes
then cull and draw processes for each pipe are paired together.

The LPoint processes are also  given their own CPUs if there are enough
available CPUs.  Otherwise, they are grouped together on a single CPU,
their own if there is on extra, else the last CPU.
.Ee
.fi
.na


The pfuProcessManager provides 2 mechanisms of extensibility: C++ subclassing and
C callbacks. The pfuBaseProcessManager and pfuDefaultProcessManager can
both be subclassed and new instances can be made the current process manager.
In addition, custom user callbacks can be assigned to hand specific types
of processes defined by the \f3\%PFU_PROC_*\fP mask or unknown processes.

\f3\%pfuProcessManager::setCreateFunc\fP
sets a custom user callback for handling the creation of new processes.  The
return value should be \f3\%PFU_CB_CONT\fP if the user has decided to let the
pfuProcessManager handle the function and \f3\%PFU_CB_TERM\fP if the user has
processed it.  The current user function and associated mask is gotten with
\f3\%pfuProcessManager::getCreateFunc\fP.

\f3\%pfuProcessManager::setPlaceFunc\fP
sets a custom user callback for handling the processor placement of new processes.
The placement function should return the chosen CPU number or (-1) if
the user has choosen to not handle the process. The cpu placement will be done
upon the explicit or automatic call by the process manager to place processes.
\f3\%pfuProcessManager::getPlaceFunc\fP
will return the current user placement function and process mask.

\f3\%pfuProcessManager::setPrioritizeFunc\fP
sets a custom user callback for handling the processor prioritization of new processes.
The function should return the chosen priority relative to the app process or \f3\%PFUMP_NOPRI\fP
if the user has choosen to not handle the process.
The process priority will be set upon the explicit or automatic call by the process manager
to place processes.
\f3\%pfuProcessManager::getPrioritizeFunc\fP
will return the current user prioritization function and process mask.


The following example is taken from the sample application source
code provided in
\f3\%c/usr/share/Performer/src/sample/C++/common/main.C\fP:
.Ex
.nf

    /* Initialization and start of pfuProcess Manager */
    pfuInitDefaultProcessManager;
    .....
    //
    // Configure OpenGL Performer and fork extra processes if
    // configured for multiprocessing.
    //
    pfConfig();

    // Tell ProcessManager to configure the system
    pfuProcessManagerMode(PFUPM_CONTINUOUS, 1);
    pfuProcessManagerMode(PFUPM_LOCK_DOWN, 1);
    pfuProcessManagerMode(PFUPM_PRIORITIES, 1);
    pfuReconfigureProcessManager;
    // Initialization of pfuProcess Manager
    pfuDefaultProcessManager::init();
    .....
    //
    // Configure OpenGL Performer and fork extra processes if
    // configured for multiprocessing.
    //
    pfConfig();

    // Tell ProcessManager to configure the system
    pfuProcessManager::setMode(PFUPM_CONTINUOUS, 1);
    pfuProcessManager::setMode(PFUPM_LOCK_DOWN, 1);
    pfuProcessManager::setMode(PFUPM_PRIORITIES, 1);
    pfuProcessManager::reconfigure();
.Ee
.fi
.na

.SH NOTES


CPU isolation or restriction functionality is not currently
supported under Linux.

Isolating a CPU to run specific processes requires super-user permission.
Setting non-degrading priorities on processes requires super-user permission.

CPU 0 should \f2never\fP be isolated.

A utility shell script is provided in /usr/share/Performer/src/tools/freeproc
that can be run as super-user to clean up after a program that exits ungracefully.

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.

.SH SEE ALSO
pfConfig,
pfConfigStage,
pfCreateProcessFunc,
pfuGetProcessMiscCPU,
realtime,
sysmp,
schedctl,
sched_setscheduler
