'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuTextureManager" "3pf"
.SH NAME
.upperok
.B pfuTextureManager
\- 
Class for managing texture lists
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfuTexList\	*\ \ \ 'u
#include <Performer/pfutil/pfuTextureManager.h>
.nf
.fi
.TP
\ 
\f3\%pfuTextureManager::pfuTextureManager\fP(void);
.nf
.fi
.TP
\ 
\f3\%pfuTextureManager::~pfuTextureManager\fP(void);
.nf
.fi
.TP
int
\f3\%pfuTextureManager::getTextureMemoryAvailable\fP(void);
.nf
.fi
.TP
int
\f3\%pfuTextureManager::getTextureMemoryUsed\fP(void);
.nf
.fi
.TP
float
\f3\%pfuTextureManager::getTextureMemoryUsedPercent\fP(void);
.nf
.fi
.TP
pfuTexList	*
\f3\%pfuTextureManager::addTextureList\fP(pfList\ *texList,
'in +2
'll -2
const\ char\ *name,
pfTexture\ *default);
.in
.ll
.nf
.fi
.TP
pfuTexList *
\f3\%pfuTextureManager::addTextureList\fP(pfNode\ *node,
'in +2
'll -2
const\ char\ *name,
pfTexture\ *default);
.in
.ll
.nf
.fi
.TP
pfuTexList *
\f3\%pfuTextureManager::addTextureList\fP(pfList\ *texList,
'in +2
'll -2
const\ char\ *name);
.in
.ll
.nf
.fi
.TP
pfuTexList *
\f3\%pfuTextureManager::addTextureList\fP(pfNode\ *node,
'in +2
'll -2
const\ char\ *name);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTextureManager::deleteTextureList\fP(pfuTexList\ *tlist);
.nf
.fi
.TP
void
\f3\%pfuTextureManager::load\fP(pfuTexList\ *texList);
.nf
.fi
.TP
float
\f3\%pfuTextureManager::load\fP(pfuTexList\ *texList,
'in +2
'll -2
float\ deadline);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTextureManager::unload\fP(pfuTexList\ *texList);
.nf
.fi
.TP
void
\f3\%pfuTextureManager::loadUnload\fP(pfuTexList\ *loadList,
'in +2
'll -2
pfuTexList\ *unloadList);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuTextureManager::getTextureMemorySize\fP(void);
.nf
.fi
.TP
float
\f3\%pfuTextureManager::getTextureMemoryUsage\fP(void);
.nf
.fi
.TP
int
\f3\%pfuTextureManager::willFit\fP(pfuTexList\ *texList);
.nf
.fi
.TP
float
\f3\%pfuTextureManager::isListLoaded\fP(pfuTexList\ *texList);
.nf
.fi
.TP
void
\f3\%pfuTextureManager::Print\fP(int\ level);
.fi
.SH DESCRIPTION


\f3\%pfuTextureManager\fP provides management of textures through texture lists.
Lists are created by either passing a pfList of pfTextures to \f3\%addTextureList\fP
or a pfNode, from which the Texture Manager will find all textures.  A master
list is kept internally and the Texture Manager attempts to resolve ambiguities of load and unload requests.  Further, the Texture Manager allows "trickle"
loads, that is, partial loads of a list allowing the loading of a texture list
to be amortized over several frames.


\f3\%pfuTextureManager::getTextureMemoryAvailable\fP(void) returns the size of unused texture memory in bytes.

\f3\%pfuTextureManager::getTextureMemoryUsed\fP(void) return size in bytes of the texture memory used
by the textures that the TextureManager if managing.

\f3\%pfuTextureManager::getTextureMemoryUsedPercent\fP(void) returns a percentage (0.0 to 100.0 of texture memory
used by textures that the Texture Manager is managing.

\f3\%pfuTextureManager::*addTextureList\fP(pfList *texList, const char *name, pfTexture *default) adds a texture list
to be managed from a pfList of pfTextures \f3\%texList\fP.  \f3\%name\fP is an arbitary name that is used for identifying the
texture list in a \f3\%Print\fP.  \f3\%default\fP is a pfTexture whose image will be substituded in the case of an \f3\%unload\fP.
This way, no textures will be accidentally loaded if something is drawn which references the texture.
\f3\%addTextureList\fP returns a pfTextureManager::pfuTexList type which is solely used to pass to \f3\%deleteTextureList\fP,
\f3\%load\fP, \f3\%unload\fP, \f3\%loadUnload\fP, \f3\%willFit\fP, and \f3\%isListLoaded\fP.

\f3\%pfuTextureManager::*addTextureList\fP(pfNode *node, const char *name, pfTexture *default) adds a texture list
to be managed based on a pfNode \f3\%node\fP.  \f3\%name\fP is an arbitary name that is used for identifying the texture list
in a \f3\%Print\fP.  \f3\%default\fP is a pfTexture whose image will be substituded in the case of an \f3\%unload\fP.  This way,
no textures will be accidentally loaded if something is drawn which references the texture.
\f3\%addTextureList\fP returns a pfTextureManager::pfuTexList type which is solely used to pass to \f3\%deleteTextureList\fP,
\f3\%load\fP, \f3\%unload\fP, \f3\%loadUnload\fP, \f3\%willFit\fP, and \f3\%isListLoaded\fP.


\f3\%pfuTextureManager::*addTextureList\fP(pfList *texList, const char *name) adds a texture list to be managed
based on a pfList \f3\%texList\fP of pfTextures.  \f3\%name\fP is an arbitary name that is used for identifying the texture
list in a \f3\%Print\fP.  This version does not supply a default texture and it is therefore possible to accidentally
load a texture outside of the Texture Manager's control if the object is drawn.
\f3\%addTextureList\fP returns a pfTextureManager::pfuTexList type which is solely used to pass to \f3\%deleteTextureList\fP,
\f3\%load\fP, \f3\%unload\fP, \f3\%loadUnload\fP, \f3\%willFit\fP, and \f3\%isListLoaded\fP.


\f3\%pfuTextureManager::*addTextureList\fP(pfNode *node, const char *name)
adds a texture list based on a pfNode \f3\%node\fP.  \f3\%name\fP is an arbitary name
that is used for identifying the texture list in a \f3\%Print\fP.  This version
does not supply a default texture and it is therefore possible to accidentally
load a texture outside of the Texture Manager's control if the object is drawn.
\f3\%addTextureList\fP returns a pfTextureManager::pfuTexList type which is solely
used to pass to \f3\%deleteTextureList\fP, \f3\%load\fP, \f3\%unload\fP, \f3\%loadUnload\fP,
\f3\%willFit\fP, and \f3\%isListLoaded\fP.

\f3\%pfuTextureManager::deleteTextureList\fP(pfuTexList *tlist) deletes a
texture list from the group  of lists the Texture Manager is managing.

\f3\%pfuTextureManager::load\fP(pfuTexList *texList) loads the list referred
to by \f3\%texList\fP to texture memory.  This must be called from the DRAW
process.

\f3\%pfuTextureManager::load\fP(pfuTexList *texList, float deadline) loads
as much of the texture list referred to by \f3\%texList\fP to texture memory as
it can before \f3\%deadline\fP, and returns a value between 0.0f and 1.0f indicating
the level of completion.  This function can be called multiple times until
it returns 1.0f.  This is useful when there is a need to load a large list of
textures.  The load may be  amortized over several frames.  This function
can only be called in the DRAW process.

\f3\%pfuTextureManager::unload\fP(pfuTexList *texList) unload a texture
list from texture memory.  If a default texture is associated with this
list, it will point the image pointers of all of the list of pfTextures to
the default texture image.

\f3\%pfuTextureManager::loadUnload\fP(pfuTexList *loadList, pfuTexList *unloadList) effects a load of \f3\%loadlist\fP and an unload of \f3\%unloadList\fP in
one operation.  This is useful when \f3\%loadList\fP and \f3\%unloadList\fP that have
textures in common to avoid unloading and reloading the common textures.

\f3\%pfuTextureManager::getTextureMemorySize\fP(void) returns the size
of texture memory in bytes.

\f3\%pfuTextureManager::getTextureMemoryUsage\fP(void) returns the size
of texture memory in bytes used by textures managed by the texture manager.

\f3\%pfuTextureManager::willFit\fP(pfuTexList *texList) returns a boolean
indicating whether \f3\%texList\fP will all fit in texture memory.

\f3\%pfuTextureManager::isListLoaded\fP(pfuTexList *texList) returns a
boolean indicating whether all pfTextures in \f3\%texList\fP are loaded in
texture memory.

\f3\%pfuTextureManager::Print\fP(int level) prints a formated message
about managed textures and lists and is useful for debuggin or observing
the Texture Manager's behavior.  Three levels are currently supported:
1) prints only the name of each texture list, it's overall size and a
fraction indicating the level of residence in texture memory,  2) a more
comprehensive list of each list, including status of individual texture
on the list, and 3) a more verbose description.

.SH SEE ALSO
pfTexture(3pf)
