'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfBox" "3pf"
.SH NAME
.upperok
.B pfMakeEmptyBox,
.B pfBoxExtendByPt,
.B pfBoxExtendByBox,
.B pfBoxAroundPts,
.B pfBoxAroundBoxes,
.B pfBoxAroundSpheres,
.B pfBoxAroundCyls,
.B pfBoxContainsPt,
.B pfBoxContainsBox,
.B pfBoxContainsSphere,
.B pfBoxIsectSeg,
.B pfXformBox
\- 
Operate on axis-aligned bounding boxes
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfMakeEmptyBox\fP(pfBox\ *box);
.nf
.fi
.TP
void
\f3\%pfBoxExtendByPt\fP(pfBox\ *dst,
'in +2
'll -2
const\ pfVec3\ pt);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBoxExtendByBox\fP(pfBox\ *dst,
'in +2
'll -2
const\ pfBox\ *box);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBoxAroundPts\fP(pfBox\ *dst,
'in +2
'll -2
const\ pfVec3\ *pts,
int\ npt);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBoxAroundBoxes\fP(pfBox*\ dst,
'in +2
'll -2
const\ pfBox\ **boxes,
int\ nbox);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBoxAroundSpheres\fP(pfBox\ *dst,
'in +2
'll -2
const\ pfSphere\ **sphs,
int\ nsph);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfBoxAroundCyls\fP(pfBox\ *dst,
'in +2
'll -2
const\ pfCylinder\ **sphs,
int\ ncyl);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBoxContainsPt\fP(const\ pfBox*\ box,
'in +2
'll -2
const\ pfVec3\ pt);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBoxContainsBox\fP(const\ pfBox\ *box1,
'in +2
'll -2
const\ pfBox\ *box2);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBoxContainsSphere\fP(const\ pfBox*\ box,
'in +2
'll -2
const\ pfSphere\ *sphere);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBoxIsectSeg\fP(const\ pfBox*\ box,
'in +2
'll -2
const\ pfSeg*\ seg,
float*\ d1,
float*\ d2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfXformBox\fP(pfBox\ *dst,
'in +2
'll -2
const\ pfBox\ *box,
const\ pfMatrix\ xform);
.in
.ll
.fi

.Ex
.nf

typedef struct
{
    pfVec3      min;
    pfVec3      max;
} pfBox;
.Ee
.fi
.na

.SH DESCRIPTION

A pfBox is an axis-aligned box which can be used for intersection tests
and for maintaining bounding information about geometry.  A box
represents the axis-aligned hexahedral volume: (x, y, z) where min[0]
<= x <= max[0], min[1] <= y <= max[1] and min[2] <= z <= max[2].  pfBox
is a public struct whose data members \f2min\fP and \f2max\fP may be
operated on directly.


\f3\%pfMakeEmptyBox\fP sets \f2dst\fP to appear empty to extend operations.

\f3\%pfBoxExtendByPt\fP extends the size of box
\f2dst\fP to include the point \f2pt\fP.

\f3\%pfBoxExtendByBox\fP extends the size of box
\f2dst\fP to include the box \f2box\fP.

\f3\%pfBoxAroundPts\fP,
\f3\%pfBoxAroundBoxes\fP
\f3\%pfBoxAroundCyls\fP and
\f3\%pfBoxAroundSpheres\fP
set \f2dst\fP to be an axis-aligned box encompassing
the given primitives.  \f2npt\fP, \f2nbox\fP, \f2ncyls\fP and \f2nsph\fP are
the number of points, boxes, and spheres in the respective primitive
lists.

\f3\%pfBoxContainsPt\fP returns
\f3\%TRUE\fP or \f3\%FALSE\fP depending on whether the point \f2pt\fP is in the
interior of the specified box.

The return value from \f3\%pfBoxContainsBox\fP is the OR of one or more bit fields.  The returned
value may be:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
The intersection of the second box argument and the
first box is empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP:
The intersection of the second box argument and the
first box is definitely non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP:
The second box argument is non-empty and lies entirely
inside the first box.
.RE

The return value from \f3\%pfBoxContainsSphere\fP is the OR of one or more bit fields.  The returned
value may be:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
The intersection of the box and the sphere is empty, or the box is empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP:
The intersection of the box and the sphere is non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP:
The sphere lies entirely inside the box, or the sphere is empty.
.RE

\f3\%pfBoxIsectSeg\fP intersect the line segment
\f2seg\fP with the volume of an axis-aligned box \f2box\fP.
The possible return values include all of the above as well as:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
\f2seg\fP lies entirely in the exterior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_START_IN\fP:
The starting point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_END_IN\fP:
The ending point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP | \f3\%PFIS_START_IN\fP | \f3\%PFIS_END_IN\fP:
Both end points of \f2seg\fP lie in the interior.
.RE

If \f2d1\fP and \f2d2\fP are non-NULL, on return from
\f3\%pfBoxIsectSeg\fP
they contain the starting and ending positions of the line segment
(0 <= d1 <= d2 <= seg->length) intersected with the specified volume.

\f3\%pfXformBox\fP sets \f2dst\fP to
a box which contains \f2box\fP as transformed by the matrix \f2xform\fP,
i.e. a box around (\f2box\fP * \f2xform\fP).  Because transformed boxes
must be axis-aligned, most rotations cause the box to grow, and the
transformation is not reversed by the inverse rotation.

.SH NOTES

The bit fields returned by the contains functions are structured so
that bitwise AND-ing the results of sequential tests can be used to
compute composite results, e.g. testing exclusion against a number
of half spaces.

Because pfBoxes are axially aligned, they tend to grow when
transformed.  Hence, they are best for static geometry or other cases
in which the bounding geometry does not need to be transformed.
.SH SEE ALSO
pfSeg,
pfSphere
