'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfBuffer" "3pf"
.SH NAME
.upperok
.B pfNewBuffer,
.B pfSelectBuffer,
.B pfMergeBuffer,
.B pfBufferScope,
.B pfGetBufferScope,
.B pfBufferAdd,
.B pfBufferRemove,
.B pfBufferInsert,
.B pfBufferReplace,
.B pfBufferSet,
.B pfAsyncDelete,
.B pfGetCurBuffer
\- 
Create, select, and merge a pfBuffer.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfBuffer\ *\ \ \ 'u
#include <Performer/pf.h>
.nf
.fi
.TP
pfBuffer *
\f3\%pfNewBuffer\fP(void);
.nf
.fi
.TP
void
\f3\%pfSelectBuffer\fP(pfBuffer*\ buf);
.nf
.fi
.TP
int
\f3\%pfMergeBuffer\fP(void);
.nf
.fi
.TP
void
\f3\%pfBufferScope\fP(pfBuffer\ *buf,
'in +2
'll -2
pfObject\ *obj,
int\ scope);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfGetBufferScope\fP(pfBuffer\ *buf,
'in +2
'll -2
pfObject\ *obj);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBufferAdd\fP(void\ *parent,
'in +2
'll -2
void\ *child);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBufferRemove\fP(void\ *parent,
'in +2
'll -2
void\ *child);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBufferInsert\fP(void\ *parent,
'in +2
'll -2
int\ index,
void\ *child);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBufferReplace\fP(void\ *parent,
'in +2
'll -2
void\ *oldChild,
void\ *newChild);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfBufferSet\fP(void\ *parent,
'in +2
'll -2
int\ index,
void\ *child);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfAsyncDelete\fP(void\ *mem);
.nf
.fi
.TP
pfBuffer*
\f3\%pfGetCurBuffer\fP(void);
.fi
.SH PARAMETERS
.TP \w'buf\ \ 'u
\f2buf\fP
identifies a pfBuffer
.TP
\f2obj\fP
identifies a pfObject
.SH DESCRIPTION

A pfBuffer is a data structure that logically encompasses \f3\%libpf\fP
objects such as pfNodes. Newly created objects are automatically
"attached" to the current pfBuffer specified by
\f3\%pfSelectBuffer\fP.  Later, any objects
created in \f2buf\fP may be merged into the main
OpenGL Performer processing stream with
\f3\%pfMergeBuffer\fP. In conjunction with a forked
\f3\%DBASE\fP process (see \f3\%pfMultiprocess\fP and \f3\%pfDBaseFunc\fP), the
pfBuffer mechanism supports asynchronous parallel creation and deletion
of database objects.  This is the foundation of a real-time database
paging system.

\f3\%pfNewBuffer\fP creates and returns a handle to a pfBuffer.

\f3\%pfSelectBuffer\fP makes \f2buf\fP the current pfBuffer. Once \f2buf\fP is
current, all subsequently created \f3\%libpf\fP objects will be
automatically associated with \f2buf\fP and these
objects may only be accessed through OpenGL Performer routines when
\f2buf\fP is the current pfBuffer (except for
\f3\%pfBufferAddChild\fP and
\f3\%pfBufferRemoveChild\fP, see the
pfGroup man page).  A given pfBuffer should only be current in a single
process at any given time.  In this way, a pfBuffer restricts access to
a given object to a single process, avoiding hard-to-find errors due to
multiprocessed data collisions. \f3\%pfGetCurBuffer\fP returns the current
pfBuffer.

Only \f3\%libpf\fP objects are subject to pfBuffer access restrictions.
\f3\%libpf\fP objects include pfNodes such as pfGroup, pfGeode and
pfUpdatables such as pfLODState, pfChannel, pfEarthSky. \f3\%libpr\fP
objects such as pfGeoSets, pfGeoStates, and pfMaterials have no
pfBuffer restrictions so they may be accessed by any process at any
time although care must be taken by the application to avoid
multiprocessed collisions on these data structures.

\f3\%pfMergeBuffer\fP merges the current pfBuffer
with the main OpenGL Performer pfBuffer.  This main pfBuffer is created
by \f3\%pfConfig\fP and will resist deletion and merging and should only be
made current in the \f3\%APP\fP process (however, it is legal to select a
different buffer in the \f3\%APP\fP process). If called in a process other
than the \f3\%APP\fP, \f3\%pfMergeBuffer\fP will block
until the \f3\%APP\fP calls \f3\%pfSync\fP, at which time the \f3\%APP\fP will
merge the current pfBuffer into the main pfBuffer and then allow the
process that requested the merge to continue. If called in the \f3\%APP\fP,
\f3\%pfMergeBuffer\fP will immediately execute the
merge.  After \f3\%pfMergeBuffer\fP returns, any
objects that were created in the current pfBuffer may only be accessed
in the \f3\%APP\fP process when the \f3\%APP\fP pfBuffer has been selected as
the current pfBuffer.  In other words, the merged pfBuffer has been
"reset" and its objects now "exist" only in the \f3\%APP\fP pfBuffer. The
addresses of \f3\%libpf\fP objects are not changed by
\f3\%pfMergeBuffer\fP.

Any number of pfBuffers may be used and merged
(\f3\%pfMergeBuffer\fP) by any number of processes
for multithreaded database manipulation, subject to the following
restrictions:

.RS
.TP
1.
A given pfBuffer should be current
(via \f3\%pfSelectBuffer\fP) in only a single
process at any given time.
.TP
2.
Each process which selects a pfBuffer must be \f3\%fork\fPed, not
\f3\%sproc\fPed.
.RE

Specifically, pfBuffer usage is not restricted to the
\f3\%DBASE\fP process (see \f3\%pfConfig\fP).

\f3\%pfBufferAddChild\fP and
\f3\%pfBufferRemoveChild\fP provide
access to nodes that do not exist in the current pfBuffer.
Either, none, or both of \f2group\fP and \f2node\fP may exist outside the current pfBuffer.
\f3\%pfBufferAddChild\fP and
\f3\%pfBufferRemoveChild\fP act just
like their non-buffered counterparts
\f3\%pfAddChild\fP and
\f3\%pfRemoveChild\fP except that the
addition or removal request is not carried out immediately but is
recorded by the current pfBuffer. The request is delayed until the
first \f3\%pfMergeBuffer\fP when both
\f2group\fP and \f2node\fP are found in the main OpenGL
Performer pfBuffer. The list of
\f3\%pfBufferAddChild\fP and
\f3\%pfBufferRemoveChild\fP requests is
traversed in \f3\%pfMergeBuffer\fP after all nodes
have been merged.
\f3\%pfBufferAddChild\fP and
\f3\%pfBufferRemoveChild\fP return
\f3\%TRUE\fP if the request was recorded and \f3\%FALSE\fP otherwise.

\f3\%pfBufferSet\fP sets the specified child \f2index\fP of a
\f2parent\fP node to the given \f2child\fP. It acts as a replace command but uses
an index to identify the replaced child. The current implementation supports
\f3\%pfBufferSet\fP only for setting a source array in a
\f3\%pfAlign\fP node.

In addition to the pfGroup-specific
\f3\%pfBufferAddChild\fP and
\f3\%pfBufferRemoveChild\fP
routines, a pfBuffer allows generic list management for pfGroup,
pfGeode, pfText, and pfPipeWindow objects. These functions,
\f3\%pfBufferAdd\fP,
\f3\%pfBufferRemove\fP,
\f3\%pfBufferInsert\fP,
\f3\%pfBufferReplace\fP
can be used to manage a pfGroup's list of pfNodes,
a pfGeode's list of pfGeoSets,
a pfText's list of pfStrings,
or a pfPipeWindow's list of pfChannels
respectively.
These routines infer the proper action to take from the argument types.
For example, the following code fragment is equivalent to
\f3\%pfBufferAddChild(group, geode)\fP:

.Ex
.nf

pfGroup	*group;
pfGeode	*geode;

pfBufferAdd(group, geode);
.Ee
.fi
.na


\f3\%pfBufferAdd\fP,
\f3\%pfBufferRemove\fP,
\f3\%pfBufferInsert\fP,
\f3\%pfBufferReplace\fP,
\f3\%pfBufferSet\fP
all act similarly in that they do not have effect until
\f3\%pfMergeBuffer\fP is called and all parties
have been merged into the main OpenGL Performer buffer. They return -1 if
the argument types are not consistent (e.g., pfBufferRemove(group,
geoset)), 0 if the request is
immediately processed (this happens when all parties already have scope
in the current pfBuffer), and 1 if the request is buffered until the
next \f3\%pfMergeBuffer\fP.

\f3\%pfBufferScope\fP sets the scope of \f2obj\fP
with respect to pfBuffer \f2buf\fP. If \f2scope\fP is TRUE,
then \f2obj\fP is "added" to \f2buf\fP so that when
\f2buf\fP is made current
(\f3\%pfSelectBuffer\fP) in a process, \f2obj\fP may
be accessed through OpenGL Performer routines in that same process.  When
\f2scope\fP is FALSE, \f2obj\fP is "removed" from \f2buf\fP.  \f3\%pfBufferScope\fP's primary
purpose is to move objects between pfBuffers, particularly from the
main \f3\%APP\fP pfBuffer into an application pfBuffer typically used for
asynchronous database manipulations.  In this case the object's scope
would be set to FALSE in the old pfBuffer and TRUE in the new pfBuffer.
It is undefined when an object has scope in multiple pfBuffers since
this violates the multiprocessing data exclusion requirement of OpenGL
Performer.  \f3\%pfGetBufferScope\fP returns
TRUE or FALSE indicating the scope of \f2obj\fP in pfBuffer
\f2buf\fP.

When using pfBuffers for database paging, it is sometimes desirable
to retain certain, common database models ("library models") in memory.
Examples are trees, houses, and other "culture" which are instanced on
paged terrain patches. One instancing mechanism is to create the
library models in one pfBuffer and later use
\f3\%pfBufferAddChild\fP to attach the models to
scene graphs created in another pfBuffer. This is classic instancing
which uses transformations (pfSCS) to properly position the models.
However, this mechanism suffers from 2 performance problems:
.RS
.TP
1.
\f3\%pfMergeBuffer\fP will adversely impact the
\f3\%APP\fP process, proportional
to the number of \f3\%pfBufferAddChild\fP and \f3\%pfBufferRemoveChild\fP requests.
.TP
2.
Transformations in the scene graph reduce OpenGL Performer's ability
to sort the database (see \f3\%pfChanBinSort\fP)
and matrix operations have some cost in the graphics pipeline.
.RE

An alternative to classic instancing is "flattening" which creates
a clone of the instanced subtree and then applies the transformation
to all geometry in the cloned subtree. This method eliminates
the performance problems listed above but does increase memory usage.

.IP
pfNode* \f3\%pfBufferClone\fP(pfNode *node, int mode, pfBuffer *buf)
.PP
is a version of \f3\%pfClone\fP which clones \f2node\fP and its subtree,
which resides in \f2buf\fP, into the current pfBuffer. \f2mode\fP
is the same argument as that passed to \f3\%pfClone\fP (it is currently
ignored). Once cloned, a subtree may be flattened with \f3\%pfFlatten\fP:

Example 1: Instancing with pfBufferAddChild

.Ex
.nf


libraryBuffer = pfNewBuffer();
pfSelectBuffer(libraryBuffer);

loadLibraryObjects();

pagingBuffer = pfNewBuffer();
pfSelectBuffer(pagingBuffer);

while (!done)
{
    pfNode	*newStuff;
    pfSCS	*treeLocation;

    /* Load new terrain tile or whatever */
    newStuff = loadStuff();

    /* Create pfSCS which is location of tree */
    treeLocation = pfNewSCS(treeMatrix);

    /* Add library model of a tree to treeLocation */
    pfBufferAddChild(treeLocation, libraryTree);

    /* Add instanced tree to newly loaded stuff */
    pfAddChild(newStuff, treeLocation);
}

.Ee
.fi
.na


Example 2: Instancing with pfBufferClone and pfFlatten

.Ex
.nf


libraryBuffer = pfNewBuffer();
pfSelectBuffer(libraryBuffer);

loadLibraryObjects();

pagingBuffer = pfNewBuffer();
pfSelectBuffer(pagingBuffer);

while (!done)
{
    pfNode	*newStuff;
    pfSCS	*treeLocation;

    /* Load new terrain tile or whatever */
    newStuff = loadStuff();

    /* Create pfSCS which is location of tree */
    treeLocation = pfNewSCS(treeMatrix);

    /* Clone tree model from library into current, paging buffer */
    newTree = pfBufferClone(libraryTree, 0, libraryBuffer);

    /* Transform cloned tree */
    pfAddChild(treeLocation, newTree);
    pfFlatten(treeLocation);

    /* Get rid of unneeded treeLocation */
    pfRemoveChild(treeLocation, newTree);
    pfDelete(treeLocation);

    /* Add cloned, flattened tree to newly loaded stuff */
    pfAddChild(newStuff, newTree);
}

.Ee
.fi
.na


\f3\%pfAsyncDelete\fP is a special version of \f3\%pfDelete\fP which is
useful for asynchronous database deletion. Instead of having immediate
effect, \f3\%pfAsyncDelete\fP simply registers a deletion request at the
time of invocation.  These deletion requests are then processed in the
\f3\%DBASE\fP trigger routine, \f3\%pfDBase\fP (\f3\%pfDBase\fP is automatically
called if you have not registered a \f3\%DBASE\fP callback with
\f3\%pfDBaseFunc\fP).  Thus, if the \f3\%DBASE\fP processing stage is
configured as its own process via \f3\%pfMultiprocess\fP, then the
deletion will be carried out asynchronously without affecting (slowing
down) the main processing pipelines.

\f3\%pfAsyncDelete\fP may be called from any process and returns -1 if
\f2mem\fP is NULL or not derived from pfMemory and returns TRUE
otherwise.  Note that unlike \f3\%pfDelete\fP \f3\%pfAsyncDelete\fP does not
check \f2mem\fP's reference count and return TRUE or FALSE indicating
whether \f2mem\fP was successfully deleted or not. Instead, the
reference count check is delayed until the next call to \f3\%pfDBase\fP.
At this time there is no way to query the success of an
\f3\%pfAsyncDelete\fP request.

Note that \f3\%pfDBase\fP should only be called from within the
database callback function (\f3\%pfDBaseFunc\fP) in the \f3\%DBASE\fP process
just like \f3\%pfCull\fP and \f3\%pfDraw\fP should only be called in the
pfChannel \f3\%CULL\fP and \f3\%DRAW\fP callbacks respectively (\f3\%pfChanTravFunc\fP).

Example 2: How to use a pfBuffer
.Ex
.nf


/* Must create these in shared memory */
static pfGroup	**Tiles;
static int	*TileStatus;

/*
 * Load new tiles and delete old ones.
 */
void
pageDBase(void *data)
{
    static pfBuffer 	*buf = NULL;
    pfGroup		*root;

    if (buf == NULL)
    {
  	buf = pfNewBuffer();
  	pfSelectBuffer(buf);
    }

    /* Asynchronously delete unneeded tiles and update their status */
    for (allUnneededTiles)
    {
	/*
         * Scene does not have scope in 'buf' so use pfBufferRemoveChild
	 * Tiles[i] is not really removed until pfMergeBuffer
	 */
	pfBufferRemoveChild(Scene, Tiles[i]);

	/* Delete Tiles[i] at pfDBase time if Tiles[i] only has Scene as
	   a parent.
	 */
    	pfAsyncDelete(Tiles[i]);

	/* Update tile status */
	TileStatus[i] = TILE_DELETED;
    }

    /*
     * Synchronously load needed tiles and update their status.
     */
    LoadNeededDatabaseTiles(Tiles, TileStatus);

    for (allLoadedTiles)
    {
	/*
         * Scene does not have scope in 'buf' so use pfBufferAddChild
	 * loadedTile[i] is not really added until pfMergeBuffer
	 */
	pfBufferAddChild(Scene, loadedTile[i]);
    }

    /*
     * Merge newly loaded tiles into main pfBuffer then carry out
     * all pfBufferAdd/RemoveChild requests.
     */
    pfMergeBuffer();

    /*
     * Carry out pfAsyncDelete requests. Call *after* pfMergeBuffer()
     * so that all pfBufferRemoveChild requests have been processed
     * and child reference counts have been properly decremented.
     */
    pfDBase();
}

    :

pfInit();
Tiles = pfMalloc(sizeof(pfGroup*) * NUM_TILES, pfGetSharedArena());
TileStatus = pfMalloc(sizeof(int) * NUM_TILES, pfGetSharedArena());
pfMultiprocess(PFMP_APP_CULL_DRAW | PFMP_FORK_DBASE);
pfConfig();
    :
pfDBaseFunc(pageDBase);

while(!done)
{
    pfSync();

    /* Remove and request deletion of unneeded tiles */
    UpdateTileStatus(Tiles, TileStatus);

    pfFrame();
}

.Ee
.fi
.na

.SH NOTES

\f3\%pfGetCurBuffer\fP will return the \f3\%APP\fP pfBuffer immediately
after \f3\%pfConfig\fP returns.
.SH SEE ALSO
pfBufferAddChild,
pfBufferRemoveChild,
pfConfig,
pfDBaseFunc,
pfDelete,
pfFrame,
pfMultiprocess,
pfGroup
