'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfComputeFunc" "3pf"
.SH NAME
.upperok
.B pfComputeFunc,
.B pfGetComputeFunc,
.B pfAllocComputeData,
.B pfGetComputeData,
.B pfPassComputeData,
.B pfCompute
\- 
Set callback for compute process, allocate and pass compute data.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfComputeFuncType\ \ \ 'u
#include <Performer/pf.h>
.nf
.fi
.TP
void
\f3\%pfComputeFunc\fP(pfComputeFuncType\ func);
.nf
.fi
.TP
pfComputeFuncType
\f3\%pfGetComputeFunc\fP(void);
.nf
.fi
.TP
void *
\f3\%pfAllocComputeData\fP(int\ bytes);
.nf
.fi
.TP
void *
\f3\%pfGetComputeData\fP(void);
.nf
.fi
.TP
void
\f3\%pfPassComputeData\fP(void);
.nf
.fi
.TP
void
\f3\%pfCompute\fP(void);
.fi

.Ex
.nf

typedef void (*pfComputeFuncType)(void *userData);
.Ee
.fi
.na

.SH DESCRIPTION

The \f2func\fP argument to \f3\%pfComputeFunc\fP specifies the compute
callback function.  This function will be invoked by \f3\%pfFrame\fP and
will be passed a pointer to a data buffer allocated by
\f3\%pfAllocComputeData\fP.  If a separate process is allocated for asynchronous
computation by the \f3\%PFMP_FORK_COMPUTE\fP mode to \f3\%pfMultiprocess\fP, then
\f3\%pfFrame\fP will cause \f2func\fP to be called in the separate
(\f3\%COMPUTE\fP) process.  \f3\%pfGetComputeFunc\fP returns the compute callback
or \f3\%NULL\fP if none is set.

The compute function's primary purpose is to provide a place for per-frame
asynchronous computation.  The results of such computation may be placed in
pfFlux objects for use in the scene graph or by the asynchronous rendering pipeline;
see the man page for \f3\%pfNewFlux\fP). The compute
process is automatically used by some OpenGL Performer objects, such as pfASD,
that require extensive computation that can be done asynchronously and potentially
extend beyond the time for a frame.  see \f3\%pfNewASD\fP).

When the compute function is in a separate process, it will run
asynchronously with the rest of the rendering pipeline. Specifically,
if the compute function takes more than a frame time, the rendering
pipeline will not be affected.

Unlike the intersection and cull processes, there is no copy of the scene
graph in the compute process.  Only pfFlux objects can be safely inspected
or edited by the compute process.  It is not valid to look at any pfNode
in the compute process unless specific care for multibuffering has been
done: creating of a pfBuffer and calling \f3\%pfBufferScope\fP to move the node
to the compute process. See the man pages for \f3\%pfDBaseFunc\fP and
\f3\%pfNewBuffer\fP for more information.
Additionally, unlike the database process, there is no automatic method
for merging computed results back into the application process.  However,
if the results are stored in a pfFlux, this should not be necessary.

If a compute function has been specified by \f3\%pfComputeFunc\fP, it must
call \f3\%pfCompute\fP to carry out default OpenGL Performer compute
processing. \f3\%pfCompute\fP should only be called from within the
\f3\%COMPUTE\fP callback in the \f3\%COMPUTE\fP process
just like \f3\%pfCull\fP and \f3\%pfDraw\fP should only be called in the
pfChannel \f3\%CULL\fP and \f3\%DRAW\fP callbacks (\f3\%pfChanTravFunc\fP)
respectively. If a compute function has not been specified or
is \f3\%NULL\fP, OpenGL Performer automatically calls \f3\%pfCompute\fP
from \f3\%pfFrame\fP.

\f3\%pfAllocComputeData\fP returns a pointer to a chunk of shared memory of
\f2bytes\fP bytes.  This memory buffer may be used to communicate
information between the compute function and application.  Compute
data should only be allocated once.  \f3\%pfGetComputeData\fP returns the
previously allocated compute data.

When the compute function is forked, \f3\%pfPassComputeData\fP should be
used to copy the compute data into internal OpenGL Performer memory when
the next \f3\%pfFrame\fP is called.  Once \f3\%pfFrame\fP is called, the
application may modify data in the compute data buffer without fear of
colliding with the forked compute function. However, modifications
to the compute data chunk made by the \f3\%COMPUTE\fP process will not
be visible to the \f3\%APP\fP process, i.e, there is no "upstream"
propagation of passthrough data.  For managing sending upstream
or asynchronous processes to receive the results of the compute process,
you can use pfFlux objects.

.SH SEE ALSO
pfConfig,
pfFrame,
pfMultiprocess,
pfNewFlux,
pfNewBuffer
