'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfCullFace" "3pf"
.SH NAME
.upperok
.B pfCullFace,
.B pfGetCullFace
\- 
Specify face culling mode
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfCullFace\fP(int\ mode);
.nf
.fi
.TP
int
\f3\%pfGetCullFace\fP(void);
.fi
.SH PARAMETERS
.TP \w'mode\ \ 'u
\f2mode\fP
is a symbolic constant and is one of:
.RS
.TP 20
\f3\%PFCF_OFF\fP
Face culling is off,
.TP
\f3\%PFCF_BACK\fP
Polygons that are back-facing will be culled.
.TP
\f3\%PFCF_FRONT\fP
Polygons that are front-facing will be culled.
.TP
\f3\%PFCF_BOTH\fP
Polygons that are front and back-facing will be culled.
.RE
.SH DESCRIPTION

\f3\%pfCullFace\fP sets the face culling mode used to cull all subsequent
polygons.  A polygon is considered to be backfacing if its vertices are
in clockwise order (screen coordinates).  Frontfacing polygon vertex
ordering is counterclockwise.

\f3\%pfGetCullFace\fP returns the current face culling mode.

The face culling mode state element is identified by the
\f3\%PFSTATE_CULLFACE\fP token.  Use this token with \f3\%pfGStateMode\fP
to set the face culling mode of a pfGeoState and with \f3\%pfOverride\fP
to override subsequent face culling mode changes.

\f3\%pfCullFace\fP is a display-listable command.  If a pfDispList has been
opened by \f3\%pfOpenDList\fP, \f3\%pfCullFace\fP will not have immediate effect
but will be captured by the pfDispList and will only have effect when
that pfDispList is later drawn with \f3\%pfDrawDList\fP.

Example 1:
.Ex
.nf

/* Set up 'face-culled' pfGeoState */
pfGStateMode(gstate, PFSTATE_CULLFACE, PFCF_BACK);

/* Attach gstate to gset */
pfGSetGState(gset, gstate);

/* Draw face-culled gset */
pfDrawGSet(gset);
.Ee
.fi
.na


Example 2:
.Ex
.nf

/* Override face culling mode to PFCF_OFF */
pfCullFace(PFCF_OFF);
pfOverride(PFSTATE_CULLFACE, PF_ON);
.Ee
.fi
.na

.SH NOTES

Backface culling with \f3\%pfCullFace\fP(\f3\%PFCF_BACK\fP) can significantly
improve performance for "solid" databases whose polygons are oriented
consistently and where objects are closed.  With these databases you
cannot see backfacing polygons since they are always obscured by
nearer front-facing ones.  The graphics hardware can quickly reject
backfacing polygons so use of backface culling is \f2strongly\fP
encouraged to increase performance.

Face culling should be disabled when using two-sided lighting, since
the two-sided lighting is only useful for distinguishing backfacing
objects.
.SH SEE ALSO
backface,
frontface,
pfDispList,
pfGeoState,
pfOverride,
pfState
