'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfCylinder" "3pf"
.SH NAME
.upperok
.B pfMakeEmptyCyl,
.B pfCylAroundSegs,
.B pfCylAroundPts,
.B pfCylAroundBoxes,
.B pfCylAroundSpheres,
.B pfCylExtendByBox,
.B pfCylExtendBySphere,
.B pfCylExtendByCyl,
.B pfCylContainsPt,
.B pfCylIsectSeg,
.B pfOrthoXformCyl
\- 
Operations on cylinder definitions.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfMakeEmptyCyl\fP(pfCylinder\ *cyl);
.nf
.fi
.TP
void
\f3\%pfCylAroundSegs\fP(pfCylinder\ *dst,
'in +2
'll -2
const\ pfSeg\ **segs,
int\ nseg);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylAroundPts\fP(pfCylinder\ *dst,
'in +2
'll -2
const\ pfVec3\ *pts,
int\ npt);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylAroundBoxes\fP(pfCylinder\ *dst,
'in +2
'll -2
const\ pfBox\ **boxes,
int\ nbox);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylAroundSpheres\fP(pfCylinder\ *dst,
'in +2
'll -2
const\ pfSphere\ **sphs,
int\ nsph);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylExtendByBox\fP(pfCylinder\ *dst,
'in +2
'll -2
const\ pfBox\ *box);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylExtendBySphere\fP(pfCylinder\ *dst,
'in +2
'll -2
const\ pfSphere\ *sph);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfCylExtendByCyl\fP(pfCylinder\ *dst,
'in +2
'll -2
const\ pfCylinder\ *cyl);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfCylContainsPt\fP(const\ pfCylinder*\ cyl,
'in +2
'll -2
const\ pfVec3\ pt);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfCylIsectSeg\fP(const\ pfCylinder*\ cyl,
'in +2
'll -2
const\ pfSeg*\ seg,
float*\ d1,
float*\ d2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfOrthoXformCyl\fP(pfCylinder\ *dst,
'in +2
'll -2
const\ pfCylinder\ *cyl,
const\ pfMatrix\ xform);
.in
.ll
.fi

.Ex
.nf

typedef struct
{
    pfVec3   center;
    float    radius;
    pfVec3   axis;
    float    halfLength;
} pfCylinder;
.Ee
.fi
.na

.SH DESCRIPTION

A pfCylinder represents a cylinder of finite length.  The routines
listed here provide means of creating and extending cylinders for use
as bounding geometry around groups of line segments.  The cylinder is
defined by its \f2center\fP, \f2radius\fP, \f2axis\fP and \f2halfLength\fP.
The routines assume \f2axis\fP is a vector of unit length, otherwise
results are undefined.  pfCylinder is a public struct whose data
members \f2center\fP, \f2radius\fP, \f2axis\fP and \f2halfLength\fP may be
operated on directly.


\f3\%pfMakeEmptyCyl\fP sets \f2dst\fP so that it appears empty to other operations.

\f3\%pfCylAroundSegs\fP,
\f3\%pfCylAroundPts\fP,
\f3\%pfCylAroundSpheres\fP
and \f3\%pfCylAroundBoxes\fP set \f2dst\fP to a cylinder which
contains a set of line segments, points, spheres or boxes,
respectively.  These routines are passed the address of an array of
pointers to the objects to be encompassed along with the number of
objects.

\f3\%pfCylExtendByBox\fP,
\f3\%pfCylExtendBySphere\fP, and \f3\%pfCylExtendByCyl\fP set \f2dst\fP to a cylinder
which contains both the pfSphere \f2dst\fP and the box \f2box\fP,
the sphere \f2sph\fP or the cylinder \f2cyl\fP, respectively.

\f3\%pfCylContainsPt\fP returns \f3\%TRUE\fP or
\f3\%FALSE\fP depending on whether the point \f2pt\fP is in the interior of
the specified pfCylinder.

\f3\%pfCylIsectSeg\fP intersects the line segment \f2seg\fP with the volume
of the cylinder \f2cyl\fP.  The possible return values
are:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
\f2seg\fP lies entirely in the exterior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_START_IN\fP:
The starting point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_END_IN\fP:
The ending point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP | \f3\%PFIS_START_IN\fP | \f3\%PFIS_END_IN\fP:
Both end points of \f2seg\fP lie in the interior.
.RE

If \f2d1\fP and \f2d2\fP are non-NULL, on return from
\f3\%pfCylIsectSeg\fP they contain the starting
and ending positions of the line segment (0 <= d1 <= d2 <=
seg->length) intersected with the cyl.

\f3\%pfOrthoXformCyl\fP sets \f2dst\fP to the cylinder \f2cyl\fP transformed by the orthogonal
transformation \f2xform\fP.
\f2dst\fP = \f2cyl\fP * \f2xform\fP.
If \f2xform\fP is not an orthogonal transformation the results are undefined.
.SH SEE ALSO
pfBox,
pfCylinder,
pfSeg,
pfSphere,
pfVec3
