'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfDBaseFunc" "3pf"
.SH NAME
.upperok
.B pfDBaseFunc,
.B pfGetDBaseFunc,
.B pfAllocDBaseData,
.B pfGetDBaseData,
.B pfPassDBaseData,
.B pfDBase
\- 
Set database callback, allocate and pass database data.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfDBaseFuncType\ \ \ 'u
#include <Performer/pf.h>
.nf
.fi
.TP
void
\f3\%pfDBaseFunc\fP(pfDBaseFuncType\ func);
.nf
.fi
.TP
pfDBaseFuncType
\f3\%pfGetDBaseFunc\fP(void);
.nf
.fi
.TP
void *
\f3\%pfAllocDBaseData\fP(int\ bytes);
.nf
.fi
.TP
void *
\f3\%pfGetDBaseData\fP(void);
.nf
.fi
.TP
void
\f3\%pfPassDBaseData\fP(void);
.nf
.fi
.TP
void
\f3\%pfDBase\fP(void);
.fi

.Ex
.nf

typedef void (*pfDBaseFuncType)(void *userData);
.Ee
.fi
.na

.SH DESCRIPTION

The \f2func\fP argument to \f3\%pfDBaseFunc\fP specifies the database
callback function.  This function will be invoked by \f3\%pfFrame\fP and
will be passed a pointer to a data buffer allocated by
\f3\%pfAllocDBaseData\fP.  If a separate process is allocated for database
processing by the \f3\%PFMP_FORK_DBASE\fP mode to \f3\%pfMultiprocess\fP, then
\f3\%pfFrame\fP will cause \f2func\fP to be called in the separate
(\f3\%DBASE\fP) process.  \f3\%pfGetDBaseFunc\fP returns the database callback
or \f3\%NULL\fP if none is set.

The database function's primary purpose is to provide asynchronous
database creation and deletion when using the pfBuffer mechanism and a
forked \f3\%DBASE\fP process (see \f3\%PFMP_FORK_DBASE\fP, \f3\%pfMultiprocess\fP,
and \f3\%pfNewBuffer\fP).

When the database function is in a separate process, it will run
asynchronously with the rest of the rendering pipeline. Specifically,
if the database function takes more than a frame time, the rendering
pipeline will not be affected.

If a database function has been specified by \f3\%pfDBaseFunc\fP, it must
call \f3\%pfDBase\fP to carry out default OpenGL Performer database
processing. \f3\%pfDBase\fP should only be called from within the
\f3\%DBASE\fP callback in the \f3\%DBASE\fP process
just like \f3\%pfCull\fP and \f3\%pfDraw\fP should only be called in the
pfChannel \f3\%CULL\fP and \f3\%DRAW\fP callbacks (\f3\%pfChanTravFunc\fP)
respectively. If a database function has not been specified or
is \f3\%NULL\fP, OpenGL Performer automatically calls \f3\%pfDBase\fP
from \f3\%pfFrame\fP.

\f3\%pfAllocDBaseData\fP returns a pointer to a chunk of shared memory of
\f2bytes\fP bytes.  This memory buffer may be used to communicate
information between the database function and application.  Database
data should only be allocated once.  \f3\%pfGetDBaseData\fP returns the
previously allocated database data.

When the database function is forked, \f3\%pfPassDBaseData\fP should be
used to copy the database data into internal OpenGL Performer memory when
the next \f3\%pfFrame\fP is called.  Once \f3\%pfFrame\fP is called, the
application may modify data in the database data buffer without fear of
colliding with the forked database function. However, modifications
to the database data chunk made by the \f3\%DBASE\fP process will not
be visible to the \f3\%APP\fP process, i.e, there is no "upstream"
propagation of passthrough data.

.SH NOTES

Currently, \f3\%pfDBase\fP carries out asynchronous deletion requests
made with \f3\%pfAsyncDelete\fP.

.SH SEE ALSO
pfAsyncDelete,
pfConfig,
pfFrame,
pfMultiprocess,
pfNewBuffer
