'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfDecal" "3pf"
.SH NAME
.upperok
.B pfDecal,
.B pfGetDecal,
.B pfApplyDecalPlane,
.B pfGetCurDecalPlane
\- 
Set and get decal mode for drawing coplanar polygons
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfPlane\ *\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfDecal\fP(int\ mode);
.nf
.fi
.TP
int
\f3\%pfGetDecal\fP(void);
.nf
.fi
.TP
void
\f3\%pfApplyDecalPlane\fP(pfPlane\ *plane);
.nf
.fi
.TP
pfPlane *
\f3\%pfGetCurDecalPlane\fP();
.fi
.SH PARAMETERS
.TP \w'mode\ \ 'u
\f2mode\fP
is a symbolic constant specifying a decaling mode and is one of:
.RS
.TP 5
\f3\%PFDECAL_OFF\fP
Decaling is off
.TP
\f3\%PFDECAL_BASE\fP
Subsequent drawn geometry is considered to be 'base'
geometry. Use the default decaling mechanism.
.TP
\f3\%PFDECAL_LAYER\fP
Subsequent drawn geometry is considered to be 'layered'
geometry. Use the default decaling mechanism.
.TP
\f3\%PFDECAL_BASE_FAST\fP, \f3\%PFDECAL_LAYER_FAST\fP
Use a decaling mechanism appropriate to the hardware that
produces the fastest, but not necessarily the highest quality, decaling.
.TP
\f3\%PFDECAL_BASE_HIGH_QUALITY\fP, \f3\%PFDECAL_LAYER_HIGH_QUALITY\fP
Use a decaling mechanism appropriate to the hardware
that produces the highest quality, but not necessarily the fastest, decaling.
.TP
\f3\%PFDECAL_BASE_DISPLACE\fP, \f3\%PFDECAL_LAYER_DISPLACE\fP
Use the polygon displacement technique (\f3\%glPolygonOffsetEXT\fP in OpenGL)
to slightly displace the depth
values of layer geometry \f2toward\fP the eyepoint.
.TP
\f3\%PFDECAL_BASE_DISPLACE\fP, \f3\%PFDECAL_LAYER_DISPLACE_AWAY\fP
Use the polygon displacement technique
(\f3\%glPolygonOffsetEXT\fP in OpenGL) to slightly displace the depth
values of layer geometry \f2away\fP from the eyepoint.
.TP
\f3\%PFDECAL_BASE_STENCIL\fP, \f3\%PFDECAL_LAYER_STENCIL\fP
Use the stencil buffer technique
(\f3\%glStencilFunc\fP, \f3\%glStencilOp\fP, and \f3\%glEnable(GL_STENCIL_TEST)\fP
in OpenGL) to determine visual priority.
.TP
\f3\%PFDECAL_PLANE\fP
Use the current decal reference plane to force all decal geometry to be
rendered with the same plane equation.
.RE

The constant \f3\%PFDECAL_PLANE\fP may be specified alone or OR'ed with one of the
above modes to enable the use of reference planes for the base and layer
geometry. When these planes are specified, subsequent geometry is forced
to be truly coplanar with the reference plane.  Displace and stencil
operations happen on the planed geometry.  Reference planes may be specified
in the global state with \f3\%pfApplyDecalPlane\fP, in the pfGeoState with the
\f3\%PFSTATE_DECAL_PLANE\fP attribute, and on the pfGeoSet itself with
\f3\%pfGSetDecalPlane\fP.  Setting reference
planes on pfGeoSets directly may help improve performance by allowing
better re-use of pfGeoStates but will also slightly slow the handling
of the pfGeoSet.
Reference planes are only supported under OpenGL
operation and require the reference_plane SGI extension to the OpenGL.
The \f3\%PFQFTR_DECAL_PLANE\fP to \f3\%pfQueryFeature\fP can be used for run-time
queries for the support of decal reference planes.

.SH DESCRIPTION

In some cases, such as when drawing stripes on a runway, it is easier to
draw coplanar polygons than it is to model the geometry without coplanar
faces.  However, on Z-buffer based machines, coplanar polygons can cause
unwanted visual artifacts because the visual priorities of the coplanar
polygons are subject to the finite numerical precision of the graphics
pipeline. This results in a "torn" appearance and "twinkling" from frame
to frame.

Decaled geometry can be thought of as a stack where each
layer has visual priority over the geometry beneath it in the stack.
An example of a 3 layer stack consists of stripes which are layered
over a runway which is layered over the ground. The bottommost
layer is called the "base" while the other layers are called "decals" or "layers".
When using certain hardware mechanisms (\f3\%PFDECAL_BASE_STENCIL\fP)
to implement pfDecal, the "base" is special because it defines
the depth values which are used to determine layer visibility with
respect to other scene geometry and which are written to the depth buffer.

Certain decaling mechanisms (currently only \f3\%DISPLACE\fP) require that
each layer in the layer stack be identified for proper rendering.
The \f3\%PFDECAL_LAYER_1\fP through \f3\%PFDECAL_LAYER_7\fP tokens are provided for
this purpose and should be logically OR'ed into the layer mode, e.g.,
\f3\%PFDECAL_LAYER_DISPLACE\fP | \f3\%PFDECAL_LAYER_2\fP. Note that the
layer identifier is extracted from the mode as follows:
.Ex
.nf

layerId = (mode & \f3\%PFDECAL_LAYER_MASK\fP) >> \f3\%PFDECAL_LAYER_SHIFT\fP;
.Ee
.fi
.na


\f3\%pfDecal\fP is used to draw visually correct coplanar polygons
that are arranged as 'base' and 'layer' polygons as shown here:
.Ex
.nf

/* Prepare to draw base polygons */
pfDecal(PFDECAL_BASE_DISPLACE);
 :
/* draw base geometry using OpenGL Performer or graphics library */
 :
/* Prepare to draw first layer polygons */
pfDecal(PFDECAL_LAYER_DISPLACE);
 :
/* draw layer geometry using OpenGL Performer or graphics library */
 :
/* Prepare to draw second layer polygons */
pfDecal(PFDECAL_LAYER_DISPLACE | PFDECAL_LAYER_1);
 :
/* draw layer geometry using OpenGL Performer or graphics library */
 :
/* exit decal mode */
pfDecal(PFDECAL_OFF);
.Ee
.fi
.na

.ne 12
The different \f3\%pfDecal\fP modes offer quality-feature tradeoffs
listed in the table below:

.TS
center tab($);
C | C | C | C
L | L | L | L.
$\f3\%DISPLACE\fP$\f3\%STENCIL\fP$\f3\%(DISPLACE | OFFSET)\fP
_
Quality$medium$high$high
Order$not required$required$not required
Coplanarity$not required$required$not required
Containment$not required$required$not required
.TE

The \f3\%STENCIL\fP mechanism offers the best image quality but at a
performance cost since the base and layer geometry must be rendered in
strict order. When multisampling on RealityEngine, this mechanism
also significantly reduces pixel fill performance. An additional
constraint is that \f3\%STENCIL\fPed layers must be coplanar or decal
geometry may incorrectly show through base geometry. For proper results, each
layer in the "stack" must be completely contained within the boundaries of the
base geometry.

The \f3\%DISPLACE\fP mechanism offers the best performance since layers can
be sorted by graphics state, because the displace call itself is
usually faster than other mode changes, and because there is no pixel
fill rate penalty when it is in use.

The \f3\%DISPLACE\fP mechanism does not require that geometry within a single layer
be coplanar. The main disadvantage is that decal geometry may incorrectly poke
through other geometry due to the displacement of the decal geometry.

The performance differences between \f3\%STENCIL\fP and \f3\%DISPLACE\fP modes
are hardware-dependent so some experimentation and benchmarking is
required to determine the most suitable method for your application.

\f3\%pfDecal\fP is a display-listable command.  If a pfDispList has been
opened by \f3\%pfOpenDList\fP, \f3\%pfDecal\fP will not have immediate effect
but will be captured by the pfDispList and will only have effect when
that pfDispList is later drawn with \f3\%pfDrawDList\fP.

\f3\%pfGetDecal\fP returns the current decal mode.

The decaling mode state element is identified by the \f3\%PFSTATE_DECAL\fP
token.  Use this token with
\f3\%pfGStateMode\fP to set the decaling mode
of a pfGeoState and with \f3\%pfOverride\fP to override subsequent decaling
mode changes.

\f3\%pfApplyDecalPlane\fP will make \f2plane\fP the current decal reference
plane and is a display-listable command.
\f3\%pfGetCurDecalPlane\fP returns the current decal reference plane.

.SH EXAMPLES
Example 1:
.Ex
.nf

/* Set up 'base' pfGeoState */
pfGStateMode(gstate, PFSTATE_DECAL, PFDECAL_BASE);

/* Attach pfGeoState to pfGeoSet */
pfGSetGState(gset, gstate);

/* Draw base pfGeoState */
pfDrawGSet(gset);
.Ee
.fi
.na


Example 2:
.Ex
.nf

/* Override decaling mode to PFDECAL_OFF */
pfDecal(PFDECAL_OFF);
pfOverride(PFSTATE_DECAL, PF_ON);
.Ee
.fi
.na

.SH NOTES

\f3\%PFDECAL_BASE_FAST\fP currently implies \f3\%displacepolygon\fP on machines
that support this feature.  The use of displacements for rendering
coplanar geometry can cause visual artifacts such as decals "Z
fighting" or "flimmering" when viewed perpendicularly and punching
through geometry that should be in front of them when viewed
obliquely.  In these cases, use \f3\%PFDECAL_LAYER_OFFSET\fP,
modify the database should by cutting
away overlapping polygons to eliminate the need for coplanar rendering
or use \f3\%PFDECAL_BASE_HIGH_QUALITY\fP or \f3\%PFDECAL_BASE_STENCIL\fP.

\f3\%PFDECAL_BASE_STENCIL\fP is implemented with stencil planes and
requires the framebuffer to be configured with at least one stencil
bit (see \f3\%stensize(3g)\fP and \f3\%mssize(3g)\fP). The first stencil bit
should be considered as reserved for \f3\%pfDecal\fP.

When using \f3\%PFDECAL_LAYER_OFFSET\fP, the minimum depth buffer range
set with \f3\%lsetdepth\fP must be incremented an extra 1024 * max layers
so the negative displacement of the layers does not wrap.
\f3\%pfInitGfx\fP does this automatically.

The \f3\%PFDECAL_LAYER_DISPLACE\fP decal mode implements a Z-depth bias
that is influenced by several issues including near clipping plane
distance, far clipping plane distance, the ratio of these values,
and the effective depth precision in the Z-buffer. Clipping plane
distances can be inquired from the pfChannel via channel API and
the Z-buffer configuration of the current pfWindow is available
from the \f3\%pfQueryWin\fP function, using the \f3\%PFQWIN_MIN_DEPTH_VAL\fP,
\f3\%PFQWIN_MAX_DEPTH_VAL\fP, and \f3\%PFQWIN_DEPTH_BITS\fP arguments.
.SH SEE ALSO
mssize,
pfDispList,
pfGStateMode,
pfGeoState,
pfOverride,
pfState,
pfInitGfx,
pfQueryWin,
pfChannel,
stencil,
stensize
