'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfEnable" "3pf"
.SH NAME
.upperok
.B pfEnable,
.B pfDisable,
.B pfGetEnable
\- 
Enable and disable graphics modes.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfEnable\fP(int\ mode);
.nf
.fi
.TP
void
\f3\%pfDisable\fP(int\ mode);
.nf
.fi
.TP
int
\f3\%pfGetEnable\fP(int\ mode);
.fi
.SH PARAMETERS
.TP \w'mode\ \ 'u
\f2mode\fP
is a symbolic constant that specifies the enable target
which is to be enabled or disabled.  The enable targets are:
.RS
.TP 30
\f3\%PFEN_LIGHTING\fP
Hardware lighting
.TP
\f3\%PFEN_TEXTURE\fP
Hardware texturing
.TP
\f3\%PFEN_FOG\fP
Hardware fogging
.TP
\f3\%PFEN_WIREFRAME\fP
Wireframe display mode
.TP
\f3\%PFEN_COLORTABLE\fP
Colortable display mode
.TP
\f3\%PFEN_HIGHLIGHTING\fP
Highlight display mode
.TP
\f3\%PFEN_LPOINTSTATE\fP
Light point mode
.TP
\f3\%PFEN_TEXGEN\fP
Automatic texture coordinate generation
.TP
\f3\%PFEN_TEXLOD\fP
Use of texture LOD objects
.TP
\f3\%PFEN_TEXMAT\fP
Use of current texture matrix
.RE

.SH DESCRIPTION

\f3\%pfEnable\fP and \f3\%pfDisable\fP enable and disable various graphics
library and OpenGL Performer modes. By default all modes listed above
are disabled, i.e., when a pfState is first created, its enable
modes are all \f3\%PF_OFF\fP.

\f3\%pfGetEnable\fP returns the enable status of the indicated\f2mode\fP.

.ne 17
Each \f3\%pfEnable\fP/\f3\%pfDisable\fP mode token corresponds to a \f3\%PFSTATE_\fP
token that identifies the state element and is used in pfGeoState routines
and \f3\%pfOverride\fP. The \f3\%PFEN_\fP / \f3\%PFSTATE_\fP correspondence is
illustrated in the following table:

.TS
center tab($) ;
c | c
lB | lB .
Enable Token$State Token
_
PFEN_LIGHTING$PFSTATE_ENLIGHTING
PFEN_TEXTURE$PFSTATE_ENTEXTURE
PFEN_FOG$PFSTATE_ENFOG
PFEN_WIREFRAME$PFSTATE_ENWIREFRAME
PFEN_COLORTABLE$PFSTATE_ENCOLORTABLE
PFEN_HIGHLIGHTING$PFSTATE_ENHIGHLIGHTING
PFEN_LPOINTSTATE$PFSTATE_ENLPOINTSTATE
PFEN_TEXGEN$PFSTATE_ENTEXGEN
PFEN_TEXLOD$PFSTATE_ENTEXLOD
PFEN_TEXMAT$PFSTATE_ENTEXMAT
.TE

Once enabled or disabled, these mode changes will not take effect
until their associated OpenGL Performer state elements are applied.
The following table lists the modes, state objects, and provoking
activation routine for each mode.

.TS
center tab($) ;
c | c | c
lB | l | l .
Enable Mode$Required State$Activation Routine
_
PFEN_LIGHTING$pfLightModel$\f3\%pfApplyLModel\fP
$$pfMaterial$\f3\%pfApplyMtl\fP
$$pfLight$\f3\%pfLightOn\fP
$$normals$\f3\%n3f\fP

PFEN_TEXTURE$pfTexture$\f3\%pfApplyTex\fP
$$pfTexEnv$\f3\%pfApplyTEnv\fP
$$texture coordinates$\f3\%t2f, t3f\fP

PFEN_FOG$pfFog$\f3\%pfApplyFog\fP

PFEN_COLORTABLE$pfColortable$\f3\%pfApplyCtab\fP

PFEN_HIGHLIGHTING$pfHighlight$\f3\%pfApplyHlight\fP

PFEN_LPOINTSTATE$pfLPointState$\f3\%pfApplyLPState\fP

PFEN_TEXGEN$pfTexGen$\f3\%pfApplyTGen\fP

PFEN_TEXLOD$pfTexLOD$\f3\%pfApplyTLOD\fP
PFEN_TEXMAT$pfTexMat$\f3\%pfApplyTMat\fP

PFEN_WIREFRAME$none$none
.TE

Use the \f3\%PFSTATE_\fP tokens with \f3\%pfGStateMode\fP to set the enable modes
of a pfGeoState and with \f3\%pfOverride\fP to override subsequent
enable mode changes:

Example 1:
.Ex
.nf

/* Set up textured pfGeoState */
pfGStateMode(gstate, PFSTATE_ENTEXTURE, PF_ON);
pfGStateAttr(gstate, PFSTATE_TEXTURE, tex);
pfGStateAttr(gstate, PFSTATE_TEXENV, texEnv);

/* Attach gstate to gset */
pfGSetGState(gset, gstate);

/* Draw textured gset */
pfDrawGSet(gset);
.Ee
.fi
.na


Example 2:
.Ex
.nf

/* Override lighting and texturing enable modes to PF_OFF */
pfDisable(PFEN_LIGHTING);
pfDisable(PFEN_TEXTURE);
pfOverride(PFSTATE_ENLIGHTING | PFSTATE_ENTEXTURE, PF_ON);
.Ee
.fi
.na


\f3\%pfEnable\fP and \f3\%pfDisable\fP are display-listable commands.  If a
pfDispList has been opened by \f3\%pfOpenDList\fP, then \f3\%pfEnable\fP and
\f3\%pfDisable\fP will not have immediate effect.  They will be captured
by the pfDispList and will take effect when that pfDispList is later
drawn with \f3\%pfDrawDList\fP.

\f3\%pfBasicState\fP disables all of the above modes.

notes
When lighting is disabled, lmcolor mode is set to \f3\%LMC_COLOR\fP which
effectively turns it off.  Then when enabled, the lmcolor mode is
restored to that of the current front material if there is one.
.SH SEE ALSO
pfDispList,
pfGeoSet,
pfGeoState,
pfOverride,
pfState
