'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfEventView" "3pf"
.SH NAME
.upperok
.B pfInitializeEvents,
.B pfEventSampleOn,
.B pfEventSampleOff,
.B pfWriteEvents,
.B pfResetEvents
\- 
Accumulate internal Performer timing events.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pf.h>
.nf
.fi
.TP
void
\f3\%pfInitializeEvents\fP(void);
.nf
.fi
.TP
void
\f3\%pfEventSampleOn\fP(void);
.nf
.fi
.TP
void
\f3\%pfEventSampleOff\fP(void);
.nf
.fi
.TP
void
\f3\%pfWriteEvents\fP(char\ *filename);
.nf
.fi
.TP
void
\f3\%pfResetEvents\fP(void);
.fi
.SH DESCRIPTION

OpenGL Performer is instrumented to generate internal timing events and write
them into a file upon application program request. Each event has a start time a
nd an end time. Together they bind the execution of important parts of a
Performer frame in the various Perforfmer processes.

The captured event durations are similar to the durations presented in
\f3\%pfFrameStats\fP. However, \f3\%EventView\fP events can be written to a file and
analyzed offline - after the execution of a Performer program completes.
In addition, an application program can design its private set of events and
can merge them into the same output file as the Performer internal events.

OpenGL Performer supplies an analysis program called \f3\%evanalyzer\fP for
generating a graphic display of the captured events.

\f3\%pfInitializeEvents\fP initializes the internal Performer event capture
mechanism. An application must call this function before calling \f3\%pfConfig\fP
in order to initialize the event-capture data structures. All other event
capture functions should be called after \f3\%pfInitializeEvents\fP.

\f3\%pfEventSampleOn\fP starts event capturing. Immediately after this function
is called, all Performer generated timing events will be logged for future
saving into a file. Performer will log a limited number of events (currently
10000 events). No events will be logged after this limit is reached.

\f3\%pfEventSampleOff\fP stops event capturing. No events will be logged after
this call.

\f3\%pfWriteEvents\fP writes out the currently logged events into a disk file
\f2filename\fP. The written events can be later viewed using \f3\%evanalyzer\fP.

\f3\%pfResetEvents\fP resets the list of captured events.

.SS Sample use of pfEventView:
.SS The simplest case.

The simplest way to use pfEventView is to use only Performer-internal events.
In this case, an application program may look like:

.Ex
.nf

main()
{
    pfInit();

    // Initialize event capture.
    pfInitializeEvents();

    pfConfig();

    // Create window & scene graph.
    ....

    // Start event capture
    pfEventSampleOn();

    for (i = 0 ; i < numFrames ; i ++)
    {
        pfSync();
        pfFrame();
    }

    // Stop event capture.
    pfEventSampleOff();

    // Write to file.
    pfWriteEvents("events.timefile");
}
.Ee
.fi
.na


After the program completes, one can analyze the generated events using the
command: \f2evanalyzer events.timefile\fP.

.SS Sample use of pfEventView:
.SS A more complex case.

In a more complex case, an application has some private events that it wishes
to add to the captured event file. We achieve this in two stages:

\f3\%Stage I:\fP
.br
We define application private events in a file. The following example
defines a single event named \f2myDrawCB\fP and assigns a single information slot
of type integer for this event.

.Ex
.nf

file "myEvents.timefile";
max_events 10000;
c_file "myEvents.c";
h_file "myEvents.h";

active true;

conditional_sampling true;

malloc "pfSharedMalloc";

event "myDrawCB"
    {
    info "CB_Info"
	    {
	    name "Something" ;
	    type int ;
	    };
    };
.Ee
.fi
.na


Running \f3\%evpp\fP on above file creates the two files \f2myEvents.c\fP and
\f2myEvents.h\fP. The application should compile and link with the newly
generated file \f2myEvents.c\fP. The file \f2myEvents.h\fP contains macros that
the application uses for controling event capture.

\f3\%Stage II:\fP
.br
In the application code below, we generate an event of type myDrawCB in a DRAW
callback of some node in the scene graph. Every time we generate an event, we
assign a value to its information slot:

.Ex
.nf


int pre_draw_cb(pfTraverser *trav, void *userData)
{
    int		someValue;

    // Mark the start of private event MyDrawCB.
    EVENT_START_MYDRAWCB;

    // Assign value to the information field of this event.
    EVENT_MYDRAWCB_INFO_CB_Info (someValue);

    ... Run callback code.

    // Mark the end of private event MyDrawCB.
    EVENT_END_MYDRAWCB;
}

main()
{
    pfGeode *geode;

    pfInit();

    // Initialize Performer-internal event capture.
    pfInitializeEvents();

    // Initialize application-private event capture.
    EVENT_INIT;

    pfConfig();

    // Create window & scene graph.
    ....

    // Setup DRAW callback on a node.
    pfNodeTravFuncs(geode, PFTRAV_DRAW, pre_draw_cb, NULL);

    // Start event capture on all initialized event sets (Performer-internal
    // and application-private).
    EVENT_SAMPLE_ALL_ON;

    for (i = 0 ; i < numFrames ; i ++)
    {
        pfSync();
        pfFrame();
    }

    // Stop event capture on all initialized event sets..
    EVENT_SAMPLE_ALL_OFF;

    // Write all captured events from all initialized sets to a file.
    // Since we named a file in the evpp input file,
    // we need not specify it here.
    EVENT_WRITE_ALL;
}

.Ee
.fi
.na


As in the first example, after the program completes, one can analyze the
generated events using the command: \f2evanalyzer myEvents.timefile\fP. This time
the file contains Performer-internal and application-private (myDrawCB) events.
Clicking the right-hand mouse button on the line marked myDrawCB can toggle
the display of the information field on this event.

Note that we didn't have to start/stop sampling of Performer-internal events.
The macros \f3\%EVENT_SAMPLE_ALL_ON\fP and \f3\%EVENT_SAMPLE_ALL_OFF\fP start and stop
sampling for all currently initialized event sets.

The manual page for \f3\%evpp\fP contains more information about generating
application-private event types.

.SH NOTES


EventView was originally written by Ran Yakir from BVR Systems.

.SH SEE ALSO
\f3\%evpp\fP,
\f3\%evhist\fP,
\f3\%evanalyzer\fP
