'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfFPConfig" "3pf"
.SH NAME
.upperok
.B pfFPConfig,
.B pfGetFPConfig
\- 
Specify floating-point tolerances and exception handling
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'float\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfFPConfig\fP(int\ which,
'in +2
'll -2
float\ val);
.in
.ll
.nf
.fi
.TP
float
\f3\%pfGetFPConfig\fP(int\ which);
.fi
.SH DESCRIPTION

\f3\%pfFPConfig\fP allows you to set some tolerances used by the OpenGL
Performer math routines.  The \f2which\fP field specifies which floating
point attribute is to be set.  \f2val\fP specifies the value that
attribute should take. Supported values are:
.RS
.TP 5
\f3\%PFFP_UNIT_ROUNDOFF\fP
Specifies the tolerance applied to testing for equality, usually scaled
by the magnitude of the operand.  The default is 1.0e-5.  For
performance reasons, the appropriately scaled tolerance is not used
pervasively.
.TP
\f3\%PFFP_ZERO_THRESH\fP
Specifies how large in magnitude a floating point number can be before
it should be considered non-zero.  The default is 1.0e-15.
.TP
\f3\%PFFP_TRAP_FPES\fP
Enables and disables trapping of floating point exceptions with
\f2value\fPs 1.0 and 0.0, respectively.  Normally, these floating point
errors are handled through kernel exceptions or by the floating point
hardware, and may be nearly invisible to an application except from
performance degradation, sometimes very significant, which they can
cause.  When enabled, pfNotify events are generated for the floating
point exceptions mentioned above and messages displayed or passed to
the user supplied pfNotify handler.
.RE

An application can also turn on floating point exception handling via
the general Performer pfNotify notification mechanism. Calling
\f3\%pfNotifyLevel\fP with the \f3\%PFNFY_FP_DEBUG\fP notification level
configures \f3\%pfFPConfig\fP to enable exceptions.  Alternately, an
appropriate setting of the environment variable \f3\%PFNFYLEVEL\fP will
enable this processing as well. Use of the \f3\%pfNotifyLevel\fP function
is preferable because then the PFNFYLEVEL environment variable can be
used when necessary to override the specification.
.SH BUGS

Enabling floating point exceptions may cause the values returned
from exceptions to be different from the system defaults.  Once a
\f3\%PFNFY_FP_INVALID\fP exception has been reported, all subsequent
exceptions will generate incorrect return values.
.SH SEE ALSO
handle_sigfpes,
pfNotifyLevel
