'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfFeature" "3pf"
.SH NAME
.upperok
.B pfQueryFeature,
.B pfMQueryFeature,
.B pfFeature
\- 
Graphics feature availability routines
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
int
\f3\%pfQueryFeature\fP(int\ which,
'in +2
'll -2
int\ *dst);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfMQueryFeature\fP(int\ *which,
'in +2
'll -2
int\ *dst);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfFeature\fP(int\ which,
'in +2
'll -2
int\ val);
.in
.ll
.fi
.SH DESCRIPTION

OpenGL Performer make runtime determinations regarding the existence
and relative speed of certain graphics features for the
underlying graphics library (OpenGL) on the current
hardware configuration.
These functions provide the ability to both query these results,
and to override the existence of a given feature.
OpenGL Performer makes use of the following useful routines in determining
its information: \f3\%getgdesc(3g)\fP, \f3\%XGetVisualInfo(3X11)\fP,
\f3\%glGetString(3g)\fP, \f3\%glXQueryExtensionsString(3g)\fP, and
\f3\%getinvent(3)\fP.

\f3\%pfQueryFeature\fP takes a \f3\%PFQFTR_\fP token and returns in \f2dst\fP
a token that indicates whether or not that feature exists and
whether the feature is reasonable to use in a real-time application.
The return value is the number of bytes successfully written.
The tokens are documented below and special note is made where
performance is commonly an issue.
These \f3\%pfQueryFeature\fP return values will be one of:
.RS
.TP
\f3\%PFQFTR_FALSE\fP
indicates that the feature is not available on the current hardware configuration.
.TP
\f3\%PFQFTR_TRUE\fP
indicates that the feature is available on the current hardware configuration.
.TP
\f3\%PFQFTR_FAST\fP
indicates that the feature is reasonable for real-time on the current hardware
configuration.
.RE

The \f3\%pfQueryFeature\fP tokens must be one of:
.RS
.TP
\f3\%PFQFTR_VSYNC\fP
queries the status of the graphics video clock.
See the \f3\%pfInitVClock\fP man page for more information.
.TP
\f3\%PFQFTR_VSYNC_SET\fP
queries the write-ability of the graphics video clock.
See the \f3\%pfInitVClock\fP man page for more information.
.TP
\f3\%PFQFTR_GANGDRAW\fP
queries the availability of "gang" swapbuffers where multiple
graphics pipelines may be forced to swap framebuffers simultaneously.
See the \f3\%pfChanShare\fP man page
for more information.
.TP
\f3\%PFQFTR_HYPERPIPE\fP
queries the support for hyperpipe hardware. See the \f3\%pfHyperpipe\fP
man page for more information.
.TP
\f3\%PFQFTR_STEREO_IN_WINDOW\fP
queries the support for doing stereo with multiple buffers in a single window.
See the OpenGL \f3\%glDrawBufferMode(3g)\fP man
pages for more information.
.TP
\f3\%PFQFTR_PBUFFER\fP
queries the support for OpenGL pbuffer drawables for off-screen rendering.
See the OpenGL/X \f3\%glXCreateGLXPbufferSGIX(3g)\fP man
page for more information on pbuffers.
.TP
\f3\%PFQFTR_MULTISAMPLE\fP
queries the support and relative performance of multisampled antialiasing.
See the \f3\%pfAntialias\fP man page for more information.
.TP
\f3\%PFQFTR_MULTISAMPLE_TRANSP\fP
queries the support and relative performance of multisampled transparency.
See the \f3\%pfTransparency\fP man page for more information.
.TP
\f3\%PFQFTR_MULTISAMPLE_ROUND_POINTS\fP
queries the support and relative performance of round multisampled light points.
.TP
\f3\%PFQFTR_MULTISAMPLE_STENCIL\fP
queries the support and relative performance of multisampled stencil.
.TP
\f3\%PFQFTR_COLOR_ABGR\fP
queries the support and relative performance of image data in the
ABGR format.  This format may be slow or unsupported
in some OpenGL implementations.
This is relevant to the OpenGL \f3\%glDrawPixels(3g)\fP command.
.TP
\f3\%PFQFTR_DISPLACE_POLYGON\fP
queries the support for polygon displacement in screen Z used
for doing decals.  See the \f3\%pfDecal\fP man page for
more information.
.TP
\f3\%PFQFTR_POLYMODE\fP
queries the support for polygon fill modes.
See the OpenGL \f3\%glPolygonMode(3g)\fP
man pages for more information.
.TP
\f3\%PFQFTR_TRANSPARENCY\fP
queries the support for and relative performance of transparency.
.TP
\f3\%PFQFTR_AALINES\fP
queries the support for and relative performance of antialiased lines.
.TP
\f3\%PFQFTR_AAPOINTS\fP
queries the support for and relative performance of antialiased points.
.TP
\f3\%PFQFTR_FOG_SPLINE\fP
queries the support for spline fog.
See the \f3\%pfFog\fP man page for more information.
.TP
\f3\%PFQFTR_FOG_LAYERED\fP
queries the support for layered fog using special OpenGL extensions.
.TP
\f3\%PFQFTR_ALPHA_FUNC\fP
queries the support for alpha functions.
See the \f3\%pfAlphaFunc\fP man page and the
OpenGL \f3\%glBlendFunc(3g)\fP
man pages for more information.
.TP
\f3\%PFQFTR_ALPHA_FUNC_COMPARE_REF\fP
queries the support for comparative alpha functions.
See the \f3\%pfAlphaFunc\fP man page for more information.
.TP
\f3\%PFQFTR_BLENDCOLOR\fP
queries the support for specification of a blend color to use
with alpha functions.
Refer to the OpenGL extension
\f3\%glBlendColor(3g)\fP for more information.
.TP
\f3\%PFQFTR_BLEND_FUNC_SUBTRACT\fP
queries the support for additional differencing alpha blending functions.
.TP
\f3\%PFQFTR_BLEND_FUNC_MINMAX\fP
queries the support for additional min/max alpha blending functions.
.TP
\f3\%PFQFTR_GLSPRITE\fP
queries the existence of GL support for sprites (billboards).
This can mean faster billboard drawing and also allows for
correct rendering of billboards in GL display lists.
See the pfSprite and pfBillboard man pages for more information.
.TP
\f3\%PFQFTR_DVR\fP
queries the support for dynamic video resizing, useful for
load management of fill limited applications.
See the \f3\%pfPipeVideoChannel\fP man page for more information.
.TP
\f3\%PFQFTR_TEXTURE\fP
queries the support and relative performance of texture mapping.
.TP
\f3\%PFQFTR_TEXTURE_16BIT_IFMTS\fP
queries the support and relative performance of 16-bit texel formats.
These formats take up less texture memory and can provide a significant
performance improvement at the cost of some loss of image quality.
See the \f3\%pfTexFormat\fP man page for more information.
.TP
\f3\%PFQFTR_TEXTURE_SUBTEXTURE\fP
queries the support for dynamic loading of parts or all of textures
after the texture has been defined.
.TP
\f3\%PFQFTR_TEXTURE_TRILINEAR\fP
queries the support for trilinear MIPmapping for minification
filtering of texture maps.
.TP
\f3\%PFQFTR_TEXTURE_DETAIL\fP
queries the support for detailing of magnified texture maps.
See the \f3\%pfTexFilter\fP and \f3\%pfTexDetail\fP man pages for more information.
.TP
\f3\%PFQFTR_TEXTURE_SHARPEN\fP
queries the support for sharpening of magnified texture maps.
See the \f3\%pfTexFilter\fP man page for more information.
.TP
\f3\%PFQFTR_TEXTURE_3D\fP
queries the support for three-dimensional textures.
.TP
\f3\%PFQFTR_TEXTURE_5551\fP
queries whether graphics hardware supports the 5551 texture format
(PFTEX_UNSIGNED_SHORT_5_5_5_1).
.TP
\f3\%PFQFTR_TEXTURE_PROJECTIVE\fP
queries the support for projected textures (lights). Projected shadows needs also the TEXTURE_SHADOW feature.
.TP
\f3\%PFQFTR_TEXTURE_MINFILTER_BILINEAR_CMP\fP
queries the support for special bilinear LEQUAL/GEQUAL minification filters
for doing real-time shadows.
\f3\%PPFQFTR_TEXTURE_SHADOW\fP
queries the support for shadows texture extension.
.TP
\f3\%PFQFTR_TEXTURE_CLIPMAP\fP
queries the support of clipmapping, or virtual MIPmapping, for the
minification filtering of very large textures.
.TP
\f3\%PFQFTR_TEXTURE_LOD_RANGE\fP
queries the support of texture LOD range control for selecting which
texture levels are to be used.
.TP
\f3\%PFQFTR_TEXTURE_LOD_BIAS\fP
queries the support of texture LOD bias control.
.TP
\f3\%PFQFTR_READ_MSDEPTH_BUFFER\fP
queries the support for reading the multisample depth buffer.
.TP
\f3\%PFQFTR_COPY_MSDEPTH_BUFFER\fP
queries the support for copying to/from the multisample depth buffer.
.TP
\f3\%PFQFTR_READ_TEXTURE_MEMORY\fP
queries the support for reading texture memory.
.TP
\f3\%PFQFTR_COPY_TEXTURE_MEMORY\fP
queries the support for copying to/from texture memory.
.TP
\f3\%PFQFTR_MTL_CMODE\fP
queries the support and speed of material color mode. On most
graphics platforms, this mode yields significant performance
improvements for management of multiple materials.  However, on some older
low-end platforms, it can have additional cost and should not
be used if multiple materials are not in used.
.TP
\f3\%PFQFTR_LIGHT_ATTENUATION\fP
queries the support for light attenuation on the light.
This is OpenGL style light attenuation.
See the OpenGL \f3\%glLightModel(3g)\fP man page for more information.
.TP
\f3\%PFQFTR_LIGHT_CLR_SPECULAR\fP
queries the support for specular color components on lights.
This is only supported in OpenGL.
See the OpenGL \f3\%glLight(3g)\fP man page for more information.
.TP
\f3\%PFQFTR_CALLIGRAPHIC\fP
queries the support for calligraphic support.
This is to know if the Graphic Pipe Z buffer can be used for calligraphics light points,
but it doesn't tell you if a board is connected to the system.
.TP
\f3\%PFQFTR_DECAL_PLANE\fP
queries the support for decal reference planes for use with pfLayers and pfDecal
geometry.
.TP
\f3\%PFQFTR_PIPE_STATS\fP
queries the hardware support for graphics pipeline statistics.
The InfiniteReality graphics subsystem supports time stamps in the graphics pipeline.
.TP
\f3\%PFQFTR_TEXTURE_ANISOTROPIC\fP
queries the hardware support for anisotropic texture filtering.
NVIDIA-based graphics subsystems support anisotropic texture filtering.
.RE

\f3\%pfMQueryFeature\fP takes an NULL-terminated array of query tokens and a destination buffer
and will do multiple queries.
The return value is the number of bytes successfully written.
This routine is more efficient than \f3\%pfQueryFeature\fP
if multiple queries are desired.

\f3\%pfFeature\fP takes a \f3\%PFFTR_\fP token \f2which\fP and a boolean value \f2val\fP and
allows the overriding of OpenGL Performer's determination
of the existence of certain features.  This can force OpenGL Performer
to use, or to stop using, a specific feature.  This is useful for
running on new graphics platforms that may have considerations that OpenGL Performer
did not predict, or for making one machine behave like another for a specific
feature.  Note that if a particular feature is forced on and it happens to
require hardware support that does not exist, the program may not run.
The features that may be set are:
.RS
.TP
\f3\%PFFTR_VSYNC\fP
.TP
\f3\%PFFTR_VSYNC_SET\fP
.TP
\f3\%PFFTR_MULTISAMPLE\fP
.TP
\f3\%PFFTR_MULTISAMPLE_ROUND_POINT\fP
.TP
\f3\%PFFTR_ALPHA_FUNC_ALL\fP
.TP
\f3\%PFFTR_DISPLACE_POLYGON\fP
.TP
\f3\%PFFTR_POLYMODE\fP
.TP
\f3\%PFFTR_FOG_SPLINE\fP
.TP
\f3\%PFFTR_FOG_LAYERED\fP
.TP
\f3\%PFFTR_GANGDRAW\fP
.TP
\f3\%PFFTR_HYPERPIPE\fP
.TP
\f3\%PFFTR_PBUFFER\fP
.TP
\f3\%PFFTR_DVR\fP
.TP
\f3\%PFFTR_FAST_TRANSPARENCY\fP
.TP
\f3\%PFFTR_FAST_AALINES\fP
.TP
\f3\%PFFTR_FAST_AAPOINTS\fP
.TP
\f3\%PFFTR_TEXTURE_FAST\fP
.TP
\f3\%PFFTR_TEXTURE_16BIT_IFMTS\fP
.TP
\f3\%PFFTR_TEXTURE_SUBTEXTURE\fP
.TP
\f3\%PFFTR_TEXTURE_3D\fP
.TP
\f3\%PFFTR_TEXTURE_5551\fP
.TP
\f3\%PFFTR_TEXTURE_DETAIL\fP
.TP
\f3\%PFFTR_TEXTURE_SHARPEN\fP
.TP
\f3\%PFFTR_TEXTURE_OBJECT\fP
.TP
\f3\%PFFTR_TEXTURE_PROJECTIVE\fP
.TP
\f3\%PFFTR_TEXTURE_TRILINEAR\fP
.TP
\f3\%PFFTR_TEXTURE_CLIPMAP\fP
.TP
\f3\%PFFTR_DECAL_PLANE\fP
.TP
\f3\%PFFTR_PIPE_STATS\fP
.RE
.SH NOTES

\f3\%pfQueryWin\fP can be used to query the configuration
parameters of a given pfWindow.  \f3\%pfQuerySys\fP can be used to query the
specific configuration parameters of the current hardware configuration.
.SH SEE ALSO
getgdesc,
XGetVisualInfo,
glGetString,
glXQueryExtensionsString,
getinvent
