'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfInit" "3pf"
.SH NAME
.upperok
.B pfInit,
.B pfIsInited,
.B pfExit
\- 
Initialize and terminate OpenGL Performer processes.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pf.h>
.nf
.fi
.TP
int
\f3\%pfInit\fP(void);
.nf
.fi
.TP
int
\f3\%pfIsInited\fP(void);
.nf
.fi
.TP
void
\f3\%pfExit\fP(void);
.fi
.SH DESCRIPTION

\f3\%pfInit\fP initializes internal OpenGL Performer data structures and must be
the first OpenGL Performer call in an application except for the following:
.IP
pfNotifyLevel

pfSharedArenaSize

pfSharedArenaBase

pfTmpDir
.PP

\f3\%pfInit\fP is required by all Performer applications whether they
use libpf or not.  But \f3\%pfInit\fP has slightly different behavior
applications that only use libpr and do not include pf.h.  for
these applications, \f3\%pfInit\fP does not set up any shared memory
arenas.  If shared memory is required, it should be explicitly set
up by calling \f3\%pfInitArenas\fP before \f3\%pfInit\fP.
\f3\%pfInit\fP returns a value of 1 if successful and -1 if an error is
detected.
\f3\%pfIsInited\fP returns a value of 1 if \f3\%pfInit\fP has successfully
been called and 0 otherwise.

\f3\%pfExit\fP closes graphics windows, frees all OpenGL Performer data
structures, deletes all OpenGL Performer shared memory arenas (see
\f3\%pfGetSharedArena\fP), kills all spawned OpenGL Performer processes,
then returns control to the application.  \f3\%pfExit\fP also turns off the
video retrace clock (see \f3\%pfVClock\fP).  After calling \f3\%pfExit\fP an
application may restart OpenGL Performer with \f3\%pfInit\fP.

User processes forked or sproced after \f3\%pfConfig\fP will be terminated
by \f3\%pfExit\fP.  Those forked or sproced before \f3\%pfConfig\fP will be sent
a \f3\%SIGCLD\fP signal.
.SH NOTES

Since \f3\%pfExit\fP deletes all shared memory arenas, any memory used by
the application that was created out of OpenGL Performer shared memory
is now invalid.
.SH BUGS

Currently \f3\%pfExit\fP returns directly to the operating system,
terminating the simulation application as well.  However, it does turn
off video retrace CPU interrupts while exiting (see \f3\%pfVClock\fP).
.SH SEE ALSO
pfConfig,
pfGetSharedArena,
pfMalloc,
pfVClock
