'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfMemory" "3pf"
.SH NAME
.upperok
.B pfGetMemoryClassType,
.B pfGetType,
.B pfIsOfType,
.B pfIsExactType,
.B pfGetTypeName,
.B pfGetMemory,
.B pfGetData,
.B pfRef,
.B pfUnref,
.B pfUnrefDelete,
.B pfUnrefGetRef,
.B pfGetRef,
.B pfCopy,
.B pfDelete,
.B pfIsFluxed,
.B pfCompare,
.B pfPrint,
.B pfMalloc,
.B pfCalloc,
.B pfRealloc,
.B pfFree,
.B pfGetArena,
.B pfGetSize,
.B pfGetMemoryArenaBytesUsed,
.B pfStrdup
\- 
Reference, copy, delete, print and query pfMemory
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'const\ char\ *\ \ \ 'u
#include <Performer/pf.h>
.nf
.fi
.TP
pfType *
\f3\%pfGetMemoryClassType\fP(void);
.nf
.fi
.TP
pfType *
\f3\%pfGetType\fP(const\ void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfIsOfType\fP(const\ void\ *ptr,
'in +2
'll -2
pfType\ *type);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfIsExactType\fP(const\ void\ *ptr,
'in +2
'll -2
pfType\ *type);
.in
.ll
.nf
.fi
.TP
const char *
\f3\%pfGetTypeName\fP(const\ void\ *ptr);
.nf
.fi
.TP
pfMemory *
\f3\%pfGetMemory\fP(const\ void\ *ptr);
.nf
.fi
.TP
void *
\f3\%pfGetData\fP(const\ void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfRef\fP(void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfUnref\fP(void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfUnrefDelete\fP(void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfUnrefGetRef\fP(void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfGetRef\fP(const\ void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfCopy\fP(void\ *dst,
'in +2
'll -2
void\ *src);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfDelete\fP(void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfIsFluxed\fP(void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfCompare\fP(const\ void\ *ptr1,
'in +2
'll -2
const\ void\ *ptr2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfPrint\fP(const\ void\ *ptr,
'in +2
'll -2
uint\ which,
uint\ verbose,
FILE\ *file);
.in
.ll
.nf
.fi
.TP
void *
\f3\%pfMalloc\fP(size_t\ nbytes,
'in +2
'll -2
void\ *arena);
.in
.ll
.nf
.fi
.TP
void *
\f3\%pfCalloc\fP(size_t\ numelem,
'in +2
'll -2
size_t\ elsize,
void\ *arena);
.in
.ll
.nf
.fi
.TP
void *
\f3\%pfRealloc\fP(void\ *ptr,
'in +2
'll -2
size_t\ nbytes);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfFree\fP(void\ *ptr);
.nf
.fi
.TP
void *
\f3\%pfGetArena\fP(void\ *ptr);
.nf
.fi
.TP
size_t
\f3\%pfGetSize\fP(void\ *ptr);
.nf
.fi
.TP
int
\f3\%pfGetMemoryArenaBytesUsed\fP();
.nf
.fi
.TP
char *
\f3\%pfStrdup\fP(const\ char\ *str,
'in +2
'll -2
void\ *arena);
.in
.ll
.fi
.SH DESCRIPTION


pfMemory is the base class from which all major OpenGL Performer classes
are derived and is also the type used by the OpenGL Performer memory
allocation routines such as \f3\%pfMalloc\fP
and \f3\%pfFree\fP.

Because most OpenGL Performer data structures are derived from pfMemory,
they inherit the functionality of the pfMemory routines described here.
In practice this means you can use the pfMemory routines listed above
with most any OpenGL Performer object, such as pfMaterial, pfList, pfFog,
pfFrustum, pfChannel, pfGroup, pfGeode or with a data pointer returned
by \f3\%pfMalloc\fP.

pfMemory supports the following:
.RS
.TP
1.
Typed data structures.
.TP
2.
Memory arena allocation.
.TP
3.
Memory chunks which know their size.
.TP
4.
Reference counting.
.RE

with only a 4 word overhead.

Although the OpenGL Performer general memory allocation routines
(\f3\%pfMalloc\fP) create pfMemories, they return
void* so the application can treat the allocation as raw data.
Consequently, all routines that would normally take a pfMemory* take a
void* and infer the pfMemory handle so that applications can treat
pfMemory as raw memory. However, one caveat is that routines which
take raw memory such as \f3\%pfGSetAttr\fP or
\f3\%pfFree\fP should not be passed a pointer to
static data since the routines may not be able to successfully infer
the pfMemory handle from the void*.

.Ex
.nf

         ---------------  <------ pfMemory*
        | pfMemory      |
        | header        |
        |               |
         ---------------  <------ void* returned by
        |               |         allocation routines
        | raw data      |
        .               .
        .               .
        .               .
        |               |
         ---------------
.Ee
.fi
.na


Routines which convert between pfMemory* and void* are:
.IP
void* -> pfMemory*: \f3\%pfGetMemory\fP
.IP
pfMemory* -> void*: \f3\%pfGetData\fP
.PP

Note that is it legal to pass either a pfMemory* or a void* to
those routines which are prototyped as accepting a void*, e.g.,
\f3\%pfRef\fP. In this way, a single set of routines
supports the same feature set including reference counts, copy,
and delete for pfMemories used as OpenGL Performer data types
like pfGeoSet as well as for pfMemories used as raw data like
pfGeoSet attribute arrays.

\f3\%pfGetMemoryClassType\fP returns the
\f3\%pfType\fP* for the class \f3\%pfMemory\fP.  The \f3\%pfType\fP* returned
by \f3\%pfGetMemoryClassType\fP is the
same as the \f3\%pfType\fP* returned by invoking \f3\%pfGetType\fP on any instance of class
\f3\%pfMemory\fP.  Because OpenGL Performer allows subclassing of
built-in types, when decisions are made based on the type of an
object, it is usually better to use \f3\%pfIsOfType\fP to test if an object is of a type derived from
a Performer type rather than to test for strict equality of the
\f3\%pfType\fP*'s.

All objects derived from pfMemory have a type identifier (pfType*)
that is returned by \f3\%pfGetType\fP

Example 1: API sharing.
.Ex
.nf

dcs = pfNewDCS();

/* pfDCS uses pfGroup routine */
pfAddChild(dcs, geode);

/* pfDCS uses pfNode routine */
pfNodeTravMask(dcs, PFTRAV_ISECT, DCS_MASK, PFTRAV_SELF | PFTRAV_DESCEND, PF_SET);
.Ee
.fi
.na


Each data type derived from pfMemory has an associated routine for
getting a pointer to its corresponding pfType, e.g.
\f3\%pfGetDCSClassType()\fP returns the
pfType* corresponding to the pfDCS class.  The exact type of an object
is tested by comparing its pfType* to that returned by one of these
\f3\%pfGet<*>ClassType\fP routines or with the
\f3\%pfIsExactType\fP test, e.g.

.Ex
.nf

	if (pfGetType(obj) == pfGetGroupClassType()) ...

	if (pfIsExactType(obj, pfGetGroupClassType())) ...

.Ee
.fi
.na


But since OpenGL Performer allows subclassing and the creation of new
types in C++, it's more often desirable to know whether a particular
object is of a type derived from a particular type defined by OpenGL
Performer.  In particular, exact type tests makes application code
more likely to fail on scene graphs produced by database loaders that
use subclassing.  \f3\%pfIsOfType\fP performs
this test and returns TRUE if \f2mem\fP's type is
derived from \f2type\fP:
.Ex
.nf

	if (pfIsOfType(obj, pfGetGroupClassType())) ...
.Ee
.fi
.na

If 'obj' is a pfDCS, then the above conditional would evaluate TRUE since
pfDCS is derived from pfGroup.

\f3\%pfGetTypeName\fP returns a string that
identifies the type of \f2mem\fP.  For example, if
\f2mem\fP is a pfDCS, the string returned is "pfDCS".

All pfMemories have a reference count which indicates how many times
the pfMemory is referenced, either by other pfMemories or by the
application. Reference counts are crucial for many database
operations, particularly deletion, since it is highly dangerous to
delete a pfMemory which is still being used, i.e., its reference count
is greater than 0.

Reference counts may be incremented and decremented by
\f3\%pfRef\fP and
\f3\%pfUnref\fP respectively.
\f3\%pfGetRef\fP returns the reference count of
\f2mem\fP.
\f3\%pfUnrefGetRef\fP
decrements the reference count and returns the new value,
as an atomic operation, i.e. it is guaranteed that no other
process can change the value or free the memory between the
decrement and the time the value is read.
\f3\%pfUnrefDelete\fP will decrement the
reference count of \f2mem\fP and delete it if the
count is <= 0.  Thus it is equivalent to calling
\f3\%pfUnref\fP followed by
\f3\%pfDelete\fP, as an atomic operation.

\f3\%pfDelete\fP frees the memory associated
with \f2mem\fP if its reference count is <= 0.  When
an object is freed, it decrements the reference count of all
pfMemories that it once referenced and will delete any of these
pfMemories with reference counts that are <= 0.  Thus,
\f3\%pfDelete\fP will follow all reference
chains until it encounters a pfMemory which it cannot delete. Note
that the reference count of a pfNode is incremented each time it is
added as a child to a pfGroup. Thus, a pfNode must be removed from all
its parents before it can be deleted.

When multiprocessing in a \f3\%libpf\fP application,
pfNodes should be \f3\%pfDelete\fPed only in the APP or DBASE processes
as should \f3\%libpr\fP objects that are referenced directly or indirectly
by pfNodes, like pfGeoSets and pfGeoStates. If you wish to delete
objects in processes other than the APP or DBASE, use \f3\%pfAsyncDelete\fP.

Example 2: Deletion

.Ex
.nf

pfMaterial	*mtl;
pfTexture	*tex;
pfGeoState	*brickStyle, *woodStyle;
pfGeoSet	*brickWall, *woodWall;

mtl = pfNewMtl(arena);

brickStyle = pfNewGState(arena);
tex = pfNewTex(arena);
pfLoadTexFile(tex, "brick.rgb");
pfGStateAttr(brickStyle, PFSTATE_TEXTURE, tex);
pfGStateAttr(brickStyle, PFSTATE_FRONTMTL, mtl);

woodStyle = pfNewGState(arena);
tex = pfNewTex(arena);
pfLoadTexFile(tex, "wood.rgb");
pfGStateAttr(woodStyle, PFSTATE_TEXTURE, tex);
pfGStateAttr(woodStyle, PFSTATE_FRONTMTL, mtl);

brickWall = pfNewGSet(arena);
pfGSetGState(brickWall, brickStyle);
pfGSetAttr(brickWall, PFGS_COORD3, PFGS_PER_VERTEX, coords);

woodWall = pfNewGSet(arena);
pfGSetAttr(woodWall, PFGS_COORD3, PFGS_PER_VERTEX, coords);
pfGSetGState(woodWall, woodStyle);

pfDelete(woodWall);

/* At this point woodWall, woodStyle, and the wood texture
 * have been deleted. coords and mtl have not been deleted
 * since they are referenced by brickWall and brickStyle respectively.
*/

.Ee
.fi
.na


\f3\%pfDelete\fP returns the following:

.RS
.TS
tab($) ;
c l .
FALSE$\f2mem\fP was not deleted
TRUE$\f2mem\fP was deleted
-1$\f2mem\fP is not a pfMemory
.TE
.RE

\f3\%pfDelete\fP is implemented for all OpenGL Performer objects except the
following:

.RS
pfPipe
.br
pfChannel
.br
pfEarthSky
.br
pfBuffer
.br
pfPipeWindow
.br
pfTraverser
.br
pfState
.br
pfDataPool
.RE

\f3\%pfIsFluxed\fP returns TRUE if a pfMemory
is actually the data portion of a pfFluxMemory that is part of a pfFlux.
It returns FALSE otherwise.  Calling
\f3\%pfRef\fP,
\f3\%pfUnref\fP,
\f3\%pfUnrefDelete\fP,
\f3\%pfUnrefGetRef\fP,
\f3\%pfGetRef\fP, or
\f3\%pfDelete\fP
on a Fluxed pfMemory, will actually call the function on the parent pfFlux.

\f3\%pfCopy\fP copies \f2src\fP into \f2dst\fP.
\f3\%pfCopy\fP is not recursive - it does not
follow reference chains but instead copies only the first- level
references. The reference counts of objects newly referenced by
\f2dst\fP are incremented by one while those counts of objects
previously reverenced by \f2dst\fP are decremented by one. Objects whose
reference counts reach 0 during \f3\%pfCopy\fP are
\f2not\fP deleted.  A value of \f3\%TRUE\fP will be returned if the copy
successfully completes and \f3\%FALSE\fP will be returned if an
error is encountered.  If the destination object is a
growable object (such as a \f3\%pfList\fP) then it will automatically
grow as needed and as memory permits to hold the copy of the source object.
If the destination object is not a growable
object and the \f2src\fP object is larger than
the destination object, no copy will be done and a value of \f3\%FALSE\fP
will be returned.

.ne 6
\f3\%pfCopy\fP is currently not implemented for any libpf data structures
and is not implemented for the following libpr data structures:

.RS
pfState
.br
pfDataPool
.RE

\f3\%pfCompare\fP compares the object \f2p1\fP
to the object \f2p2\fP.  If the objects are identical, a value of 0 will
be returned.  Upon the first difference, a value of (1) will be
returned if \f2p1\fP had a greater value and a value of (-1) will be
returned if \f2p2\fP had a greater value.
\f3\%pfCompare\fP is recursive on known defined
objects and their sub objects but is not recursive on user data or
generic buffers or lists that hold undefined data.
Derived classes first compare their parent classes.  If objects
of different types are compared, they will be found to be different
and the return value will be based on a comparison of the \f2p1\fP
and \f2p2\fP pointer values.

.ne 13
\f3\%pfPrint\fP Prints information to a file about
the specified pfMemory \f2mem\fP.  The \f2file\fP
argument specifies the file.
If \f2file\fP is \f3\%NULL\fP, the listing is
printed to stderr.  \f3\%pfPrint\fP takes a
verbosity indicator, \f2verbose\fP.  Valid selections in order of
increasing verbosity are:

.RS
.TS
tab($) ;
l l .
\f3\%PFPRINT_VB_OFF\fP$no printing
\f3\%PFPRINT_VB_ON\fP$minimal printing (default)
\f3\%PFPRINT_VB_NOTICE\fP$minimal printing (default)
\f3\%PFPRINT_VB_INFO\fP$considerable printing
\f3\%PFPRINT_VB_DEBUG\fP$exhaustive printing
.TE
.RE

If \f2mem\fP is a type of pfNode, then \f2which\fP specifies whether the
print traversal should only traverse the current node
(\f3\%PFTRAV_SELF\fP) or print out the entire scene graph rooted by node
\f2mem\fP by traversing \f2node\fP and its descendents in the graph
(\f3\%PFTRAV_SELF\fP | \f3\%PFTRAV_DESCEND\fP).  If \f2mem\fP is a pfFrameStats,
then \f2which\fP specifies a bitmask of frame statistics classes that
should be printed.  If \f2mem\fP is a pfGeoSet, then \f2which\fP is
ignored and information about that pfGeoSet is printed according to
the verbosity indicator.  The output contains the types, names and
bounds of the nodes and pfGeoSets in the hierarchy.  This routine is
provided for debugging purposes only and the content and format may
change in future releases.

Example 3: Print entire contents of a pfGeoSet, \f2gset\fP, to stderr.
.Ex
.nf

pfPrint(gset, NULL, PFPRINT_VB_DEBUG, NULL);
.Ee
.fi
.na


Example 4: Print entire scene graph under node to a file \f2file\fP with
default verbosity.
.Ex
.nf

file = fopen ("scene.out","w");
pfPrint(scene, PFTRAV_SELF | PFTRAV_DESCEND, PFPRINT_VB_ON, fp);
fclose(file);
.Ee
.fi
.na


Example 5: Print select classes of a pfFrameStats structure, \f2stats\fP,
to stderr.
.Ex
.nf

pfPrint(stats, PFSTATS_ENGFX | PFFSTATS_ENDB | PFFSTATS_ENCULL, PFSTATS_ON, NULL);
.Ee
.fi
.na


pfMalloc and the related routines provide a
consistent method to allocate memory, either from the user's heap
(using the C-library \f3\%malloc\fP function) or from a shared memory
arena (using the IRIX \f3\%amalloc\fP function).  In addition, these
routines provide a reference counting mechanism used by OpenGL Performer
to efficiently manage memory.

\f3\%pfMalloc\fP operates identically to the
C-library \f3\%malloc\fP function, except that a shared memory arena may
be specified to allocate the memory from.  If \f2arena\fP is NULL,
memory is allocated from the heap, otherwise memory is allocated from
\f2arena\fP which must be a previously configured shared memory arena
(see pfSharedMem).  

Shared memory arenas can be created using \f3\%acreate\fP and can be found
by using \f3\%pfGetSharedArena\fP.

\f3\%pfCalloc\fP and
\f3\%pfRealloc\fP function just as their Unix
counterparts, except that they may use shared arenas.

In all cases, a pointer to the allocated memory block is returned or
NULL if there is not enough available memory.

The data pointer returned by \f3\%pfMalloc\fP,
\f3\%pfCalloc\fP, and
\f3\%pfRealloc\fP is actually part of a pfMemory
object that, among other things, provides a reference count.
Reference counts are used to keep track of how many times each
allocated block of memory is referenced or instanced.  All OpenGL
Performer libpr objects (pfMemory) are created with
\f3\%pfMalloc\fP so their reference counts are
updated by appropriate libpr routines. Examples of references
follow:

Example 6:
.Ex
.nf

tex = pfNewTex(NULL);

/* Attach 'tex' to gstate0 and gstate1 */
pfGStateAttr(gstate0, PFSTATE_TEXTURE, tex);
pfGStateAttr(gstate1, PFSTATE_TEXTURE, tex);

/* The reference count of 'tex' is now 2 */

/* Remove 'tex' from gstate1 */
pfGStateAttr(gstate1, PFSTATE_TEXTURE, NULL);

/* The reference count of 'tex' is now 1 */
.Ee
.fi
.na


Example 7:
.Ex
.nf

coords = (pfVec3*) pfMalloc(sizeof(pfVec3) * numVerts, arena);

/* Attach 'coords' to non-indexed pfGeoSet, 'gset' */
pfGSetAttr(gset, PFGS_COORD3, PFGS_PER_VERTEX, coords, NULL);

/* The reference count of 'coords' is now 1 */
.Ee
.fi
.na


.ne 7
Example 8:
.Ex
.nf

/* Attach 'gstate0' to 'gset' */
pfGSetGState(gset, gstate0);

/* The reference count of 'gstate0' is now incremented by 1 */
.Ee
.fi
.na


\f3\%pfFree\fP frees the memory associated with
\f2ptr\fP.  It is an error to \f3\%pfFree\fP memory
that was not allocated by \f3\%pfMalloc\fP,
\f3\%pfCalloc\fP, or \f3\%pfRealloc\fP.  It is also
an error to use any method other than
\f3\%pfFree\fP or
\f3\%pfDelete\fP to free memory allocated by
\f3\%pfMalloc\fP,
\f3\%pfCalloc\fP, or
\f3\%pfRealloc\fP.

\f3\%pfFree\fP does not honor the reference count
of \f2ptr\fP.  This means that you can free a chunk of memory that is
still being used (which means that its reference count is > 0) with
potentially disastrous results. Typical failure modes are in the form
of mysterious memory corruption and segmentation violations.

\f3\%pfDelete\fP, however, does honor the
reference count of \f2ptr\fP and will not delete any memory whose
reference count is > 0.  \f3\%pfDelete\fP
returns -1 if \f2ptr\fP is not a pfMalloc pointer, TRUE if \f2ptr\fP was
deleted, and FALSE otherwise.
\f3\%pfDelete\fP is recommended if you are
not sure of the reference count of a piece of memory.  See the
pfObject reference page for more details on
\f3\%pfDelete\fP.

\f3\%pfGetArena\fP returns the arena
pointer which \f2ptr\fP was allocated from or NULL if \f2ptr\fP was
allocated from the process heap.

\f3\%pfGetSize\fP returns the size in
bytes of the memory referenced by \f2ptr\fP or 0 if \f2ptr\fP is not a
\f3\%pfMalloc\fP pointer.

\f3\%pfGetMemoryArenaBytesUsed\fP returns the
number of bytes currently allocated from the shared arena.  It takes into
account all the Performer calls that allocate or free shared arena memory
(\f3\%pfMalloc\fP, \f3\%pfCalloc\fP, \f3\%pfRealloc\fP, \f3\%pfDelete\fP, \f3\%pfFree\fP, new on
any Performer class, etc).  This call is only available in the \f2DEBUG\fP
libraries.  It is meant for debugging memory leaks and therefore is not
necessary in the optimized libraries. If called with optimized libraries, it
returns (-1). This call makes it easier to distinguish between memory leaks and
memory fragmentation: If the arena grows but the number of bytes used doesn't
then the program in question causes memory fragmentation but doesn't leak.

\f3\%pfStrdup\fP duplicates the NULL-terminated string \f2str\fP by allocating
storage in the shared memory arena defined by the \f2arena\fP argument.

.SH BUGS

\f3\%pfPrint\fP is not yet implemented for pfGeoStates and other state
structures, and is not implemented for pfPaths or pfLists.
.SH SEE ALSO
acreate,
calloc,
free,
malloc,
realloc,
pfInitArenas
