'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfNotify" "3pf"
.SH NAME
.upperok
.B pfNotify,
.B pfNotifyLevel,
.B pfGetNotifyLevel,
.B pfNotifyHandler,
.B pfGetNotifyHandler,
.B pfDefaultNotifyHandler,
.B pfNotifyLock,
.B pfNotifyUnlock
\- 
Control error handling, signal errors or log messages
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfNotifyFuncType\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfNotify\fP(int\ severity,
'in +2
'll -2
int\ error,
char\ *format,
...);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfNotifyLevel\fP(int\ severity);
.nf
.fi
.TP
int
\f3\%pfGetNotifyLevel\fP(void);
.nf
.fi
.TP
void
\f3\%pfNotifyHandler\fP(pfNotifyFuncType\ handler);
.nf
.fi
.TP
pfNotifyFuncType
\f3\%pfGetNotifyHandler\fP(void);
.nf
.fi
.TP
void
\f3\%pfDefaultNotifyHandler\fP(pfNotifyData\ *data);
.nf
.fi
.TP
void
\f3\%pfNotifyLock\fP(void);
.nf
.fi
.TP
void
\f3\%pfNotifyUnlock\fP(void);
.fi

.Ex
.nf

typedef struct
{
    int severity;
    int pferrno;
    char *emsg;
} pfNotifyData;

typedef void (*pfNotifyFuncType)(pfNotifyData*);
.Ee
.fi
.na

.SH DESCRIPTION

These functions provide a general purpose error message and
notification handling facility for applications using OpenGL
Performer.  This facility is used internally by OpenGL Performer
for error, warning, and status notifications and can be used by
user developed programs as well.

\f3\%pfNotifyHandler\fP sets \f2handler\fP as the user error handling
routine.  All errors, warnings and notices will call \f2handler\fP with
a pointer to a pfNotifyData structure that describes the error or
message.  The default notification handler \f3\%pfDefaultNotifyHandler\fP
prints out a message to stderr of the form:
.Ex
.nf

<PID> PF <LEVEL>/<PFERROR>(<ERRNO>)	<MESSAGE>
.Ee
.fi
.na

where PID is the process id of the process that encountered the error
(PID is only emitted when the notification threshold is >= PFNFY_DEBUG),
LEVEL is a string indicating the severity of the error, PFERROR is
the type of error detected, ERRNO is
the value of the system global \f3\%errno\fP (see perror(3C)), and MESSAGE is
the formatted error message given \f3\%pfNotify\fP.
The default handler
zeros the system global errno.
If PFERROR is \f3\%PFNFY_MORE\fP, the message is considered to be a continuation
of the previous message and the print format is:
.Ex
.nf

PF			<MESSAGE>
.Ee
.fi
.na



The companion function \f3\%pfGetNotifyHandler\fP returns the address of
the installed handler function.  It is possible to inquire this address
and provide it to user installed handlers in order to chain multiple
notification handlers to any desired level.

The functions \f3\%pfNotifyLock\fP and \f3\%pfNotifyUnlock\fP provide access
to the lock used by \f3\%pfDefaultNotifyHandler\fP.
This can be used to atomically print messages to stderr
that won't get scrambled with the output from \f3\%pfDefaultNotifyHandler\fP
(or other routines that use this lock) in other processes.

\f3\%pfNotifyLevel\fP sets the threshold for notification.  A notification
must have a level less than or equal to the threshold for the default
handler to print a message.  The notification handler itself is
invoked regardless of the notification level.  The levels are in
decreasing severity:
.RS
\f3\%PFNFY_ALWAYS\fP
.br
\f3\%PFNFY_FATAL\fP
.br
\f3\%PFNFY_WARN\fP
.br
\f3\%PFNFY_NOTICE\fP
.br
\f3\%PFNFY_INFO\fP
.br
\f3\%PFNFY_DEBUG\fP
.br
\f3\%PFNFY_FP_DEBUG\fP.
.RE

The default notification threshold is \f3\%PFNFY_NOTICE\fP.

Call \f3\%pfGetNotifyLevel\fP to query the current notification level.
The meaning of these notification levels is as follows:

.TS
center tab($) ;
c | c
l | l .
Error Level$Description
_
\f3\%PFNFY_ALWAYS\fP$Always print regardless of notify level
\f3\%PFNFY_FATAL\fP$Fatal error, the dying gasp of a doomed process
\f3\%PFNFY_WARN\fP$Serious warning, rarely used for frame-time errors
\f3\%PFNFY_NOTICE\fP$Warning, may be used for frame time errors
\f3\%PFNFY_INFO\fP$Information on progress as well as errors
\f3\%PFNFY_DEBUG\fP$Debug information of significant verbosity
\f3\%PFNFY_FP_DEBUG\fP$Debug information and floating point exceptions
.TE

Setting the notification level to PFNFY_FP_DEBUG also enables floating
point exceptions for overflow, underflow and invalid operations.
Normally, these floating point errors are handled through kernel
exceptions or by the floating point hardware, and may be nearly
invisible to an application except from the performance degradation,
sometimes very significant, which they can cause.  When enabled,
pfNotify events are generated for the floating point exceptions
mentioned above and messages displayed or passed to the user supplied
pfNotify handler.

The environment variable PFNFYLEVEL can be set to override the
value specified in pfNotifyLevel.  If the notification level is set
via PFNFYLEVEL it can not be changed by an application.

A notification level of PFNFY_FATAL causes the program to exit after
notification; less severe levels do not.

\f3\%pfNotify\fP generates an error message.  \f2severity\fP must be one of
the above listed constants.  \f2error\fP may be any integer value,
however, OpenGL Performer uses the following values internally:
.RS
\f3\%PFNFY_USAGE\fP
.br
\f3\%PFNFY_RESOURCE\fP
.br
\f3\%PFNFY_SYSERR\fP
.br
\f3\%PFNFY_ASSERT\fP
.br
\f3\%PFNFY_PRINT\fP
.br
\f3\%PFNFY_INTERNAL\fP
.br
\f3\%PFNFY_FP_OVERFLOW\fP
.br
\f3\%PFNFY_FP_DIVZERO\fP
.br
\f3\%PFNFY_FP_INVALID\fP
.br
\f3\%PFNFY_FP_UNDERFLOW\fP
.br
\f3\%PFNFY_MORE\fP
does a continuation of the previous message.
.RE

The severity must be less than or equal to the severity set in
\f3\%pfNotifyLevel\fP for the error message to be output.
.SH NOTES

Notification level is managed on a per process basis.  Processes
forked off after \f3\%pfNotifyLevel\fP is called inherit the specified
level.
.SH BUGS

Enabling floating point exceptions may cause the values returned from
exceptions to be different than the system defaults.  After an
\f3\%_INVALID\fP operation, all subsequent exceptions will generate
incorrect return values.
.SH SEE ALSO
errno,
handle_sigfpes,
perror
