'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfPlane" "3pf"
.SH NAME
.upperok
.B pfMakePtsPlane,
.B pfMakeNormPtPlane,
.B pfDisplacePlane,
.B pfClosestPtOnPlane,
.B pfPlaneIsectSeg,
.B pfHalfSpaceIsectSeg,
.B pfHalfSpaceContainsPt,
.B pfHalfSpaceContainsBox,
.B pfHalfSpaceContainsSphere,
.B pfHalfSpaceContainsCyl,
.B pfOrthoXformPlane
\- 
Set and operate on planes
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfMakePtsPlane\fP(pfPlane\ *dst,
'in +2
'll -2
const\ pfVec3\ pt1,
const\ pfVec3\ pt2,
const\ pfVec3\ pt3);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfMakeNormPtPlane\fP(pfPlane\ *dst,
'in +2
'll -2
const\ pfVec3\ norm,
const\ pfVec3\ pos);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfDisplacePlane\fP(pfPlane\ *dst,
'in +2
'll -2
float\ d);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfClosestPtOnPlane\fP(const\ pfPlane\ *pln,
'in +2
'll -2
const\ pfVec3\ pt,
pfVec3\ dst);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfPlaneIsectSeg\fP(const\ pfPlane\ *pln,
'in +2
'll -2
const\ pfSeg\ *seg,
float\ *d);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfHalfSpaceIsectSeg\fP(const\ pfPlane*\ hs,
'in +2
'll -2
const\ pfSeg*\ seg,
float*\ d1,
float*\ d2);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfHalfSpaceContainsPt\fP(const\ pfPlane*\ p,
'in +2
'll -2
const\ pfVec3\ pt);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfHalfSpaceContainsBox\fP(const\ pfPlane*\ p,
'in +2
'll -2
const\ pfBox\ *box);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfHalfSpaceContainsSphere\fP(const\ pfPlane\ *hs,
'in +2
'll -2
const\ pfSphere\ *sph);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfHalfSpaceContainsCyl\fP(const\ pfPlane\ *hs,
'in +2
'll -2
const\ pfCylinder\ *cyl);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfOrthoXformPlane\fP(pfPlane\ *dst,
'in +2
'll -2
const\ pfPlane\ *pln,
const\ pfMatrix\ xform);
.in
.ll
.fi

.Ex
.nf

typedef struct
{
    pfVec3      normal;
    float       offset;
} pfPlane;
.Ee
.fi
.na

.SH DESCRIPTION

A pfPlane represents an infinite 2D plane as a normal and a distance
offset from the origin in the normal direction.  A point on the plane
satisfies the equation \f2normal\fP dot (x, y, z) = offset.  pfPlane is
a public struct whose data members \f2normal\fP and \f2offset\fP may be
operated on directly.


\f3\%pfMakePtsPlane\fP sets \f2dst\fP to the plane which passes through the
three points \f2pt1\fP, \f2pt2\fP and \f2pt3\fP.

\f3\%pfMakeNormPtPlane\fP sets \f2dst\fP to the plane which passes through
the point \f2pt\fP with normal \f2norm\fP.

\f3\%pfDisplacePlane\fP moves the plane \f2dst\fP by a distance \f2d\fP in the
direction of the plane normal.

\f3\%pfClosestPtOnPlane\fP
returns in \f2dst\fP
the closest point to \f2pt\fP
which lies in the plane \f2pln\fP.  The line segment connecting \f2pt\fP and
\f2dst\fP is perpendicular to \f2pln\fP.

\f3\%pfHalfSpaceContainsPt\fP
returns \f3\%TRUE\fP or \f3\%FALSE\fP depending on whether the point
\f2pt\fP is in the interior of the specified half-space.
The half-space is defined with plane normal pointing to the exterior.

\f3\%pfHalfSpaceContainsSphere\fP, \f3\%pfHalfSpaceContainsBox\fP and \f3\%pfHalfSpaceContainsCyl\fP test whether the half space specified by pfPlane
contains a non-empty portion of the volume specified by the
second argument, a sphere, box or cylinder, respectively.

The return value from the these functions is the OR of one or
more bit fields.  The returned value may be:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
The intersection of the primitive
and the half space is empty.
.TP
\f3\%PFIS_MAYBE\fP:
The intersection of the primitive and the
half space might be non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP:
The intersection of the primitive and the
half space is definitely non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP:
The primitive
is non-empty and lies entirely inside the half space.
.RE

indicating indicate that the second argument is entirely
outside, potentially partly inside, partially inside or entirely
inside the half space specified by the pfPlane.

\f3\%pfPlaneIsectSeg\fP tests the line segment \f2seg\fP for intersection
with the half space \f2pln\fP.
The possible test results are:
.RS
.TP 5
\f3\%PFIS_FALSE\fP:
\f2seg\fP lies entirely in the exterior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_START_IN\fP:
The starting point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_END_IN\fP:
The ending point of \f2seg\fP lies in the interior.
.RE

If \f2d\fP is non-NULL, on return it contains the position along the
line segment (0 <= d <= seg->length) at which the intersection occurred.

\f3\%pfHalfSpaceIsectSeg\fP intersects the line
segment \f2seg\fP with the half space \f2hs\fP and has return values the same as
\f3\%pfPlaneIsectSeg\fP
except that it also returns a non-zero value when both points are
inside the half-space.  In this case it returns:
.RS
.TP 5
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP | \f3\%PFIS_START_IN\fP | \f3\%PFIS_END_IN\fP:
Both end points of \f2seg\fP lie in the interior.
.RE

If \f2d1\fP and \f2d2\fP are non-NULL, on return from
\f3\%pfHalfSpaceIsectSeg\fP they contain the
starting and ending positions of the line segment (0 <= d1 <= d2 <=
seg->length) intersected with the half space.

\f3\%pfOrthoXformPlane\fP sets \f2dst\fP
to the plane as transformed by the orthogonal transformation \f2xform\fP;
\f2dst\fP = \f2pln\fP * \f2xform\fP.  If \f2xform\fP is not an orthogonal transformation the
results are undefined.
.SH NOTES

The bit fields returned by the contains functions are structured so
that bitwise AND-ing the results of sequential tests can be used to
compute composite results, e.g. testing exclusion against a number
of half spaces.

.SH SEE ALSO
pfBox,
pfMatrix,
pfSeg,
pfSphere,
pfVec3
