'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfSeg" "3pf"
.SH NAME
.upperok
.B pfClipSeg,
.B pfMakePtsSeg,
.B pfMakePolarSeg,
.B pfClosestPtsOnSeg,
.B pfTriIsectSeg
\- 
Set and operate on line segments
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfClipSeg\fP(pfSeg\ *dst,
'in +2
'll -2
const\ pfSeg\ *seg,
float\ d1,
float\ d2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfMakePtsSeg\fP(pfSeg\ *dst,
'in +2
'll -2
const\ pfVec3\ p1,
const\ pfVec3\ p2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfMakePolarSeg\fP(pfSeg\ *dst,
'in +2
'll -2
const\ pfVec3\ pos,
float\ azi,
float\ elev,
float\ len);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfClosestPtsOnSeg\fP(const\ pfSeg\ *seg1,
'in +2
'll -2
const\ pfSeg\ *seg2,
pfVec3\ ptOn1,
pfVec3\ ptOn2);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfTriIsectSeg\fP(const\ pfVec3\ v1,
'in +2
'll -2
const\ pfVec3\ v2,
const\ pfVec3\ v3,
const\ pfSeg\ *seg,
float\ *d);
.in
.ll
.fi

.Ex
.nf

typedef struct
{
    pfVec3      pos;
    pfVec3      dir;
    float       length;
} pfSeg;
.Ee
.fi
.na

.SH DESCRIPTION


A \f3\%pfSeg\fP represents a line segment starting at \f2pos\fP, extending
for a length \f2length\fP in the direction \f2dir\fP.  The routines assume
that \f2dir\fP is of unit length, otherwise the results are undefined.
pfSeg is a public struct whose data members \f2pos\fP, \f2dir\fP and
\f2length\fP may be operated on directly.


\f3\%pfClipSeg\fP is used to select a subset of the
input segment \f2seg\fP.  It sets \f2dst\fP to the portion
of segment \f2seg\fP clipped to start at distance
\f2d1\fP and end at \f2d2\fP.  When \f2d1\fP = 0 and \f2d2\fP =
\f2seg\fP->length, \f3\%pfClipSeg\fP returns the
original segment.  Values of \f2d1\fP < 0 and \f2d2\fP > \f2seg\fP->length
can be used to extend the segment.

\f3\%pfMakePtsSeg\fP sets \f2dst\fP to the segment which starts at the point
\f2p1\fP and ends at the point \f2p2\fP.

\f3\%pfMakePolarSeg\fP sets \f2dst\fP to the segment which starts at \f2pos\fP
and has length \f2length\fP and points in the direction specified by
\f2azi\fP and \f2elev\fP.  \f2azi\fP specifies the azimuth (or heading),
which is the angle which the projection of the segment in the X-Y plane
makes with the +Y axis.  \f2elev\fP specifies the elevation (or pitch),
the angle with respect to the X-Y plane.  The positive Y axis is
\f2azi\fP=0 and \f2elev\fP=0.  Azimuth follows the right hand rule about
the Z axis, i.e. +90 degrees is the -X axis.  Similarly, elevation
follows the right hand rule about the X axis, i.e. +90 degrees is the
+Z axis.

\f3\%pfClosestPtsOnSeg\fP returns the two closest
points on the
two segments \f2seg1\fP
and \f2seg2\fP.  If the two segments are parallel FALSE
is returned and the contents of ptOn1 and ptOn2 are undefined.

\f3\%pfTriIsectSeg\fP tests the line segment \f2seg\fP for intersection with
the triangle defined by the three vertices \f2v1\fP, \f2v2\fP, and \f2v3\fP.
\f3\%pfTriIsectSeg\fP returns TRUE or FALSE.  If \f2d\fP is non-null, on
return it contains the length position of the intersection between 0
and seg->length.

.SH SEE ALSO
pfNodeIsectSegs,
pfGSetIsectSegs,
pfVec3
