'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfSharedMem" "3pf"
.SH NAME
.upperok
.B pfInitArenas,
.B pfFreeArenas,
.B pfGetSharedArena,
.B pfGetSemaArena,
.B pfSharedArenaSize,
.B pfGetSharedArenaSize,
.B pfSharedArenaBase,
.B pfGetSharedArenaBase,
.B pfSemaArenaSize,
.B pfGetSemaArenaSize,
.B pfSemaArenaBase,
.B pfGetSemaArenaBase,
.B pfTmpDir,
.B pfGetTmpDir
\- 
Shared Memory Functions
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'const\ char\ *\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
int
\f3\%pfInitArenas\fP(void);
.nf
.fi
.TP
int
\f3\%pfFreeArenas\fP(void);
.nf
.fi
.TP
void*
\f3\%pfGetSharedArena\fP(void);
.nf
.fi
.TP
void*
\f3\%pfGetSemaArena\fP(void);
.nf
.fi
.TP
void
\f3\%pfSharedArenaSize\fP(size_t\ size);
.nf
.fi
.TP
size_t
\f3\%pfGetSharedArenaSize\fP(void);
.nf
.fi
.TP
void
\f3\%pfSharedArenaBase\fP(void\ *base);
.nf
.fi
.TP
void *
\f3\%pfGetSharedArenaBase\fP(void);
.nf
.fi
.TP
void
\f3\%pfSemaArenaSize\fP(size_t\ size);
.nf
.fi
.TP
size_t
\f3\%pfGetSemaArenaSize\fP(void);
.nf
.fi
.TP
void
\f3\%pfSemaArenaBase\fP(void\ *base);
.nf
.fi
.TP
void *
\f3\%pfGetSemaArenaBase\fP(void);
.nf
.fi
.TP
void
\f3\%pfTmpDir\fP(char\ *dirname);
.nf
.fi
.TP
const char *
\f3\%pfGetTmpDir\fP(void);
.fi
.SH DESCRIPTION


\f3\%pfInitArenas\fP creates arenas that can be used to allocate shared
memory, locks and semaphores from (see \f3\%pfMalloc\fP, \f3\%usnewlock\fP,
\f3\%usnewsema\fP).  In a libpf application, this function is called by
\f3\%pfInit\fP so it is not necessary to call it directly.  However, a
libpr application that wishes to use Performer's arenas, must call
\f3\%pfInitArenas\fP before calling \f3\%pfInit\fP to ensure that the type
system is created in shared memory.

In addition to creating a shared memory arena, \f3\%pfInitArenas\fP uses
\f3\%usinit\fP to create an arena for locks and semaphores.

\f3\%pfTmpDir\fP and the environment variable \f3\%PFTMPDIR\fP control where
\f3\%pfInitArenas\fP creates its arenas.  If neither is specified, the
semaphore arena is created in \f3\%/usr/tmp\fP and the shared memory arena
is created in /dev/zero (swap space).  If a temporary directory is
specified, then files are created in that directory for the shared
memory and semaphore arenas.  Both these files are unlinked so that
they are removed from the file system when the process terminates.
Once created, these arenas cannot grow beyond the specified size, so
OpenGL Performer tries to create a large (256MB by default) shared
memory arena and a 128KB semaphore arena.  The semaphore arena always
uses a memory mapped file.

For the shared memory arena, the default option uses swap space.  This
is usually preferable to using a memory mapped file in a directory
specified by \f3\%PFTMPDIR\fP.  Using an actual file is slower at
allocation time and requires actual disk space for extending the file
length to equal the amount of memory allocated (\f3\%pfMalloc\fP) from the
arena.  Because the temporary file is unlinked after creation, any
memory actually allocated will show up as used under df(1), but not
under du(1).  The application size in ps(1) will reflect the maximum
specified size of the arena, e.g. 256MB.  But space is not reserved
until accessed, i.e. until required by \f3\%pfMalloc\fP.  So the large
arena created by \f3\%pfInitArenas\fP does not consume any substantial
disk or swap space resources until needed.

\f3\%pfSharedArenaSize\fP can be used to override the arena size that OpenGL
Performer uses by default (256MB).  \f2size\fP specifies the desired
size in bytes.  Arena size is limited by the largest contiguous
possible memory mapping, currently slightly more than 1.7GB in an
application linked with DSOs.  When attempting large arena mappings,
first make sure that the real and virtual memory usage limits set in
the shell or with setrlimit() are adequate.  \f3\%pfSharedArenaSize\fP
must be called before \f3\%pfInitArenas\fP to have effect.
\f3\%pfGetSharedArenaSize\fP returns the arena size in bytes.


The comparable calls for the semaphore arena are \f3\%pfSemaArenaSize\fP
and \f3\%pfGetSemaArenaSize\fP.

\f3\%pfSharedArenaBase\fP sets the base address for the mapping of the
shared memory arena.  Normally, IRIX chooses these base addresses
automatically.  Direct specification is only useful if the application
needs closer control over the layout of virtual address space, e.g. to
avoid conflicts with other mappings.  \f3\%pfGetSharedArenaBase\fP returns
the base address for the arena.

The comparable calls for the semaphore arena are \f3\%pfSemaArenaBase\fP
and \f3\%pfGetSemaArenaBase\fP.

\f3\%pfGetSemaArena\fP returns a handle to the lock arena and
\f3\%pfGetSharedArena\fP returns a pointer to the shared memory arena.
This pointer cannot be used directly, only as an argument to
\f3\%pfMalloc\fP.  \f3\%pfGetTmpDir\fP returns the temporary directory set
using \f3\%pfTmpDir\fP.
.SH NOTES


Shared Arena functionality is not currently supported under
the single processor version of Performer on Linux.

These arenas can only be used by related processes.  Related in this
context means processes that are created by \f3\%fork\fP or \f3\%sproc\fP once
\f3\%pfInitArenas\fP has been called.  Use \f3\%pfDataPool\fP for sharing memory
between unrelated processes.  \f3\%pfInitArenas\fP should be called before
any \f3\%fork\fP calls are made.
.SH SEE ALSO
acreate,
pfFree,
pfMalloc,
usinit,
usnewlock,
usnewsema
