'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfSphere" "3pf"
.SH NAME
.upperok
.B pfMakeEmptySphere,
.B pfSphereExtendByPt,
.B pfSphereExtendBySphere,
.B pfSphereExtendByCyl,
.B pfSphereAroundPts,
.B pfSphereAroundSpheres,
.B pfSphereAroundCyls,
.B pfSphereAroundBoxes,
.B pfSphereContainsPt,
.B pfSphereContainsSphere,
.B pfSphereContainsCyl,
.B pfSphereContainsBox,
.B pfSphereIsectSeg,
.B pfOrthoXformSphere
\- 
Set, transform and extend a sphere
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfMakeEmptySphere\fP(pfSphere\ *sph);
.nf
.fi
.TP
void
\f3\%pfSphereExtendByPt\fP(pfSphere*\ dst,
'in +2
'll -2
const\ pfVec3\ pt);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphereExtendBySphere\fP(pfSphere*\ dst,
'in +2
'll -2
const\ pfSphere*\ sph);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphereExtendByCyl\fP(pfSphere*\ dst,
'in +2
'll -2
const\ pfCylinder*\ cyl);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphereAroundPts\fP(pfSphere\ *dst,
'in +2
'll -2
const\ pfVec3\ *pts,
int\ npt);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphereAroundSpheres\fP(pfSphere\ *dst,
'in +2
'll -2
const\ pfSphere\ **sphs,
int\ nsph);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphereAroundCyls\fP(pfSphere\ *dst,
'in +2
'll -2
const\ pfCylinders\ **sphs,
int\ ncyl);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfSphereAroundBoxes\fP(pfSphere\ *dst,
'in +2
'll -2
const\ pfBox\ **boxes,
int\ nbox);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfSphereContainsPt\fP(const\ pfSphere*\ sp,
'in +2
'll -2
const\ pfVec3\ pt);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfSphereContainsSphere\fP(const\ pfSphere\ *sph1,
'in +2
'll -2
const\ pfSphere\ *sph2);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfSphereContainsCyl\fP(const\ pfSphere\ *sph,
'in +2
'll -2
const\ pfCylinder\ *cyl);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfSphereContainsBox\fP(const\ pfSphere\ *sph,
'in +2
'll -2
const\ pfBox\ *box);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfSphereIsectSeg\fP(const\ pfSphere*\ sph,
'in +2
'll -2
const\ pfSeg*\ seg,
float*\ d1,
float*\ d2);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfOrthoXformSphere\fP(pfSphere\ *dst,
'in +2
'll -2
const\ pfSphere\ *sph,
const\ pfMatrix\ xform);
.in
.ll
.fi

.Ex
.nf

typedef struct
{
    pfVec3 center;
    float radius;
} pfSphere;
struct pfSphere
{
    pfVec3 center;
    float radius;
};
.Ee
.fi
.na

.SH DESCRIPTION


A pfSphere represents a sphere as a \f2center\fP and a \f2radius\fP.  The
routines listed here provide means of creating and extending spheres
for use as bounding geometry.  pfSphere is a public struct whose data
members \f2center\fP and \f2radius\fP may be operated on directly.


\f3\%pfMakeEmptySphere\fP sets
\f2sph\fP so that it appears empty to extend and
around operations.

\f3\%pfSphereExtendByPt\fP,
\f3\%pfSphereExtendBySphere\fP, and \f3\%pfSphereExtendByCyl\fP set \f2dst\fP to a sphere
which contains both \f2dst\fP and the point \f2pt\fP,
the sphere \f2sph\fP or the cylinder \f2cyl\fP, respectively.

\f3\%pfSphereAroundPts\fP,
\f3\%pfSphereAroundBoxes\fP,
\f3\%pfSphereAroundSpheres\fP and \f3\%pfSphereAroundCyls\fP set \f2dst\fP to a sphere
which contains a set of points, boxes, spheres or cylinders,
respectively.  These routines are passed the address of an array of
pointers to the objects being bounded along with the number of
objects.

\f3\%pfSphereContainsPt\fP,
returns \f3\%TRUE\fP or \f3\%FALSE\fP depending on whether the point \f2pt\fP is
in the interior of the specified sphere.

\f3\%pfSphereContainsSphere\fP, \f3\%pfSphereContainsCyl\fP and \f3\%pfSphereContainsBox\fP test whether the sphere  specified by the first
argument contains a non-empty portion of the volume specified by
the secondsecond argument and the sphere is empty.
.TP
\f3\%PFIS_MAYBE\fP:
The intersection of the second argument
and the sphere might be non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP:
The intersection of the second
argument and the sphere is  definitely non-empty.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP:
The second
argument is non-empty and lies entirely inside the sphere.
.RE

\f3\%pfSphereIsectSeg\fP intersects the line
segment \f2seg\fP with the volume of the pfSphere \f2sphere\fP.  The
possible return values are:

.RS
.TP 5
\f3\%PFIS_FALSE\fP:
\f2seg\fP lies entirely in the exterior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_START_IN\fP:
The starting point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_END_IN\fP:
The ending point of \f2seg\fP lies in the interior.
.TP
\f3\%PFIS_MAYBE\fP | \f3\%PFIS_TRUE\fP | \f3\%PFIS_ALL_IN\fP | \f3\%PFIS_START_IN\fP | \f3\%PFIS_END_IN\fP:
Both end points of \f2seg\fP lie in the interior.
.RE

If \f2d1\fP and \f2d2\fP are non-NULL, on return from
\f3\%pfSphereIsectSeg\fP they contain the starting
and ending positions of the line segment (0 <= d1 <= d2 <=
seg->length) intersected with the sphere.

\f3\%pfOrthoXformSphere\fP
returns in \f2dst\fP
the sphere \f2sph\fP
transformed by the orthogonal transform \f2xform\fP.

.SH NOTES

The bit fields returned by the contains functions are structured so
that bitwise AND-ing the results of sequential tests can be used to
compute composite results, e.g. testing exclusion against a number
of half spaces.

Some of the extend and around operations are time consuming and should
be used sparingly.  In general, the quality of a bound generated by a
series of extend operations will be no better, and sometimes much
worse, than a bound generated by a single around operation.


.SH SEE ALSO
pfBox,
pfCylinder,
pfSeg,
pfVec3
