'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfTime" "3pf"
.SH NAME
.upperok
.B pfInitClock,
.B pfGetTime,
.B pfClockMode,
.B pfGetClockMode,
.B pfClockName,
.B pfGetClockName,
.B pfWrapClock
\- 
Initialize and query high resolution clock
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'const\ char\ *\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
pid_t
\f3\%pfInitClock\fP(double\ time);
.nf
.fi
.TP
double
\f3\%pfGetTime\fP(void);
.nf
.fi
.TP
void
\f3\%pfClockMode\fP(int\ mode);
.nf
.fi
.TP
int
\f3\%pfGetClockMode\fP(void);
.nf
.fi
.TP
void
\f3\%pfClockName\fP(char\ *name);
.nf
.fi
.TP
const char *
\f3\%pfGetClockName\fP(char\ *name);
.nf
.fi
.TP
void
\f3\%pfWrapClock\fP(void);
.fi
.SH DESCRIPTION

Most SGI platforms have high resolution hardware timers.  The routines
described in this man page provide a simple interface to these timers.

On the first call to \f3\%pfInitClock\fP, OpenGL Performer finds the highest
resolution clock available and initializes it, setting the initial time
to \f2time\fP seconds.  Subsequent calls simply reset the initial time.

\f3\%pfGetTime\fP returns a high resolution clock time in seconds that is
relative to the initial time set by \f3\%pfInitClock\fP.  It determines the
highest resolution clock available and uses that clock in all
subsequent calls.  On Indy, Indigo, Indigo2, 4D/35, Power Series
systems with IO3 boards, and Onyx, the resolution of the clock is
submicrosecond.  If the hardware does not support a high resolution
counter, the time of day clock is used which typically has 10ms
resolution (see below).

By default, processes forked after the first call to \f3\%pfInitClock\fP
share the same clock.  All such related processes receive the same time
from \f3\%pfGetTime\fP and see the effects of any subsequent calls to
\f3\%pfInitClock\fP.

Unrelated processes can similarly share a single clock by invoking
\f3\%pfClockName\fP before the first call to \f3\%pfInitClock\fP.
\f3\%pfClockName\fP allows a character string name to be associated with a
clock before it is initialized.  \f3\%pfClockName\fP has no effect after
the clock has been initialized.  \f3\%pfGetClockName\fP returns the name of
the clock.

OpenGL Performer periodically checks to see if the underlying hardware
counter has wrapped by having an interval timer (see setitimer(2))
regularly invoke \f3\%pfWrapClock\fP.  By default, this interval timer and
the associated alarm signal are handled in a separate process created
by \f3\%pfInitClock\fP.  The wrapping behavior can be changed by calling
\f3\%pfClockMode\fP before the first call to \f3\%pfInitClock\fP.  A \f2mode\fP
of \f3\%PFCLOCK_APPWRAP\fP causes wrapping to be handled in the process
that called \f3\%pfInitClock\fP.  In this case, the SIGALRM signals
generated by the interval timer are generated and handled in that
process.  Wrap checking can be disabled altogether with a \f2mode\fP of
\f3\%PFCLOCK_NOWRAP\fP.  In this case, the application should invoke
\f3\%pfWrapClock\fP with sufficient frequency to avoid missing a cycle of
the counter (typically the hardware counters wrap every 60-120 secs,
but the R3000 Indigo's clock wraps every 500ms).  \f3\%pfGetClockMode\fP
returns the clock mode.

When wrapping is done in a separate process, only one process is
spawned, regardless of the number of processes using the same clock.
If the calling process has super-user privileges, both the spawned
process and the kernel clock functions are forced to run on CPU 0.
\f3\%pfInitClock\fP returns the process ID of the process handling clock
wrapping and 0 otherwise.  A value of -1 indicates failure.

.SH NOTES

If none of the hardware clocks are available, the time of day clock is
used, but with only 10ms resolution it is insufficient for many
applications such as frame rate control.  In this case, it may be
useful to enable the fast clock (see ftimer(1)).

The pfDataPool file for clock data is created in the directory
\f2/usr/tmp\fP or in the directory indicated by the \f3\%PFTMPDIR\fP
environment variable, if it is specified.

Processes specifying a clock of the same name, share that clock through
a pfDataPool.  With an unnamed clock, the pfDataPool is deleted on
exit, but when using a named clock, the application must explicitly
call \f3\%pfReleaseDPool\fP with the clock name to remove the pfDataPool.
.SH SEE ALSO
fork,
ftimer,
setitimer,
lboot
