'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfTransparency" "3pf"
.SH NAME
.upperok
.B pfTransparency,
.B pfGetTransparency
\- 
Set/get the transparency mode
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfTransparency\fP(int\ type);
.nf
.fi
.TP
int
\f3\%pfGetTransparency\fP(void);
.fi
.SH PARAMETERS
.TP \w'type\ \ 'u
\f2type\fP
is a symbolic constant and is one of:
.RS
.TP 5
\f3\%PFTR_OFF\fP
Disable transparency
.TP
\f3\%PFTR_ON\fP
OpenGL Performer will use a default transparency mechanism depending on
the machine being used.
.TP
\f3\%PFTR_HIGH_QUALITY\fP
OpenGL Performer will use a transparency mechanism that provides the
highest quality, not necessarily the fastest, transparency.
.TP
\f3\%PFTR_FAST\fP
OpenGL Performer will use a transparency mechanism that provides the
fastest, not necessarily the highest quality, transparency.
.TP
\f3\%PFTR_BLEND_ALPHA\fP
OpenGL Performer will use the OpenGL \f3\%glBlendFunc(3g)\fP
method of transparency.
.TP
\f3\%PFTR_MS_ALPHA\fP
OpenGL Performer will use the the
OpenGL \f3\%glEnable(GL_SAMPLE_ALPHA_TO_ONE_SGIS)\fP method of transparency
when multisampling is available and enabled.
Source alpha values will be converted to
1.0 (full opacity) before writing to the framebuffer.
.TP
\f3\%PFTR_MS_ALPHA_MASK\fP
OpenGL Performer will use the OpenGL \f3\%glEnable(GL_SAMPLE_ALPHA_TO_MASK_SGIS)\fP
method of transparency
when multisampling is enabled. Source alpha values will not be modified.
.RE
\f2type\fP may be OR-ed with \f3\%PFTR_NO_OCCLUDE\fP if transparent geometry
is not to occlude other geometry.
.SH DESCRIPTION

.EQ
.EN
\f3\%pfTransparency\fP sets the transparency mode to \f2mode\fP.  Enabling
transparency is not enough to render transparent geometry.  Geometry
colors must have alpha values that are less than the maximum
(alpha < 1 for \f3\%c4f\fP and alpha < 255 for \f3\%cpack\fP)
in order to be transparent.  When alpha is less than maximum, it
defines the blending of geometry color with framebuffer color
according to the following equation.

.RS
finalColor = alpha * geometryColor + ( 1 - alpha ) * colorInFramebuffer
.RE

In other words, alpha is the "inverse" of transparency.

The transparency mode value may be OR-ed with \f3\%PFTR_NO_OCCLUDE\fP.
\f3\%PFTR_NO_OCCLUDE\fP disables writes to the depth buffer so that any
geometry rendered after \f3\%pfTransparency\fP is called with this value
will not modify the depth buffer and so will not be able to occlude any
other geometry.  Since you can "see-through" transparent geometry, this
is a useful option when using \f3\%PFTR_BLEND_ALPHA\fP type transparency
and you are unable to render transparent geometry back to front.

\f3\%pfGetTransparency\fP returns the current transparency mode.

The transparency mode state element is identified by the
\f3\%PFSTATE_TRANSPARENCY\fP token.  Use this token with \f3\%pfGStateMode\fP
to set the transparency mode of a pfGeoState and with \f3\%pfOverride\fP to
override subsequent transparency mode changes:

Example 1:
.Ex
.nf

/* Set up transparent pfGeoState */
pfGStateMode(gstate, PFSTATE_TRANSPARENCY, PFTR_HIGH_QUALITY);

/* Attach gstate to gset */
pfGSetGState(gset, gstate);

/*
 * Draw transparent gset. 'gset' must have alpha values
 * that are < 1.0f for transparency to have effect.
 */
pfDrawGSet(gset);
.Ee
.fi
.na


Example 2:
.Ex
.nf

/* Override transparency mode to PFTR_OFF */
pfTransparency(PFTR_OFF);
pfOverride(PFSTATE_TRANSPARENCY, PF_ON);
.Ee
.fi
.na


The \f3\%MS_ALPHA\fP transparency methods only work when the window
is configured for multisampling. In this case alpha values are converted
to a multisample mask, a "screen door" if you will, that allows
the geometry color to only partially influence each pixel. This
kind of transparency is most efficient when multisampling and has
the important benefit of *not* requiring sorting of
transparent geometry. \f3\%PFTR_BLEND_ALPHA\fP on the other hand
actually blends the geometry color with what is already in the
framebuffer. Thus it requires the following for proper results:

.RS
.TP 5
1.
Transparent geometry be rendered after opaque geometry.
.TP
2.
Transparent geometry be rendered back to front.
.RE

\f3\%pfTransparency\fP
is a display-listable command.  If a pfDispList has been opened by
\f3\%pfOpenDList\fP,
\f3\%pfTransparency\fP
will not have immediate effect but will be captured by the pfDispList
and will only have effect when that pfDispList is later drawn with
\f3\%pfDrawDList\fP.

.SH BUGS

\f3\%pfTransparency\fP modifies the \f3\%zwritemask\fP but does not restore it.

.SH SEE ALSO
blendfunction,
msalpha,
msmask,
glEnable,
glSampleMaskSGIS,
pfDispList,
pfGeoState,
pfState,
zwritemask
