'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfVClock" "3pf"
.SH NAME
.upperok
.B pfStartVClock,
.B pfStopVClock,
.B pfInitVClock,
.B pfGetVClockOffset,
.B pfVClockOffset,
.B pfGetVClock,
.B pfVClockSync
\- 
Initialize and query vertical retrace clock
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pr.h>
.nf
.fi
.TP
void
\f3\%pfStartVClock\fP(void);
.nf
.fi
.TP
void
\f3\%pfStopVClock\fP(void);
.nf
.fi
.TP
void
\f3\%pfInitVClock\fP(int\ ticks);
.nf
.fi
.TP
int
\f3\%pfGetVClockOffset\fP(void);
.nf
.fi
.TP
void
\f3\%pfVClockOffset\fP(int\ offset);
.nf
.fi
.TP
int
\f3\%pfGetVClock\fP(void);
.nf
.fi
.TP
int
\f3\%pfVClockSync\fP(int\ rate,
'in +2
'll -2
int\ offset);
.in
.ll
.fi
.SH DESCRIPTION

A pfVClock (Video Clock) is a clock which runs at the video retrace
rate.  There is one clock for each hardware graphics pipeline which runs
at the video rate of that pipeline. A pipeline's video rate is defined by
its video format (see \f3\%setmon\fP). To access the video clock
a Graphics Library window must be opened and made current although
the window does not need to be mapped to the display surface
(\f3\%noport\fP in IRIS GL). The screen of the current window determines
which pipe's video clock is accessed on multipipe machines.

\f3\%pfStartVClock\fP starts the video clock by enabling CPU interrupts
from the graphics pipeline while \f3\%pfStopVClock\fP disables CPU interrupts.
At this time, the video clock can be started/stopped only on
VGX and VGXT graphics hardware. On other hardware such as InfiniteReality
and RealityEngine, CPU interrupts are always enabled.

\f3\%pfInitVClock\fP sets the initial value of the video clock to \f2ticks\fP.
Note that \f3\%pfInitVClock\fP does not set the video clock, rather it
computes an offset which is added to the real video clock's value.
This offset is unique to a given address space - \f3\%fork\fPed processes
each have their own offset while \f3\%sproc\fPed processes share the
offset. The offset computed by \f3\%pfInitVClock\fP is returned by
\f3\%pfGetVClockOffset\fP and may be set directly with \f3\%pfVClockOffset\fP,
simplifying clock synchronization across processes.

\f3\%pfGetVClock\fP returns the current video retrace count relative to
the initial value set by \f3\%pfInitVClock\fP.

\f3\%pfVClockSync\fP puts the calling process to sleep until \f2count\fP
modulo \f2rate\fP is equal to \f2offset\fP.  For instance, if the count
is 0, the \f2rate\fP is 10 and \f2offset\fP is 4, then \f3\%pfVClockSync\fP
will return when the count is 4.  Subsequent calls with the same values
will return when the count is 14, 24, etc.  \f2offset\fP must be a
non-negative number less than \f2rate\fP and \f2rate\fP must be positive.

If the retrace count modulo \f2rate\fP is equal to \f2offset\fP at the
time \f3\%pfVClockSync\fP is called, the caller will not return immediately
but will go to sleep until \f2rate\fP ticks later.

The following code fragment illustrates a true video clock which will
wake up each vertical retrace period (subject to process priorities):
.Ex
.nf

while (1)
{
    /* wait for next vertical retrace */
    pfVClockSync(1, 0);

    /* perform per-interval actions */
     :
}
.Ee
.fi
.na

.SH NOTES


pfVClock functionality is not currently supported under Linux.

A GL window (which may be a \f3\%noport\fP window) must be open before
any pfVClock routines are called.

The video clock count is *not* the \f3\%swapbuffers\fP count.

