'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfiInputXformSpheric" "3pf"
.SH NAME
.upperok
.B pfiGetIXformSphericClassType,
.B pfiNewIXformSpheric,
.B pfiIXformSphericMode,
.B pfiGetIXformSphericMode,
.B pfiIXformSphericParameter,
.B pfiGetIXformSphericParameter,
.B pfiCreate2DIXformSpheric,
.B pfiUpdate2DIXformSpheric,
.B pfiIXformSphericSetWorld,
.B pfiIXformSphericReadPathFile,
.B pfiIXformSphericPrintPathStuff
\- 
pfiInputXformSpheric functions.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfiInputXformSpheric\ *\ \ \ 'u
#include <Performer/pfui.h>
.nf
.fi
.TP
pfType *
\f3\%pfiGetIXformSphericClassType\fP(void);
.nf
.fi
.TP
pfiInputXformSpheric *
\f3\%pfiNewIXformSpheric\fP(void\ *arena);
.nf
.fi
.TP
void
\f3\%pfiIXformSphericMode\fP(pfiInputXformSpheric\ *spheric,
'in +2
'll -2
int\ mode,
int\ val);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfiGetIXformSphericMode\fP(pfiInputXformSpheric\ *spheric,
'in +2
'll -2
int\ mode);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfiIXformSphericParameter\fP(pfiInputXformSpheric\ *spheric,
'in +2
'll -2
int\ _param,
float\ _val);
.in
.ll
.nf
.fi
.TP
float
\f3\%pfiGetIXformSphericParameter\fP(pfiInputXformSpheric\ *spheric,
'in +2
'll -2
int\ _param);
.in
.ll
.nf
.fi
.TP
pfiInputXformSpheric *
\f3\%pfiCreate2DIXformSpheric\fP(void\ *arena);
.nf
.fi
.TP
int
\f3\%pfiUpdate2DIXformSpheric\fP(pfiInputXform\ *spheric,
'in +2
'll -2
pfiInputCoord\ *icoord,
void\ *data);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfiIXformSphericSetWorld\fP(pfiInputXformSpheric\ *spheric,
'in +2
'll -2
int\ worldNumber,
int\ in_or_out);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfiIXformSphericReadPathFile\fP(pfiInputXformSpheric\ *spheric,
'in +2
'll -2
char\ *filename);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfiIXformSphericPrintPathStuff\fP(pfiInputXformSpheric\ *spheric);
.fi
.SH DESCRIPTION

\f3\%pfiInputXformSpheric\fP functions.

A pfiInputXformSpheric is a child of the pfiInputXform class and
so pfiInputXform routines may be called with a pfiInputXformSpheric.
See the pfiInputXform reference page for information on pfiInputXform
functionality.
The pfiInputXformSpheric has a complete C++ API
following the conventions of general OpenGL Performer C++ API; C++ methods
are declared in /usr/include/Performer/pfui/pfiSpheric.h.

The motion model is based on a spheric coordinate system - there is the concept
of a center (or origin - but it is not limited to 0,0,0).  The view is always
towards the center - with the ability to change the radius, rotation around
(think of spinning a globe), tilt (tilting a globe), and even roll (rolling
your head to the side while looking at a globe).

.SS Mouse motion

Left mouse button - hold down then up and down motion lengthens or shortens the
radius.

Middle mouse button - hold down then up and down motion tilts up and down.
Left and right motion spins around.

Right mouse button - hold down then up and down motion proceeds along rails to
the next or previous "worlds".  If there are no destinations then this button
will do nothing.

This motion model can be constrained with minimum and maximum values for any of
these parameters.  These values can be set using the C get and set functions:
\f3\%pfiGetIXformSphericParameter\fP and \f3\%pfiIXformSphericParameter\fP. In C++:
\f3\%getParameter\fP and \f3\%setParameter\fP.  These functions all use parameters
defined in \f3\%pfui.h\fP.

This motion model also has the concept of multiple "worlds".  This allows you
to within a space without always having the same center.  Imaging rotating
around a room containing an object of interest set off to one side.  When
viewing the whole room more rotational freedom is allowed, the center of motion
is the center of the room, etc.  Once interested in viewing the object the
center of motion becomes the object and rotational freedom is limited in order
to simplify viewing the model.  This can be accomplished by having one world
defined by the center of the room, and this has a transition to a second world
defined by the center of the object.  These parameters are of the form of
radius for the "in" point(s) and the out point (radius, around, tilt).  There
is the ability to have multiple children "worlds" per world - the limit is
currently two.

To make setting these parameters less bulky there is a configuration file that
can be loaded using \f3\%pfiIXformSphericReadPathFile\fP.  To jump to a particular
world use \f3\%pfiIXformSphericSetWorld\fP.  To print out information about these
paths from world to world use \f3\%pfiIXformSphericPrintPathStuff\fP.

.SS Path file format

This file is formatted as follows (this is a legal file also):

.Ex
.nf

 # at the beginning of a line comments the entire line
 #
 # The first tag that must be used is world.  These must occur in
 # monotonically increasing order (0 1 2 3 ...).
 world 0
 #
 # A world can have 0-2 children.  Each must be specified on a
 # separate line and refer to a world number defined in this file.
 child 1
 #
 # This is the center of motion - X Y Z floating point numbers.
 center 0 0 0
 #
 # The remaining parameters are the same names as the definitions
 # in \f3\%pfui.h\fP but using C++ style lowercase names.  That means
 # the names are all lowercase except the first letter of words other
 # than the first word.  For example PFIXSPH_NEAR_MIN_AROUND becomes
 # nearMinAround.  And all parameters other than those specified
 # above (world, child, center) are single floating point numbers.
 minRadius 12.040635
 maxRadius 100
 inRadius 16.4774
 outRadius 100
 inAround -70.6169
 outAround 0
 inTilt 40.5644
 outTilt 30
 nearScaleLinearVelocity 0.0048
 farScaleLinearVelocity 0.7
 nearMaxAngularVelocity 0.7
 farMaxAngularVelocity 4
 nearMaxRollVelocity 0.2
 farMaxRollVelocity 0.2
 #
 # This is the next world...
 world 1
 center 0 0 0
 minRadius 2.001
 maxRadius 3
 outRadius 3
 inRadius 2
 outAround -90
 inAround -90
 outTilt 90
 inTilt 90
.Ee
.fi
.na


\f3\%pfiInit\fP should be called once before any pfiInputXformSpheric routines and
before the forked creation of any additional processes that will be calling
pfiInputXformSpheric routines.

.SH NOTES

The libpfui source code, object code and documentation are provided as
unsupported software.  Routines are subject to change in future releases.

.SS CAVEATS

This motion model doesn't really pay attention to collisions or position
resets.  It is best to turn off collision checking while using the spheric
motion model.  And hitting reset will reset the position until any movement -
then the position prior to the reset is used again.
