'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuBarrier" "3pf"
.SH NAME
.upperok
.B pfuBarrierCreate,
.B pfuBarrierDestroy,
.B pfuBarrierEnter
\- 
spin-free multiprocess barrier routines
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfuBarrier\ *\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
pfuBarrier *
\f3\%pfuBarrierCreate\fP(usptr_t\ *usptr);
.nf
.fi
.TP
void
\f3\%pfuBarrierDestroy\fP(pfuBarrier\ *b);
.nf
.fi
.TP
void
\f3\%pfuBarrierEnter\fP(pfuBarrier\ *b,
'in +2
'll -2
unsigned\ int\ n);
.in
.ll
.fi
.SH DESCRIPTION

These functions provide a rendezvous mechanism for synchronizing processes.
The API and functionality is analogous to IRIX libc's barrier(),
but the implementation uses semaphores instead of spinning,
and is therefore hundreds of times faster than barrier()
when there is contention for processors.

\f3\%pfuBarrierCreate()\fP takes a usptr_t as an argument
to indicate the shared arena from which to allocate the barrier.
This should be either Performer's semaphore arena
obtained by pfGetSemaArena(), or an arena handle obtained
through a call to usinit().  If the barrier cannot be created
then NULL is returned and the system errno is set to indicate the error.

\f3\%pfuBarrierDestroy()\fP releases all storage associated with \f3\%b\fP.

\f3\%pfuBarrierEnter()\fP takes a pointer to a previously allocated
barrier structure (as returned by \f3\%pfuBarrierCreate()\fP)
and the number of participating processes.
As each process enters the barrier, it waits (using a semaphore)
until all \f3\%n\fP processes have entered, at which time all are
released and continue executing.  The barrier may be reused immediately.

.SH NOTES

The implementation does not depend on any other part of Performer.
It uses two semaphores allocated from the semaphore arena.

There is no analogue of libc's init_barrier() to reset a barrier,
since it's nonsense.

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfGetSemaArena,
usinit,
oserror,
pfuRendezvous,
barrier
