'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuBoxLOD" "3pf"
.SH NAME
.upperok
.B pfuBoxLOD,
.B pfuMakeBoxGSet
\- 
Calculate node bounding boxes and build LODs from boxes.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfGeoSet\ *\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
pfLOD *
\f3\%pfuBoxLOD\fP(pfGroup\ *grp,
'in +2
'll -2
int\ flat,
pfVec4\ *color);
.in
.ll
.nf
.fi
.TP
pfGeoSet *
\f3\%pfuMakeBoxGSet\fP(pfBox\ *box,
'in +2
'll -2
pfVec4\ color,
int\ flat);
.in
.ll
.fi
.SH DESCRIPTION


These functions can be used to automatically generate very simple
level-of-detail representations of a subgraph from the bounding boxes
of the geometric objects contained in that subgraph.  Sending a node
to \f3\%pfuBoxLOD\fP creates multiple representations for that node by
finding bounding boxes at different levels of the scene graph and then
using those boxes as level-of-detail models under \f3\%pfLOD\fP nodes.
The highest level-of-detail under the \f3\%pfLOD\fP is simply the
original subgraph.  The next highest level-of-detail consists of the
subgraph with each \f3\%pfGeode's\fP \f3\%pfGeoSets\fP replaced by a single
\f3\%pfGeoSet\fP depicting a box.  Next every \f3\%pfGeode\fP
is replaced by a single "box" \f3\%pfGeode\fP.  This continues to progressively
lower levels-of-detail as the "box" \f3\%pfGeodes\fP contain larger and larger
subgraphs, until the box rendered for the lowest level-of-detail
contains the entire subgraph.  The new partial subgraphs are created
using \f3\%pfClone\fP.  The LOD transition ranges (see \f3\%pfLODRange\fP) are
set as integer multiples of the spatial extent of the subgraph.

\f3\%pfuBoxLOD\fP takes a subgraph \f2grp\fP, a color vector \f2color\fP, and a
shading type \f2flat\fP.  \f2grp\fP specifies the existing scene
graph hierarchy to be processed.  The \f2color\fP argument provides the
color definition used for the boxes that will be built.  The shading
type \f2flat\fP determines each \f3\%pfGeoSet\fP's primitive type and normals.  If
\f2flat\fP is non-zero, the generated \f3\%pfGeoSet\fP's are made up of
\f3\%PFGS_FLAT_TRISTRIP\fP primitives with face directed normals.  If
\f2flat\fP is zero, they consist of \f3\%PFGS_TRISTRIP\fP primitives with
normals directed radially outwards.  The former produces sharply
defined cubes, the latter marshmallows.

To determine the appropriate size of the box geometry, \f3\%pfuBoxLOD\fP
uses \f3\%pfuTravCalcBBox\fP to get a tighter bound on the subgraphs
than \f3\%pfGetNodeBSphere\fP provides.

\f3\%pfuMakeBoxGSet\fP takes a \f3\%pfBox\fP \f2box\fP and returns a \f3\%pfGeoSet\fP
containing drawable geometry that represents the box with color
\f2color\fP and a shading type determined by \f2flat\fP, as explained in the
description of \f3\%pfuBoxLOD\fP above.  The \f3\%pfGeoSet\fP and associated
arrays are allocated from the current shared memory arena (see
\f3\%pfGetSharedArena\fP).
.SH NOTES

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH BUGS

The LOD transition range setting is very primitive.
.SH SEE ALSO
pfGeoSet,
pfLOD,
pfLODRange,
pfuTravCalcBBox,
pfNodeBSphere,
pfScene,
pfClone,
pfGetSharedArena
