'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuAddClipTextureToPipes" "3pf"
.SH NAME
.upperok
.B pfuAddMPClipTextureToPipes,
.B pfuAddMPClipTexturesToPipes,
.B pfuInitClipTexConfig,
.B pfuMakeClipTexture,
.B pfuFreeClipTexConfig,
.B pfuInitImgCacheConfig,
.B pfuMakeImageCache,
.B pfuFreeImgCacheConfig
\- 
Connect an MPClipTexture to one or more Pipes
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfGroup\ *\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
pfGroup *
\f3\%pfuAddMPClipTextureToPipes\fP(pfMPClipTextures\ *master,
'in +2
'll -2
pfPipe\ *masterpipe,
pfPipe\ *pipes[]);
.in
.ll
.nf
.fi
.TP
pfGroup *
\f3\%pfuAddMPClipTexturesToPipes\fP(pfList\ *mpcliptextures,
'in +2
'll -2
pfPipe\ *masterpipe,
pfPipe\ *pipes[]);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuInitClipTexConfig\fP(pfuClipTexConfig\ *config);
.nf
.fi
.TP
void
\f3\%pfuMakeClipTexture\fP(pfuClipTexConfig\ *config);
.nf
.fi
.TP
void
\f3\%pfuFreeClipTexConfig\fP(pfuClipTexConfig\ *config);
.nf
.fi
.TP
void
\f3\%pfuInitImgCacheConfig\fP(pfuImgCacheConfig\ *config);
.nf
.fi
.TP
void
\f3\%pfuMakeImageCache\fP(pfuImgCacheConfig\ *config);
.nf
.fi
.TP
void
\f3\%pfuFreeImgCacheConfig\fP(pfuImgCacheConfig\ *config);
.fi
.SH DESCRIPTION

pfMPClipTextures must be attached to pipes if they are to update properly.
In order to connect the same pfMPClipTexture to more than one pipe, the
pfMPClipTexture must be cloned; the original pfMPClipTexture becomes the
master, and the new pfMPClipTexture becomes the slave.

This is done automatically if the same pfMPClipTexture is added to more than
one pipe. This routine makes the process more convenient, by allowing the
user to specify which pipe should contain the master cliptexture, what set
of pipes should have the pfMPClipTextures attached, and the pfMPClipTexture(s)
that should be attached.

This routine does additional checking to avoid erroneous operations. For
example, it checks to avoid common mistakes such as; trying to attach the same
pfMPClipTexture to the same pfPipe more than once, not adding a slave to a
pfPipe that already contains the slave's master, etc. This makes it possible
to run the routine multiple times without generating warnings.

\f3\%pfuAddMPClipTextureToPipes\fP takes a pointer to an pfMPClipTexture \f2master\fP
which is put in \f2masterpipe\fP. All other pipes in \f2pipes\fP are given slaves
to this cliptexture.

Both the \f2masterpipe\fP and \f2pipes\fP arguments can be NULL. If \f2masterpipe\fP
is NULL, the first available pfPipe is used to contain the master (if it isn't
already attached to a different pipe). If \f2pipes\fP is NULL, then all the
pfPipes have the pfMPClipTexture or its slaves attached to it.

If only a single pfMPClipTexture needs to be attached, use the
\f3\%pfuAddMPClipTextureToPipes\fP routine. If a number of pfMPClipTextures need to
be attached to pfPipes, you can call \f3\%pfuAddMPClipTextureToPipes\fP multiple
times or create a list of them and pass it to \f3\%pfuAddMPClipTexturesToPipes\fP
using \f2mpcliptextures\fP. This is particulary convenient if you are using the
clipcenter nodes, since \f3\%pfuProcessClipCenters\fP fills in the pfList you give
it with the list of pfMPClipTexture objects in the scene graph.

\f3\%pfuInitClipTexConfig\fP, \f3\%pfuMakeClipTexture\fP, and \f3\%pfuFreeClipTexConfig\fP
are used together to create and configure clip textures. These routines are
used by \f3\%pfdLoadClipTexture\fP, and can be used by applications to build
cliptextures dynamically or to design custom clip texture configuration
file formats. pfuMakeClipTexture takes a single argument, a pointer to
a properly filled in pfuClipTexConfig structure, and uses the information
in it to create and configure a cliptexture, its image caches, and
its pyramid image tiles. The pfuInitClipTexConfig called before
pfuMakeClipTexture; it's purpose is to fill in the pfuClipTexConfig structure
with default values. pfuFreeClipTexConfig is called last, after the cliptexture
has been created. It correctly frees any elements of the structure that
have been created (although it doesn't free the structure itself).

The recommended proceedure is to allocate a pfuClipTexConfig structure,
call pfuInitClipTexConfig to initialize it, fill in the values of the
structure to describe the clip texture desired, then call
pfuMakeClipTexture with a pointer to the initialized and filled in structure.
Finally, call pfuFreeClipTexConfig to free any structure elements, then
delete the pfuClipTexConfig structure itself. The advantage of this
proceedure is that structure elements are filled in with reasonable default
values, allowing optional values to be left out. It also improves error
checking, since the error checking code knows what the default values should
be, allowing it to warn the application if mandatory values weren't filled in.

The pfuClipTexConfig structure has three types of values: configuration data
for the cliptexture itself, function pointers to user-supplied functions
that load the image caches and image tiles that the clip texture requires,
and pointers to user-defined data structures used by the user-supplied
configuration functions.

The user-supplied image cache configuration function should be an
argument of the following format:
pfImageCache *func(pfClipTexture *ct, int level, pfuClipTexConfig *config)

This function should use
\f3\%pfuInitImgCacheConfig\fP, \f3\%pfuMakeImageCache\fP, and \f3\%pfuFreeImgCacheConfig\fP
in the same way as was done for configuring cliptextures, and return a pointer
to the created and configured image cache. This function can access the
user-supplied data in the icData element of the _pfuClipTexConfig structure
for additional information.

The user-supplied image tile configuration function should be an
argument of the following format:
pfImageTile *func(pfClipTexture *ct, int level, pfuClipTexConfig *config)

This function should create and configure the required image tile and
return a pointer to it. This function can access the
user-supplied data in the tileData element of the _pfuClipTexConfig
structure for additional information.

The configuration functions allow the application to customize how
image caches and image tiles are configured when an clip texture is created.
The pfdLoadImage.c file uses this functionality and should be used as
an example.

The
\f3\%pfuInitImgCacheConfig\fP, \f3\%pfuMakeImageCache\fP, and \f3\%pfuFreeImgCacheConfig\fP
functions are used in the same way as described for
\f3\%pfuInitClipTexConfig\fP, \f3\%pfuMakeClipTexture\fP, and \f3\%pfuFreeClipTexConfig\fP.
The major difference is that the pfuImgCacheConfig structure doesn't require
user-supplied function or data pointers.

.SH NOTES


ClipTexture functionality is not supported under Linux.

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH BUGS


.SH SEE ALSO
pfuProcessClipCenters,
pfuNewClipCenterNode,
pfClipTexture,
pfImageCache,
pfImageTile,
pfMPClipTexture,
pfMPImageCache,
pfTexLoad,
pfTexture,
pfuGetClosestPoint
