'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuCollide" "3pf"
.SH NAME
.upperok
.B pfuCollisionChan,
.B pfuGetCollisionChan,
.B pfuCollideSetup,
.B pfuCollideGrnd,
.B pfuCollideObj,
.B pfuCollideGrndObj
\- 
Terrain following and collision routines.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfChannel\ *\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
void
\f3\%pfuCollisionChan\fP(pfChannel\ *chan);
.nf
.fi
.TP
pfChannel *
\f3\%pfuGetCollisionChan\fP(void);
.nf
.fi
.TP
void
\f3\%pfuCollideSetup\fP(pfNode\ *node,
'in +2
'll -2
int\ mode,
int\ mask);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuCollideGrnd\fP(pfCoord\ *coord,
'in +2
'll -2
pfNode\ *node,
pfVec3\ zpr);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuCollideObj\fP(pfSeg\ *seg,
'in +2
'll -2
pfNode\ *objNode,
pfVec3\ hitPos,
pfVec3\ hitNorm);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuCollideGrndObj\fP(pfCoord\ *coord,
'in +2
'll -2
pfNode\ *grndNode,
pfVec3\ zpr,
pfSeg\ *seg,
pfNode\ *objNode,
pfVec3\ hitPos,
pfVec3\ hitNorm);
.in
.ll
.fi
.SH DESCRIPTION

This collection of simple routines provides intersection traversals
for basic collision detection and ground following.
Examples of \f3\%pfuCollide\fP usage can be found in the OpenGL
Performer transformation utility, \f3\%pfuXformer\fP.

\f3\%pfuCollisionChan\fP sets the channel from which
to derive LOD scale and viewing information. This is used to
determine which LOD to collide against.

\f3\%pfuGetCollisionChan\fP returns the current channel set by \f3\%pfuCollisionChan.\fP

\f3\%pfuCollideSetup\fP sets the intersection mask of the subgraph rooted
by \f2node\fP to \f2mask\fP. The values \f2mask\fP can take are the same as
in \f3\%pfNodeTravMask\fP. \f2mode\fP is either
.RS
.TP
\f3\%PFUCOLLIDE_STATIC\fP
Geometry below \f2node\fP is considered static and
intersection caching will be enabled (see \f3\%pfNodeTravMask\fP) or
.TP
\f3\%PFUCOLLIDE_DYNAMIC\fP
Geometry below \f2node\fP is considered dynamic and
intersection caching will not be enabled (see \f3\%pfNodeTravMask\fP)
.RE

Best intersection performance is achieved for static geometry.

\f3\%pfuCollideGrnd\fP fires a ray downward (in negative Z direction) from
\f2coord\fP and returns \f3\%PFUCOLLIDE_GROUND\fP if an intersection was
found with the subgraph rooted by \f2node\fP and \f3\%FALSE\fP otherwise.
The height and orientation of the
intersected geometry is returned in \f2zpr\fP which contains the height,
and the pitch and roll angles of the surface at the intersection point.

\f3\%pfuCollideObj\fP intersects \f2seg\fP with the subgraph
rooted by \f2objNode\fP and returns \f3\%PFUCOLLIDE_OBJECT\fP if an intersection was
found and \f3\%FALSE\fP otherwise.  The intersection point and surface normal
is returned in \f2hitPos\fP and \f2hitNorm\fP respectively.  \f2seg\fP is
typically a velocity vector used to detect collisions in the direction
of travel.

\f3\%pfuCollideGrndObj\fP is the combination of \f3\%pfuCollideObj\fP and
\f3\%pfuCollideGrnd\fP. It may provide better performance over calling
these routines separately when \f2grndNode\fP and \f2objNode\fP are
identical.  The return value is a bitmask of \f3\%PFUCOLLIDE_GROUND\fP and
\f3\%PFUCOLLIDE_OBJECT\fP indicating whether the downward-directed ray
and/or the line segment hit anything.

.SH NOTES

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfHit,
pfNode,
pfNodeIsectSegs,
pfNodeTravMask
