'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuGridify" "3pf"
.SH NAME
.upperok
.B pfuGridifySaveSize,
.B pfuGridifyMem,
.B pfuUnGridifyMem,
.B pfuGridifyFileP,
.B pfuGridifyFile,
.B pfuGridifyImageTile,
.B pfuUnGridifyImageTile,
.B pfuGridifyImageCache,
.B pfuUnGridifyImageCache,
.B pfuGridifyClipTexture,
.B pfuUnGridifyClipTexture,
.B pfuGridifyMPClipTexture,
.B pfuUnGridifyMPClipTexture,
.B pfuGridifyAnyClipTexture,
.B pfuUnGridifyAnyClipTexture
\- 
Runtime clip texture gridification
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
int
\f3\%pfuGridifySaveSize\fP(int\ width,
'in +2
'll -2
int\ height,
int\ pixeldepth,
int\ borderwidth);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuGridifyMem\fP(unsigned\ char\ *buf,
'in +2
'll -2
int\ width,
int\ height,
int\ pixeldepth,
int\ borderwidth,
const\ unsigned\ char\ color[/*\ pixeldepth\ */],
unsigned\ char\ savecolors[/*\ pfuGridifySaveSize()\ */]);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuUnGridifyMem\fP(unsigned\ char\ *buf,
'in +2
'll -2
int\ width,
int\ height,
int\ pixeldepth,
int\ borderwidth,
const\ unsigned\ char\ savedcolors[/*\ pfuGridifySaveSize()\ */]);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuGridifyFileP\fP(const\ char\ *infilename_for_errors,
'in +2
'll -2
FILE\ *infp,
const\ char\ *outfilename_for_errors,
FILE\ *outfp,
int\ width,
int\ height,
int\ pixeldepth,
int\ borderwidth,
unsigned\ char\ color[/*\ pixeldepth\ */]);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuGridifyFile\fP(const\ char\ *filename,
'in +2
'll -2
int\ width,
int\ height,
int\ pixeldepth,
int\ borderwidth,
unsigned\ char\ color[/*\ pixeldepth\ */]);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuGridifyImageTile\fP(pfImageTile\ *itile,
'in +2
'll -2
int\ pixeldepth,
const\ unsigned\ char\ color[/*\ pixeldepth\ */],
int\ isproto);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuUnGridifyImageTile\fP(pfImageTile\ *itile);
.nf
.fi
.TP
int
\f3\%pfuGridifyImageCache\fP(pfImageCache\ *icache,
'in +2
'll -2
int\ pixeldepth,
unsigned\ char\ color[/*\ pixeldepth\ */]);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuUnGridifyImageCache\fP(pfImageCache\ *icache);
.nf
.fi
.TP
int
\f3\%pfuGridifyClipTexture\fP(pfClipTexture\ *cliptex);
.nf
.fi
.TP
int
\f3\%pfuUnGridifyClipTexture\fP(pfClipTexture\ *cliptex);
.nf
.fi
.TP
int
\f3\%pfuGridifyMPClipTexture\fP(pfMPClipTexture\ *mpcliptex);
.nf
.fi
.TP
int
\f3\%pfuUnGridifyMPClipTexture\fP(pfMPClipTexture\ *mpcliptex);
.nf
.fi
.TP
int
\f3\%pfuGridifyAnyClipTexture\fP(pfNode\ *scene,
'in +2
'll -2
int\ i);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuUnGridifyAnyClipTexture\fP(pfNode\ *scene,
'in +2
'll -2
int\ i);
.in
.ll
.fi
.SH DESCRIPTION

The \f3\%pfuGridify\fP functions overlay color-coded
grids on image tile data at runtime,
to facilitate visual verification of texel size,
tile size, and which mipmap levels of detail
are being displayed.
Functions are provided to gridify image data that is represented
in any of the following forms: pfClipTexture, pfImageCache, pfImageTile,
raw tile in memory, or raw tile in file.
For all but the last of these,
the image pixels clobbered by the grid is saved in a compact form
so that the corresponding \f3\%pfuUnGridify\fP function
can retrieve them and restore the original underlying image quickly.
This allows an application to efficiently toggle gridification at runtime.
In the sample application perfly, the 'g' key toggles
clipmap gridification (if the scene contains a clipmap).

If the image data is in an RGB format (PFTEX_RGBA_8 or PFTEX_RGB5_A1),
the grid color is chosen as follows, depending
on the destination clipmap level size:
.RS
.nf
cyan     1x1    64x64     4096x4096    262144x262144
blue     2x2   128x128    8192x8192    524288x524288
magenta  4x4   256x256   16384x16384  1048576x1048576
red      8x8   512x512   32767x32768  2097152x2097152
yellow  16x16 1024x1024  65535x65535  4194304x4194304
green   32x32 2048x2048 131072x131072 8388608x8388608
.fi
.RE
If the data is in luminance (PFTEX_I_8) format,
the grid colors alternate
between black and white: black for 1x1, 4x4, 16x16, etc. and
white for 2x2, 8x8, 32x32, etc.

The grid for a pfImageTile or raw memory or file tile consists of:
.RS
.TP 5
--
A 2-pixel border around the entire tile
.TP
--
A column of 4x8-pixel ticks, spaced 8 pixels apart,
down the center of the tile vertically, and a similar row of them
across the center horizontally
.TP
--
Two columns of 2x4-pixel ticks, spaced 16 pixels apart,
at the 1/4 and 3/4 tile position, and similar rows of them.
.RE

.bp
For example, this is the grid overlayed on a 128x128 tile
(only the upper-left quarter is shown):
.nf
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
::
::                                                          ::::::::
::                                                          ::::::::
::
::
::                            ....                          ::::::::
::                            """"                          ::::::::
::
::
::                                                          ::::::::
::                                                          ::::::::
::
::
::             ::             .::.             ::           ::::::::
::             ::             "::"             ::           ::::::::
::
::
::                                                          ::::::::
::                                                          ::::::::
::
::
::                            ....                          ::::::::
::                            """"                          ::::::::
::
::
::                                                          ::::::::
::                                                          ::::::::
::
::    ::::    ::::    ::::    ::::    ::::    ::::    ::::    ::::
::    ::::    ::::    ::::    ::::    ::::    ::::    ::::  ::::::::
::    ::::    ::::    ::::    ::::    ::::    ::::    ::::  ::::::::
::    ::::    ::::    ::::    ::::    ::::    ::::    ::::    ::::
::
::                                                          ::::::::
.fi

.bp
And this is the grid used for a 64x64 tile:
.nf
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::                                                            ::
::                                                            ::
::                          ::::::::                          ::
::                          ::::::::                          ::
::                                                            ::
::                                                            ::
::            .::.          ::::::::          .::.            ::
::            "::"          ::::::::          "::"            ::
::                                                            ::
::                                                            ::
::                          ::::::::                          ::
::                          ::::::::                          ::
::                                                            ::
::    ::::    ::::    ::::    ::::    ::::    ::::    ::::    ::
::    ::::    ::::    ::::  ::::::::  ::::    ::::    ::::    ::
::    ::::    ::::    ::::  ::::::::  ::::    ::::    ::::    ::
::    ::::    ::::    ::::    ::::    ::::    ::::    ::::    ::
::                                                            ::
::                          ::::::::                          ::
::                          ::::::::                          ::
::                                                            ::
::                                                            ::
::            .::.          ::::::::          .::.            ::
::            "::"          ::::::::          "::"            ::
::                                                            ::
::                                                            ::
::                          ::::::::                          ::
::                          ::::::::                          ::
::                                                            ::
::                                                            ::
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
.fi


Thus a user can look at geometry that is textured
with a gridified clipmap and deduce the following kinds of information:
.RS
Texel size and size of tiles and imagery in texels:
.RS
Use the known size of the ticks (4x8 texels and 2x4 texels),
the size of the tile border (2 texels wide around each tile,
so the boundary line between adjacent tiles is 4 texels wide),
and the spacing between ticks (little ticks are 16 texels apart,
big ticks are 8 texels apart) to measure sizes.
.RE
Which mipmap levels of detail are being displayed:
.RS
Look it up by color in the table above.  For
RGB data, the colors repeat every 6 levels,
so some deduction by other means may be necessary
(e.g. familiarity with the image data itself,
or fiddling with the minLOD or maxLOD slider).
Usually it is only necessary to think hard once
in order to "get one's bearings"; after that, it is
easy see transitions between levels and to figure out
the size of a given level from knowledge of adjacent levels.
.RE

The \f3\%pfuGridifySaveSize\fP function takes the size (\f2width\fP,\f2height\fP) and
bytes-per-pixel (\f2pixeldepth\fP) of raw image tile in memory,
and returns the size in bytes of the buffer needed for saving
and restoring the image pixels that will get clobbered by
\f3\%pfuGridifyMem\fP.

\f3\%pfuGridifyMem\fP takes a pointer \f2buf\fP to a tile of raw image data
of size \f2width\fP,\f2height\fP with \f2pixeldepth\fP bytes per pixel,
and a given \f2borderwidth\fP (2 is standard, but any number can be given;
e.g. a value larger than width/2 will change the tile to a solid color),
and gridifies it using the specified \f2color\fP for the grid.
If \f2savecolors\fP is not \f3\%NULL\fP, it must be a pointer
to a buffer of size previously returned by \f3\%pfuGridifySaveSize\fP
to be used for saving the image pixels that will get clobbered by
the grid.
\f3\%pfuUnGridifyMem\fP undoes the effect of \f3\%pfuGridifyMem\fP.

\f3\%pfuGridifyFile\fP is just like \f3\%pfuGridifyMem\fP except that it
operates on a raw image tile in a file (specified by \f2filename\fP)
rather than in memory.  There is no corresponding ungridify function,
since it is easy enough to simply save a copy of the original file.
The function return value is 1 indicating success,
or 0 (and a pfNotify message of level PFNFY_WARN)
indicating failure to open, read from, or write to the file.

\f3\%pfuGridifyFileP\fP is a lower-level interface to the functionality
of \f3\%pfuGridifyFile\fP.  It takes file pointers (\f2infp\fP, \f2outfp\fP)
and the input file may be different from the output file.
The parameters \f2infilename_for_errors\fP and \f2outfilename_for_errors\fP
are used for printing in error messages, should an error occur.
The function return value is 1 indicating success,
or 0 (and a pfNotify message of level PFNFY_WARN)
indicating failure to read from \f2infp\fP or write to \f2outfp\fP.

\f3\%pfuGridifyImageTile\fP gridifies the pfImageTile \f2itile\fP
using a border width of 2 and \f2pixeldepth\fP bytes per pixel
(which must be the actual number of bytes per pixel in \f2itile\fP),
with the given \f2color\fP.
If \f2isproto\fP is FALSE, this means waiting for
the tile to become valid and then gridifying the underlying memory.
If \f2isproto\fP is true, it means setting the readFunc so
that all pfImageTiles cloned from itile in the future
will automatically get gridified as they are read in from disk.
\f3\%pfuUnGridifyImageTile\fP undoes the effect of \f3\%pfuGridifyImageTile\fP.
Both functions return 1 on success,
or 0 (and a pfNotify message of level PFNFY_WARN)
if an error or unexpected state is encountered (e.g. trying to gridify an
already-gridified image cache, or trying to ungridify one that has not been
gridified).

\f3\%pfuGridifyImageCache\fP gridifies the given image cache \f2icache\fP
by calling \f3\%pfuGridifyImageTile\fP with the given arguments
on the prototype tile and all tiles in the cache.
\f3\%pfuUnGridifyImageCache\fP undoes the effect of \f3\%BpfuGridifyImageCache\fP.
Both function return 1 on success,
or 0 (and a pfNotify message of level PFNFY_WARN) if an error is encountered.

\f3\%pfuGridifyClipTexture\fP gridifies the pfClipTexture \f2cliptex\fP
by calling \f3\%pfuGridifyImageCache\fP on each level
(with grid colors assigned to levels as described earlier),
and then marking all destination texture memory as invalid
so that it will be reloaded.
\f3\%pfuUnGridifyClipTexture\fP undoes the effect of \f3\%pfuGridifyClipTexture\fP.
Both functions return 1 on success,
or 0 (and a pfNotify message of level PFNFY_WARN) if an error is encountered.

\f3\%pfuGridifyAnyClipTexture\fP is a convenience function that
calls \f3\%pfuGridifyClipTexture\fP on the \f2i\fP'th texture encountered
during traversal of the scene graph rooted at \f2scene\fP.
\f3\%pfuUnGridifyAnyClipTexture\fP undoes the effect of
\f3\%pfuGridifyAnyClipTexture\fP.

.SH NOTES

In a multiprocess Performer application, these functions
should only be called from the DRAW process.

While a pfClipTexture, pfImageCache, or pfImageTile is in a gridified state,
its user data is used to store the saved pixels and callback information,
so applications cannot use the objects' user data for other purposes.

The implementation of pfuGridifyImageCache (and thus also pfuGridifyClipTexture
which uses it) installs a wrapper readFunc on the prototype tile.  This
wrapper calls the original readFunc and gridifies the data
before returning, allocating an array to store the saved pixels.
(The corresponding pfuUnGridify functions restore the saved pixels,
free the saved pixel array, and restore the original readFunc.)
This implementation has two important consequences: first, while an object is
in a gridified state, it can incur per-frame memory allocations and frees
(a practice that is frowned upon when trying to
achieve consistent frame rates).  Second, applications that install
a custom readFunc must take care to do so *before* gridifying
so that the gridification function will wrap it properly.

\f3\%pfuGridifyMPClipTexture\fP and \f3\%pfuUnGridifyMPClipTexture\fP
are not implemented.

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.

.SH SEE ALSO
pfClipTexture,
pfImageCache,
pfImageTile,
pfMultiprocess,
pfNotify,
perfly
