'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuHash" "3pf"
.SH NAME
.upperok
.B pfuNewHTable,
.B pfuDelHTable,
.B pfuResetHTable,
.B pfuEnterHash,
.B pfuRemoveHash,
.B pfuFindHash,
.B pfuHashGSetVerts,
.B pfuCalcHashSize
\- 
Hash table utility library.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfuHashTable\ *\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
pfuHashTable *
\f3\%pfuNewHTable\fP(int\ numb,
'in +2
'll -2
int\ eltsize,
void*\ arena);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuDelHTable\fP(pfuHashTable*\ ht);
.nf
.fi
.TP
void
\f3\%pfuResetHTable\fP(pfuHashTable*\ ht);
.nf
.fi
.TP
pfuHashElt *
\f3\%pfuEnterHash\fP(pfuHashTable*\ ht,
'in +2
'll -2
pfuHashElt*\ elt);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuRemoveHash\fP(pfuHashTable*\ ht,
'in +2
'll -2
pfuHashElt*\ elt);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuFindHash\fP(pfuHashTable*\ ht,
'in +2
'll -2
pfuHashElt*\ elt);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuHashGSetVerts\fP(pfGeoSet\ *gset);
.nf
.fi
.TP
int
\f3\%pfuCalcHashSize\fP(int\ size);
.fi

.Ex
.nf

typedef struct _pfuHashElt
{
    int		id;
    int		listIndex;
    uint	key;
    void	*data;
} pfuHashElt;

typedef struct _pfuHashBucket
{
    int		nelts;
    pfuHashElt	*elts;
    struct _pfuHashBucket	*next;

} pfuHashBucket;

typedef struct _pfuHashTable
{
    void	*arena;
    int		eltSize;
    int   	realeltSize;
    int		numBuckets;
    pfuHashBucket	**buckets;

    /* Flat list of hash elements provides linear ordering */
    int		listCount;
    int		listAvail;
    pfuHashElt	**list;
} pfuHashTable;
.Ee
.fi
.na

.SH DESCRIPTION

\f3\%pfuNewHTable\fP returns a new hash table allocated from
the shared memory arena \f2arena\fP with \f2numb\fP elements each
of size \f2eltsize\fP.  \f2eltsize\fP is in bytes and must be a multiple
of four.

\f3\%pfuDelHTable\fP deletes the hash table \f2ht\fP.

\f3\%pfuResetHTable\fP resets the hash table \f2ht\fP.

\f3\%pfuEnterHash\fP puts element \f2elt\fP into hash table \f2ht\fP.
If the element is already in the table, it returns the address
of that element, otherwise it returns NULL and adds the element to
the \f2list\fP member of the \f3\%pfuHashTable\fP structure.

\f3\%pfuRemoveHash\fP removes element \f2elt\fP from the hash table \f2ht\fP, returning
\f3\%TRUE\fP if \f2elt\fP was found and \f3\%FALSE\fP otherwise.

\f3\%pfuFindHash\fP looks for element \f2elt\fP in hash table \f2ht\fP, returning
\f3\%TRUE\fP if \f2elt\fP was found and \f3\%FALSE\fP otherwise.

\f3\%pfuHashGSetVerts\fP takes a \f3\%pfGeoSet\fP of type \f3\%PFGS_TRIS\fP and
attempts to share all \f3\%PFGS_PER_VERTEX\fP attributes.
\f3\%pfuHashGSetVerts\fP will convert a non-indexed \f3\%pfGeoSet\fP into an
indexed one and may delete the old attribute and index arrays and
create new ones.  Consequently you may wish to \f3\%pfRef\fP your arrays
to avoid their deletion.

An example of \f3\%pfuHashGSetVerts\fP usage is found in \f3\%pfdMeshGSet\fP.

\f3\%pfuCalcHashSize\fP returns the smallest prime number larger than
\f2size\fP. This is useful since hash tables are more memory efficient
when their table size is prime.
.SH NOTES

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfGeoSet,
pfRef,
pfuMeshGSet,
pfuEventQueue,
pfuGUI
