'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuInitInput" "3pf"
.SH NAME
.upperok
.B pfuInitInput,
.B pfuInitMultiChanInput,
.B pfuExitInput,
.B pfuGetMouse,
.B pfuGetEvents,
.B pfuInputHandler,
.B pfuCollectInput,
.B pfuCollectGLEventStream,
.B pfuCollectXEventStream,
.B pfuMapMouseToChan,
.B pfuMouseInChan,
.B pfuMouseButtonClick,
.B pfuMouseButtonRelease,
.B pfuMapXTime
\- 
Initialize, process and reset input devices.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'double\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
void
\f3\%pfuInitInput\fP(pfPipeWindow\ *pipeWin,
'in +2
'll -2
int\ mode);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuInitMultiChanInput\fP(pfChannel\ **chanArray,
'in +2
'll -2
int\ numChans,
int\ mode);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuExitInput\fP(void);
.nf
.fi
.TP
void
\f3\%pfuGetMouse\fP(pfuMouse\ *mouse);
.nf
.fi
.TP
void
\f3\%pfuGetEvents\fP(pfuEventStream\ *events);
.nf
.fi
.TP
void
\f3\%pfuInputHandler\fP(pfuEventHandlerFuncType\ userFunc,
'in +2
'll -2
uint\ mask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuCollectInput\fP(void);
.nf
.fi
.TP
void
\f3\%pfuCollectGLEventStream\fP(pfuEventStream\ *events,
'in +2
'll -2
pfuMouse\ *mouse,
int\ handlerMask,
pfuEventHandlerFuncType\ handlerFunc);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuCollectXEventStream\fP(pfWSConnection\ dsp,
'in +2
'll -2
pfuEventStream\ *events,
pfuMouse\ *mouse,
int\ handlerMask,
pfuEventHandlerFuncType\ handlerFunc);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuMapMouseToChan\fP(pfuMouse\ *mouse,
'in +2
'll -2
pfChannel\ *chan);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuMouseInChan\fP(pfuMouse\ *mouse,
'in +2
'll -2
pfChannel\ *chan);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuMouseButtonClick\fP(pfuMouse\ *mouse,
'in +2
'll -2
int\ button,
int\ x,
int\ y,
double\ time);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuMouseButtonRelease\fP(pfuMouse\ *mouse,
'in +2
'll -2
int\ button,
int\ x,
int\ y,
double\ time);
.in
.ll
.nf
.fi
.TP
double
\f3\%pfuMapXTime\fP(double\ xtime);
.fi

.Ex
.nf


typedef struct _pfuMouse
{
    int		flags;		/* for PDEV_MOUSE_*_DOWN and PFUDEV_MOD_* bitmasks */
    int		modifiers;	/* modifier keys only  */

    int 	xpos, ypos;	/* Screen coordinates of mouse */
    float	xchan, ychan;	/* Normalized coordinates of mouse */
    double	posTime;	/* msec timestamp on current mouse position */

    /* These are used by the GUI and pfiXformer
     * GUI needs Last click positional info
     * Xformers need last and middle click and release info
     */
		    /* Mask of clicks seen last frame */
    int		click;
		    /* Last click position for each mouse button */
    int 	clickPos[PFUDEV_MOUSE_DOWN_MASK][2];
		    /* Screen coordinates where a mouse button was last clicked */
    int		clickPosLast[2];
		    /* mask of mouse releases seen last frame */
    int		release;
		    /* last release position for each mouse button */
    int 	releasePos[PFUDEV_MOUSE_DOWN_MASK][2];
		    /* Screen coordinates where a mouse button was last released*/
    int		releasePosLast[2];
		    /* Last click time for each mouse button */
    double	clickTime[PFUDEV_MOUSE_DOWN_MASK];
		    /* Time of last button click */
    double	clickTimeLast;
		    /* Last release time for each mouse button */
    double	releaseTime[PFUDEV_MOUSE_DOWN_MASK];
		    /* Time of last button release */
    double	releaseTimeLast;


    int      	winSizeX;	/* Window Size */
    int      	winSizeY;

    int	inWin;		/* Window focus flag */

} pfuMouse;


typedef void (*pfuEventHandlerFuncType)(int dev, void* val,
    pfuCustomEvent *pfuevent);
.Ee
.fi
.na

.SH DESCRIPTION

There are a variety of automatic and explicit event collection utilities
in libpfutil.  Automatic X or GL input event collection is started with
\f3\%pfuInitInput\fP and the resulting events can be queried with
\f3\%pfuInputHandler\fP and \f3\%pfuGetEvents\fP.

\f3\%pfuInitInput\fP initializes mouse and keyboard input to be read from the
specified \f3\%pfPipeWindow\fP \f2pipeWin\fP.  \f2mode\fP is one of:
.RS
.TP 5
\f3\%PFUINPUT_X\fP
Read mouse and keyboard from a forked process using X device commands.
\f2pipe\fP must have a GLX window.
See the \f3\%pfuGLXWinopen\fP reference page for more information.
.TP
\f3\%PFUINPUT_GL\fP
Read mouse and keyboard from the draw process using GL device
commands.  \f2pipe\fP must have a GL window.  This option is
no longer supported.
.RE
\f3\%pfuInitMultiChanInput\fP is a companion function to \f3\%pfuInit\fP but for
forked X input will create a separate forked input process for each
channel in the \f2chanArray\fP. This is useful for multipipe programs.

\f3\%pfuCollectInput\fP should be called from the draw process if the mode
is \f3\%PFUINPUT_GL\fP and will poll the mouse and collect all queued
devices.  The first time it is called, \f3\%pfuCollectInput\fP will queue
the following GL devices:
.Ex
.nf

WINQUIT
REDRAW
KEYBD
LEFTMOUSE
MIDDLEMOUSE
RIGHTMOUSE
INPUTCHANGE
.Ee
.fi
.na

.na

Any other required GL devices, such as function keys, should be queued
explicitly by the application.  \f3\%pfuCollectInput\fP is no longer supported.

If the mode is \f3\%PFUINPUT_X\fP then \f3\%pfuCollectInput\fP does not need to
be called since the device input is automatically collected by the forked
process.

\f3\%pfuInputHandler\fP installs the custom handler \f2userFunc\fP, which will
then be called to process each input event included in \f2mask\fP.
If the mode is \f3\%PFUINPUT_X\fP, then \f2mask\fP can be set to the
bitwise-or of an X input
mask with \f3\%PFUINPUT_CATCH_UNKNOWN\fP, \f3\%PFUINPUT_CATCH_SIM\fP,
or \f3\%PFU_CATCH_ALL\fP.  The \f3\%PFUINPUT_GL\fP mode is no longer supported.

\f3\%pfuGetMouse\fP copies the current mouse values from the libpfutil
event collector (initially triggered with \f3\%pfuInitInput\fP) into \f2mouse\fP and
\f3\%pfuGetEvents\fP copies the events of the current frame into \f2events\fP.
\f3\%pfuGetEvents\fP also resets the internal event queue.

\f3\%pfuMapMouseToChan\fP maps the mouse screen coordinates (mouse->xpos,
mouse->ypos) into coordinates in the range [-1, 1] (mouse->xchan, mouse->ychan)
based on \f2chan\fP's viewport.  Either \f3\%TRUE\fP or \f3\%FALSE\fP is returned to
indicate that the mouse is in or out of the \f2chan\fP's viewport.

\f3\%pfuMouseInChan\fP does the \f3\%pfuMapMouseToChan\fP mapping.
In addition, its return
value considers mouse focus if a mouse button is recorded as being down
in \f2mouse\fP.  In this case, the recorded position of where the
mouse button was clicked will determine if the current channel has focus.
The mouse will be considered to be "in" the channel of focus.

\f3\%pfuCollectGLEventStream\fP will do immediate IRIS GL input collection into
the provided pfuEventStream and pfuMouse structures. If the provided
event stream or mouse pointer is NULL, it will be ignored.
This routine must be called in the draw process.  This function is no longer
supported.

\f3\%pfuCollectXEventStream\fP will do immediate X input collection from the
provided pfWSConnection into
the provided pfuEventStream and pfuMouse structures. If the provided
event stream or mouse pointer is NULL, it will be ignored.

\f3\%pfuExitInput\fP must be called to terminate the forked X input process.

Call \f3\%pfuMouseButtonClick\fP to insert a mouse button click event into
the designated pfuMouse structure. The arguments \f2button\fP, \f2x\fP,
\f2y\fP, and \f2time\fP allow the button-click event to be localized in
screen-based space-time coordinates.

\f3\%pfuMouseButtonRelease\fP provides a similar capability for button-up
(release) events. As before, \f2button\fP specifies which button, \f2x\fP
and \f2y\fP identify the location of the virtual event, and \f2time\fP
denotes when the even is to be considered to have occurred.

\f3\%pfuMapXTime\fP performs the mapping from an X-event time \f2xtime\fP to
the OpenGL Performer time system.
.SH NOTES

\f3\%pfuInit\fP should be called immediately after \f3\%pfConfig\fP when using
these routines to initialize shared memory used by the utility library.
OpenGL Performer recommends that you use X device input. The OpenGL Performer
sample application, \f3\%perfly\fP, is shipped with \f3\%PFUINPUT_X\fP as
the default.  X device input is recommended for the following reasons:

.RS
OpenGL does not contain device input routines and all input must be
managed through X.

Collecting X device input in an asynchronous process can improve
real-time characteristics.
.RE

See the \f3\%pfuEventQueue\fP man page for a description of the
\f3\%pfuEventStream\fP structure.

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfuGLXWinopen,
pfuInit,
pfuEventQueue,
pfuGUI
