'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuLockCPU" "3pf"
.SH NAME
.upperok
.B pfuFreeAllCPUs,
.B pfuRunProcOn,
.B pfuLockDownProc,
.B pfuLockDownApp,
.B pfuLockDownCull,
.B pfuLockDownDraw,
.B pfuLockDownLPoint,
.B pfuPrioritizeProcs,
.B pfuRunDiskProcsOn
\- 
Priority, processes and processor assignment functions.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'int\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
int
\f3\%pfuFreeAllCPUs\fP(void);
.nf
.fi
.TP
int
\f3\%pfuRunProcOn\fP(int\ cpu);
.nf
.fi
.TP
int
\f3\%pfuLockDownProc\fP(int\ cpu);
.nf
.fi
.TP
int
\f3\%pfuLockDownApp\fP(void);
.nf
.fi
.TP
int
\f3\%pfuLockDownCull\fP(pfPipe\ *);
.nf
.fi
.TP
int
\f3\%pfuLockDownDraw\fP(pfPipe\ *);
.nf
.fi
.TP
int
\f3\%pfuLockDownLPoint\fP(pfPipe\ *);
.nf
.fi
.TP
int
\f3\%pfuPrioritizeProcs\fP(int\ pri);
.nf
.fi
.TP
int
\f3\%pfuRunDiskProcsOn\fP(int\ cpu);
.fi
.SH DESCRIPTION

These routines assign processes to CPUs and implement a policy
specifically designed for locking down the OpenGL Performer application,
cull, and draw processes.  The routines implementing these features
utilize the IRIX REACT facilities.  Refer to the IRIX REACT
technical report, and the \f3\%sysmp(2)\fP reference page for detailed
information on these concepts.

\f3\%pfuFreeAllCPUs\fP frees any CPUs which may have been previously restricted.

The routine \f3\%pfuRunProcOn\fP can be used to force a process to run on
a specified CPU and does not require super-user permission.  This is often
used to force extra processes that can run asynchronously from the
draw, such as those receiving and generating input, onto CPU 0 without
isolating that CPU from standard UNIX scheduling.
\f3\%pfuRunProcOn\fP forces the calling process to run on the specified CPU
and does not require super-user permission.

All of the \f3\%pfuLock<*>\fP routines force a process to run on the
specified CPU.  They also attempt to isolate the processor to run only
those processes that have specified that they must run on that CPU.
Isolating a CPU also protects it from seeing unnecessary cache and TLB
flushes generated by processes that have not specified that they must
run on this CPU.

\f3\%pfuLockDownProc\fP locks the calling process onto CPU \f2cpu\fP.  The CPU
is isolated to running only processes that have specified that they must
run on this CPU.  This CPU isolation requires super-user permission.

.SH OBSOLETE

The process control functionality  below is considered obsolete in Performer 2.2.
It is still provided for compatibility had has been updated to include new processes
and to use the POSIX scheduling if available on the current system
(requires 6.2 + patch1717, patchset 31 and patchset43 or 6.4 or later version of IRIX).
However, this functionality has been obsoleted by the much simpler and
more extensible \f3\%pfuProcessManager\fP.

The \f3\%pfuLockDown<App,Cull,Draw>\fP routines implement a policy for
selecting CPUs for different processors given the program and machine
configuration.

The locking and assignment policy implemented by these routines is
implemented in the various stages of an OpenGL Performer application
as follows.

1. CPU 0 is never isolated.

2. In the \f3\%APPCULLDRAW\fP mode, the processor assignment is handled by
the \f3\%APP\fP process which takes CPU 1.

3. In the \f3\%APP_CULLDRAW\fP mode, the processor assignment is handled
separately by the \f3\%APP\fP and \f3\%DRAW\fP processes.

4. In the \f3\%APP_CULL_DRAW\fP mode, each process handles itself.

When there is only one pipe, processors are mapped to processes as
follows. If there are three CPUs, each of \f3\%APP, CULL\fP and \f3\%DRAW\fP
gets its own process. If there are only two CPUs, \f3\%APP\fP is put on
CPU 0, which is not isolated, and \f3\%DRAW\fP and \f3\%CULL\fP share CPU 1.
The \f3\%LPOINT\fP process is given its own CPU if there are enough available CPUs.
Otherwise, it will be given then last CPU.

Multipipe mappings are as follows.
.Ex
.nf

If NumCPUs >= 2 + 2*NumPipes
then each cull and draw process can have its own CPU, with the application
getting CPU 1, and UNIX getting CPU 0.

If NumCPUs == 1 + 2*NumPipes
then the application shares CPU 0 with UNIX.

Otherwise, if NumCPUs >= 2 + NumPipes
then cull and draw processes for each pipe are paired together.

The LPoint processes are also  given their own CPUs if there are enough
available CPUs.  Otherwise, they are grouped together on a single CPU,
their own if there is on extra, else the last CPU.
.Ee
.fi
.na


If there are fewer CPUs then indicated above, then the application is
assigned to CPU 0 with UNIX, APP and CULL process are paired, and when
only one free CPU remains, all remaining processes are assigned to the
last CPU.

Each of these routines return 1 if successful and 0 if an error is
encountered.
\f3\%pfuLockDownApp\fP locks the APP process to a CPU determined by the
policy above.  The CPU is isolated to running only processes that have
specified that they must run on this CPU.  This CPU isolation requires
super-user permission.

\f3\%pfuLockDownCull\fP locks the CULL process of a pfPipe (there is one
CULL process per pfPipe) to a CPU determined by the policy above.  T
policy above.  This routine should be called from application's
\f3\%pfConfigStage\fP callback for the \f3\%PFPROC_CULL\fP stage.
The CPU is isolated to running only processes that
have specified that they must run on this CPU.  This CPU isolation
requires super-user permission.

\f3\%pfuLockDownDraw\fP locks the DRAW process of a pfPipe (there is one
draw process per pfPipe) to a CPU determined by the policy above.  This
routine should be called from application's \f3\%pfConfigStage\fP callback for
the \f3\%PFPROC_DRAW\fP stage.
See the \f3\%pfConfigStage\fP reference page for more information.  The CPU
is isolated to running only processes that have specified that they
must run on this CPU.  This CPU isolation requires super-user permission.

\f3\%pfuLockDownLPoint\fP locks the LPOINT process of a pfPipe (there up to
one optional light point process per pfPipe) to a CPU determined by the
policy above.  This routine should be called from application's
\f3\%pfConfigStage\fP callback for the \f3\%PFPROC_LPOINT\fP stage.  The
is isolated to running only processes that have specified that they
must run on this CPU.  This CPU isolation requires super-user permission.

\f3\%pfuRunDiskProcsOn\fP forces all threads servicing pfQueue's to run
on a certain cpu.  See the \f3\%pfQueue\fP man page for how pfQueue's can
be used to service disk threads.  A value of -1 passed as the \f2cpu\fP
argument will cause the routine to choose cpu 2*NumPipes+3 if it
exists or cpu 0 otherwise.

\f3\%pfuPrioritizeProcs\fP should be called after \f3\%pfConfig\fP and will
set or remove non-degrading priorities from all Performer processes.
\f2pri\fP is a boolean: if TRUE all OpenGL Performer processes will be
assigned a non-degrading priority of NDPHIMAX+2 (see \f3\%schedctl\fP); if
FALSE any non-degrading priorities will be removed.  You must have
super-user permission to enable non-degrading priorities but not to
remove them.

If you wish to assign different priorities to different processes, simply
modify \f3\%pfuPrioritizeProcs\fP to suit your needs.
.SH NOTES


Processor lock-down functionality is not currently available under Linux.

Isolating a CPU to specific processes requires super-user permission.

CPU 0 should \f2never\fP be isolated.

A utility shell script is provided in /usr/share/Performer/src/tools/freeproc
that can be run as super-user to clean up after a program that exits ungracefully.

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.

.SH SEE ALSO
pfuProcessManager,
pfConfig,
pfConfigStage,
sysmp,
schedctl,
sched_setscheduler
