'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuMPipeStats" "3pf"
.SH NAME
.upperok
.B pfuManageMPipeStats
\- 
Multipipe/multichannel stats utility.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'int\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
int
\f3\%pfuManageMPipeStats\fP(int\ nframes,
'in +2
'll -2
nSampledPipes);
.in
.ll
.fi
.SH DESCRIPTION


This utility obtains time stamp stats for each Performer channel/pipe
in an automatic fashion. The complete stats log is stored in a file
for later review. Timing information for pre- and post-callbacks is
also provided.

The routine \f3\%pfuManageMPipeStats\fP can be used to obtain timing
information during a number of frames per each channel for pre-draw,
\f3\%pfDraw\fP, post-draw, pre-cull, \f3\%pfCull\fP, post-cull as well as
\f3\%ISECT\fP and \f3\%APP\fP.

\f2nframes\fP specifies the number of frames to be measured.
\f2nSampledPipes\fP is the number of pipes for which statistics
are to be gathered.

The first time that \f3\%pfuManageMPipeStats\fP is called it automatically
determines the pipe and channel configuration.  It also sets up a
minimally configured \f3\%pfStats\fP in order to obtain the data.


During the requested number of frames, the utility grabs the latest
\f3\%pfStats\fP buffer into main memory. After the requested number of
frames, the utility creates a new file called \f3\%MPipeStats0.log\fP.
The file number is incremented each time that the utility is
used from the same execution, so that the next time it is used the
file have the name \f3\%MPipeStats1.log\fP and so on.

\f3\%pfuManageMPipeStats\fP returns 1 while it is getting time stamps and
0 when it has finished its collection and created the historical file.

\f3\%pfuManageMPipeStats\fP must be called in the APP. It may be used
multiple times.

Following is an example of use in an application loop:

.Ex
.nf

/* DumpMPipeStats is controlled by the user interface */

if(SharedArena->DumpMPipeStats && !mpstats_running)
    mpstats_running = pfuManageMPipeStats(10, 1));

if(mpstats_running)
    mpstats_running = pfuManageMPipeStats(10, 1));
.Ee
.fi
.na


The file format used by \f3\%pfuManageMPipeStats\fP is offered only as an example.
The meaning and lay out is as follows:

.Ex
.nf

====================================================================
Frame:<frame>
_____________________________APP____________________________________
Absolute Timeline information (sec)
...................................
AppFrame:<frame>
AppStart:<start>
enterSync:<enterSync> afterClean:<afterClean> afterSync:<afterSync>
pfFrameStart:<frameStart> pfFrameEnd:<frameEnd>
_____________________________ISECT__________________________________
Absolute Timeline information (sec)
...................................
IsectFrame: <frame>
Start:<start> End:<end>

Relative Timeline information (msec)
....................................
Total Isect Time:<end-start>
_____________________________CULL & DRAW____________________________
Chan: <Channel_number>
Frame: <frame_number>

Absolute Timeline information (sec)
...................................
CullFrame: <cullFrame>
    Start: <cullStart> End: <cullEnd>
    BeginUpDate: <begUpdate> EndUpdate: <endUpdate>
DrawFrame: <drawFrame>
    Start: <drawStart> End: <drawEnd>
    pfDrawStart: <pfDrawStart> pfDrawEnd: <pfDrawEnd> AfterSwap: <AfterSwap>

Relative Timeline information (msec)
....................................
Total Cull Time:<cullEnd-cullStart>
Total Draw Time:<drawEnd-drawStart>
    PreDraw: <preDraw>
    pfDraw:  <pfDraw>
    PostDraw:<postDraw>
--------------------------------------------------------------------
.Ee
.fi
.na


You are encouraged to change the default file format as well as the data
being collected.

.SH NOTES


Depending on the multiprocess mode being used a number of initial
frames could be filled by nonsensical data.

The use of \f3\%pfuManageMPipeStats\fP modifies the \f3\%pfStats\fP mode.
After it completes it sets the \f3\%pfStats\fP modes to their default values.

Hence, if you wish to reuse settings from the previous \f3\%pfStats\fP you should save
them before the first call to \f3\%pfuManageMPipeStats\fP.

CPU 0 should \f2never\fP be isolated.

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.

.SH SEE ALSO
pfFrameStats
