'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuPath" "3pf"
.SH NAME
.upperok
.B pfuNewPath,
.B pfuClosePath,
.B pfuCopyPath,
.B pfuSharePath,
.B pfuPrintPath,
.B pfuFollowPath,
.B pfuAddArc,
.B pfuAddDelay,
.B pfuAddFile,
.B pfuAddFillet,
.B pfuAddPath,
.B pfuAddSpeed
\- 
Simple path-following utility
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfuPath\ *\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
pfuPath *
\f3\%pfuNewPath\fP(void);
.nf
.fi
.TP
pfuPath *
\f3\%pfuClosePath\fP(pfuPath\ *path);
.nf
.fi
.TP
pfuPath *
\f3\%pfuCopyPath\fP(pfuPath\ *copy);
.nf
.fi
.TP
pfuPath *
\f3\%pfuSharePath\fP(pfuPath\ *share);
.nf
.fi
.TP
int
\f3\%pfuPrintPath\fP(pfuPath\ *path);
.nf
.fi
.TP
int
\f3\%pfuFollowPath\fP(pfuPath\ *path,
'in +2
'll -2
float\ seconds,
pfVec3\ where,
pfVec3\ orient);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuAddArc\fP(pfuPath\ *path,
'in +2
'll -2
pfVec3\ center,
float\ radius,
pfVec2\ angles);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuAddDelay\fP(pfuPath\ *path,
'in +2
'll -2
float\ delay);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuAddFile\fP(pfuPath\ *path,
'in +2
'll -2
char\ *name);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuAddFillet\fP(pfuPath\ *path,
'in +2
'll -2
float\ radius);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuAddPath\fP(pfuPath\ *path,
'in +2
'll -2
pfVec3\ start,
pfVec3\ final);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuAddSpeed\fP(pfuPath\ *path,
'in +2
'll -2
float\ desired,
float\ rate);
.in
.ll
.fi
.SH DESCRIPTION

The \f3\%pfuPath\fP functions provide a simple way to move one or more
simulated vehicles or eyepoints along a mathematically defined path.
The path is a general series of arcs and line segments.  Once a path is
created, it can be followed each simulation frame to provide position
and orientation information.

\f3\%pfuNewPath\fP allocates and initializes a new \f3\%pfuPath\fP structure.  Once
the path is opened, segments (both straight and curved) can be added
using the \f3\%pfuAddPath\fP and \f3\%pfuAddArc\fP commands as described
below.  The initial path speed is set at 1.0 database units per
second.

\f3\%pfuClosePath\fP connects the end point of the final segment of path
\f2path\fP with the start point of the path's first segment.  This
creates a closed (also known as looping) path that can be followed
endlessly. Paths can be either open or closed, but they should only be
closed once. The closed path is returned.

\f3\%pfuCopyPath\fP creates and returns a new \f3\%pfuPath\fP structure that has
path segments which are an exact copy of those in the path indicated by
the \f2path\fP argument. This is a deep copy, and will create a new
instance of each element in the original path definition.

\f3\%pfuSharePath\fP creates a new \f3\%pfuPath\fP structure, but causes the linked
list of path segments in the path structure \f2path\fP to be shared by
both old and new path structures. This allows multiple simulated
objects to follow the same path without the redundant allocation of
path storage, and also allows changes to the path segment definition to
effect all \f3\%pfuPath\fP structures that share it.

\f3\%pfuPrintPath\fP prints the path following control information for path
object \f2path\fP, and then prints the definition of each segment in the
path itself. All printing is done using the OpenGL Performer \f3\%pfNotify\fP
mechanism with a severity level of \f3\%PFNFY_DEBUG\fP.

\f3\%pfuFollowPath\fP performs the actual simulation of moving an object
along the path indicated by \f2path\fP. The \f2seconds\fP argument
specifies the simulated duration, and internal data in the \f3\%pfuPath\fP
structure supplies the speed. From this time and speed information, the
vehicle's simulated distance of travel is computed. Then, the vehicle
is moved this distance along the path segments defined within the
\f3\%pfuPath\fP object. The resulting simulated position is returned in the
\f3\%pfVec3\fP argument \f2where\fP and the orientation of the vehicle is
returned in \f2orient\fP.

\f3\%pfuAddArc\fP adds a circular arc path segment to the \f3\%pfuPath\fP structure
\f2path\fP. The arc is defined by the \f2center\fP and \f2radius\fP values,
and a pair of \f2angles\fP. The first angle, \f2angle[0]\fP, is the start
angle for the path, and is a point on the circle defined by \f2center\fP
and \f2radius\fP with the indicated counterclockwise angle from the
positive X axis.  The second angle, \f2angle[1]\fP, represents the turn
angle, the angle between the arc's start and end points as measured
from the designated center point. Positive turn angles indicate
counterclockwise turns, and negative angles represent clockwise
turns. An arc from the +X axis to the +Y axis would be defined with
a start angle of 0 degrees and a turn angle of 90 degrees. The same
arc in the opposite direction of travel is defined by a start angle
of 90 degrees and a turn angle of -90 degrees.

\f3\%pfuAddDelay\fP adds a zero-length segment to \f2path\fP that causes
the simulated vehicle to stop for the \f2delay\fP seconds at that point
in the path. Once this much simulated time elapses, the simulation
will continue with the next segment in the path. Delay segments can
be used to simulate motor vehicles paused waiting for traffic signals
and similar latent delay sources along a path.

\f3\%pfuAddFile\fP adds a series of path segments defined by a simple
ASCII file format to the \f3\%pfuPath\fP structure \f2path\fP. This function
provides an easy way to load user specified paths for vehicles into
simulation applications, and is used in the popular SGI
"Performer Town" demonstration program to load the paths for the
truck into the program.

\f3\%pfuAddFillet\fP is used to create circular arcs that join two
adjacent straight path segments. Fillet creation is a three step
process.

.RS
.TP 5
1.
A straight segment is added to a path using \f3\%pfuAddPath\fP. This
is the segment that will lead into the fillet.
.TP
2.
A fillet request is added using \f3\%pfuAddFillet\fP. The fillet's
radius is defined in this call.
.TP
3.
A second straight segment is added, again with \f3\%pfuAddPath\fP. This
segment must start where the segment of step one ended.
.RE

When the second segment is added in step three, a test is performed
to see if the first point in the second segment has the same X,
Y, and Z values as the last point in the previous segment. If
so, and if there is a fillet request between the two segments, then
a matching fillet of the specified radius is computed. In addition,
the endpoints (and thus lengths) of the two straight segments are
adjusted so as to match with the fillet endpoints.

For example, the request

.RS
line from (0,0) to (1,0)
.br
fillet of radius 0.25
.br
line from (1,0) to (1,1)
.RE

will cause the fillet arc and line segment lengths to be automatically
computed as

.RS
line from (0,0) to (0.75,0)
.br
fillet of radius 0.25 with center (0.75,0.25) and angles 270 and 90.
.br
line from (1,0.25) to (1,1)
.RE

This automatic fillet construction is seen to be very convenient
once the alternative manual fillet construction is attempted. When
the fillet radius is too large to allow the arc to be created (as
would be the case in the example above with a radius greater than
one), a fillet is not constructed and a sharp turn will exist in
the path.

\f3\%pfuAddPath\fP adds a straight line segment to \f2path\fP. The line
segment is defined as extending from \f2start\fP to \f2final\fP. As
mentioned above, if the segment preceding the line segment is an
unevaluated fillet, and the value of \f2start\fP is equal to that of
the \f2final\fP point of the segment before the fillet, then an
automatic fillet will be constructed if the fillet's radius
specification is sufficiently small to allow it.

\f3\%pfuAddSpeed\fP adds a speed changing segment to \f2path\fP. The
new speed is indicated by \f2desired\fP and the rate of adjustment
from the current path speed to the new speed is given by \f2rate\fP.

.SH NOTES

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfNotify
