'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuRendezvous" "3pf"
.SH NAME
.upperok
.B pfuInitRendezvous,
.B pfuMasterRendezvous,
.B pfuSlaveRendezvous
\- 
Multiprocessing master and slave rendezvous routines
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
void
\f3\%pfuInitRendezvous\fP(pfuRendezvous\ *rvous,
'in +2
'll -2
int\ numSlaves);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuMasterRendezvous\fP(pfuRendezvous\ *rvous);
.nf
.fi
.TP
void
\f3\%pfuSlaveRendezvous\fP(pfuRendezvous\ *rvous,
'in +2
'll -2
int\ id);
.in
.ll
.fi

.Ex
.nf

\#define PFURV_MAXSLAVES         2

\#define PFURV_GARBAGE           -1
\#define PFURV_READY             10
\#define PFURV_SYNC              11
\#define PFURV_SYNCACK           12
\#define PFURV_RESUME            13

typedef struct _pfuRendezvous
{
    int master;
    int numSlaves;
    int slaves[PFURV_MAXSLAVES];
} pfuRendezvous;
.Ee
.fi
.na

.SH DESCRIPTION

These rendezvous functions are useful for synchronizing master and
slave processes in a multiprocessing environment.

In the case of multiple processes, a rendezvous is a method for
synchronizing each independent process.  A process chosen as the
"master" waits for the remaining processes, designated "slaves",
to indicate through the rendezvous that they are ready to synchronize.
As each slave indicates its readiness, it then waits on the master
process.  The master process releases the slaves after all slaves
have made the rendezvous.  The rendezvous token should be allocated in
a shared arena; all processes require access to it.

\f3\%pfuInitRendezvous\fP initializes the rendezvous token \f2rvous\fP for
one master process and \f2numSlaves\fP slave processes.

In order to synchronize multiple processes, the master should call
\f3\%pfuMasterRendezvous\fP and each of the slaves should call
\f3\%pfuSlaveRendezvous\fP with its slave ID in \f2id\fP.  Slave IDs can
range from 0 to (\f3\%PFURV_MAXSLAVES\fP - 1).

.SH NOTES

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfGetSharedArena,
pfMalloc,
pfuBarrier
