'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuStyle" "3pf"
.SH NAME
.upperok
.B pfuPreDrawStyle,
.B pfuPostDrawStyle
\- 
Functions to produce fancy drawing styles.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'void\ \ \ 'u
#include <Performer/pf.h>
.nf
.fi
.TP
#include <Performer/pfutil.h>
.nf
.fi
.TP
void
\f3\%pfuPreDrawStyle\fP(int\ style,
'in +2
'll -2
pfVec4\ scribeColor);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuPostDrawStyle\fP(int\ style);
.fi
.SH PARAMETERS
.TP \w'style\ \ 'u
\f2style\fP
identifies a drawing style.
.SH DESCRIPTION

Both \f3\%pfuPreDrawStyle\fP and \f3\%pfuPostDrawStyle\fP use multi-pass
rendering to implement special draw styles that are not directly
provided by the hardware or graphics library. These effects, such
as hidden line rendering and haloed polygons, can however be
achieved with high-performance Z-buffered graphics rendering
hardware as these functions demonstrate.

\f3\%pfuPreDrawStyle\fP should be called in the Performer draw callback
just before \f3\%pfDraw\fP. \f3\%pfuPostDrawStyle\fP must then be called after
\f3\%pfDraw\fP. These two functions are a pair and both must be called as
described.

The \f2style\fP argument can take the following values.
.RS
.TP 5
\f3\%PFUSTYLE_POINTS\fP
This causes polygon vertices to be drawn as small points.
In many cases, internal structural aspects can be observed
from the resulting "cloud of points" image, especially when
that image is in motion on the screen.
.TP
\f3\%PFUSTYLE_LINES\fP
This causes polygons to be drawn in wireframe mode using
the Performer \f3\%PFGS_WIREFRAME\fP draw style. The lines are
drawn in the same color as the base geometry.
.TP
\f3\%PFUSTYLE_DASHED\fP
This is a wireframe drawing style that does something slightly
tricky: it draws the front-facing polygons of an object in the
normal wireframe mode and the back-facing polygons in a dashed
line mode, creating a typical draftsman's dashed-occlusion hint
style of hidden line drawing.
.TP
\f3\%PFUSTYLE_HALOED\fP
Polygons are drawn in wireframe mode with haloed edges. This is
the drawing style of electronic schematics, where lines that
cross but are not connected have a slight gap or break in the
lower line. This same idea can be extended to 3D geometry, and
was proposed as a rendering mode by graphics pioneer
Dr.\ Arthur Appel.
.TP
\f3\%PFUSTYLE_HIDDEN\fP
Hidden polygons are not drawn while visible polygons are drawn in
wireframe mode. This is the traditional hidden-line removal
display mode.
.TP
\f3\%PFUSTYLE_FILLED\fP
Draws polygons as filled solids. This hidden-surface removal
display mode is the standard display style.
.TP
\f3\%PFUSTYLE_SCRIBED\fP
Polygons are drawn filled with hidden surfaces removed. The
boundary of each visible polygon is highlighted in a wire
frame highlight. This mode can be very useful in understanding
the geometric complexity of textured scenes.
.RE

When \f2style\fP takes the values \f3\%PFUSTYLE_POINTS\fP or \f3\%PFUSTYLE_DASHED\fP,
\f3\%pfuPostDrawStyle\fP restores polygon drawing to the normal filled mode.
Finally, for every value of \f2style\fP, it restores the state before
\f3\%pfuPreDrawStyle\fP was called.

The \f2scribeColor\fP argument is the desired color for scribed lines. The
first three elements of the array represent the red, green, and blue values
in the range 0 to 1, and the final element is the alpha value for the
scribed lines. This argument is not used in the other style modes.

The following code fragment displays how \f3\%pfuPreDrawStyle\fP and
\f3\%pfuPostDrawStyle\fP would typically be used in an application. See
the files:
.Ex
.nf

/usr/share/Performer/src/sample/C/perfly.c
/usr/share/Performer/src/sample/C++/perfly.C
.Ee
.fi
.na

for details.

Example 1:
.Ex
.nf

/* convey draw style from localPreDraw to localPostDraw */
static int selectedDrawStyle = 0;

void
localPreDraw(pfChannel *chan, void *data)
{
  :
  :
    /*
     * remember draw style in case it changes between now
     * and the time localPostDraw() gets called.
     */
    selectedDrawStyle = ViewState->drawStyle;

    /* handle draw style */
    pfuPreDrawStyle(selectedDrawStyle, ViewState->drawColor);
}

void
localPostDraw(pfChannel *chan, void *data)
{
    /* handle draw style */
    pfuPostDrawStyle(selectedDrawStyle);
  :
  :
}
.Ee
.fi
.na

.SH SEE ALSO
pfDraw
