'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuTimer" "3pf"
.SH NAME
.upperok
.B pfuNewTimer,
.B pfuInitTimer,
.B pfuStartTimer,
.B pfuEvalTimers,
.B pfuEvalTimer,
.B pfuStopTimer,
.B pfuActiveTimer
\- 
Benchmarking and interval timing facilities.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfuTimer\ *\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
pfuTimer *
\f3\%pfuNewTimer\fP(void\ *arena,
'in +2
'll -2
int\ size);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuInitTimer\fP(pfuTimer\ *timer,
'in +2
'll -2
double\ start,
double\ delta,
void\ (*func)(pfuTimer*),
void\ *data);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuStartTimer\fP(pfuTimer\ *timer);
.nf
.fi
.TP
void
\f3\%pfuEvalTimers\fP(void);
.nf
.fi
.TP
int
\f3\%pfuEvalTimer\fP(pfuTimer\ *timer);
.nf
.fi
.TP
void
\f3\%pfuStopTimer\fP(pfuTimer\ *timer);
.nf
.fi
.TP
int
\f3\%pfuActiveTimer\fP(pfuTimer\ *timer);
.fi

.Ex
.nf

struct _pfuTimer
{
    double tstart, tstop, tdelta;

    int	 frames;
    double tnow;
    double fraction;

    void	 (*func)(struct _pfuTimer *timer);
    void	 *data;
    int	 dataSize;
};
typedef struct _pfuTimer pfuTimer;
.Ee
.fi
.na

.SH DESCRIPTION

\f3\%pfuTimers\fP provide a real-time, frame rate independent mechanism for
defining time-based behavior.  A \f3\%pfuTimer\fP is typically initialized with
start and stop times, a callback function, and is triggered with
\f3\%pfuStartTimer\fP.  Then each time \f3\%pfuEvalTimer\fP is called,
the function callback will be invoked and can then carry out
actions based on the current time.

\f3\%pfuNewTimer\fP creates a new \f3\%pfuTimer\fP structure in \f2arena\fP as well
as a user-data memory block of \f2size\fP bytes that is referenced by the
\f2data\fP member of the \f3\%pfuTimer\fP structure.

\f3\%pfuInitTimer\fP initializes the starting time (\f2tstart\fP), duration
(\f2tdelta\fP), function
callback (\f2func\fP), and user data (\f2data\fP) of \f2timer\fP to
\f2start\fP, \f2delta\fP,
\f2func\fP
and \f2data\fP.  User data is copied by value, not by reference.

\f3\%pfuStartTimer\fP starts \f2timer\fP and adds it to a static list of
active timers.  All timers in this list are evaluated by \f3\%pfuEvalTimers\fP
and a single, activated timer may be evaluated by \f3\%pfuEvalTimer\fP.
A timer must be evaluated for its function callback to be invoked.

When a timer is evaluated, the current time is checked against its
active interval defined by [\f2tstart\fP, \f2tstop\fP].  If the current time
is within this interval, \f2func\fP will be invoked with a pointer to
the \f3\%pfuTimer\fP.  The following elements of the timer will be set:
.RS
.TP
\f2frames\fP
The number of times the pfuTimer has been evaluated,
.TP
\f2tnow\fP
The current time in seconds,
.TP
\f2fraction\fP
The fraction of the interval that has passed,
(\f2tnow\fP-\f2tstart\fP)/(\f2tstop\fP-\f2tstart\fP). This ranges from 0 to 1.
.TP
\f2data\fP
The user-data memory block referenced by the timer.
.RE

If the current time is not within the active interval, the timer will
be removed from the internal timer list.

\f3\%pfuStopTimer\fP stops \f2timer\fP and removes it from the internal timer list.

\f3\%pfuActiveTimer\fP returns \f3\%TRUE\fP if \f2timer\fP is active and \f3\%FALSE\fP
otherwise.
.SH NOTES

\f3\%pfuTimer\fP utilizes \f3\%pfGetTime\fP and its accuracy is dependent on the
resolution of available system timers.  See \f3\%pfGetTime\fP for more details.

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfGetTime,
pfInitClock
