'\"! eqn | tbl | mmdoc
'\"macro stdmacro
.TH "pfuTraverser" "3pf"
.SH NAME
.upperok
.B pfuInitTraverser,
.B pfuTraverse,
.B pfuTravPrintNodes,
.B pfuTravGLProf,
.B pfuTravCountDB,
.B pfuTravNodeHlight,
.B pfuTravNodeAttrBind,
.B pfuTravCalcBBox,
.B pfuTravCountNumVerts,
.B pfuTravSetDListMode,
.B pfuTravCompileDLists,
.B pfuTravDrawDLists,
.B pfuTravCreatePackedAttrs,
.B pfuFillGSetPackedAttrs,
.B pfuDelGSetAttrs,
.B pfuTravCachedCull,
.B pfuCalcDepth,
.B pfuLowestCommonAncestor,
.B pfuLowestCommonAncestorOfGeoSets,
.B pfuFindTexture,
.B pfuFindClipTextures
\- 
Useful scene graph traversals.
.SH FUNCTION SPECIFICATION
.nf
.fi
.TP \w'pfNode\ *\ \ \ 'u
#include <Performer/pfutil.h>
.nf
.fi
.TP
void
\f3\%pfuInitTraverser\fP(pfuTraverser*\ trav);
.nf
.fi
.TP
int
\f3\%pfuTraverse\fP(pfNode*\ node,
'in +2
'll -2
pfuTraverser*\ trav);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTravPrintNodes\fP(pfNode\ *node,
'in +2
'll -2
const\ char\ *fname);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTravGLProf\fP(pfNode\ *node,
'in +2
'll -2
int\ mode);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTravCountDB\fP(pfNode\ *node,
'in +2
'll -2
pfFrameStats\ *fstats);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTravNodeHlight\fP(pfNode\ *node,
'in +2
'll -2
pfHighlight\ *hl);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTravNodeAttrBind\fP(pfNode\ *node,
'in +2
'll -2
unsigned\ int\ attr,
unsigned\ int\ bind);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTravCalcBBox\fP(pfNode\ *node,
'in +2
'll -2
pfBox\ *box);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuTravCountNumVerts\fP(pfNode\ *node);
.nf
.fi
.TP
void
\f3\%pfuTravSetDListMode\fP(pfNode\ *node,
'in +2
'll -2
int\ enable);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTravCompileDLists\fP(pfNode\ *node,
'in +2
'll -2
int\ delMask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTravDrawDLists\fP(pfNode\ *node);
.nf
.fi
.TP
void
\f3\%pfuTravCreatePackedAttrs\fP(pfNode\ *node,
'in +2
'll -2
int\ format,
int\ delMask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuFillGSetPackedAttrs\fP(pfGeoSet\ *gset,
'in +2
'll -2
int\ mask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuDelGSetAttrs\fP(pfGeoSet\ *gset,
'in +2
'll -2
int\ delMask);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuTravCachedCull\fP(pfNode*\ node,
'in +2
'll -2
int\ numChans);
.in
.ll
.nf
.fi
.TP
int
\f3\%pfuCalcDepth\fP(pfNode\ *node);
.nf
.fi
.TP
pfNode *
\f3\%pfuLowestCommonAncestor\fP(pfNode\ *scene,
'in +2
'll -2
int\ (*fun)(pfNode*,
void*),
void\ *arg,
int\ travmode);
.in
.ll
.nf
.fi
.TP
pfNode *
\f3\%pfuLowestCommonAncestorOfGeoSets\fP(pfNode\ *scene,
'in +2
'll -2
int\ (*fun)(pfGeoSet*,
void*),
void\ *arg,
int\ travmode);
.in
.ll
.nf
.fi
.TP
pfNode *
\f3\%pfuFindTexture\fP(pfNode\ *scene,
'in +2
'll -2
int\ n,
(*fun)(pfGeoState*,
pfTexture*,
void*),
void\ *arg,
int\ travmode);
.in
.ll
.nf
.fi
.TP
void
\f3\%pfuFindClipTextures\fP(pfList\ *list,
'in +2
'll -2
pfNode\ *scene);
.in
.ll
.fi

.Ex
.nf

struct _pfuTraverser
{
    pfuTravFuncType       preFunc;
    pfuTravFuncType       postFunc;
    int                   mode;
    int                   depth
    pfNode               *node;
    pfMatStack           *mstack;
    void                 *data;
};

typedef struct _pfuTraverser pfuTraverser;
typedef int (*pfuTravFuncType)(pfuTraverser *trav);
.Ee
.fi
.na

.SH DESCRIPTION

\f3\%pfuTraverser\fP provides a customizable, recursive traversal of an
OpenGL Performer scene graph.  Traversals are useful for many things
including database queries like "find and activate all the
\f3\%pfSequence\fP nodes in this scene graph".

The \f3\%pfuTraverser\fP facility is used by initializing a \f3\%pfuTraverser\fP
structure and invoking \f3\%pfuTraverse\fP with the \f3\%pfuTraverser\fP and a
target \f3\%pfNode\fP.  Custom database processing is carried out in the
pre- and post-traversal callbacks that are provided by the user.

\f3\%pfuInitTraverser\fP initializes \f2trav\fP to the following:
.Ex
.nf

preFunc = postFunc = NULL
mode = \f3\%PFUTRAV_SW_ALL\fP | \f3\%PFUTRAV_LOD_ALL\fP | \f3\%PFUTRAV_SEQ_ALL\fP
mstack = NULL
data = NULL
.Ee
.fi
.na


\f2mode\fP is a bitmask indicating how to traverse nodes which have an
explicit (\f3\%pfSwitch\fP) or implicit (\f3\%pfLOD\fP) switch.  The \f3\%ALL\fP,
\f3\%NONE\fP, and \f3\%CUR\fP forms of the \f3\%PFUTRAV\fP tokens indicate that
the traversal should traverse all, none, or just the current child of
the specified node type, e.g. \f3\%PFUTRAV_SEQ_CUR\fP will traverse the
currently selected child of all \f3\%pfSequence\fP nodes.

\f2preFunc\fP and \f2postFunc\fP are callbacks which are invoked before and
after each node in the hierarchy is visited.  Callbacks are passed a pointer
to the current \f3\%pfuTraverser\fP structure and may access elements of the
\f3\%pfuTraverser\fP.

The \f2node\fP member references the current node in the traversal.
\f2data\fP is a pointer to user-data and if non-NULL, \f2mstack\fP will
contain the matrix stack of the traversal, i.e. all \f3\%pfSCS\fP and
\f3\%pfDCS\fP nodes will push, multiply and pop the \f3\%pfMatStack\fP
represented by \f2mstack\fP.  Memory management of the \f3\%pfMatStack\fP`
is the responsibility of the application.

The following functions provide a few specific examples of the \f3\%pfuTraverser\fP
utility.  The specific traversals implemented are:
.RS
.TP 5
\f3\%pfuTravPrintNodes\fP
Recursively prints all nodes below \f2node\fP into file \f2fname\fP.
.TP
\f3\%pfuTravGLProf\fP
Outputs GLprof output tags.  If \f2mode\fP is true then it places glprof tag
callbacks.
.TP
\f3\%pfuTravCountDB\fP
Accumulates stats for scene graph \f2node\fP into stats structure \f2fstats\fP
by traversing \f2node\fP with stats open.
.TP
\f3\%pfuTravNodeHlight\fP
Highlights all nodes below \f2node\fP by recursive traversal and calling
a highlight routine on each.
.TP
\f3\%pfuTravNodeAttrBind\fP
Recursively traverses \f2node\fP setting \f2attr\fP
of each node under \f2node\fP to the value \f2bind\fP.
.TP
\f3\%pfuTravCalcBBox\fP
Computes the bounding box of \f2node\fP and returns
it in the parameter \f2box\fP.  If the \f2node\fP parameter is \f3\%NULL\fP,
the returned box will be an empty box.
.TP
\f3\%pfuTravCountNumVerts\fP
Returns the number of vertices in the scene
graph rooted by \f2node\fP.
.TP
\f3\%pfuTravCachedCull\fP
Installs callback functions for each node which cache CULL results
between frame updates.
.TP
\f3\%pfuCalcDepth\fP
Returns the depth of the scene graph pointed to by \f2node\fP.  A single
root node with no children is counted as having depth 1.
.TP
\f3\%pfuLowestCommonAncestor\fP
Return the lowest common ancestor node of all nodes in the scene graph
rooted by \f2scene\fP for which \f2fun\fP(node, \f2arg\fP) != 0,
using the traversal mode given by \f2travmode\fP.
.TP
\f3\%pfuLowestCommonAncestorOfGeoSets\fP
Return the lowest common ancestor node of all geosets in the scene graph
rooted by \f2scene\fP for which \f2fun\fP(geoset, \f2arg\fP) != 0,
using the traversal mode given by \f2travmode\fP.
.TP
\f3\%pfuFindTexture\fP
Return the \f2n\fP'th texture occuring in the traversal of the scene graph
rooted by \f2scene\fP for which \f2fun\fP(geostate, texture, \f2arg\fP) != 0,
using the traversal mode given by \f2travmode\fP.
Note that textures occuring multiple times will be counted multiple times;
this behavior is a bug and may change in future versions.
.TP
\f3\%pfuTravSetDListMode\fP
Enables and disables the pfGeoSet display-list
rendering mode on all nodes beneath \f2node\fP. The value of \f2enable\fP
guides the processing: non-zero \f2enable\fP will force display list mode
and a zero value will prevent it.   If display lists for pfGeoSets
in the subtree are dirty or non-existant, they will be recreated
as they are drawn.  Note that pfGeoSets under pfLODs or pfSwitch nodes
may not be immediately drawn, as pfGeoSets not in view.  To make
sure all pfGeoSets are compiled immediately, the \f3\%pfuTravCompleDLists\fP
traversal should be used.
.TP
\f3\%pfuTravCompleDLists\fP
Will immediately compile all pfGeoSets beneath \f2node\fP.
The \f2delMask\fP argument indicates
which (if any) of the pfGeoSet attributes (vertex position, color,
normal, and texture coordinate) are to be deleted once the
display list is built to minimize storage consumption. Once these
attributes are deleted, then returning to immediate-mode rendering
via a second call to \f3\%pfuTravSetDListMode\fP will result in empty
pfGeoSets and lost geometry. This traversal must be executed in
the DRAW process, or in a thread that is sproced or in the same
address as the DRAW as it requires an active GL contect to create
the GL display lists for the DRAW.  This traversal will reach all
pfGeoSets immediately for compilation.  \f3\%pfuTravSetDListMode\fP
can then be used to select the display list rendering mode.
.TP
\f3\%pfuTravDrawDLists\fP
will draw all pfGeoSets under \f2node\fP
that have a draw mode of \f3\%PFGS_DRAW_GLOBJ\fP.  On an
InfiniteReality graphics platform, this will  cause the GL display
lists to be downloaded into the local display list memory in the
graphics subsystem for future fast rendering.
Even on other graphics platforms,  there is other additional optimization
that might be done on the first rendering of a display list.
.TP
\f3\%pfuTravCreatePackedAttrs\fP
This traversal builds a special packed pfGeoSet attribute array
representation (based on the OpenGL VertexArray facility) that can
speed rendering in many cases by reducing host overhead in data
transmission to the pipeline. The \f2format\fP argument indicates
which style of packed attribute data should be used. Refer to the
header files for details of the supported formats, such as
PFGS_PA_C4UBN3ST2FV3F and PFGS_PA_C4UBN3ST2F).
.RE

\f3\%pfuFillGSetPackedAttrs\fP constructs the packed attribute array
representation of \f2gset\fP using \f2mask\fP to determine what to
pack and how to pack it.

\f3\%pfuDelGSetAttrs\fP deletes the packed attribute arrays of \f2gset\fP
using \f2delMask\fP as a guide for the deletion operation.

\f3\%pfuFindClipTexture\fP
traverses the nodes in the scenegraph whose root node is scene, finding
all the cliptextures in the scene, and collecting pointers to them in
the pfList list.

.SH NOTES

The libpfutil source code, object code and documentation are provided
as unsupported software.  Routines are subject to change in future
releases.
.SH SEE ALSO
pfFrameStats,
pfGeoSet,
pfNode,
pfTraverser
